<?php
namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Mail;
use App\Models\Retails\Location;
use App\Models\Account\User;
use App\Models\Account\SubscriptionPlans;
use Carbon\Carbon;
use Illuminate\Support\Facades\Log;
use App\Mail\SubscriptionReminderMail;

class SendSubscriptionReminders extends Command
{
    protected $signature = 'send:subscription-reminders';
    protected $description = 'Send subscription reminder emails to users';

    public function __construct()
    {
        parent::__construct();
    }

    public function handle()
    {
        Log::channel('slack')->info('SendSubscriptionReminders command started.');

        // Fetch subscriptions that are about to expire in 7 days, 3 days, and 1 day
        $reminders = [
            '7 days' => Carbon::now()->addDays(7)->format('Y-m-d'),
            '3 days' => Carbon::now()->addDays(3)->format('Y-m-d'),
            '1 day' => Carbon::now()->addDay()->format('Y-m-d')
        ];

        foreach ($reminders as $reminderPeriod => $reminderDate) {
            Log::channel('slack')->info("Checking subscriptions for reminder period: {$reminderPeriod} ({$reminderDate})");

            // Get subscriptions with the exact end date
            $subscriptions = Location::whereDate('end_date', $reminderDate)->get();
            Log::channel('slack')->info("Subscriptions found: " . $subscriptions->count());

            if ($subscriptions->isEmpty()) {
                Log::channel('slack')->info("No subscriptions found for date: {$reminderDate}");
            }

            foreach ($subscriptions as $subscription) {
                $user = User::find($subscription->user_id);
                $plan = SubscriptionPlans::find($subscription->app_plan_id);

                if ($plan) {
                    $subscription->plan_name = $plan->core_name;
                    $subscription->formatted_end_date = Carbon::parse($subscription->end_date)->format('Y-m-d');


                    Log::channel('slack')->info("Sending email to user: {$subscription->email} for subscription ending on: {$subscription->end_date}");
                    Mail::to($subscription->email)->send(new SubscriptionReminderMail($subscription, $reminderPeriod));
                } else {
                    Log::channel('slack')->warning("Subscription ID {$subscription->id} does not have a valid user or plan.");
                }
            }
        }

        Log::channel('slack')->info('Subscription reminders sent successfully.');
        $this->info('Subscription reminders sent successfully.');
    }
}
