<?php

namespace App\Enums;

enum OriginEnum: int
{
    case WEB = 3;
    case MOBILE = 1;
    case DESKTOP = 2;

    /**
     * Get the label for each enum case.
     *
     * @return string
     */
    public function label(): string
    {
        return match ($this) {
            self::WEB => 'Web',
            self::MOBILE => 'Mobile',
            self::DESKTOP => 'Desktop',
        };
    }

    /**
     * Get all values.
     *
     * @return array
     */
    public static function values(): array
    {
        return array_column(self::cases(), 'value');
    }

    /**
     * Get all names and values as an associative array.
     *
     * @return array
     */
    public static function toArray(): array
    {
        return array_reduce(
            self::cases(),
            fn ($carry, $case) => $carry + [$case->name => $case->value],
            []
        );
    }

    /**
     * Get an instance by value.
     *
     * @param int $value
     * @return ?self
     */
    public static function fromValue(int $value): ?self
    {
        foreach (self::cases() as $case) {
            if ($case->value === $value) {
                return $case;
            }
        }
        return null;
    }
}
