<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\AppBaseController;
use App\Models\PaymentTransaction;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class PaystackWithdrawal extends AppBaseController
{
    public function listBank(Request $request)
    {
        $curl = curl_init();

        curl_setopt_array($curl, [
                CURLOPT_URL => 'https://api.paystack.co/bank?currency=NGN',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 30,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'GET',
                CURLOPT_HTTPHEADER => [
                'Authorization: Bearer PAYSTACK_SECRET_LIVE_KEY',
                'Cache-Control: no-cache',
                ],
            ]);

        $response = curl_exec($curl);
        $err = curl_error($curl);
        $res = json_decode($response, true);

        return Response::json(['data' => $res['data'], 'success' => true, 'message' => 'Bank successfully retrieved'], 200);
    }

    public function createTransferRecipent(Request $request)
    {
        $url = 'https://api.paystack.co/transferrecipient';

        $validator = Validator::make($request->all(), [
            'account_number' => 'required',
            'bank_code' => 'required',
        ]);

        if ($validator->fails()) {
            return Response::json(['message' => $validator->errors()->first(), 'success' => false], 400);
        }

        $fields = [
          'type' => 'nuban',
        //   'name' => $request->name,
          'account_number' => $request->account_number,
          'bank_code' => $request->bank_code,
          'currency' => 'NGN',
        ];

        $fields_string = http_build_query($fields);

        //open connection
        $ch = curl_init();

        //set the url, number of POST vars, POST data
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer sk_live_c8d66cf012d742eab34fedce4535c4a183e6921d',
            'Cache-Control: no-cache',
        ]);

        //So that curl_exec returns the contents of the cURL; rather than echoing it
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        //execute post
        $response = curl_exec($ch);
        $err = curl_error($ch);
        $transaction = json_decode($response, true);
        //  var_dump($transaction);
        //return $recipentCode = $transaction['data']['recipient_code'];
        if($transaction['status'] == false ){
            return Response::json(['data' => '', 'success' => false, 'message' => 'Error verifiying account number'], 400);
        }else{
            return Response::json(['data' => $transaction, 'success' => true, 'message' => 'Transfer recipient created successfully'], 200);
        }
    }

    public function initiateTransfer(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'recipient_code' => 'required',
            'amount' => 'required',
            'pin' => 'required',
        ]);

        if ($request->pin != Auth::user()->pin) {
            return Response::json(['message' => 'Invalid Pin', 'success' => false], 400);
        }

        if ($validator->fails()) {
            return Response::json(['message' => $validator->errors()->first(), 'success' => false], 400);
        }

        if ($request->amount > Auth::user()->wallet_balance) {
            return Response::json(['data' => '', 'success' => false, 'message' => 'Low Wallet Balance'], 400);
        }

        if ($request->amount > 5000) {
            return Response::json(['data' => '', 'success' => false, 'message' => 'Limit withdrawal reached'], 400);
        }

        $url = 'https://api.paystack.co/transfer';
        $ref = Str::uuid();

        $fields = [
          'source' => 'balance',
          'amount' => ($request->amount * 100),
          'reference' => $ref,
          'recipient' => $request->recipient_code,
          'reason' => 'Cash4Trash ',
        ];

        $fields_string = http_build_query($fields);

        //open connection
        $ch = curl_init();

        //set the url, number of POST vars, POST data
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
          'Authorization: Bearer sk_live_c8d66cf012d742eab34fedce4535c4a183e6921d',
          'Cache-Control: no-cache',
        ]);

        //So that curl_exec returns the contents of the cURL; rather than echoing it
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        //execute post
        $result = curl_exec($ch);
        $transaction = json_decode($result, true);
        // echo $result;

        if ($transaction['status'] == false) {
            return Response::json(['data' => $transaction['message'], 'success' => false, 'message' => 'Error in processing transaction'], 400);
        }

        $data = ['user_id' => Auth::user()->id, 'effect' => 'DR', 'payment_ref' => $ref, 'amount' => $request->amount, 'source' => 'wallet', 'payment_for' => 'Withdrawal', 'status' => '1', 'country' => Auth::user()->country];
        //inseet Transaction record
        PaymentTransaction::insert($data);
        // var_dump($transaction);
        $unit_after_payment = Auth::user()->wallet_balance - $request->amount;
        User::where('id', Auth::user()->id)->update(['wallet_balance' => $unit_after_payment]);

        return Response::json(['data' => $transaction, 'success' => true, 'message' => 'Transaction Successful'], 200);
    }

    // Str::uuid();
}
