<?php

namespace App\Http\Controllers\Account\CashLodgement;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Controllers\Account\Log\TxnLogController;
use Validator;
use Carbon\Carbon;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Folio\Bank\BankAccountController;
use Illuminate\Support\Str;
use DB;
use App\Models\Retails\TxnLog;



class CashLodgementController extends Controller
{
    //
    protected $txnLogController;
    protected $chartOfAccountController;
    protected $currencyController;
    protected $bankAccountController;
 
    public function __construct(
        TxnLogController $txnLogController,
        ChartOfAccountController $chartOfAccountController,
        CurrencyController $currencyController,
        BankAccountController $bankAccountController
    ) {
        $this->txnLogController = $txnLogController;
        $this->chartOfAccountController = $chartOfAccountController;
        $this->currencyController = $currencyController;
        $this->bankAccountController = $bankAccountController;
    }

    public function lodgeCash(Request $request) {

        $validationRules = [
            'amount' => 'required',
            'bank_account_id' => 'required',
            'date' => 'required',
            'purpose' => 'required'
        ];
        $validator = Validator::make($request->all(), $validationRules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        if($this->txnLogController->chartIdBalance(CHART_CASH) < $request->amount) {
            return response()->json(['message' =>'Insufficient funds', 'error' => true], 400);
        }
        
        $accLegs = $this->chartOfAccountController->getAccountLegs(CHART_BANK, CHART_CASH, Auth::user()->client_id);
       
        $tokenUid = Str::uuid()->toString();

        $currencyId = $this->currencyController->getPrimaryCurrency(Auth::user()->client_id, Auth::user()->location_id);

        $currencyUnit = $this->currencyController->getCurrencyUnit(Auth::user()->client_id, Auth::user()->location_id, $currencyId);
        $txnDate = $request->date;

        $bank_id = $this->bankAccountController->getBankIdByBankAccountId($request->bank_account_id);
        
        $this->txnLogController->createTxnLog($accLegs, $request->amount, $tokenUid, $currencyUnit, $txnDate, "", $request->purpose, "", 'cl', $currencyId, "", $request->bank_account_id, $bank_id);

        return response()->json(['message' => 'Cash Lodgement saved successfully', 'data' => "", 'success' => true], 201);
         
    }



    public function editCashLodge(Request $request, $token) {
        // Define validation rules
        $validationRules = [
            'amount' => 'required',
            'bank_account_id' => 'required',
            'date' => 'required',
            'purpose' => 'required'
        ];
    
        // Create a validator with the incoming request and rules
        $validator = Validator::make($request->all(), $validationRules);
    
        // Check if validation fails
        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }
    
            // Retrieve and update all records in a single query
        $updatedRows = TxnLog::where('token', $token)
        ->update([
            'bank_account_id' => $request->bank_account_id,
            'txn_date' => $request->date,
            'being' => $request->purpose,
            'jvalue' => $request->amount,
            'jvdr' => DB::raw('CASE WHEN effect = "dr" THEN ' . $request->amount . ' ELSE STATUS END'),
            'jvcr' => DB::raw('CASE WHEN effect = "cr" THEN ' . $request->amount . ' ELSE STATUS END')
        ]);

    if ($updatedRows == 0) {
    return response()->json(['message' => 'Transaction not found', 'error' => true], 404);
    }

    // Fetch the updated records for response
    $txnLogs = TxnLog::where('token', $token)->get();

    return response()->json(['message' => 'Cash Lodgement edited successfully', 'data' => $txnLogs, 'success' => true], 200);
    }
    



    public function deleteCashLodge(Request $request, $token) {
    
        $updatedRows = TxnLog::where('token', $token)
                            ->update([
                                'control_id' => 1,
                                'control_user_id' => Auth::user()->id
                            ]);
    
        if ($updatedRows == 0) {
            return response()->json(['message' => 'Transaction not found', 'error' => true], 404);
        }
    
        return response()->json(['message' => 'Cash Lodgement deleted successfully', 'success' => true], 200);
    }
    
    


    

    public function getAllCashLodgement(Request $request)
{
    $client_id = Auth::user()->client_id;
    $posted_by = Auth::user()->name;

    $searchValue = $request->input('search') ?? '';
    $search = !empty($searchValue) ? "AND txn.being LIKE '%$searchValue%'" : '';

    $locationSort = !empty($request->location_id) ? "AND txn.location_id='".$request->location_id."'" : "";
    $limit = "LIMIT $request->count, 50";

    $userSort = !empty($request->user_id) ? "AND txn.user_id='$request->user_id'" : "";
    $txnDate = !empty($request->from) && !empty($request->to) ? "AND txn.txn_date BETWEEN '$request->from' AND '$request->to'" : "";

    $query = "
    WITH cash_lodge_txn as (
        SELECT
            txn.being,
            txn.token,
            txn.txn_date,
            txn.tm,
            txn.jvalue,
            txn.bank_account_id,
            ba.name as bank_account_name,
            txn.currency_unit,
            txn.pmethod,
            '$posted_by' as posted_by
        FROM " . ONLINE_RETAILS . ".txn_log txn
        LEFT JOIN " . ONLINE_RETAILS . ".bank_account ba ON ba.id = txn.bank_account_id
        WHERE txn.effect = '".EFFECT_DEBIT."' AND txn.chart_id = ".CHART_BANK." AND txn.prefix = 'cl' AND txn.client_id = '$client_id' AND txn.control_id = ".STATUS." $locationSort $search $userSort $txnDate  ORDER BY txn.txn_date DESC $limit
    ),
    Total as (
        SELECT
            COALESCE(SUM(txn.jvalue), 0) AS total_value
        FROM " . ONLINE_RETAILS . ".txn_log txn
        WHERE txn.effect = '".EFFECT_DEBIT."' AND txn.chart_id = ".CHART_BANK." AND txn.prefix = 'cl' AND txn.client_id = '$client_id'  AND txn.control_id = ".STATUS." $locationSort $search
    )
    SELECT 
        *
    FROM cash_lodge_txn c
    CROSS JOIN Total t;
    ";

    $data = DB::select($query);

    return response()->json(['data' => $data, 'success' => true], 200);
}



    


    
}
