<?php

namespace App\Http\Controllers\Account\ChartOfAccount;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Retails\ChartOfAccount;
use Illuminate\Support\Facades\Auth;
use DB;

class ChartOfAccountController extends Controller
{
    public function getAccountTypes(Request $request) {
        $accountTypes = ChartOfAccount::where('sort_code', SORT_REVENUE)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->get();

        $formattedStore= [];
        foreach ($accountTypes as $act) {
            $formattedAct[] = [
                'value' => $act->code,
                'title' => $act->name
            ];
        }
        return response()->json([
            'message' => 'Account types fetched successfully',
            'data' => $formattedAct,
            'success' => true
        ], 200);
    }

    public function getAccountLegs($dr, $cr, $client_id)
    {
        $sCR = $this->getSortCodeByChartId($cr, $client_id);
        $sDR = $this->getSortCodeByChartId($dr, $client_id);
        $aCR = $this->getAccountCategoryIdByChartId($cr, $client_id);
        $aDR = $this->getAccountCategoryIdByChartId($dr, $client_id);

        return [
            'sortCodeCR' => $sCR,
            'sortCodeDR' => $sDR,
            'accountCateCR' => $aCR,
            'accountCateDR' => $aDR,
            'chartIdCR' => $cr,
            'chartIdDR' => $dr,
        ];
    }

    public function getSingleAccountLeg($chart_id, $client_id)
    {
        $sortCode = $this->getSortCodeByChartId($chart_id, $client_id);
        $accountCate = $this->getAccountCategoryIdByChartId($chart_id, $client_id);

        return [
            'sortCode' => $sortCode,
            'accountCate' => $accountCate,
            'chartId' => $chart_id,
        ];
    }

    public function getSortCodeByChartId($chart_id, $client_id)
    {
        return ChartOfAccount::where('code', $chart_id)
            ->where('client_id', $client_id)
            ->value('sort_code');
    }

    public function getRevenueAccounts()
    {
        $data = ChartOfAccount::where('sort_code', SORT_REVENUE)
            ->where('client_id', Auth::user()->client_id)
            ->select('code', 'name')
            ->get();

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function getAccountCategoryIdByChartId($chart_id, $client_id)
    {
        return ChartOfAccount::where('code', $chart_id)
            ->where('client_id', $client_id)
            ->value('acct_cate');
    }

    public function getDefaultAccount() {
        return ChartOfAccount::where('client_id', STATUS)->get();
    }

    

    public function isChartId($chart_id, $location_id, $client_id)
    {
        return ChartOfAccount::where('code', $chart_id)
        ->where('location_id', $location_id)
        ->where('client_id', $client_id)->first();
    }

    public function getStockAdjustMentAccounts(){
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;

        $query = "
            SELECT 
                code as value,
                CASE
                WHEN name = 'suspense' THEN 'no change to my account'
                ELSE name
                END AS title
            FROM " . TBL_CHART_OF_ACCOUNTS . "

            WHERE client_id='$client_id' AND location_id='$location_id' AND status = " . STATUS . ";
        ";
       
        $data = DB::select($query);

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function chartIdSummary($chartId, $request){
        $client_id = Auth::user()->client_id;

        $locationSort = !empty($request->location_id) ? "AND location_id='$request->location_id'" : "";

        $query = "SELECT   
        CASE WHEN dr.DebitBalance IS NULL 
            THEN 0
            ELSE dr.DebitBalance
        END AS Credit,

        CASE WHEN cr.CreditBalance IS NULL 
            THEN 0
            ELSE cr.CreditBalance
        END AS Debit,

        CASE WHEN cr.CreditBalance IS NOT NULL 
            THEN ROUND(dr.DebitBalance - cr.CreditBalance, 2) 
            ELSE dr.DebitBalance
        END AS balance,

        CASE WHEN dr.effect IS NULL 
            THEN cr.effect
            ELSE dr.effect
        END AS effect

        FROM " . ONLINE_RETAILS .".txn_log 

        INNER JOIN ( 
            SELECT SUM(jvalue) AS DebitBalance, b.effect 
            FROM " . ONLINE_RETAILS .".txn_log a 
            INNER JOIN " . ONLINE_RETAILS .".account_type b ON b.sort_code = a.account_type
            WHERE a.effect = 'dr' AND a.chart_id = ".$chartId." AND a.client_id = '$client_id' AND a.control_id != '".ONE_STATUS."' $locationSort
        ) AS dr 

        JOIN ( 
            SELECT SUM(jvalue) AS CreditBalance, b.effect  
            FROM " . ONLINE_RETAILS .".txn_log a 
            INNER JOIN " . ONLINE_RETAILS .".account_type b ON b.sort_code = a.account_type
            WHERE a.effect = 'cr' AND a.chart_id = ".$chartId." AND a.client_id = '$client_id' AND a.control_id != '".ONE_STATUS."' $locationSort
        ) cr 

        WHERE control_id != '".ONE_STATUS."' AND client_id = '$client_id' $locationSort LIMIT 1";

   return $data = DB::select($query);
    }
}
