<?php

namespace App\Http\Controllers\Account\Float;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use DB;
use Illuminate\Support\Str;
use Validator;
use App\Models\Retails\TxnLog;
class FloatController extends Controller
{
    protected $currencyController;
    protected $txnLogController;

    protected $chartOfAccountController;

    public function __construct(CurrencyController $currencyController, TxnLogController $txnLogController, ChartOfAccountController $chartOfAccountController){
        $this->currencyController = $currencyController;
        $this->txnLogController = $txnLogController;
        $this->chartOfAccountController = $chartOfAccountController;
    }

    public function currencyAndAccForFloat()
    {
        $clientId = Auth::user()->client_id;
        $locationId = Auth::user()->location_id;

        // Define the raw SQL query with CTEs
        $query = "
            WITH CurrencyData AS (
                SELECT
                    currency.id AS value,
                    CONCAT(UPPER(SUBSTRING(currency_cate.name, 1, 1)), SUBSTRING(currency_cate.name, 2)) AS title,
                    currency.primary_currency
                FROM
                    " . ONLINE_RETAILS . ".currency
                INNER JOIN  " . ONLINE_RETAILS . ".currency_cate ON currency.currency_cate_id = currency_cate.id
                WHERE
                    currency.status = 0
                    AND currency.client_id='$clientId' AND currency.location_id='$locationId'
            ),
            ChartOfAccountsData AS (
                SELECT
                    *
                FROM
                    " . ONLINE_RETAILS . ".chart_cate
                WHERE
                    acct_type_id = ".SORT_EXPENSES." AND client_id='$clientId' AND location_id='$locationId'
            ),

            directExpenses as (
                SELECT
                    *
                FROM
                    " . ONLINE_RETAILS . ".chart_of_accounts
                WHERE
                    sort_code = ".SORT_EXPENSES." AND acct_cate = ".CATEGORY_DIRECT_EXPENSES." AND client_id='$clientId' AND location_id='$locationId'
            ),

            operationalExpenses as (
                SELECT
                    *
                FROM
                    " . ONLINE_RETAILS . ".chart_of_accounts
                WHERE
                    sort_code = ".SORT_EXPENSES." AND acct_cate = ".CATEGORY_OPERATIONAL_EXPENSES." AND client_id='$clientId' AND location_id='$locationId'
            )

            SELECT
                (SELECT JSON_ARRAYAGG(
                    JSON_OBJECT('value', value, 'title', title, 'primary_currency', primary_currency)
                ) FROM CurrencyData) AS currency,

                (SELECT JSON_ARRAYAGG(
                    JSON_OBJECT('id', id, 'name', name, 'code', cate_id)
                ) FROM ChartOfAccountsData) AS chart_of_account,

                (SELECT JSON_ARRAYAGG(
                    JSON_OBJECT('id', id, 'name', name, 'code', code)
                ) FROM directExpenses) AS directExpense,

                (SELECT JSON_ARRAYAGG(
                    JSON_OBJECT('id', id, 'name', name, 'code', code)
                ) FROM operationalExpenses) AS operationalExpense
        ";

        $result = DB::select($query);

        $currency = json_decode($result[0]->currency, true);
        $chartOfAccount = json_decode($result[0]->chart_of_account, true);
        $directExpense = json_decode($result[0]->directExpense, true);
        $operationalExpense = json_decode($result[0]->operationalExpense, true);

        // Construct the response
        return response()->json([
            'data' => [
                'currency' => $currency,
                'chart_of_account' => $chartOfAccount,
                'directExpense' => $directExpense,
                'operationalExpense' => $operationalExpense
            ],
            'success' => true
        ], 200);
    }


    public function postFloat(Request $request) {

        $validationRules = [
            'amount' => 'required',
            'currency' => 'required',
            'chart_id' => 'required',
            'date' => 'required',
            'purpose' => 'required'
        ];
        
        $validator = Validator::make($request->all(), $validationRules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        if($this->txnLogController->chartIdBalance(CHART_PETTY_CASH) < $request->amount) {
            return response()->json(['message' =>'Insufficient funds', 'error' => true], 400);
        }

        $accLegs = $this->chartOfAccountController->getAccountLegs($request->chart_id, CHART_PETTY_CASH, Auth::user()->client_id);

        $currencyUnit = $this->currencyController->getCurrencyUnit(Auth::user()->client_id, Auth::user()->location_id, $request->currency);
       
        $tokenUid = Str::uuid()->toString();
        $txnDate = $request->date;
        
        $this->txnLogController->createTxnLog($accLegs, $request->amount, $tokenUid, $currencyUnit, $txnDate, "", $request->purpose, "", 'f', $request->currency);

        return response()->json(['message' => 'Float saved successfully', 'data' => "", 'success' => true], 201);
         
    }


    public function getFloatBalance() {
        $balance = $this->txnLogController->chartIdBalance(CHART_PETTY_CASH);
        return response()->json(['data' => $balance, 'success' => true], 200);
    }


    public function getFloat(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        $token = !empty($request->token) ? " AND t.token LIKE '%$request->token%'" : "";
        $purpose = !empty($request->purpose) ? " AND t.being LIKE '%$request->purpose%'" : "";
        $date = !empty($request->from) && !empty($request->to) ? "AND t.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $sortUser = !empty($request->user) ? "AND t.user_id='$request->user'" : "";
        $limit = "LIMIT $request->count, 50";
         
        //currency name
        //category_id
        ///categrory_name
        $query = "
            WITH petty_cash AS (
                SELECT 
                    ROW_NUMBER() OVER (ORDER BY t.txn_date, t.tm) as rowNum, 
                    t.txn_date,
                    t.tm,
                    t.token,
                    t.being,
                    t.jvalue,
                    t.currency_id,
                    us.name as posted_by,
                    coalesce(t.jvcr,0) as jvcr,
                    coalesce(t.jvdr,0) as jvdr,
                    t.effect,
                    cc.name,
                    cc.cate_id,
                    ct.name as currency_name,
                    t.currency_unit
                FROM " . TBL_LOG . " t
                LEFT JOIN " . ONLINE_ACCOUNT . ".user us ON us.id = t.user_id AND us.client_id='$client_id'

                LEFT JOIN ".ONLINE_RETAILS.".chart_cate cc ON cc.cate_id=t.chart_cate AND cc.client_id='$client_id' AND cc.location_id='$location_id'

                LEFT JOIN ".ONLINE_RETAILS.".currency c ON c.id=t.currency_id AND c.client_id='$client_id' AND c.location_id='$location_id'

                LEFT JOIN ".ONLINE_RETAILS.".currency_cate ct ON c.currency_cate_id=ct.id

                WHERE t.client_id='$client_id' AND t.location_id='$location_id' AND t.control_id != 1 AND t.chart_id=" . CHART_PETTY_CASH . " $token $date $sortUser $purpose
                ORDER BY t.txn_date, t.tm DESC
                $limit
            ), 

            runningBal as (
                SELECT s1.*, sum(s2.jvdr-s2.jvcr) as runningBal FROM petty_cash s1 
                LEFT JOIN petty_cash s2 ON
                s1.rowNum  >= s2.rowNum
                group by s1.rowNum
            ),

            total_balance as (
                SELECT sum(jvdr-jvcr) as total, sum(jvdr) as debit, sum(jvcr) as credit FROM petty_cash 
            )
                
            select * from runningBal
            CROSS JOIN
            total_balance;

        ";

   //  echo $query;exit;

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }



    public function getCash(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $token = !empty($request->token) ? " AND t.token LIKE '%$request->token%'" : "";
        $date = !empty($request->from) && !empty($request->to) ? "AND t.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $sortUser = !empty($request->user) ? "AND t.user_id='$request->user'" : "";
        $limit = "LIMIT $request->count, 50";
        
         
        //currency name
        //category_id
        ///categrory_name
        $query = "
            WITH petty_cash AS (
                SELECT 
                    ROW_NUMBER() OVER (ORDER BY t.txn_date, t.tm) as rowNum, 
                    t.txn_date,
                    t.tm,
                    t.token,
                    t.being,
                    t.jvalue,
                    t.currency_id,
                    us.name as posted_by,
                    coalesce(t.jvcr,0) as jvcr,
                    coalesce(t.jvdr,0) as jvdr,
                    t.effect,
                    cc.name,
                    t.pmethod,
                    cc.cate_id,
                    t.currency_unit,
                    cus.name as customer_name
                FROM " . TBL_LOG . " t
                LEFT JOIN " . ONLINE_ACCOUNT . ".user us ON us.id = t.user_id AND us.client_id='$client_id'

                LEFT JOIN ".ONLINE_RETAILS.".chart_cate cc ON cc.cate_id=t.chart_cate AND cc.client_id='$client_id' AND cc.location_id='$location_id'

                LEFT JOIN " . TBL_CUSTOMERS . " cus ON cus.id = t.coy AND cus.client_id='$client_id' AND cus.location_id='$location_id' AND cus.status = 0

                WHERE t.client_id='$client_id' AND t.location_id='$location_id' AND t.control_id != 1 AND t.chart_id=" . CHART_CASH . " $token $date $sortUser 
                
                $limit
            ), 

            runningBal as (
                SELECT s1.*, sum(s2.jvdr - s2.jvcr) as runningBal FROM petty_cash s1 
                LEFT JOIN petty_cash s2 ON
                s1.rowNum  >= s2.rowNum
                group by s1.rowNum
                ORDER BY s1.rowNum DESC
            ),

            total_balance as (
                SELECT sum(jvdr-jvcr) as total, sum(jvdr) as debit, sum(jvcr) as credit FROM petty_cash
            )
                
            select * from runningBal
            CROSS JOIN
            total_balance;

        ";

   //  echo $query;exit;

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }



    public function editFloat(Request $request, $token) {
        $validationRules = [
            'cate_id' => 'required',
            'amount' => 'required',
            'currency' => 'required',
            'date' => 'required',
            'purpose' => 'required'
        ];
        $validator = Validator::make($request->all(), $validationRules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }
    
            // Retrieve and update all records in a single query
        $updatedRows = TxnLog::where('token', $token)
        ->update([
            'chart_cate' => $request->cate_id,
            'currency_id' => $request->currency,
            'txn_date' => $request->date,
            'being' => $request->purpose,
            'jvalue' => $request->amount,
            'jvdr' => DB::raw('CASE WHEN effect = "dr" THEN ' . $request->amount . ' ELSE STATUS END'),
            'jvcr' => DB::raw('CASE WHEN effect = "cr" THEN ' . $request->amount . ' ELSE STATUS END')
        ]);

    if ($updatedRows == 0) {
    return response()->json(['message' => 'Transaction not found', 'error' => true], 404);
    }

    // Fetch the updated records for response
    $txnLogs = TxnLog::where('token', $token)->get();

    return response()->json(['message' => 'Cash Lodgement edited successfully', 'data' => $txnLogs, 'success' => true], 200);
    }
    



    public function deleteFloat(Request $request, $token) {
    
        $updatedRows = TxnLog::where('token', $token)
                            ->update([
                                'control_id' => 1,
                                'control_user_id' => Auth::user()->id
                            ]);
    
        if ($updatedRows == 0) {
            return response()->json(['message' => 'Transaction not found', 'error' => true], 404);
        }
    
        return response()->json(['message' => 'Cash Lodgement deleted successfully', 'success' => true], 200);
    }







}
