<?php

namespace App\Http\Controllers\Account\FloatDisbursement;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Controllers\Account\Log\TxnLogController;
use Validator;
use Carbon\Carbon;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Folio\Bank\BankAccountController;
use Illuminate\Support\Str;
use DB;
use App\Models\Retails\TxnLog;

class FloatDisbursementController extends Controller
{
    protected $txnLogController;
    protected $chartOfAccountController;
    protected $currencyController;
    protected $bankAccountController;
 
    public function __construct(
        TxnLogController $txnLogController,
        ChartOfAccountController $chartOfAccountController,
        CurrencyController $currencyController,
        BankAccountController $bankAccountController
    ) {
        $this->txnLogController = $txnLogController;
        $this->chartOfAccountController = $chartOfAccountController;
        $this->currencyController = $currencyController;
        $this->bankAccountController = $bankAccountController;
    }

    public function disburseCash(Request $request) {

        $validationRules = [
            'amount' => 'required',
            'p_method' => 'required',
            'date' => 'required',
            'purpose' => 'required' 
        ];
        $validator = Validator::make($request->all(), $validationRules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        if($request->p_method == CASH AND $this->txnLogController->chartIdBalance(CHART_CASH) < $request->amount) {
            return response()->json(['message' =>'Insufficient funds', 'error' => true], 400);
        }

        if(($request->p_method == BANK_TRANSFER || $request->p_method == POS) AND $this->txnLogController->chartIdBalance(CHART_BANK) < $request->amount) {
            return response()->json(['message' =>'Insufficient funds in bank', 'error' => true], 400);
        }

        $locationId = empty($request->location_id) ?  Auth::user()->location_id : $request->location_id;
        $pmethodLeg = "";
        $bank_id = "";
        if($request->p_method == BANK_TRANSFER || $request->p_method == POS){
            $bank_id = $this->bankAccountController->getBankIdByBankAccountId($request->bank_account_id);
            $pmethodLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_BANK, Auth::user()->client_id);
        } else {
            $pmethodLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_CASH, Auth::user()->client_id);
        }
        
        $pettyCashLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_PETTY_CASH, Auth::user()->client_id);
       
        $tokenUid = Str::uuid()->toString();

        $currencyId = $this->currencyController->getPrimaryCurrency(Auth::user()->client_id, $locationId);

        $currencyUnit = $this->currencyController->getCurrencyUnit(Auth::user()->client_id, $locationId, $currencyId);
        $txnDate = $request->date;

        $currencyValue = $currencyUnit * $request->amount;

        $this->txnLogController->createSingleTxnLog($pmethodLeg, EFFECT_CREDIT, $request->amount, $tokenUid, $currencyUnit, $txnDate, "", $request->purpose, "", "fl", $currencyId, $currencyValue, $request->p_method,  $request->bank_account_id, $bank_id);

        $this->txnLogController->createSingleTxnLog($pettyCashLeg, EFFECT_DEBIT, $request->amount, $tokenUid, $currencyUnit, $txnDate, "", $request->purpose, "", "fl", $currencyId, $currencyValue, $request->p_method, "","", "", "", "", $locationId);

        return response()->json(['message' => 'Cash Disbursement saved successfully', 'data' => "", 'success' => true], 201);
         
    }


    public function getAllCashDisbursement(Request $request)
    {
        $client_id = Auth::user()->client_id;
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $posted_by = Auth::user()->name;
    
        $searchValue = $request->input('search') ?? '';
        $search = !empty($searchValue) ? "AND txn.being LIKE '%$searchValue%'" : '';
    
        $locationSort = !empty($request->location_id) ? "AND txn.location_id='$request->location_id'" : "";
        $userSort = !empty($request->user_id) ? "AND txn.user_id='$request->user_id'" : "";
        $txnDate = !empty($request->from) && !empty($request->to) ? "AND txn.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $limit = "LIMIT $request->count, 50";
        $query = "
        WITH cash_disbursement_txn as (
            SELECT
                txn.being,
                txn.token,
                txn.txn_date,
                txn.tm,
                txn.jvalue,
                tx.bank_account_id,
                ba.name as bank_account_name,
                lo.name as location_name,
                us.name as posted_by,
                txn.pmethod,
                txn.currency_unit
            FROM " . ONLINE_RETAILS . ".txn_log txn
            LEFT JOIN " . ONLINE_RETAILS . ".location lo ON lo.id = txn.location_id AND lo.client_id='$client_id'
            LEFT JOIN " . ONLINE_RETAILS . ".txn_log tx ON tx.token = txn.token AND tx.chart_id=" . CHART_BANK . " AND tx.client_id='$client_id' AND tx.location_id='$location_id'
            LEFT JOIN " . ONLINE_RETAILS . ".bank_account ba ON ba.id = tx.bank_account_id AND ba.client_id='$client_id' AND ba.location_id='$location_id'
            LEFT JOIN " . ONLINE_ACCOUNT . ".user us ON us.id = tx.user_id AND us.client_id='$client_id' AND us.location_id='$location_id'
            

            WHERE txn.effect = '".EFFECT_DEBIT."' AND txn.chart_id = ".CHART_PETTY_CASH." AND txn.prefix = 'fl' AND txn.client_id = '$client_id' AND txn.control_id != ".ONE_STATUS." $locationSort $userSort $txnDate $search  ORDER BY txn.txn_date, txn.tm DESC $limit
        ),
        Total as (
            SELECT
                COALESCE(SUM(txn.jvalue), 0) AS total_value
            FROM " . ONLINE_RETAILS . ".txn_log txn
            WHERE txn.effect = '".EFFECT_DEBIT."' AND txn.chart_id = ".CHART_PETTY_CASH." AND txn.prefix = 'fl' AND txn.client_id = '$client_id'  AND txn.control_id != ".ONE_STATUS." $locationSort $search $txnDate
        )
        SELECT 
            *
        FROM cash_disbursement_txn c
        CROSS JOIN Total t;
        ";
    
        $data = DB::select($query);
    
        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function getGroupedCashDisbursementByLocation(Request $request)
    {
        $client_id = Auth::user()->client_id;
     //   $posted_by = Auth::user()->name;

        $searchValue = $request->input('search') ?? '';
        $search = !empty($searchValue) ? "AND txn.being LIKE '%$searchValue%'" : '';
        $limit = "LIMIT $request->count, 50";

        $query = "
        WITH disbursement_grouped AS (
            SELECT
                txn.location_id,
                lo.name as location_name,
                COALESCE(SUM(txn.jvdr - txn.jvcr), 0) AS total_value
            FROM " . ONLINE_RETAILS . ".txn_log txn
            LEFT JOIN " . ONLINE_RETAILS . ".location lo ON lo.id = txn.location_id
            WHERE txn.chart_id = ".CHART_PETTY_CASH." 
             AND txn.client_id = '$client_id' AND txn.control_id = ".STATUS." $search
            GROUP BY txn.location_id, lo.name  ORDER BY txn.txn_date DESC $limit
        ),
        Total as (
            SELECT
                COALESCE(SUM(txn.jvalue), 0) AS total_disbursed
            FROM " . ONLINE_RETAILS . ".txn_log txn
            WHERE txn.effect = '".EFFECT_DEBIT."' AND txn.chart_id = ".CHART_PETTY_CASH." 
            AND txn.prefix = 'fl' AND txn.client_id = '$client_id' AND txn.control_id = ".STATUS." $search
        )
        SELECT 
            d.location_id,
            d.location_name,
            d.total_value,
            t.total_disbursed
        FROM disbursement_grouped d
        CROSS JOIN Total t;
        ";

        $data = DB::select($query);

        return response()->json(['data' => $data, 'success' => true], 200);
    }


    public function getFloatDisbursementByLocation(Request $request, $location_id) {
        $client_id = Auth::user()->client_id;
        $token = !empty($request->token) ? " AND t.token LIKE '%$request->token%'" : "";
        $date = !empty($request->from) && !empty($request->to) ? "AND t.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $sortUser = !empty($request->user) ? "AND t.user_id='$request->user'" : "";
        $limit = "LIMIT $request->count, 50";
        
        $query = "
            WITH disbursement AS (
                SELECT 
                    ROW_NUMBER() OVER (ORDER BY t.txn_date, t.tm) as rowNum, 
                    t.txn_date,
                    t.tm,
                    t.being,
                    t.jvalue,
                    us.name as posted_by,
                    coalesce(t.jvcr,0) as jvcr,
                    coalesce(t.jvdr,0) as jvdr,
                    t.effect,
                    t.pmethod,
                    t.currency_unit
                FROM " . TBL_LOG . " t
                LEFT JOIN " . ONLINE_ACCOUNT . ".user us ON us.id = t.user_id

                WHERE t.client_id='$client_id' AND  t.location_id='$location_id' AND t.control_id != 1 AND t.chart_id=" . CHART_PETTY_CASH . " $token $date $sortUser
                ORDER BY t.txn_date, t.tm DESC
                $limit
            ), 

            runningBal as (
                SELECT s1.*, sum(s2.jvdr-s2.jvcr) as runningBal FROM disbursement s1 
                LEFT JOIN disbursement s2 ON
                s1.rowNum  >= s2.rowNum
                group by s1.rowNum
            ),

            total_balance as (
                SELECT sum(jvdr-jvcr) as  total FROM disbursement
            )
                
            select * from runningBal
            CROSS JOIN
            total_balance;

        ";

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }


}


