<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\General\GeneralController;
use Illuminate\Http\Request;
use App\Models\Account\User;
use Illuminate\Support\Facades\Auth;
use Validator;
use Response;


class ChangePasswordController extends GeneralController
{

     /**
     * Authenticate and log in a user based on the provided credentials.
     *
     * This method handles user authentication by validating the provided email and password
     * and checking various conditions such as account status, email verification, and login
     * permission before generating an API token upon successful authentication.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\JsonResponse
     *
     * @author Idowu Adeyinka
     */
 
    public function changePassword(Request $request){
        $validator = Validator::make($request->all(), [
            'password' => 'required',
            'confirmPassword' => 'required|same:password'
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation failed', 'errors' => $validator->errors()], 400);
        }

        $salt = $this->salt();
        $password = $this->hashPass($request->password, $salt);

        User::where('id', Auth::user()->id)
        ->where('location_id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->update([
            'password' => $password,
            'salt' => $salt
        ]);

        return response()->json(['message' => 'Password updated successfully', 'success' => true], 200);
    }
}
