<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\General\GeneralController;
use App\Http\Controllers\General\LoadDefaultController;
use App\Models\Account\User;
use Illuminate\Http\Request;
use JWTAuth;
use Response;
use Validator;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Settings\User\UserController;
use App\Http\Controllers\Settings\Device\DeviceController; 
use App\Http\Controllers\Folio\Location\LocationController;
use App\Http\Controllers\Inventory\Product\ProductController;
use App\Http\Controllers\Settings\License\LicenseController;
use App\Http\Controllers\Settings\Store\StoreController;

class LoginController extends GeneralController
{

    protected $loadDefaultController;
    protected $userController;
    protected $generalController;
    protected $deviceController;
    protected $locationController;
    protected $productController;
    protected $licenseController;
    protected $storeController;

    public function __construct(LoadDefaultController $loadDefaultController, UserController $userController, GeneralController $generalController, DeviceController $deviceController, LocationController $locationController, ProductController $productController, LicenseController $licenseController, StoreController $storeController) {
        $this->loadDefaultController = $loadDefaultController;
        $this->userController = $userController;
        $this->generalController = $generalController;
        $this->deviceController = $deviceController;
        $this->locationController = $locationController;
        $this->productController = $productController;
        $this->licenseController = $licenseController;
        $this->storeController = $storeController;
    }

    /**
     * Authenticate and log in a user based on the provided credentials.
     *
     * This method handles user authentication by validating the provided email and password
     * and checking various conditions such as account status, email verification, and login
     * permission before generating an API token upon successful authentication.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\JsonResponse
     *
     * @author Idowu Adeyinka
     */
    private function sendOTP($phone, $userId)
    {
        // Prepare data for the request
        $data = array(
            "api_key" => "TLYL7YLhThb8NcOaX4xpa0BtLR0GQ0LlD6exoB1EYkr5pY7uho7jhzy4g38ycQ",
            "message_type" => "NUMERIC",
            "to" => $this->standardPhone($phone),
            "from" => "N-Alert",
            "channel" => "dnd",
            "pin_attempts" => 10,
            "pin_time_to_live" => 5,
            "pin_length" => 5,
            "pin_placeholder" => "< 12345 >",
            "message_text" => "Your pin is < 12345 >",
            "pin_type" => "NUMERIC"
        );
    
        // Encode data to JSON
        $post_data = json_encode($data);
    
        // Initialize cURL
        $curl = curl_init();
    
        // Set cURL options
        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.ng.termii.com/api/sms/otp/send",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $post_data,
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json"
            ),
        ));
    
        // Execute cURL request and get the response
        $response = curl_exec($curl);
    
        // Close cURL session
        curl_close($curl);
    
        // Decode the response
        $decode = json_decode($response);
    
        // var_dump($decode); exit;
    
        // Check if pinId is present in the response
        if (isset($decode->pinId)) {
            // Update user's pin_id in the database
            $pinId = $decode->pinId;
            User::where('id', $userId)->update(['pin_id' => $pinId]);
    
            return true;
        }
    
        // If pinId is not present, return false or handle as needed
        return false;
    }
    public function login(Request $request)
    {
        // try{
            $validator = Validator::make($request->all(), [
                'email' => 'required',
                'password' => 'required'
            ]);

            if ($validator->fails()) {
                return response()->json(['message' => 'Validation failed', 'errors' => $validator->errors()], 400);
            }

            $user = $this->getUser($request->email);

            if (!$user) {
                return response()->json(['message' => 'Invalid Username or Password', 'success' => false], 401);
            }

            session(['client_id' => $user->client_id]);
            session(['location_id' => $user->location_id]);
            session(['user_id' => $user->id]);

            if ($user->status == 1) {
                return response()->json(['message' => 'Your account has been disabled', 'success' => false], 401);
            }

            if($request->origin != 3){
                if ($user->verification_status != 0) {
                    $res = ['email' => $user->username, 'phone' => $user->phone];

                    return response()->json(['message' => 'OTP not verified', 'data' => $res, 'success' => false], 401);
                }
            }

            // if ($user->email_verification != 0) {
            //     $res = ['email' => $user->username, 'phone' => $user->phone];
            //     return response()->json(['message' => 'You have not verified your email', 'data' => $res, 'success' => false], 401);
            // }

            if ($this->isLoginAllowed($user, $request->origin)) {
                $hash = $this->hashPass($request->password, $user->salt);
                $hash2 = $this->hashPass($request->password, $user->salt2);
   
                $userMatch = $this->authenticateUser($request->email, $hash, $hash2);
         
                if ($userMatch) {

                    if ($userMatch->password2 === $hash2 && $userMatch->support_pin_time <= Carbon::now()) {
                        return response()->json(['message' => 'Support pin has expired', 'success' => false], 401);
                    }

                    $primaryStore = $this->storeController->getPrimaryStoreName($user->client_id, $user->location_id);
                    if($primaryStore === "front store"){
                        $this->storeController->updatePrimaryStoreName($user->client_id, $user->location_id);
                    }
                    
                    $expiry_status = $this->locationController->isExpired($user->client_id, $user->location_id);
                    $app_plan_id = $this->locationController->locationAppPlan($user->location_id, $user->client_id);
                    
                    if($expiry_status === "expired" && $app_plan_id != ONE_STATUS) {
                        $this->locationController->activateExpiration($user->client_id, $user->location_id);
                    }
                    // validating expiration for location that is not the head quaters
                    $headQuatersEmail = $this->licenseController->licenseEmail($user->client_id);
                    $locationEmail = $this->locationController->locationEmail($user->location_id, $user->client_id);
                   
                    if($expiry_status == 'expired' && $request->origin != 3) {
                        if($headQuatersEmail != $locationEmail) {
                            return response()->json(['message' => 'Your location does not have an active subscription', 'success' => false], 401);
                        }
                    }
                    // getting subscription id
                    $plan_id = $this->locationController->getPlanId($user->client_id, $user->location_id);
                    // loading default data and rights for first time user
                    $this->loadDefaultController->loadDefault($user, $request, $plan_id);

                    // getting desktop database structure
                    $tables = $this->readDatabaseFile("destktopDB");
                    // getting user's desktop rights
                    $defaultRights = $this->userController->getUserDesktopRights($user->client_id, $user->location_id, $user->id, $request->origin);

                    // echo json_encode($defaultRights);exit;
                    // getting default currency
                    $defaultCurrency = $this->generalController->getDefaultCurrency();
                    // getting default vendor
                    $defaultVendor = $this->generalController->getDefaultVendor($user->client_id);
                    // getting default walk-in customer
                    $defaultWalkInCustomer = $this->generalController->getDefaultWalkInCustomer($user->client_id);
                    // getting app banks
                    $defaultAppBanks = $this->generalController->getDefaultAppBanks();
                    // getting default location
                    $defaultLocations = $this->generalController->getDefaultLocations($user->client_id, $user->location_id, $user->id, $user->location_overview);
                    // getting default users
                    $allUsers = $this->generalController->getDefaultUsers($user->client_id, $user->overview, $user->id, $user->location_id);
                    // getting users pincode
                    $pinCodeUsers = $this->generalController->getDefaultUsersPinCode($user->client_id, $user->location_id);
                    // getting default account type
                    $accountClass = $this->generalController->getDefaultAccountClass();
                    // getting default account category
                    $accountCategory = $this->generalController->getChartCateInfo($user->client_id, $user->location_id);
                    // getting default chart of accounts
                    $accounts = $this->generalController->getChartOfAccountInfo($user->client_id, $user->location_id);
                    // getting default measurement category
                    $defaultMeasurementCategory = $this->generalController->getDefaultMeasurementCategory($user->client_id);
                    // getting default measurement
                    $defaultMeasurement = $this->generalController->getDefaultMeasurement($user->client_id, $user->location_id);
                    // getting device id
                    $deviceId = $this->deviceController->getDeviceId($user->client_id, $user->location_id, $request);
                    
                    if($request->origin != 3){
                        if($user->username != 'demo@klakpad.com'){
                            if (!$deviceId) {
                                // send otp to location admin phone number
                                $locationAdminPhone = $this->locationController->locationPhone($user->location_id, $user->client_id);
                                $this->sendOTP($locationAdminPhone, $user->id);
                                
                                return response()->json(['message' => 'Your device is not authorised, meet your administrator for the otp code to authorise your device', 'data' => ['email' => $user->username, 'phone' => $locationAdminPhone], 'success' => false], 401);
                            }
                        }
                    }
                    
                    $allowed_items = $this->locationController->allowedItems($user->client_id, $user->location_id);
                    $allowed_store = $this->locationController->allowedStore($user->client_id, $user->location_id);
                    $syncDefault = $this->generalController->getSyncSetting($user->client_id, $user->location_id);
                    $defaultItemCategory = $this->productController->getDefaultItemCategory($user->client_id);
                    $licenseEmail = $this->licenseController->licenseEmail($user->client_id);
                    $licensePhone = $this->licenseController->licensePhone($user->client_id);
                    $defaultLoyalty = '';
                    $logo = $this->locationController->locationLogo($user->location_id, $user->client_id);
                    $deviceInfo = $this->deviceController->getDevice($user->client_id, $user->location_id, $request);
                    $inventoryEvaluation = $this->locationController->getInventoryEvaluation($user->location_id, $user->client_id);
                    
                    // generating bearer token for logged in user
                    $token =  $this->generateApiToken($user);

                    User::where('username', $user->username)->update(['expo_token' => $request->expo_token]);
                    
                    return response()->json(['message' => 'Login Successful', 'data' => $user, 'tables' => $tables, 'defaultRights' => $defaultRights, 'defaultCurrency' => $defaultCurrency, 'defaultVendor' => $defaultVendor, 'defaultWalkInCustomer' => $defaultWalkInCustomer, 'defaultAppBanks' => $defaultAppBanks, 'defaultLocations' => $defaultLocations, 'allUsers' => $allUsers, 'accountClass' => $accountClass, 'defaultMeasurementCategory' => $defaultMeasurementCategory, 'defaultMeasurement' => $defaultMeasurement, 'device_id' => $deviceId, 'expired_status' => $expiry_status, 'allowed_items' => $allowed_items, 'syncDefault' => $syncDefault, 'defaultItemCategory' => $defaultItemCategory, 'licenseEmail' => $licenseEmail, 'licensePhone' => $licensePhone, 'defaultLoyalty' => $defaultLoyalty, 'plan_id' => $plan_id, 'logo' => $logo, 'accountCategory' => $accountCategory, 'accounts' => $accounts, 'allowed_store' => $allowed_store, 'success' => true, 'api_token' => $token, 'pin_code_users' => $pinCodeUsers, 'deviceInfo' => $deviceInfo, 'inventoryEvaluation' => $inventoryEvaluation], 200);
                    
                } else {
                    return response()->json(['message' => 'Invalid Username or Password', 'success' => false], 401);
                }
            } else {
                if($request->origin == 1) {
                    return response()->json(['message' => 'You are not authorised to login on mobile', 'success' => false], 401);
                } else if($request->origin == 2) {
                    return response()->json(['message' => 'You are not authorised to login on desktop', 'success' => false], 401);
                } else {
                    return response()->json(['message' => 'You are not authorised to login on web', 'success' => false], 401);
                }
            }

            
        // } catch (\Exception $e){
        //     return response()->json(['message' => 'Something went wrong', 'error' => $e->getMessage(), 'success' => false], 500);
        // }
    }

    private function getUser($email)
    {
        return User::where('username', $email)
        ->orWhere('username2', $email)
        ->first();
    }

    /**
     * Check if the user is allowed to log in from a specific origin.
     *
     * This method evaluates whether the user is permitted to log in based on their access permissions
     * for different login origins, such as web, desktop, and mobile.
     *
     * @param User $user The user for whom login permission is being checked.
     * @param int $origin The origin of the login attempt (1 for mobile, 2 for desktop, 3 for web).
     * @return bool Returns true if the user is allowed to log in from the specified origin, or false if not.
     *
     * @author Your Name
     */
    private function isLoginAllowed($user, $origin)
    {
        $web = $user->allowed_on_web;
        $desktop = $user->allowed_on_desktop;
        $mobile = $user->allowed_on_mobile;

        if ($origin == 1 && $mobile != 1) {
            return false;
        } elseif ($origin == 2 && $desktop != 1) {
            return false;
        } elseif ($origin == 3 && $web != 1) {
            return false;
        }

        return true;
    }

    private function readDatabaseFile($fileName){
        $file = __DIR__ . "/$fileName.sql";
        if (file_exists($file)) {
            $myfile = fopen($file, "r") or die("Unable to open file!");
            if (!empty(filesize($file))) {
                $data = fread($myfile, filesize($file));
                fclose($myfile);
                return $data;
            }
        } else {
            die("File does not exist!");
        }
    }

    private function authenticateUser($email, $hash, $hash2)
    {
        return User::where(function ($query) use ($email) {
                $query->where('username', $email)
                    ->orWhere('username2', $email);
            })
            ->where(function ($query) use ($hash, $hash2) {
                $query->where('password', $hash)
                    ->orWhere(function ($query) use ($hash2) {
                        $query->where('password2', $hash2);
                    });
            })
            ->first();
    }


    /**
     * Generate an API token for the given user and update it in the database.
     *
     * This method generates an API token for the specified user and updates the user's record in the database
     * with the generated token.
     *
     * @param User $user The user for whom the API token is generated and updated.
     * @return string The generated API token.
     *
     * @author Idowu Adeyinka
     * @created Date Created
     * @updated Date Updated
     */
    public function generateApiToken($user)
    {
        // Assuming $user is your authenticated user model
        $subscription_status = $this->locationController->isExpired($user->client_id, $user->location_id);

        $customClaims = [
            'username' => $user->username,
            'phone' => $user->phone,
            'email' => $user->email,
            'location_id' => $user->location_id,
            'name' => $user->name,
            'verification_status' => $user->verification_status,
            'subscription_status' => $subscription_status
        ];

        $token = JWTAuth::claims($customClaims)->fromUser($user);

        // Update the user's api_token with the new JWT
        User::where('username', $user->username)->update(['api_token' => $token, 'sales_priviledge' => $user->sales_priviledge, 'confirm_bank_pos' => $user->confirm_bank_pos]);

        return $token;
    }

 
}
