<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\General\GeneralController;
use Illuminate\Support\Facades\DB;
use App\Models\Account\DeviceLog;
use App\Models\Account\Licensee;
use App\Models\Account\User;
use App\Models\Retails\Location;
use App\Mail\UserSignUpMail;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Auth\LaraTermii;
use Carbon\Carbon;
use Response;
use Validator;
use Illuminate\Validation\Rule;
use Illuminate\Foundation\Auth\RegistersUsers;
use Tymon\JWTAuth\Contracts\JWTSubject;
use Illuminate\Auth\Events\Registered;
use Illuminate\Contracts\Auth\MustVerifyEmail; // Add this line
use App\Http\Controllers\Settings\User\UserController;
use App\Http\Controllers\Folio\Location\LocationController;
use App\Http\Controllers\Settings\License\LicenseController;
use App\Http\Controllers\Settings\Device\DeviceController;
use App\Http\Controllers\Auth\LoginController;
use App\Services\ZeptoMailService;
use Auth;
use GuzzleHttp\Client;

class RegisterController extends GeneralController
{
    // use RegistersUsers;

    protected $UserController ;
    protected $LocationController ;

    protected $LicenseController;
    protected $DeviceController;
    protected $zeptoMailService;
    protected $LoginController;


    public function __construct(UserController $UserController, LocationController $LocationController, LicenseController $LicenseController, DeviceController $DeviceController, ZeptoMailService $zeptoMailService, LoginController $LoginController)
    {
        $this->UserController = $UserController;
        $this->LocationController = $LocationController;
        $this->LicenseController = $LicenseController;
        $this->DeviceController = $DeviceController;
        $this->zeptoMailService = $zeptoMailService;
        $this->LoginController = $LoginController;
    }
    
    /**
     * Create a new client, licensee, location, user, and related records in a transaction.
     *
     * This method orchestrates the creation of a new client, licensee, location, user, and related records
     * in a database transaction. It performs validation, generates necessary values, and creates records for
     * a new client, allowing user registration, device logging, OTP generation, and email verification.
     *
     * @param \Illuminate\Http\Request $request The HTTP request containing client registration data.
     * @return \Illuminate\Http\JsonResponse A JSON response indicating the success or failure of the client creation process.
     *
     * @author Idowu Adeyinka
     */

    public function sendEmail($to, $subject, $message)
    {
        // Capture the full response from sendEmail
        $response = $this->zeptoMailService->sendEmail($to, $subject, $message);

        // Instead of just checking for a boolean, handle the response more granularly
        if ($response->successful()) {
            return 'Email sent successfully';
        } else {
            // Log the status code and response body for debugging
            \Log::error('Email sending failed', [
                'status' => $response->status(),
                'response' => $response->body(),
            ]);

            // Provide a generic failure message or include some response details if appropriate
            return 'Email sending failed. Status: ' . $response->status();
        }
    }



    public function sendTestEmail2(Request $request)
{
    $data = ['message' => $request->message ?? 'This is a default test message.'];
    Mail::to('test@example.com')->send(new TestEmail($data));

    return response()->json(['message' => 'Email sent!'], 200);
}

public function editPhoneNumber(Request $request) {
    if(empty($request->phone)) {
        return response()->json(['message' => "Phone field cannot be empty", 'success' => false], 400);
    }

    User::where('id', Auth::user()->id)
    ->where('client_id', Auth::user()->client_id)
    ->where('location_id', Auth::user()->location_id)
    ->update([
        'phone' => $request->phone
    ]);

    Location::where('client_id', Auth::user()->client_id)
    ->where('id', Auth::user()->location_id)
    ->update([
        'phone' => $request->phone
    ]);

    Licensee::where('id', Auth::user()->client_id)
    ->update([
        'phone' => $request->phone
    ]);

    $this->sendOTP($request, Auth::user()->id);

    $user = User::where('id', Auth::user()->id)
    ->where('client_id', Auth::user()->client_id)
    ->where('location_id', Auth::user()->location_id)
    ->first();

    $token = $this->LoginController->generateApiToken($user);

    return response()->json(['message' => 'An OTP has been sent to your new phone number', 'api_token' => $token, 'success' => true], 200);
    
}


    private function sanitizeInput($input)
    {
        return preg_replace('/[^A-Za-z0-9\s]/', '', $input); // Remove all special characters except alphanumeric and spaces
    }


    // private function validateRequest(Request $request)
    // {
    //     // Sanitize inputs before validation
    //     $request->merge([
    //         'name' => $this->sanitizeInput($request->name),
    //         'address' => $this->sanitizeInput($request->address),
    //         'username' => $this->sanitizeInput($request->username),
    //     ]);

    //     $validator = Validator::make($request->all(), [
    //         'name' => 'required|unique:licensee',
    //         'password' => 'required|min:8|confirmed',
    //         'phone' => 'required|size:11|unique:licensee',
    //         'email' => 'required|email|unique:licensee|regex:/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/|max:255',
    //     ]);

    //     if ($validator->fails()) {
    //         return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 400);
    //     }

    //     return null;
    // }

 

    private function validateRequest(Request $request)
    {
        // Sanitize inputs before validation
        $request->merge([
            'name' => $this->sanitizeInput($request->name),
            'address' => $this->sanitizeInput($request->address),
            'username' => $this->sanitizeInput($request->username),
        ]);

        $validator = Validator::make($request->all(), [
            
            'email' => [
                'required',
                'email',
                'max:255',
                Rule::unique('licensee')->where(function ($query) {
                    return $query->whereNotNull('email');
                }),
            ],
            'phone' => 'required|size:11|unique:licensee',
            'name' => 'required|unique:licensee',
            'password' => 'required|min:8|confirmed'
        ], [
            'email.unique' => 'This email is already registered, please log in with it.',
            'phone.unique' => 'This phone number has been taken'
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'success' => false], 400);
        }

        return null;
    }

    public function newClient(Request $request)
    {
        try {
            DB::beginTransaction(); // Start a database transaction

            $validationResult = $this->validateRequest($request);
            if ($validationResult) {
                return $validationResult;
            }

            $usernameExist = User::where('username2', $request->username)->value('username2');

            if($usernameExist) {
                return response()->json(['message' => "$usernameExist has been taken", 'success' => false], 400);
            }

            $currentDate = Carbon::now()->format('Y-m-d');
            $gst_value = $request->country == 'Nigeria' ? 7.5 : 0;
            $salt = $this->salt();
            $pass = $this->hashPass($request->password, $salt);

            // Attempt to create Licensee
            try {
                $licensee = $this->LicenseController->createLicensee($request, $currentDate, $gst_value, $salt, $pass);

            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['message' => 'Error creating licensee', 'success' => false, 'error' => $e->getMessage()], 500);
            }

            // Attempt to create Location
            try {
                $location = $this->LocationController->createLocation($request, $currentDate, $licensee->id, $gst_value);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['message' => 'Error creating location', 'success' => false, 'error' => $e->getMessage()], 500);
            }

            // Attempt to create User
            try {
                $locationId = $this->LocationController->getLocationByName($licensee->id, $request->email);
                $overview = ONE_STATUS;
                $deletion = ONE_STATUS;
                $masterStatus = ONE_STATUS;
                $salesPriviledge = TWO_STATUS;
                $upsell = ONE_STATUS;
                $storeId = 'all';
                $locationOverview = ONE_STATUS;
                $backDate = ONE_STATUS;
                $discount = ONE_STATUS;
                $edit = ONE_STATUS;
                $emailVerification = ONE_STATUS;
                $smsVerification = ONE_STATUS;
                $position = 'control';
                $priceVisiiity = ONE_STATUS;
                $dashboard = 'dashboard';
                $user = $this->UserController->createUser($request, $currentDate, $licensee->id, $locationId, $salt, $pass, $overview, $deletion, $masterStatus, $salesPriviledge, $upsell, $storeId, $locationOverview, $backDate, $discount, $edit, $position, $emailVerification, $smsVerification, $priceVisiiity, $dashboard, ONE_STATUS, ONE_STATUS, ONE_STATUS, ONE_STATUS, ONE_STATUS);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['message' => 'Error creating user', 'success' => false, 'error' => $e->getMessage()], 500);
            }

            // Attempt to create Device Log
            try {
                $deviceName = empty($request->origin) ? $request->macAddress1 . "-" . $user->id : $request->macAddress1;
                // $this->DeviceController->createDeviceLog($licensee->id, $locationId, $deviceName, $request, $user->id, $currentDate);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['message' => 'Error creating device log', 'success' => false, 'error' => $e->getMessage()], 500);
            }

            // Commit the transaction if all operations succeed
            DB::commit();

            $otpDetails = $this->sendOTP($request, $user->id);
                
            $this->sendEmailVerification2($request, $user->id);

            $arrayData = [
                'page' => 'newclient',
                'folio' => $request->name,
                'email' => $request->email,
                'phone' => $request->phone,
                'address' => $request->address,
                'location' => '1 Location',
                'advisor_id' => $request->referral_code,
                'timezone' => $request->timezone,
                'country' => $request->country,
                'deviceName' => $deviceName,
                'origin' => $request->origin,
                'refId' => $request->referral_code,
                'dueDate' => $currentDate
            ];



            $newArr = array_merge($arrayData,["clientId" => $licensee->id, "locationId" => $locationId]);
            $url = 'https://www.klakpad.org/agents_api/l/getRegistration.php';
            $postApi = $this->postData($newArr, $url);

            // Generate API token (JWT)
            $token = $this->LoginController->generateApiToken($user);
            
            // Return success response
            return response()->json(['message' => 'Success', 'success' => true, 'api_token' => $token, 'data' => $user], 200);

        } catch (\Exception $e) {
            DB::rollBack(); // Roll back the transaction in case of any unhandled error
            return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }

    public function getAccessToken($client){
   
        $tenantId = '2c687f89-02cd-4840-a150-2332e4165154';
        $clientId = '6a4dbb01-c04c-4597-9b69-74ad8a179894';
        $clientSecret = 'Fr18Q~j_HPRwvmibRv-A2TTzgAUf7ghm-V1rxdeX';
        $tokenUrl = "https://login.microsoftonline.com/$tenantId/oauth2/v2.0/token";

        $params = [
            'grant_type' => 'client_credentials',
            'client_id' => $clientId,
            'client_secret' => $clientSecret,
            'scope' => 'https://graph.microsoft.com/.default'  // Replace with appropriate scope if necessary
        ];
        
        $response = $client->post($tokenUrl, [
            'form_params' => $params
        ]);
        
        // Parse the response and extract the access token
        $body = json_decode($response->getBody()->getContents(), true);
        $accessToken = $body['access_token'];

        return $accessToken;
    }

    public function accessToken(){
        $client = new Client();
        
        $tenantId = '2c687f89-02cd-4840-a150-2332e4165154';
        $clientId = '6a4dbb01-c04c-4597-9b69-74ad8a179894';
        $clientSecret = 'Fr18Q~j_HPRwvmibRv-A2TTzgAUf7ghm-V1rxdeX';
        $tokenUrl = "https://login.microsoftonline.com/$tenantId/oauth2/v2.0/token";

        $params = [
            'grant_type' => 'client_credentials',
            'client_id' => $clientId,
            'client_secret' => $clientSecret,
            'scope' => 'https://graph.microsoft.com/.default'  // Replace with appropriate scope if necessary
        ];
        
        $response = $client->post($tokenUrl, [
            'form_params' => $params
        ]);
        
        // Parse the response and extract the access token
        $body = json_decode($response->getBody()->getContents(), true);
        $accessToken = $body['access_token'];

        return $accessToken;
    }

    public function getCellRange($siteId, $driveId, $itemId, $sheetName, $accessToken, $client) {
      
        $usedRangeResponse = $client->get("https://graph.microsoft.com/v1.0/sites/{$siteId}/drives/{$driveId}/items/{$itemId}/workbook/worksheets/{$sheetName}/usedRange", [
            'headers' => [
                'Authorization' => "Bearer $accessToken",
                'Content-Type' => 'application/json',
            ],
        ]);
    
        $usedRangeData = json_decode($usedRangeResponse->getBody()->getContents(), true);
        
        // Get the address of the last used range
        $lastUsedRange = $usedRangeData['address']; // e.g., "A1:J10"
        
        // Step 2: Calculate the next row to insert data
        // Example: If last used range is A1:J10, next empty row is 11 (A11:J11)
        preg_match('/(\d+)$/', $lastUsedRange, $matches);
        $lastRow = intval($matches[1]); // Get the last row number
        $nextRow = $lastRow + 1; // Calculate the next row
    
        // Step 3: Define the new cell range based on the next empty row
        $cellRange = "A{$nextRow}:J{$nextRow}"; // Targeting columns A to J in the next row
    
        return $cellRange;
        
    }

    public function saveClientToMsExcelSheet($licensee, $accessToken, $client) {

        $data = [
            [
                $licensee->name,
                $licensee->address,
                $licensee->start_date,
                $licensee->email,
                $licensee->tm,
                $licensee->phone,
                $licensee->app,
                $licensee->origin,
                $licensee->timezone,
                $licensee->license_no,
            ]
        ];

        $sheetName = "sheet1";
        // $siteId="klakpad377.sharepoint.com,b0062cf1-10eb-42c2-bf1d-d8c3c023d56b,9c9c6414-637c-4bd5-a97a-3e6a0cda9f30";
        $siteId = "b0062cf1-10eb-42c2-bf1d-d8c3c023d56b";
        $driveId="b!8SwGsOsQwkK_HdjDwCPVaxRknJx8Y9VLqXo-agzanzCBSTEGp-35RKtE63firxrf";
        $itemId="01FPOKXL56Y2GOVW7725BZO354PWSELRRZ";

        $cellRange='A1:J1';//$this->getCellRange($siteId, $driveId, $itemId, $sheetName, $accessToken, $client);

        $response = $client->patch("https://graph.microsoft.com/v1.0/sites/{$siteId}/drives/{$driveId}/items/{$itemId}/workbook/worksheets/{$sheetName}/range(address='{$cellRange}')", [
            'headers' => [
                'Authorization' => "Bearer $accessToken",
                'Content-Type' => 'application/json',
            ],
            'json' => [
                'values' => $data
            ]
        ]);

        return $response;
    }

    public function newClientExcel(Request $request)
    {
        try {
            DB::beginTransaction(); // Start a database transaction

            $validationResult = $this->validateRequest($request);
            if ($validationResult) {
                return $validationResult;
            }

            $usernameExist = User::where('username2', $request->username)->value('username2');
            
            if($usernameExist) {
                return response()->json(['message' => "$usernameExist has been taken", 'success' => false], 400);
            }

            $currentDate = Carbon::now()->format('Y-m-d');
            $gst_value = $request->country == 'Nigeria' ? "7.5" : '';
            $salt = $this->salt();
            $pass = $this->hashPass($request->password, $salt);

            // Attempt to create Licensee
            try {
                $licensee = $this->LicenseController->createLicensee($request, $currentDate, $gst_value, $salt, $pass);
                
                $client = new Client();
                $accessToken = $this->getAccessToken($client);
                // save registered client to microsoft excel sheet
                $excel=$this->saveClientToMsExcelSheet($licensee, $accessToken, $client);
                return response()->json(['success' => true, 'data' => $excel], 200);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['message' => 'Error creating licensee', 'success' => false, 'error' => $e->getMessage()], 500);
            }
exit;
            // Attempt to create Location
            try {
                $location = $this->LocationController->createLocation($request, $currentDate, $licensee->id, $gst_value);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['message' => 'Error creating location', 'success' => false, 'error' => $e->getMessage()], 500);
            }

            // Attempt to create User
            try {
                $locationId = $this->LocationController->getLocationByName($licensee->id, $request->email);
                $overview = ONE_STATUS;
                $deletion = ONE_STATUS;
                $masterStatus = ONE_STATUS;
                $salesPriviledge = TWO_STATUS;
                $upsell = ONE_STATUS;
                $storeId = 'all';
                $locationOverview = ONE_STATUS;
                $backDate = ONE_STATUS;
                $discount = ONE_STATUS;
                $edit = ONE_STATUS;
                $emailVerification = ONE_STATUS;
                $smsVerification = ONE_STATUS;
                $position = 'control';
                $priceVisiiity = ONE_STATUS;
                $dashboard = 'dashboard';
                $user = $this->UserController->createUser($request, $currentDate, $licensee->id, $locationId, $salt, $pass, $overview, $deletion, $masterStatus, $salesPriviledge, $upsell, $storeId, $locationOverview, $backDate, $discount, $edit, $position, $emailVerification, $smsVerification, $priceVisiiity, $dashboard, ONE_STATUS, ONE_STATUS, ONE_STATUS, ONE_STATUS, ONE_STATUS);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['message' => 'Error creating user', 'success' => false, 'error' => $e->getMessage()], 500);
            }

            // Attempt to create Device Log
            try {
                $deviceName = empty($request->origin) ? $request->macAddress1 . "-" . $user->id : $request->macAddress1;
                // $this->DeviceController->createDeviceLog($licensee->id, $locationId, $deviceName, $request, $user->id, $currentDate);
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json(['message' => 'Error creating device log', 'success' => false, 'error' => $e->getMessage()], 500);
            }

            // Commit the transaction if all operations succeed
            DB::commit();

            $otpDetails = $this->sendOTP($request, $user->id);
                
            $this->sendEmailVerification2($request, $user->id);


            $arrayData = [
                'page' => 'newclient',
                'folio' => $request->name,
                'email' => $request->email,
                'phone' => $request->phone,
                'address' => $request->address,
                'location' => '1 Location',
                'advisor_id' => $request->referral_code,
                'timezone' => $request->timezone,
                'country' => $request->country,
                'deviceName' => $deviceName,
                'origin' => $request->origin,
                'refId' => $request->referral_code,
                'dueDate' => $currentDate
            ];



            $newArr = array_merge($arrayData,["clientId" => $licensee->id, "locationId" => $locationId]);
            $url = 'https://www.klakpad.org/agents_api/l/getRegistration.php';
            $postApi = $this->postData($newArr, $url);

            // Generate API token (JWT)
            $token = $this->LoginController->generateApiToken($user);
            
            // Return success response
            return response()->json(['message' => 'Success', 'success' => true, 'api_token' => $token, 'data' => $user], 200);

        } catch (\Exception $e) {
            DB::rollBack(); // Roll back the transaction in case of any unhandled error
            return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }


    public function postData($arrayData = [], $url)
    {
        $headers = ['Content-Type: application/json'];
        $payload = json_encode($arrayData);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        //curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        //curl_getinfo($ch, CURLINFO_HTTP_CODE );

        $response_string = curl_exec($ch);
        curl_close($ch);

        echo $response_string;
    }



    public function newClientKeeeeep(Request $request)
    {
        try {
            DB::beginTransaction(); // Start a database transaction

            $validationResult = $this->validateRequest($request);
            if ($validationResult) {
                return $validationResult;
            }

            

            $currentDate = Carbon::now()->format('Y-m-d');
            $gst_value = $request->country == 'Nigeria' ? "7.5" : '';
            $salt = $this->salt();
            $pass = $this->hashPass($request->password, $salt);

            $licensee =  $this->LicenseController->createLicensee($request, $currentDate, $gst_value, $salt, $pass);
            $location = $this->LocationController->createLocation($request, $currentDate, $licensee->id, $gst_value);

            $locationId = $this->LocationController->getLocationByName($licensee->id, $request->email);

            $overview = ONE_STATUS;
            $deletion = ONE_STATUS;
            $masterStatus = ONE_STATUS;
            $salesPriviledge = TWO_STATUS;
            $upsell = ONE_STATUS;
            $storeId = 'all';
            $locationOverview = ONE_STATUS;
            $backDate = ONE_STATUS;
            $discount = ONE_STATUS;
            $edit = ONE_STATUS;
            $emailVerification = STATUS;
            $smsVerification = ONE_STATUS;
            $position = 'control';
            $priceVisiiity = ONE_STATUS;
            $dashboard = 'dashboard';
            
            $user =  $this->UserController->createUser($request, $currentDate, $licensee->id, $locationId, $salt, $pass, $overview, $deletion, $masterStatus, $salesPriviledge, $upsell, $storeId, $locationOverview, $backDate, $discount, $edit, $position,  $emailVerification, $smsVerification, $priceVisiiity, $dashboard, ONE_STATUS, ONE_STATUS, ONE_STATUS, ONE_STATUS, ONE_STATUS);

            $deviceName = empty($request->origin) ? $request->macAddress1 . "-" . $user->id : $request->macAddress1;
            // $this->DeviceController->createDeviceLog($licensee->id, $locationId, $deviceName, $request, $user->id, $currentDate);

            $otpDetails = $this->sendOTP($request, $user->id);
            
            $this->sendEmailVerification2($request, $user->id);

            DB::commit(); // Commit the transaction if all steps are successful
        } catch (\Exception $e) {
            DB::rollBack(); // Roll back the transaction in case of an error
            return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 500);
        }

        return response()->json(['message' => 'Success', 'success' => true, 'data' => $user], 200);
    }

   


    private function sendOTP($request, $userId)
    {
        // Prepare data for the request
        $data = array(
            "api_key" => "TLYL7YLhThb8NcOaX4xpa0BtLR0GQ0LlD6exoB1EYkr5pY7uho7jhzy4g38ycQ",
            "message_type" => "NUMERIC",
            "to" => $this->standardPhone($request->phone),
            "from" => "N-Alert",
            "channel" => "dnd",
            "pin_attempts" => 10,
            "pin_time_to_live" => 5,
            "pin_length" => 5,
            "pin_placeholder" => "< 12345 >",
            "message_text" => "Your pin is < 12345 >",
            "pin_type" => "NUMERIC"
        );
    
        // Encode data to JSON
        $post_data = json_encode($data);
    
        // Initialize cURL
        $curl = curl_init();
    
        // Set cURL options
        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.ng.termii.com/api/sms/otp/send",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $post_data,
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json"
            ),
        ));
    
        // Execute cURL request and get the response
        $response = curl_exec($curl);
    
        // Close cURL session
        curl_close($curl);
    
        // Decode the response
        $decode = json_decode($response);
    
        // var_dump($decode); exit;
    
        // Check if pinId is present in the response
        if (isset($decode->pinId)) {
            // Update user's pin_id in the database
            $pinId = $decode->pinId;
            User::where('id', $userId)->update(['pin_id' => $pinId]);
    
            return true;
        }
    
        // If pinId is not present, return false or handle as needed
        return false;
    }
    


    private function sendOTPOl($request, $userId)
    {
        $termii = new \Zeevx\LaraTermii\LaraTermii('TLYL7YLhThb8NcOaX4xpa0BtLR0GQ0LlD6exoB1EYkr5pY7uho7jhzy4g38ycQ');
        $otpDetails = $termii->sendOTP($this->standardPhone($request->phone), 'N-Alert', 'ALPHANUMERIC', 10, 5, 5, '< 12345 >', 'Your pin is < 12345 >', 'dnd');
        $decode = json_decode($otpDetails);
        // var_dump($decode);
        $pinId = $decode->pinId;
        User::where('id', $userId)->update(['pin_id' => $pinId]);
    }


    private function sendEmailVerification2($request, $userId)
    {
        $verificationUrl = $this->generateVerificationUrl($request, $userId);
        $arr = ['fullname' => $request->name, 'auth_code' => '1212122', 'verificationUrl' => $verificationUrl];
        Mail::to($request->email)->send(new UserSignUpMail($arr));
    }

    private function generateVerificationUrl($request, $userId)
    {
      
        $url = env('APP_URL') . "/api/email-verification/" . $userId . "/" . sha1($userId);

        // $url = 'http://127.0.0.1:8000/email-verification/' . $userId . '/' . sha1($userId);
        return $url;
    }

    
    
    
 

}
