<?php

namespace App\Http\Controllers\CustomerEngagement;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use DB;
use Illuminate\Support\Facades\Auth;

class BuyingPatternController extends Controller
{
   
   public function getBuyingPatterns(Request $request) {
    $qDay = !empty($request->day) ? "LOWER(DAYNAME(i.txn_date)) = LOWER('{$request->day}') AND" : "";
    $qMonth = !empty($request->month) ? "MONTH(i.txn_date) = '{$request->month}' AND" : "";
    $qWeek = !empty($request->week) ? "FLOOR((DAY(i.txn_date) - 1) / 7) + 1 = '{$request->week}' AND" : "";
    $qYear = !empty($request->year) ? "YEAR(i.txn_date) = '{$request->year}' AND" : "";
    $qSearch = !empty($request->text) ? "c.name LIKE '%{$request->text}%' AND" : "";
    $qItem = !empty($request->item) ? "i.item_id = '{$request->item}' AND" : "";
    $qCategory = !empty($request->category) ? "i.item_category_id = '{$request->category}' AND" : "";
    $client_id = Auth::user()->client_id;
    $location_id = Auth::user()->location_id;

    $query = 
    "
    WITH CustomerTotals AS (
    SELECT
        c.id,
        c.name,
        c.email,
        c.dob,
        c.loyalty_status,
        SUM(COALESCE(i.amount, 0)) AS total,
        COUNT(DISTINCT i.token) AS visits
    FROM
         " . TBL_PRODUCT_LOG . " i
    JOIN
        " . TBL_CUSTOMERS . " c ON i.coy = c.id
    WHERE

        c.cate = 'lba'
        AND c.name != 'walk-in customer'
        AND c.status != 1  AND i.control_id != 1  and $qDay $qMonth $qWeek $qYear $qSearch $qItem $qCategory i.client_id = '$client_id' and i.location_id = '$location_id' 
    GROUP BY
        c.id, c.name, c.email, c.dob, c.loyalty_status
),
HighestTransaction AS (
    SELECT
        token_sums.coy AS customer_id,
        MAX(token_total) AS highest
    FROM (
        SELECT
            il.coy,
            il.token,
            SUM(COALESCE(il.amount, 0)) AS token_total
        FROM
             " . TBL_PRODUCT_LOG . " il
        GROUP BY
            il.coy, il.token
    ) AS token_sums
    GROUP BY
        token_sums.coy
),
LastPurchase AS (
    SELECT
        il.coy AS customer_id,
        SUM(COALESCE(il.amount, 0)) AS last_purchase
    FROM
        " . TBL_PRODUCT_LOG . " il
    WHERE
        il.token = (
            SELECT
                il2.token
            FROM
                 " . TBL_PRODUCT_LOG . " il2
            WHERE
                il2.coy = il.coy
            ORDER BY
                il2.txn_date DESC, il2.tm DESC
            LIMIT 1
        )
    GROUP BY
        il.coy
)
SELECT
    ct.id,
    ct.name,
    ct.email,
    ct.dob,
    ct.loyalty_status,
    ct.total,
    ht.highest,
    ct.visits,
    lp.last_purchase
FROM
    CustomerTotals ct
LEFT JOIN
    HighestTransaction ht ON ct.id = ht.customer_id
LEFT JOIN
    LastPurchase lp ON ct.id = lp.customer_id
ORDER BY
    ct.total DESC;
    ";

    // echo $query;exit;

    $data = DB::select($query);

    return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
}

public function getCustomerBuyingPatterns(Request $request) {
    $id = $request->id;
    $client_id = Auth::user()->client_id;
    $location_id = Auth::user()->location_id;

    $query = 
    "
WITH customer_stats AS (
    SELECT
        c.id,
        c.name,
        c.email,
        c.loyalty_status,
        c.gender,
        c.phone,
        c.date,
        c.dob,
        COUNT(DISTINCT i.token) AS visits,
        SUM(COALESCE(NULLIF(i.amount, ''), 0)) AS total_spent,
        (SELECT MAX(il.txn_date) FROM  " . TBL_PRODUCT_LOG . " il WHERE il.coy = c.id) AS last_visit_day,
        (SELECT il.tm
         FROM  " . TBL_PRODUCT_LOG . " il
         WHERE il.coy = c.id
         ORDER BY il.txn_date DESC, il.tm DESC
         LIMIT 1) AS last_visit_time,
 (
      SELECT SUM(COALESCE(NULLIF(il.amount, ''), 0))
      FROM  " . TBL_PRODUCT_LOG . " il
      WHERE il.coy = c.id AND il.token = (
        SELECT token
        FROM  " . TBL_PRODUCT_LOG . " il2
        WHERE il2.coy = c.id
        ORDER BY il2.txn_date DESC, il2.tm DESC
        LIMIT 1
      )
    ) AS last_purchase
    FROM
        " . TBL_CUSTOMERS . " c
    LEFT JOIN
         " . TBL_PRODUCT_LOG . " i ON i.coy = c.id
    WHERE
        c.id = '$id'
        AND c.cate = 'lba'
        AND c.name != 'walk-in customer'
        AND c.status != 1  AND i.control_id != 1 AND i.client_id = '$client_id' and i.location_id = '$location_id'
    GROUP BY
        c.id, c.name, c.email, c.loyalty_status, c.gender, c.phone, c.date, c.dob
),
highest_spent AS (
    SELECT
        coy AS customer_id,
        MAX(total_purchase) AS highest_purchase
    FROM (
        SELECT
            il.coy,
            SUM(COALESCE(NULLIF(il.amount, ''), 0)) AS total_purchase
        FROM
             " . TBL_PRODUCT_LOG . " il
        GROUP BY
            il.coy, il.token
    ) AS subquery
    GROUP BY
        coy
),
customer_summary AS (
    SELECT
        cs.id,
        cs.name,
        cs.email,
        cs.loyalty_status,
        cs.gender,
        cs.phone,
        cs.date,
        cs.dob,
        cs.visits,
        cs.total_spent AS total,
        COALESCE(hs.highest_purchase, 0) AS highest,
        cs.last_visit_day,
        cs.last_visit_time,
        cs.last_purchase,
        SUM(CASE
            WHEN MONTH(i.txn_date) = MONTH(CURRENT_DATE) THEN COALESCE(NULLIF(i.amount, ''), 0)
            ELSE 0
        END) AS total_current_month,
        SUM(CASE
            WHEN DATE(i.txn_date) = CURRENT_DATE THEN COALESCE(NULLIF(i.amount, ''), 0)
            ELSE 0
        END) AS total_current_day
    FROM
        customer_stats cs
    LEFT JOIN
        highest_spent hs ON cs.id = hs.customer_id
    LEFT JOIN
         " . TBL_PRODUCT_LOG . " i ON i.coy = cs.id
    GROUP BY
        cs.id, cs.name, cs.email, cs.loyalty_status, cs.gender, cs.phone, cs.date, cs.dob, hs.highest_purchase,
        cs.last_visit_day, cs.last_visit_time, cs.last_purchase
)

SELECT
    *
FROM
    customer_summary
ORDER BY
    total DESC;
    ";

    //  echo $query;

    $data = DB::select($query);

    return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
}

public function getCustomerMostBoughtItems(Request $request) {
    $id = $request->id;
    $client_id = Auth::user()->client_id;
    $location_id = Auth::user()->location_id;

    $query = 
    "
     SELECT 
      i.item_id,
      it.name AS item,
      COUNT(*) AS purchase_count,
      SUM(COALESCE(NULLIF(i.amount, ''), 0)) AS total,
      MAX(CONCAT(i.txn_date, ' ', i.tm)) AS last_visit

    FROM 
     " . TBL_PRODUCT_LOG . " i
    JOIN 
      " . TBL_ITEMS . " it ON i.item_id = it.id
    WHERE 
      i.coy = '$id' AND i.client_id = '$client_id' AND i.location_id = '$location_id'
    GROUP BY 
      i.item_id, it.name
    ORDER BY 
      purchase_count DESC
    LIMIT 10;
    ";

    //  echo $query;

    $data = DB::select($query);

    return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
}

public function getCustomerMostVisitedLocation(Request $request) {
    $id = $request->id;
    $client_id = Auth::user()->client_id;
    $location_id = Auth::user()->location_id;

    $query = 
    "
    SELECT 
      i.location_id,
      l.name AS name,
      COUNT(*) AS visit_count
    FROM 
      " . TBL_PRODUCT_LOG . " i
    JOIN 
       " . TBL_LOCATION . " l ON i.location_id = l.id
    WHERE 
      i.coy = '$id'
    GROUP BY 
      i.location_id, l.name
    ORDER BY 
      visit_count DESC
    LIMIT 1;
    ";

    //  echo $query;

    $data = DB::select($query);

    return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
}
    
}