<?php

namespace App\Http\Controllers\CustomerEngagement;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Retails\CustomerCampaign;
use JWTAuth;
use Response;
use Validator;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Folio\CustomerVendor\CustomerVendorController;
use App\Http\Controllers\General\GeneralController;
use App\Http\Controllers\General\SmsController;
use App\Http\Controllers\CustomerEngagement\CustomerRetargetingController;
use DB;

class CampaignController extends Controller
{

    protected $customerController;
    protected $generalController;
    protected $smsController;
    protected $customerRetargetingController;

    public function __construct(
        CustomerVendorController $customerController,
        GeneralController $generalController,
        SmsController $smsController,
        CustomerRetargetingController $customerRetargetingController
    )
    {
        $this->customerController = $customerController;
        $this->generalController = $generalController;
        $this->smsController = $smsController;
        $this->customerRetargetingController = $customerRetargetingController;
    }

    //
    public function insertCustomerCampaign() {
        // Define a method to generate common fields
        $commonFields = function($name) {
            return [
                'name' => $name,
                'sent' => STATUS,
                'status' => STATUS,
                'user_id' => STATUS,
                'device_id' => STATUS,
                'location_id' => STATUS,
                'client_id' => STATUS,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            ];
        };
    
        // Use the method to generate fields for each campaign
        $newArrival = $commonFields('new arrival');
        $sales = $commonFields('sales');
        $aboutToExpire = $commonFields('about to expire');
        $mostBoughtItems = $commonFields('most bought items');
    
        // Insert the records into the database
        CustomerCampaign::create($newArrival);
        CustomerCampaign::create($sales);
        CustomerCampaign::create($aboutToExpire);
        CustomerCampaign::create($mostBoughtItems);

        return response()->json(['message' => 'Inserted campaign successfully', 'success' => true], 200);
    }

    public function loadCustomerCampaign($user) {
        try{
            $campaign = CustomerCampaign::where('client_id', STATUS)
            ->where('location_id', STATUS)
            ->where('status', STATUS)
            ->get();

            foreach($campaign as $cus) {
                $existingCampaign = CustomerCampaign::where('client_id', $user->client_id)
                ->where('location_id', $user->location_id)
                ->where('name', $cus->name)
                ->where('status', STATUS)
                ->exists();

                if(!$existingCampaign) {
                    $fields = [
                        'name' => $cus->name,
                        'sent' => STATUS,
                        'status' => STATUS,
                        'device_id' => STATUS,
                        'user_id' => $user->id,
                        'location_id' => $user->location_id,
                        'client_id' => $user->client_id,
                        'tm' => Carbon::now()->toTimeString(),
                        'date' => Carbon::now()->format('Y-m-d'),
                        'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    ];

                    CustomerCampaign::create($fields);
                }
            }

            return response()->json(['message' => 'Loaded campaign successfully', 'success' => true], 200);
        }catch (\Exception $e) {
            return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 400);
        }
    }

    public function saveCustomerCampaign(Request $request) {

        $validationRules = [
            'name' => 'required'
        ];
    
        // Create a validator with the defined rules and the request data
        $validator = Validator::make($request->all(), $validationRules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }
        
        $existingCampaign = CustomerCampaign::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('name', $request->name)
        ->where('status', STATUS)
        ->exists();

        if(!$existingCampaign) {
            $fields = [
                'name' => $request->name,
                'sent' => STATUS,
                'status' => STATUS,
                'device_id' => STATUS,
                'user_id' => Auth::user()->id,
                'location_id' => Auth::user()->location_id,
                'client_id' => Auth::user()->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            ];

            CustomerCampaign::create($fields);

            return response()->json(['message' => 'Campaign saved successfully', 'success' => true], 200);
        }

        return response()->json(['message' => "$request->name already exists", 'success' => false], 400);
    }

    public function getCustomerCampaign() {
        return CustomerCampaign::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('status', STATUS)
        ->get();
    }


    public function getAllCustomersPhoneEmail()
    {
        $client_id=Auth::user()->client_id;
        $query = "
            SELECT 
                email,
                phone
            FROM " . TBL_CUSTOMERS . " 
            WHERE status=0 AND client_id='$client_id' AND name != 'default vendor' AND name != 'walk-in customer'
        ";

        $data = DB::select($query);

        return $data;
    }

    public function sendEmailToAllCustomers(Request $request) {
        try {
            $customers = $this->getAllCustomersPhoneEmail();
            $sent = 0;
            foreach($customers as $customer) {
                if($customer->email){
                    $this->generalController->sendEmail($customer->email, $request->subject, $request->message);
                    $sent += 1;
                }
            } 
            
            $this->calculateSent($sent, $request->id);
            
            return response()->json(['message' => 'Email sent successfully', 'data' => [], 'success' => true], 200);
        }catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage(), 'success' => false], 400);
        }
    } 
    
    private function calculateSent($sent, $id="") {
        if(!empty($id)){
            $currentSent =  CustomerCampaign::where('client_id', Auth::user()->client_id)
            ->where('location_id', Auth::user()->location_id)
            ->where('status', STATUS)
            ->where('id', $id)
            ->value('sent');

            $newSent = $currentSent + $sent;

            CustomerCampaign::where('client_id', Auth::user()->client_id)
            ->where('location_id', Auth::user()->location_id)
            ->where('status', STATUS)
            ->where('id', $id)
            ->update([
                'sent' => $newSent
            ]);
        } else {
            $currentSent =  CustomerCampaign::where('client_id', Auth::user()->client_id)
            ->where('location_id', Auth::user()->location_id)
            ->where('status', STATUS)
            ->value('sent');

            $newSent = $currentSent + $sent;

            CustomerCampaign::where('client_id', Auth::user()->client_id)
            ->where('location_id', Auth::user()->location_id)
            ->where('status', STATUS)
            ->update([
                'sent' => $newSent
            ]);
        }
    }

    public function sendSmsToAllCustomers(Request $request) {
        try {
            $customers = $this->getAllCustomersPhoneEmail();
            $sent = 0;
            foreach($customers as $customer) {
                if($customer->phone){
                    $this->smsController->sendSMS($customer->phone, $request->message);
                    $sent += 1;
                }
            } 

            $this->calculateSent($sent, $request->id);            

            return response()->json(['message' => 'SMS sent successfully', 'data' => [], 'success' => true], 200);
        }catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage(), 'success' => false], 400);
        }
    }

    public function patronage($itemId) {
        $client_id=Auth::user()->client_id;
        $location_id=Auth::user()->location_id;

        $query = "
            WITH patronage as (
                SELECT 
                    DISTINCT coy
                FROM " . TBL_PRODUCT_LOG . "
                WHERE item_id='$itemId' AND control_id != 1 AND client_id='$client_id' AND location_id='$location_id'
            ),

            customers as (
                SELECT 
                    email,
                    phone
                FROM " . TBL_CUSTOMERS . " c
                INNER JOIN patronage p ON p.coy=c.id
                WHERE c.status=0 AND c.client_id='$client_id' AND c.name != 'default vendor' AND c.name != 'walk-in customer'
            )

            SELECT * FROM customers;
        ";

        $data = DB::select($query);

        return $data;
    }

    public function sendEmailToPatronage(Request $request) {
        try {
            $customers = $this->patronage($request->itemId);
            $sent = 0;
            foreach($customers as $customer) {
                $this->generalController->sendEmail($customer->email, $request->subject, $request->message);
                $sent += 1;
            } 
            
            $this->calculateSent($sent, $request->id);
            
            return response()->json(['message' => 'Email sent successfully', 'data' => [], 'success' => true], 200);
        }catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage(), 'success' => false], 400);
        }
    }

    public function sendSmsToPatronage(Request $request) {
        try {
            $sent = 0;
            $customers = $this->patronage($request->itemId);
            foreach($customers as $customer) {
                $this->smsController->sendSMS($customer->phone, $request->message);
                $sent+=1;
            } 
            
            $this->calculateSent($sent, $request->id);
            
            return response()->json(['message' => 'SMS sent successfully', 'data' => [], 'success' => true], 200);
        }catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage(), 'success' => false], 400);
        }
    }

    public function sendEmailToCustomerRetargeting(Request $request) {
        try {
            foreach($request->data as $req) {
                if (is_array($req)) {
                    $req = (object) $req;
                }
                
                $sent = 0;
                $customers = $this->customerRetargetingController->customerRetargeting($req);
               
                foreach($customers as $customer) {
                    $this->generalController->sendEmail($customer->email, $request->subject, $request->message);
                    $sent+=1;
                } 
                
                $this->calculateSent($sent, $req->id);
            }
            
            return response()->json(['message' => 'Email sent successfully', 'data' => [], 'success' => true], 200);
        }catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage(), 'success' => false], 400);
        }
    }

    public function sendSmsToCustomerRetargeting(Request $request) {
        try {
            foreach($request->data as $req) {
                if (is_array($req)) {
                    $req = (object) $req;
                }

                $sent = 0;
                $customers = $this->customerRetargetingController->customerRetargeting($req);
                
                foreach($customers as $customer) {
                    $this->smsController->sendSMS($customer->phone, $request->message);
                    $sent+=1;
                }
                $this->calculateSent($sent, $req->id);
               
            }
            
            return response()->json(['message' => 'SMS sent successfully', 'data' => [], 'success' => true], 200);
        }catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage(), 'success' => false], 400);
        }
    }
}