<?php

namespace App\Http\Controllers\CustomerSupport;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Retails\Items;
use App\Models\Retails\ItemPrice;
use App\Models\Retails\Stock;
use App\Models\Retails\StockAdjustment;
use App\Models\Retails\TxnLog;
use Illuminate\Support\Facades\Auth;
use App\Services\ZeptoMailService;
use Illuminate\Support\Facades\Log;
use App\Models\Account\User;
use App\Http\Controllers\General\GeneralController;
use Carbon\Carbon;
use App\Http\Controllers\Transactions\Sales\SalesController;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Folio\CustomerVendor\CustomerVendorController;
use App\Http\Controllers\General\SmsController;
use App\Http\Controllers\Folio\Location\LocationController;

class CustomerSupportController extends Controller
{
    //
    protected $zeptoMailService;
    protected $generalController;
    protected $salesController;
    protected $customerVendorController;
    protected $smsController;
    protected $locationController;

    public function __construct(
        ZeptoMailService $zeptoMailService,
        GeneralController $generalController,
        SalesController $salesController,
        CustomerVendorController $customerVendorController,
        SmsController $smsController,
        LocationController $locationController
    )
    {
        $this->zeptoMailService = $zeptoMailService;
        $this->generalController = $generalController;
        $this->salesController = $salesController;
        $this->customerVendorController = $customerVendorController;
        $this->smsController = $smsController;
        $this->locationController = $locationController;
    }

    public function contactUs(Request $request) {
        $firstName = $request->firstName;
        $lastName = $request->lastName;
        $email = $request->email;
        $phone = $request->phone;
        $message = $request->message;

        $to = 'klakpad@gmail.com'; 
        $subject = "$lastName $firstName contacted us to make an enquiry";
        $htmlBody = "<div>
                        Email: $email <br/>
                        Phone: $phone <br/>
                        $message
                    </div>";
    
        $response = $this->zeptoMailService->sendEmail($to, $subject, $htmlBody);

        if ($response->successful()) {
            return response()->json(['message' => 'Email sent successfully']);
        } else {
            // Log the status code and response body for debugging
            \Log::error('Email sending failed', [
                'status' => $response->status(),
                'response' => $response->body(),
            ]);
    
            // Provide a generic failure message or include some response details if appropriate
            return 'Email sending failed. Status: ' . $response->status();
        }
        
    }

    public function updateItemAsNeeded(Request $request) {
        
        Items::where('id', $request->item_id)
        ->where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->update([
            'status' => 1
        ]);

        ItemPrice::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('item_id', $request->item_id)
        ->where('status', STATUS)
        ->update([
            'status' => 1
        ]);

        Stock::where('item_id', $request->item_id)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('control_id', STATUS)
        ->update([
            'control_id' => 1
        ]);

        TxnLog::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('control_id', STATUS)
        ->where('token', $request->token)
        ->update([
            'control_id' => 1
        ]);

        return response()->json(['message' => 'Item updated successfully', 'success' => true], 201);

    }

    public function updateItemSalesType(Request $request) {
        Items::where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->where('sales_type', '!=', 2)
        ->update([
            'sales_type' => 1
        ]);

        return response()->json(['message' => 'Item updated successfully', 'success' => true], 201);
    }

    public function updateStockAsNeeded(Request $request) {
        Stock::where('id', $request->id)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('control_id', STATUS)
        ->update([
            'control_id' => 1
        ]);
        
        return response()->json(['message' => 'Stock updated successfully', 'success' => true], 201);
    }

    public function updateStockAdjustmentAsNeeded(Request $request) {
        StockAdjustment::where('token', $request->id)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('control_id', STATUS)
        ->update([
            'control_id' => 1
        ]);

        Stock::where('token', $request->id)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('control_id', STATUS)
        ->update([
            'control_id' => 1
        ]);

        TxnLog::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('token', $request->id)
        ->update([
            'control_id' => 1
        ]);
        
        return response()->json(['message' => 'Stock adjustment updated successfully', 'success' => true], 201);
    }

    public function updateTxnLogDrAsNeeded(Request $request) {
        TxnLog::where('id', $request->id)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('control_id', STATUS)
        ->update([
            'jvalue' => $request->value,
            'jvdr' => $request->value
        ]);

        return response()->json(['message' => 'TxnLog updated successfully', 'success' => true], 201);
    }

    public function updateTxnLogCrAsNeeded(Request $request) {
        TxnLog::where('id', $request->id)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('control_id', STATUS)
        ->update([
            'jvalue' => $request->value,
            'jvcr' => $request->value
        ]);

        return response()->json(['message' => 'TxnLog updated successfully', 'success' => true], 201);
    }

    public function updateTxnLogAsNeeded(Request $request) {
        TxnLog::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('token', $request->token)
        ->update([
            'control_id' => 1
        ]);

        return response()->json(['message' => 'TxnLog updated successfully', 'success' => true], 201);
    }

    public function saveTxnLogAsNeeded(Request $request) {
        $customer = $this->customerVendorController->customerName(Auth::user()->client_id, $request->customer);
        $this->salesController->saveTxnLog($request, $customer);

        return response()->json(['message' => 'TxnLog saved successfully', 'success' => true], 201);
    }

    public function generateSupportLink(Request $request) {
        try {
            $user = User::where('username', $request->email)
            ->where('status', STATUS)
            ->first();

            $email = $this->locationController->locationEmail($user->location_id, $user->client_id);

            $url = "https://testing-temp.klakpad.dev/api/generate-support-pin/" . $user->username;
            
            $this->generalController->sendEmail($email, 'Klakpad support pin', "Click this link $url to generate a support pin.");

            User::where('username', $user->username)
            ->where('id', $user->id)
            ->where('status', STATUS)
            ->where('client_id', $user->client_id)
            ->where('location_id', $user->location_id)
            ->update([
                'support_pin_time' => Carbon::now()->addMinutes(10)->format('Y-m-d H:i:s')
            ]);

            return response()->json(['message' => 'Support link generated successfully. This link will expire in 10mins.', 'success' => true], 201);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error creating support link', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }

    public function generateSupportLinkAuth() {
        try {
            $user = User::where('id', Auth::user()->id)
            ->where('client_id', Auth::user()->client_id)
            ->where('location_id', Auth::user()->location_id)
            ->where('status', STATUS)
            ->first();

            $email = $this->locationController->locationEmail($user->location_id, $user->client_id);

            $url = "https://testing-temp.klakpad.dev/api/generate-support-pin/" . $user->username;
            
            $this->generalController->sendEmail($email, 'Klakpad support pin', "Click this link $url to generate a support pin.");

            User::where('username', $user->username)
            ->where('id', Auth::user()->id)
            ->where('client_id', Auth::user()->client_id)
            ->where('location_id', Auth::user()->location_id)
            ->where('status', STATUS)
            ->update([
                'support_pin_time' => Carbon::now()->addMinutes(10)->format('Y-m-d H:i:s')
            ]);

            return response()->json(['message' => 'Support link generated successfully. This link will expire in 10mins.', 'success' => true], 201);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error creating support link', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }
   

    public function generateSupportPin(Request $request, $email) {
        try{
            // getting user details
            $user = User::where('username', $email)
            ->where('status', STATUS)
            ->first();

            // checking if support link has expired
            if ($user->support_pin_time <= Carbon::now()) {
                return response()->json(['message' => 'Support link has expired', 'success' => false], 401);
            }

            // generating support pin code
            $code = "klakpad-support-".random_int(1000, 9999);
            $salt = $this->generalController->salt();
            $password = $this->generalController->hashPass($code, $salt);

            // send sms code to location phone number
            $phoneNumber = $this->locationController->locationPhone($user->location_id, $user->client_id);
            $this->smsController->sendSMS($phoneNumber, "Your support pin code is $code, it will expire in 10mins.");

            // update user data with encrypted code
            User::where('client_id', $user->client_id)
            ->where('location_id', $user->location_id)
            ->where('id', $user->id)
            ->update([
                'password2' => $password,
                'salt2' => $salt,
                'support_pin_time' => Carbon::now()->addMinutes(10)->format('Y-m-d H:i:s')
            ]);

            // redirect to home page or verification if successful
            return view('verification.success')->with('message', 'OTP code successfully generated.');
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error creating support pin', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }

}