<?php

namespace App\Http\Controllers\Folio\Bank;

use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Retails\BankAccount;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use Validator;
use Carbon\Carbon;
use DB;
use App\Models\Retails\TxnLog;
use App\Models\Retails\QueueLog;

class BankAccountController extends Controller
{
    
    protected $ChartOfAccountController ;
    protected $TxnLogController ;
    protected $CurrencyController ;

    public function __construct(ChartOfAccountController $ChartOfAccountController, TxnLogController $TxnLogController, CurrencyController $CurrencyController)
    {
        $this->ChartOfAccountController = $ChartOfAccountController;
        $this->TxnLogController = $TxnLogController;
        $this->CurrencyController = $CurrencyController;
    }

    public function getBankIdByBankAccountId($id) {
        return BankAccount::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('id', $id)
        ->value('bank_id');
    }

    public function getBankAccountDropdown() {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;

        $query = "
            SELECT i.id as value, CONCAT(i.name, ' - ', b.name) as title, b.logo as icon FROM " . ONLINE_RETAILS . ".bank_account i
            LEFT JOIN " . ONLINE_ACCOUNT . ".app_bank b
            ON b.id=i.bank_id AND b.status=0 AND b.status = 0
            WHERE i.client_id = '$client_id' AND i.status = 0
        ";

        $data = DB::select($query);

        return response()->json(['data' => $data, 'success' => true], 200);
    }
   
    public function saveBankAccount(Request $request)
    {
        $validationRules = [
            'bank_id' => 'required',
            'account_no' => 'required',
            'name' => 'required',
            'visibility' => 'required'
           // 'previous_balance' => 'required'
        ];
        $validator = Validator::make($request->all(), $validationRules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }
        // Check for existing records that match the given parameters
        $existingBank = $this->checkIfBankExists($request);
        // If a bank account with the same parameters exists, return an error response
        if ($existingBank) {
            return response()->json(['message' => 'Bank account already exists.', 'error' => true], 400);
        }
        $newBankAccount = $this->createBank($request);
        // Handle previous balance if provided
        if (!empty($request->previous_balance)) {
            $this->handlePreviousBalance($request, $newBankAccount);
        }

        // Return a success response with the newly created category
        return response()->json(['message' => 'Bank Account saved successfully', 'data' => $newBankAccount, 'success' => true], 201);
    }

    
    private function createBank(Request $request)
    {
        $uuid = Str::uuid()->toString();
        $uuidWithoutDashes = str_replace('-', '', $uuid);
        $fields = [
            'id' => $uuidWithoutDashes,
            'bank_id' => $request->bank_id,
            'account_no' => $request->account_no,
            'name' => $request->name,
            'visibility' => $request->visibility,
            'status' => STATUS, 
            'user_id' => Auth::user()->id,
            'location_id' => Auth::user()->location_id,
            'client_id' => Auth::user()->client_id,
            'tm' => Carbon::now()->toTimeString(),
            'date' => Carbon::now()->format('Y-m-d'),
        ];
    
        // Perform the create operation
        $res = BankAccount::create($fields);
    
      
    
        return $res;
    }

    

    private function replaceBindings($query, $bindings)
    {
        foreach ($bindings as $binding) {
            $value = is_numeric($binding) ? $binding : "'" . addslashes($binding) . "'";
            $query = preg_replace('/\?/', $value, $query, 1);
        }
        return $query;
    }
    
    

    private function checkIfBankExists(Request $request)
    {
    return BankAccount::where([
        'bank_id' => $request->bank_id,
        'account_no' => $request->account_no,
        'status' => STATUS,
        'user_id' => Auth::user()->id,
        'client_id' => Auth::user()->client_id,
        'location_id' => Auth::user()->location_id,
    ])->first();
    }


    private function handlePreviousBalance(Request $request, $newBankAccount)
    {
     
        $accLegs = $this->ChartOfAccountController->getAccountLegs(CHART_BANK, CHART_SUSPENSE, Auth::user()->client_id);
       
        $tokenUid = Str::uuid()->toString();

        $currencyId = $this->CurrencyController->getPrimaryCurrency(Auth::user()->client_id, Auth::user()->location_id);

        $currencyUnit = $this->CurrencyController->getCurrencyUnit(Auth::user()->client_id, Auth::user()->location_id, $currencyId);
        $txnDate = $request->txnDate;

        $this->TxnLogController->createTxnLog($accLegs, $request->previous_balance, $tokenUid, $currencyUnit, $txnDate, "", 'Being Bank Previous Balance', "", 'pb', $request->currency_id, "", $newBankAccount->id, $newBankAccount->bank_id);
    }


    public function editBankAccount(Request $request, $bankAccountId)
    {
        $validationRules = [
            'bank_id' => 'required',
            'account_no' => 'required',
            'name' => 'required',
            'visibility' => 'required'
        ];

        $validator = Validator::make($request->all(), $validationRules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        // Update the bank account data
        $bankAccount = BankAccount::find($bankAccountId);
        $bankAccount->bank_id = $request->bank_id;
        $bankAccount->account_no = $request->account_no;
        $bankAccount->name = $request->name;
        $bankAccount->visibility = $request->visibility;
        $bankAccount->save();

        return response()->json(['message' => 'Bank Account updated successfully', 'data' => $bankAccount, 'success' => true], 200);
    }

   
    public function getBankDetails(Request $request)
    {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;

    
        $bankName = $request->input('search') ?? '';
        $search = !empty($bankName) ? "WHERE bank_name like '%$bankName%'" : '';


        $banksearch = $request->input('banksearch') ?? '';
        $searchList = !empty($banksearch) ? "AND name like '%$banksearch%'" : '';

        $locationSort = !empty($request->location_id) ? "AND location_id='$request->location_id'" : "AND location_id='$location_id'";

        $query = "

        WITH  bank_name as (
            SELECT * FROM  " . ONLINE_ACCOUNT .".app_bank
        ),
        bank_txn as (
            SELECT
                bank_account_id,
                COALESCE(SUM(jvdr), 0) AS jvdr,
                COALESCE(SUM(jvcr), 0) AS jvcr
            FROM  " . ONLINE_RETAILS .".txn_log
            WHERE chart_id = ".CHART_BANK." AND client_id = '$client_id' $locationSort
            GROUP BY bank_account_id
        ),
        bank_details as (
            SELECT
                a.id,
                b.name as bank_name,
                b.logo as logo,
                a.name as bank_account_name,
                a.bank_id,
                COALESCE(SUM(t.jvdr - t.jvcr), 0) as balance,
                a.date,
                a.account_no,
                a.visibility,
                a.location_id  
            FROM  " . ONLINE_RETAILS .".bank_account a
            LEFT JOIN bank_name b ON a.bank_id = b.id
            LEFT JOIN bank_txn t ON t.bank_account_id = a.id
            WHERE a.client_id = '$client_id' AND a.status = 0 
            GROUP BY a.id
        ),
        bank_with_location as (
            SELECT
                bd.*,
                l.name as location_name
            FROM bank_details bd
            LEFT JOIN  " . ONLINE_RETAILS .".location l ON bd.location_id = l.id $search
        ),

        bank_summary as (
            SELECT   
            CASE WHEN dr.DebitBalance IS NULL 
                THEN 0
                ELSE dr.DebitBalance
            END AS Credit,
    
            CASE WHEN cr.CreditBalance IS NULL 
                THEN 0
                ELSE cr.CreditBalance
            END AS Debit,
    
            CASE WHEN cr.CreditBalance IS NOT NULL 
                THEN ROUND(dr.DebitBalance - cr.CreditBalance, 2) 
                ELSE dr.DebitBalance
            END AS balance,
    
            CASE WHEN dr.effect IS NULL 
                THEN cr.effect
                ELSE dr.effect
            END AS effect
    
            FROM " . ONLINE_RETAILS .".txn_log 
    
            INNER JOIN ( 
                SELECT SUM(jvalue) AS DebitBalance, b.effect 
                FROM " . ONLINE_RETAILS .".txn_log a 
                INNER JOIN " . ONLINE_RETAILS .".account_type b ON b.sort_code = a.account_type
                WHERE a.effect = 'dr' AND a.chart_id = ".CHART_BANK." AND a.client_id = '$client_id' AND a.control_id != '".ONE_STATUS."' $locationSort
            ) AS dr 
    
            JOIN ( 
                SELECT SUM(jvalue) AS CreditBalance, b.effect  
                FROM " . ONLINE_RETAILS .".txn_log a 
                INNER JOIN " . ONLINE_RETAILS .".account_type b ON b.sort_code = a.account_type
                WHERE a.effect = 'cr' AND a.chart_id = ".CHART_BANK." AND a.client_id = '$client_id' AND a.control_id != '".ONE_STATUS."' $locationSort
            ) cr 
    
            WHERE control_id != '".ONE_STATUS."' AND client_id = '$client_id' $locationSort LIMIT 1
            ), 

            bank_list as (
             SELECT * FROM " . ONLINE_ACCOUNT .".app_bank where status = '".STATUS."' $searchList
            ),


        banklocation_array AS (
            SELECT JSON_ARRAYAGG(
                JSON_OBJECT(
                    'id', id,
                    'bank_name', bank_name,
                    'logo', logo,
                    'bank_account_name', bank_account_name,
                    'bank_id', bank_id,
                    'balance', balance,
                    'date', date,
                    'account_no', account_no,
                    'visibility', visibility,
                    'location_id', location_id,
                    'location_name', location_name
                )
            ) AS bank_account
            FROM bank_with_location
        ),
        
        banksummary_array AS (
            SELECT JSON_ARRAYAGG(
                JSON_OBJECT(
                    'Credit', Credit,
                    'Debit', Debit,
                    'balance', balance,
                    'effect', effect
                )
            ) AS bank_summ
            FROM bank_summary
                ),

        banklist_array AS (
            SELECT JSON_ARRAYAGG(
                JSON_OBJECT(
                    'value', id,
                    'title', name,
                    'icon', logo
                )
            ) AS bank_list
            FROM bank_list
        )
        
        SELECT * 
        FROM banklocation_array
        JOIN banksummary_array ON 1=1
        JOIN banklist_array ON 1=1;
    ";
//    echo $allCustomersQuery;  exit;

        $jsonString = DB::select($query);

        foreach ($jsonString as &$row) {
            if (property_exists($row, 'bank_account')) {
                $row->bank_account = json_decode(stripslashes($row->bank_account), true) ?? [];
            }

            if (property_exists($row, 'bank_summ')) {
                $row->bank_summ = json_decode(stripslashes($row->bank_summ), true) ?? [];
            }

            if (property_exists($row, 'bank_list')) {
                $row->bank_list = json_decode(stripslashes($row->bank_list), true) ?? [];
            }
        }

        return response()->json([
            'data' => $jsonString,
            'success' => true
        ], 200);
    }
    
    public function getBankAccounts(Request $request)
{
    $client_id = Auth::user()->client_id;

  
    $bankName = $request->input('search') ?? '';
    $search = !empty($bankName) ? "WHERE bank_name like '%$bankName%'" : '';

    $locationSort = !empty($request->location_id) ? "AND location_id='$request->location_id'" : "";
    $query = "
    WITH  bank_name as (
        SELECT * FROM  " . ONLINE_ACCOUNT .".app_bank
    ),
    bank_txn as (
        SELECT
            bank_account_id,
            COALESCE(SUM(jvdr), 0) AS jvdr,
            COALESCE(SUM(jvcr), 0) AS jvcr
        FROM  " . ONLINE_RETAILS .".txn_log
        WHERE chart_id = ".CHART_BANK." AND client_id = '$client_id' $locationSort
        GROUP BY bank_account_id
    ),
    bank_details as (
        SELECT
            a.id,
            b.name as bank_name,
            b.logo as logo,
            a.name as bank_account_name,
            a.bank_id,
            COALESCE(SUM(t.jvdr - t.jvcr), 0) as balance,
            a.date,
            a.account_no,
            a.visibility,
            a.location_id  
        FROM  " . ONLINE_RETAILS .".bank_account a
        LEFT JOIN bank_name b ON a.bank_id = b.id
        LEFT JOIN bank_txn t ON t.bank_account_id = a.id
        WHERE a.client_id = '$client_id' AND a.status = 0 
        GROUP BY a.id
    ),
    bank_with_location as (
        SELECT
            bd.*,
            l.name as location_name
        FROM bank_details bd
        LEFT JOIN  " . ONLINE_RETAILS .".location l ON bd.location_id = l.id
    )
    SELECT * FROM bank_with_location
     $search
";

    $data = DB::select($query);

    return response()->json(['data' => $data, 'success' => true], 200);
}


public function getBankSummary(Request $request) {
    $data = $this->ChartOfAccountController->chartIdSummary(CHART_BANK, $request);
    return response()->json(['data' => $data, 'success' => true], 200);
}

public function deletebank($id) {
       
    $record = BankAccount::find($id);
    $checkTransaction = TxnLog::where('bank_account_id', $id)->first();

    if (!$record) {
        return response()->json(['message' => 'Record not found', 'success' => false], 404);
    }

    if ($checkTransaction) {
        return response()->json(['message' => 'Bank account can not be deleted', 'success' => false], 404);
    }


    $record->update(['status' => 1]);

    return response()->json(['message' => 'Bank account deleted successfully', 'success' => true], 200);
}


public function getSingleBankAccount($id) {
    $record = BankAccount::with('bank')->find($id);

    if (!$record) {
        return response()->json(['message' => 'Record not found', 'success' => false], 404);
    }


    return response()->json([
        'data' => $record,
        'message' => 'Bank account retrieved successfully',
        'success' => true
    ], 200);
}


    
    




public function getBankAccountStatement(Request $request, $bankAccountId)
{
    $client_id = Auth::user()->client_id;
    $bank = !empty($bankAccountId) ? "AND a.bank_account_id = '$bankAccountId'" : "";

    $search = !empty($request->search) ? "AND a.token = '$request->search' OR a.pmethod = '$request->search' OR a.bene = '$request->search' " : "";

    $locationSort = !empty($request->location_id) ? "AND a.location_id='$request->location_id'" : "";
    $startDate = $request->start_date;
    $endDate = $request->end_date;
    $date = !empty($startDate and $endDate) ? "AND a.txn_date BETWEEN '$startDate' AND '$endDate'" : "";
    $limit = "DESC LIMIT $request->count, 50";


    $query = "
    WITH bank_statement AS (
        SELECT    
         ROW_NUMBER() OVER (ORDER BY txn_date ASC , tm ASC) AS rowNum,
            a.txn_date,
            a.tm,
            b.name,
            a.being,
            a.bene,
            a.token,
            a.effect,
            a.pmethod,
            a.currency_unit,
            a.jvalue AS amount,
            
              case when a.jvdr = ''
              then 0
              else a.jvdr
              end as Debit,
        
             case when a.jvcr = ''
              then 0
              else a.jvcr
              end as Credit,
              
            (a.jvdr-a.jvcr) AS balance
              
            FROM  " . ONLINE_RETAILS .".txn_log a
            LEFT JOIN ". ONLINE_ACCOUNT .".app_bank b
            ON b.id=a.bank_id AND b.status=0
         WHERE a.chart_id =  '".CHART_BANK."' AND a.jvalue > 0 AND a.control_id != '".ONE_STATUS."' AND a.bank_account_id = '$bankAccountId' 
         AND a.client_id = $client_id $locationSort $search $date ORDER BY a.txn_date,a.tm $limit
    ),
    summary as (
        SELECT SUM(debit) as totalDebit, SUM(credit) as totalCredit, SUM(debit-credit) as totalBalance
        FROM
        bank_statement)

       
         
         SELECT b1.*,SUM(b2.balance) as RunningBalance,c.* FROM bank_statement b1 
         LEFT JOIN  bank_statement b2 ON 
          b1.rowNum >= b2.rowNum
          JOIN summary c ON 1=1
          GROUP BY b1.tm,b1.being, b1.bene,b1.token,b1.currency_unit,b1.amount, b1.balance";

        
    // Query to calculate total credit
    // $totalCreditQuery = "SELECT SUM(jvcr) AS totalCredit FROM  " . ONLINE_RETAILS .".txn_log WHERE bank_account_id = '$bankAccountId' AND effect = 'cr' AND chart_id = '".CHART_BANK."'"; 

    // // Query to calculate total debit
    // $totalDebitQuery = "SELECT SUM(jvdr) AS totalDebit FROM  " . ONLINE_RETAILS .".txn_log WHERE bank_account_id = '$bankAccountId' AND effect = 'dr' AND chart_id = '".CHART_BANK."' ";

    // Execute queries
    $data = DB::select($query);
    // $totalCredit = DB::select($totalCreditQuery)[0]->totalCredit;
    // $totalDebit = DB::select($totalDebitQuery)[0]->totalDebit;

    // // Calculate balance
    // $balance = $totalDebit - $totalCredit ;

    // // Merge results into the existing array
    // $summary = ['totalDebit' => $totalCredit ? $totalCredit : STATUS, 'totalCredit' => $totalDebit, 'balance' => $balance];
    // $result = ['summary' => $summary, 'data' => $data, 'success' => true];

    // return response()->json($data, 200);

    return response()->json([
        'data' => $data,
        'message' => 'Bank transaction retrieved successfully',
        'success' => true
    ], 200);
}



   

   

    


}
