<?php

namespace App\Http\Controllers\Folio\CustomerVendor;

use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Models\Retails\Customer;
use App\Models\Retails\TxnLog;
use Carbon\Carbon;
use DB;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use Validator;

class CustomerVendorController extends Controller
{
    protected $ChartOfAccountController;
    protected $TxnLogController;
    protected $CurrencyController;

    public function __construct(ChartOfAccountController $ChartOfAccountController, TxnLogController $TxnLogController, CurrencyController $CurrencyController)
    {
        $this->ChartOfAccountController = $ChartOfAccountController;
        $this->TxnLogController = $TxnLogController;
        $this->CurrencyController = $CurrencyController;
    }

    public function isDefaultCustomer($client_id)
    {
        return Customer::where('status', STATUS)
            ->where('client_id', $client_id)
            ->where('coy_name', WALK_IN_CUSTOMER)
            ->where('cate', 'lba')
            ->first();
    }

    public function customerName($client_id, $id)
    {
        return Customer::where('status', STATUS)
            ->where('client_id', $client_id)
            ->where('id', $id)
            ->value('name');
    }

    public function customerDropdown()
    {
        $data = Customer::where('status', STATUS)
            ->where('client_id', Auth::user()->client_id)
            ->where('name', '!=', DEFAULT_VENDOR)
            ->select('id as value', 'name as title')
            ->get();

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function getAllCustomerDropdown()
    {
        $data = Customer::where('status', STATUS)
            ->where('client_id', Auth::user()->client_id)
            ->where('name', '!=', WALK_IN_CUSTOMER)
            ->select('id as value', 'name as title')
            ->get();

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function isDefaultVendor($client_id)
    {
        return Customer::where('status', STATUS)
            ->where('client_id', $client_id)
            ->where('coy_name', DEFAULT_VENDOR)
            ->where('cate', 'cust')
            ->first();
    }

    public function addCustomer(Request $request)
    {
        $request->validate([
            'coy_name' => 'required',
            'gender' => 'required|string',
            'phone' => 'required|size:11',
            'cate' => 'required',
        ]);

        $cate = $request->cate;
        $email = $request->email;

        $existingFields = ['coy_name', 'phone', 'email'];
        foreach ($existingFields as $field) {
            $result = $this->checkExistingRecord($field, $request->$field, $cate, '', $email);
            if ($result) {
                return response()->json(['message' => $result, 'error' => true], 400);
            }
        }

        $newRecord = $this->createCustomer($request, $cate);

        if ($request->has('previous_balance')) {
            $this->handlePreviousBalance($request, $newRecord);
        }

        return response()->json(['message' => 'Customer saved successfully', 'success' => true], 201);
    }

    private function handlePreviousBalance(Request $request, $newRecord)
    {
        if ($request->previous_balance > STATUS) {
            $accLegs = $this->ChartOfAccountController->getAccountLegs(CHART_SUSPENSE, CHART_RECEIVABLE, Auth::user()->client_id);
        } elseif ($request->previous_balance < STATUS) {
            $accLegs = $this->ChartOfAccountController->getAccountLegs(CHART_RECEIVABLE, CHART_SUSPENSE, Auth::user()->client_id);
        }

        $tokenUid = Str::uuid()->toString();
        $currencyUnit = $this->CurrencyController->getCurrencyUnit(Auth::user()->client_id, Auth::user()->location_id, $request->currency_id);
        $txnDate = Carbon::now()->format('Y-m-d'); // $request->txnDate;

        $this->TxnLogController->createTxnLog($accLegs, $request->previous_balance, $tokenUid, $currencyUnit, $txnDate, $newRecord->id, 'Being Customer Balance', $request->coy_name, 'pb', $request->currency_id);
    }

    private function createCustomer(Request $request, $cate)
    {
        $uuid = Str::uuid()->toString();
        $uuidWithoutDashes = str_replace('-', '', $uuid);
        $newRecord = new Customer();
        $newRecord->id = $uuidWithoutDashes;
        $newRecord->coy_name = $request->coy_name;
        $newRecord->name = $request->coy_name;
        $newRecord->gender = $request->gender;
        $newRecord->phone = $request->phone;
        $newRecord->cate = $cate;
        $newRecord->dob = $request->dob;
        $newRecord->credit_limit = $request->credit_limit;
        $newRecord->email = $request->email;
        $newRecord->address = $request->address;
        $newRecord->loyalty_code = Str::uuid()->toString();
        $newRecord->year = Carbon::now()->year;
        $newRecord->month = Carbon::now()->month;
        $newRecord->day = Carbon::now()->day;
        $newRecord->user_id = Auth::user()->id;
        $newRecord->location_id = Auth::user()->location_id;
        $newRecord->client_id = Auth::user()->client_id;
        $newRecord->tm = Carbon::now()->toTimeString();
        $newRecord->date = Carbon::now()->format('Y-m-d');
        $newRecord->update_date = Carbon::now()->format('Y-m-d');
        $newRecord->status = STATUS;
        $newRecord->save();

        return $newRecord;
    }

    public function editCustomer(Request $request, $customerId)
    {
        $validationRules = [
        'coy_name' => 'required',
        'gender' => 'required|string',
        'phone' => 'required|size:11',
        'cate' => 'required',
    ];

        // Create a validator with the defined rules and the request data
        $validator = Validator::make($request->all(), $validationRules);

        // Check if validation fails and return an error response if it does
        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        $fieldsToCheck = ['coy_name', 'phone', 'email'];
        $cate = $request->cate;
        $email = $request->email;

        foreach ($fieldsToCheck as $field) {
            $result = $this->checkExistingRecord($field, $request->$field, $cate, $customerId, $email);
            if ($result) {
                return response()->json(['message' => $result, 'error' => true], 400);
            }
        }

        // Find the customer record to update
        $customer = Customer::where('id', $customerId)
    ->where('status', 0)
    ->first();

        if (!$customer) {
            return response()->json(['message' => 'Customer not found', 'error' => true], 404);
        }

        // Update the customer record with the request data
        $customer->coy_name = $request->coy_name;
        $customer->name = $request->coy_name;
        $customer->gender = $request->gender;
        $customer->phone = $request->phone;
        $customer->cate = $cate;
        $customer->dob = $request->dob;
        $customer->credit_limit = $request->credit_limit;
        $customer->email = $request->email;
        $customer->address = $request->address;
        $customer->loyalty_code = $customer->loyalty_code;
        $customer->year = Carbon::now()->year;
        $customer->month = Carbon::now()->month;
        $customer->day = Carbon::now()->day;
        $customer->user_id = Auth::user()->id;
        $customer->location_id = Auth::user()->location_id;
        $customer->client_id = Auth::user()->client_id;
        $customer->tm = Carbon::now()->toTimeString();
        $customer->date = Carbon::now()->format('Y-m-d');
        $customer->update_date = Carbon::now()->format('Y-m-d');
        $customer->status = 0; // Assuming status remains unchanged
        $customer->save();

        return response()->json(['message' => 'Customer updated successfully', 'success' => true], 200);
    }

    public function checkExistingRecord($field, $value, $cate, $customerId = null, $email = null)
    {
        $conditions = [
            'cate' => $cate,
            'client_id' => Auth::user()->client_id,
            'status' => 0,
        ];

        switch ($field) {
            case 'coy_name':
                $conditions['coy_name'] = $value;
                $message = 'Folio already exists.';
                break;

            case 'phone':
                $conditions['phone'] = $value;
                $message = 'Phone already exists.';
                break;

            case 'email':
                if (!empty($value) || $value !== null) {
                    $conditions['email'] = $value;  // Use $value instead of $email
                    $message = 'Email already exists.';
                } else {
                    return false;
                }
                break;

            default:
                return false; // Invalid field
        }

        // If $customerId is provided, exclude that record from the check
        if ($customerId) {
            $existingRecord = Customer::where($conditions)->where('id', '!=', $customerId)->first();
        } else {
            $existingRecord = Customer::where($conditions)->first();
        }

        return $existingRecord ? $message : false;
    }

    public function deleteCustomer($id)
    {
        $record = Customer::find($id);
        $checkTransaction = TxnLog::where('coy', $id)
        ->where('control_id', STATUS)
        ->first();

        if (!$record) {
            return response()->json(['message' => 'Record not found', 'success' => false], 404);
        }

        if ($checkTransaction) {
            return response()->json(['message' => 'There is a transaction tied to this customer', 'success' => false], 404);
        }

        $record->update(['status' => 1]);

        return response()->json(['message' => 'Customer deleted successfully', 'success' => true], 200);
    }

    public function getCustomerbyId($id)
    {
        $record = Customer::find($id);

        if (!$record) {
            return response()->json(['message' => 'Record not found', 'success' => false], 404);
        }

        return response()->json(['data' => $record, 'message' => 'Customer data fetched successfully', 'success' => true], 200);
    }

    ///add getBankAccountDropdown
    public function getAllCustomers(Request $request)
    {
        $cusName = $request->input('search') ?? '';
        $search = !empty($cusName) ? "AND c.coy_name like '%$cusName%'" : '';
        $credDebtFilter = $request->crdr;
        $locationSort = !empty($request->location_id) ? "AND c.location_id='".$request->location_id."'" : '';
        $locationSortOthers = !empty($request->location_id) ? "AND t.location_id='".$request->location_id."'" : '';
        $client = Auth::user()->client_id;
        $startDate = $request->start_date;
        $endDate = $request->end_date;
        $date = !empty($startDate) && !empty($endDate) ? "AND c.date BETWEEN '$startDate' AND '$endDate'" : '';
        $dateSummary = !empty($startDate) && !empty($endDate) ? "WHERE c.date BETWEEN '$startDate' AND '$endDate'" : '';
        $credDeb = $credDebtFilter === '' ? '' : ($credDebtFilter === 'debtors' ? 'AND balance < 0' : 'AND balance > 0');
        $cate = !empty($request->cate) ? "AND c.cate='".$request->cate."'" : '';

        $result = DB::select('
           
            
        WITH coys as (
                SELECT c.id, c.coy_name as name, c.cate,  c.gender, c.phone, c.email, c.dob, address, c.credit_limit,
                    loyalty_status, COALESCE(SUM(t.jvcr - t.jvdr), 0) AS balance, c.date
                FROM  '.ONLINE_RETAILS.'.customers c
                LEFT JOIN  '.ONLINE_RETAILS.".txn_log t ON t.coy = c.id AND (chart_id = 20003 OR chart_id = 40000)   $locationSortOthers
                
               
                WHERE c.client_id = '$client' 
                    AND c.coy_name != 'walk-in customer' AND c.coy_name != 'default vendor' $search  $date $locationSort $cate 
                GROUP BY c.id
        ), 
        summary as (
            SELECT count() as total_customers, (SELECT count() as new_customers
            from coys $dateSummary) as new_customers,
            b.debtor_sum, c.debtor_num,d.creditor_sum, e.creditor_num
            FROM coys a
            LEFT JOIN (select sum(balance) as debtor_sum from coys where balance<0) b
            LEFT JOIN (select sum(balance) as creditor_sum from coys where balance>0) d
            )
            SELECT * FROM coys
          LEFT JOIN summary
        ");

        return response()->json(['data' => $result, 'success' => true], 200);
    }

    // const getCustomersByRaw = (
    //     cate,
    //     loyaltyValue = "",
    //     credDebtFilter = "",
    //     newLoc
    //   ) => {
    //     const locationSort = !empty(newLoc) ? `AND c.location_id='${newLoc}'` : "";
    //     const locationSortOthers = !empty(newLoc)
    //       ? `AND t.location_id='${newLoc}'`
    //       : "";
    //     const client = clientId();
    //     const { from, to } = datefilter("MTD");
    //     const loyaltyVal =
    //       loyaltyValue == ""
    //         ? ""
    //         : loyaltyValue == "noLoyalty"
    //         ? `AND c.loyalty_status = ''`
    //         : `AND c.loyalty_status = ${loyaltyValue}`;
    //     const credDeb =
    //       credDebtFilter === ""
    //         ? ""
    //         : credDebtFilter === "debtors"
    //         ? "Where balance < 0"
    //         : "Where balance > 0";
    //     const result = customerModel.rawSQL(`
    //     WITH cus_loyalty as (
    //       SELECT  coy_id, status ,
    //             coalesce (SUM(CASE WHEN effect = 'cr' THEN amount ELSE -amount END),0) AS amount
    //       FROM loyalty_log
    //       GROUP BY coy_id
    //       ),

    //       coys as (
    //       SELECT c.id, c.coy_name as name, c.gender, c.phone, c.email ,c.dob, address ,c.credit_limit,
    //       loyalty_status,coalesce(SUM(t.jvcr-t.jvdr) ,0)as balance, coalesce( g.growing_loyalty,0) as growing_loyalty,
    //       coalesce( m.matured_loyalty,0) as  mature_loyalty,c.date
    //        FROM customers c
    //       LEFT JOIN txn_log t on t.coy = c.id and (chart_id = 20003 OR chart_id = 40000) ${locationSortOthers}
    //       LEFT JOIN (select coy_id, amount  as growing_loyalty  from cus_loyalty where  status = 0) g
    //       on g.coy_id = c.id
    //       LEFT JOIN (select coy_id, amount  as matured_loyalty  from cus_loyalty where  status = 1) m
    //       on m.coy_id = c.id
    //       WHERE c.client_id = '${client}' and  c.cate='${cate}'
    //        and c.coy_name != 'walk-in customer' and c.status=0 ${loyaltyVal}
    //       GROUP BY c.id
    //       ),
    //       summary as (
    //       SELECT count() as total_customers, (SELECT count() as new_customers
    //       from coys where date between '${from}' and '${to}') as new_customers,
    //       b.debtor_sum, c.debtor_num,d.creditor_sum, e.creditor_num
    //       FROM coys a
    //       LEFT JOIN (select sum(balance) as debtor_sum from coys where balance<0) b
    //       LEFT JOIN (select sum(balance) as creditor_sum from coys where balance>0) d
    //       )
    //       SELECT * FROM coys
    //       LEFT JOIN summary
    //       ${credDeb}
    //     `);
    //     return result;
    //   };

    public function getAllCustomersWithBankAccounts(Request $request)
    {
        $cusName = $request->input('search') ?? '';
        $search = !empty($cusName) ? "AND c.coy_name like '%$cusName%'" : '';
       // $credDebtFilter = $request->crdr;
        $locationSort = !empty($request->location_id) ? "AND c.location_id='".$request->location_id."'" : '';
        $locationSortOthers = !empty($request->location_id) ? "AND t.location_id='".$request->location_id."'" : '';
        $client = Auth::user()->client_id;
        $startDate = $request->start_date;
        $endDate = $request->end_date;
        $date = !empty($startDate) && !empty($endDate) ? "AND c.date BETWEEN '$startDate' AND '$endDate'" : '';
        $dateSummary = !empty($startDate) && !empty($endDate) ? "WHERE c.date BETWEEN '$startDate' AND '$endDate'" : '';
        $credDeb = empty($request->crdr) ? '' : ($request->crdr === 'dr' ? 'WHERE balance < 0' : 'WHERE balance >= 0');
        $cate = !empty($request->cate) ? "AND c.cate='".$request->cate."'" : '';
       // $crdr = !empty($request->crdr) ? "AND t.effect='".$request->crdr."'" : '';
        $user = !empty($request->user) ? "AND c.user_id='".$request->user."'" : '';
        $limit = "LIMIT $request->count, 50";

        // Query for all_customers
        $allCustomersQuery = 'WITH coys AS (
        SELECT
            c.id,
            c.coy_name AS name,
            c.cate,
            c.gender,
            c.phone,
            c.email,
            c.dob,
            address,
            c.credit_limit,
            loyalty_status,
            COALESCE(SUM(t.jvcr - t.jvdr), 0) AS balance,
            c.date
        FROM  '.ONLINE_RETAILS.'.customers c
        LEFT JOIN  '.ONLINE_RETAILS.".txn_log t ON t.coy = c.id AND (chart_id = 20003 OR chart_id = 40000) AND t.control_id != 1 $locationSortOthers
        WHERE
            c.client_id = '$client'
            AND c.coy_name != 'walk-in customer'
            AND c.coy_name != 'default vendor' AND c.status = 0
            $search $date $locationSort $cate $user 
        GROUP BY c.id $limit
    ),
    
    
    bank_accounts AS (
        SELECT i.id AS value,  i.name AS title, b.logo AS icon
        FROM ".ONLINE_RETAILS.'.bank_account i
        LEFT JOIN '.ONLINE_ACCOUNT.".app_bank b ON b.id=i.bank_id AND b.status=0
        WHERE i.client_id = '$client' AND i.status = 0
    ),

    summary as (
        SELECT 
        SUM(CASE WHEN balance < 0 THEN balance ELSE 0 END ) AS debtor_sum,
        SUM(CASE WHEN balance > 0 THEN balance ELSE 0 END ) AS creditor_sum
        FROM coys
    ),
    
    customer_array AS (
        SELECT JSON_ARRAYAGG(
            JSON_OBJECT(
                'id', id,
                'name', name,
                'cate', cate,
                'gender', gender,
                'phone', phone,
                'email', email,
                'dob', dob,
                'address', address,
                'credit_limit', credit_limit,
                'loyalty_status', loyalty_status,
                'balance', balance,
                'date', date        
            )
        ) AS customer
        FROM coys  $credDeb
    ),
    
    bank_account_array AS (
        SELECT JSON_ARRAYAGG(
            JSON_OBJECT(
                'value', value,
                'title', title,
                'icon', icon
            )
        ) AS bank_account
        FROM bank_accounts
            ),

    summary_array AS (
        SELECT JSON_ARRAYAGG(
            JSON_OBJECT(
                'debtor_sum', debtor_sum,
                'creditor_sum', creditor_sum
            ) 
        ) AS summaries
        FROM summary
        )
    
    SELECT * FROM customer_array
    JOIN bank_account_array ON 1=1
    JOIN summary_array ON 1=1;
    
            
    ";
        // echo $allCustomersQuery;

        $jsonString = DB::select($allCustomersQuery);

        // Decode the nested JSON strings within "customer" and "bank_account" fields
        foreach ($jsonString as &$row) {
            if (property_exists($row, 'customer')) {
                $row->customer = json_decode(stripslashes($row->customer), true) ?? [];
            }

            if (property_exists($row, 'bank_account')) {
                $row->bank_account = json_decode(stripslashes($row->bank_account), true);
            }

            if (property_exists($row, 'summaries')) {
                $row->summaries = json_decode(stripslashes($row->summaries), true);
            }
        }

        return response()->json([
        'data' => $jsonString,
        'success' => true,
    ], 200);
    }

    private function convertNestedJsonToArrays($jsonString, $keysToDecode)
    {
        // Decode the outermost JSON string
        $decodedData = json_decode($jsonString, true);

        if ($decodedData === null) {
            // Handle JSON decoding error
            return [];
        }

        // Iterate through each element in the outer array
        foreach ($decodedData as &$item) {
            // Decode the nested JSON strings for specified keys
            foreach ($keysToDecode as $key) {
                if (isset($item[$key])) {
                    $item[$key] = json_decode($item[$key], true);
                }
            }
        }

        return $decodedData;
    }

    public function makePayment(Request $request)
    {
        $validationRules = [
            'customer_id' => 'required',
            'amount' => 'required',
            'payment_type' => 'required',
            'payment_method' => 'required',
            'purpose' => 'required',
            'date' => 'required',
        ];

        $txnDate = $request->date;

        // Create a validator with the defined rules and the request data
        $validator = Validator::make($request->all(), $validationRules);

        // Check if validation fails and return an error response if it does
        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        if ($request->payment_method == BANK_TRANSFER or $request->payment_method == POS) {
            if ($request->payment_type == 'paid out') {
                $accLegs = $this->ChartOfAccountController->getAccountLegs(CHART_RECEIVABLE, CHART_BANK, Auth::user()->client_id);
            } else {
                $accLegs = $this->ChartOfAccountController->getAccountLegs(CHART_BANK, CHART_RECEIVABLE, Auth::user()->client_id);
            }
        } elseif ($request->payment_method == CASH) {
            if ($request->payment_type == 'paid out') {
                $accLegs = $this->ChartOfAccountController->getAccountLegs(CHART_RECEIVABLE, CHART_CASH, Auth::user()->client_id);
            } else {
                $accLegs = $this->ChartOfAccountController->getAccountLegs(CHART_CASH, CHART_RECEIVABLE, Auth::user()->client_id);
            }
        }

        $currencyId = $this->CurrencyController->getPrimaryCurrency(Auth::user()->client_id, Auth::user()->location_id);

        $tokenUid = Str::uuid()->toString();
        $currencyUnit = $this->CurrencyController->getCurrencyUnit(Auth::user()->client_id, Auth::user()->location_id, $currencyId);
        $coyName = $this->customerName(Auth::user()->client_id, $request->customer_id);

        $this->TxnLogController->createTxnLog($accLegs, $request->amount, $tokenUid, $currencyUnit, $txnDate, $request->customer_id, $request->purpose, $coyName, P, $currencyId, $request->payment_method, $request->bank_account_id, $request->bank_id, '', $request->payment_type);

        return response()->json(['data' => '', 'message' => 'Payment added successfully', 'success' => true], 200);
    }

    public function getCustomerTransactionList(Request $request, $customerId)
    {
        $startDate = $request->start_date;
        $endDate = $request->end_date;
        $user = $request->user;
        // $customerId;
        // $customerId = $request->customer_id;
        $locationId = $request->location;

        // $search = !empty($request->search) ? "AND t.being = '$request->search'" : "";

        $searchParams = $request->input('search') ?? "";
        $search = !empty($searchParams) ? "AND t.being like '%$searchParams%'" : "";

        // $date = " AND txn_date BETWEEN '$from' AND '$to'";
        $date = !empty($startDate) && !empty($endDate) ? "AND t1.txn_date BETWEEN '$startDate' AND '$endDate'" : "";

        $userSort = !empty($user) ? "AND t.user_id='$user'" : "";
        $locationSort = !empty($locationId) ? "AND t.location_id='$locationId'" : "";
        $client = Auth::user()->client_id;
        $limit = "LIMIT $request->count, 50";

        $result = " 
        WITH txn AS (
            SELECT
            ROW_NUMBER() OVER (ORDER BY t.txn_date ASC, t.tm ASC) AS rowNum,
            t.txn_date, t.tm, t.token, c.name, c.id as coy, t.jvdr, t.jvcr, t.being, t.pmethod, t.pmt_action, l.name as loc_name, t.chart_id, t.effect,
            COALESCE((jvcr - jvdr), 0) AS bal
            FROM  ".ONLINE_RETAILS.".txn_log t
            LEFT JOIN  ".ONLINE_RETAILS.".customers c ON c.id = t.coy
            LEFT JOIN  ".ONLINE_RETAILS.".location l ON l.id = t.location_id
            WHERE
            t.control_id != 1 AND (chart_id = ".CHART_PAYABLE." OR chart_id =  ".CHART_RECEIVABLE.") AND
            t.coy = '$customerId' AND t.jvalue > 0 AND t.client_id = $client $locationSort $userSort $search $userSort $limit 
            
            ),
             
            runningbal AS (
            SELECT t1.*, SUM(t2.bal) as runningBal
            FROM txn t1
            JOIN txn t2 ON t1.rowNum >= t2.rowNum $date
            GROUP BY t1.rowNum
            ORDER BY t1.rowNum ASC
            ),
            
            previousBal AS (
            SELECT COALESCE(SUM(bal), 0) as previousBal
            FROM txn
            WHERE  txn_date < '$startDate'
             ), 
            
            totalCredit AS ( SELECT COALESCE(SUM(jvcr), 0) as totalCredit FROM txn
                ), 
                
                totalDebit AS ( SELECT COALESCE(SUM(jvdr), 0) as totalDebit FROM txn ), 
                
                totalBal AS ( SELECT c.totalCredit,
                d.totalDebit, COALESCE((c.totalCredit - d.totalDebit), 0) as totalBal, p.previousBal FROM totalCredit c 
                             LEFT JOIN
                totalDebit d ON 1=1 LEFT JOIN previousBal p ON 1=1 ) 
                
                SELECT * FROM totalBal  JOIN runningbal ";

        // echo $result; exit;

        $data = DB::select($result);

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function getCustomerPurchaseList(Request $request, $customerId)
    {
        $startDate = $request->start_date;
        $endDate = $request->end_date;
        $user = $request->user;
        $locationId = $request->location;
        $location_id = Auth::user()->location_id;

        $search = !empty($request->search) ? "AND il.token = '$request->token'" : '';

        // $date = " AND txn_date BETWEEN '$from' AND '$to'";
        $date = !empty($startDate) && !empty($endDate) ? "AND il.txn_date BETWEEN '$startDate' AND '$endDate'" : '';
        $userSort = !empty($user) ? "AND il.user_id='$user'" : '';
        $locationSort = !empty($locationId) ? "AND il.location_id='$locationId'" : " AND il.location_id='$location_id'";
        $client = Auth::user()->client_id;
        $limit = "LIMIT $request->count, 50";

        $result = "
           with data as( SELECT il.txn_date, il.tm,
                ROUND(SUM(il.amount - il.discount), 0) AS amount,
                il.token, i.name as item, l.name as location, u.name as user, u.name as staff,
                ROUND(SUM(il.discount), 0) as discount,
                il.customer,
                il.tender,
                ROUND(SUM(il.vat_value), 0) as vat,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'item', i.name,
                        'amount', round(p.amount, 2),
                        'qty', p.qty,
                        'abr', m.abr,
                        'price', round(p.price, 2),
                        'discount', round(p.discount, 2),
                        'description', p.description,
                        'vat_value', round(p.vat_value, 2),
                        'vat_status', round(p.vat_status, 2),
                        'tender', p.tender
                    ))
    
                    FROM " . TBL_PRODUCT_LOG . " p
    
                    LEFT JOIN " . TBL_ITEMS . " i
                    ON i.id=p.item_id AND i.status = " . STATUS . " AND i.client_id='$client'
    
                    LEFT JOIN " .TBL_MEASUREMENT . " m
                    ON m.m_category_id=p.m_category_id AND m.status = " . STATUS . " AND m.client_id='$client' AND m.hierarchy=p.m_metric AND m.location_id='$location_id'
    
                    WHERE p.token = il.token
                ) as receipt,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'pmethod', i.pmethod,
                        'amount', i.jvalue
                    ))

                    FROM " . TBL_LOG . " i

                    WHERE i.token = il.token AND i.pmethod != ''
                ) as pmethod

            FROM  ".ONLINE_RETAILS.".item_log il
                JOIN  ".ONLINE_RETAILS.".items i ON il.item_id = i.id
                JOIN  ".ONLINE_RETAILS.".location l ON il.location_id = l.id
                JOIN  ".ONLINE_ACCOUNT.".user u ON il.user_id = u.id
                LEFT JOIN " . TBL_CUSTOMERS . " c ON c.id = il.coy

            WHERE il.coy = '$customerId' and il.client_id = '$client' and il.control_id != 1 $locationSort $userSort $date $search   
            GROUP BY il.token
            $limit
        ),

            res as (
                select 
                d.txn_date, 
                d.tm,
                d.amount,
                d.token, 
                d.item, 
                d.location, 
                d.user, 
                d.staff,
                d.discount,
                d.customer,
                CASE 
                    WHEN t.pmt_split = 1 THEN 0 
                    ELSE d.tender 
                END as tender,
                d.receipt,
                d.vat,
                d.pmethod,
                t.comp as authoriser, 
                t.effect, 
                t.prefix, 
                t.pmt_action, 
                t.pmt, 
                t.chart_id, 
                t.being, 
                t.pmt_split

                from data d
                left join " . TBL_LOG . " t ON t.token = d.token

                group by t.token

            )

            select * from res
        ";

        $data = DB::select($result);

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function getCustomerBalance(Request $request)
    {
        $client_id = Auth::user()->client_id;
        $txn_date = !empty($request->from) && !empty($request->to) ? " AND txn_date BETWEEN '$request->from' AND '$request->to' " : '';

        $query = "
            WITH customerBalance AS (
                SELECT
                    SUM(jvcr-jvdr) as bal
                FROM
                    ".ONLINE_RETAILS.".txn_log
                WHERE
                    chart_id = ".CHART_RECEIVABLE." AND coy = '$request->id' AND client_id='$client_id' AND status = 0 AND control_id != 1 $txn_date
            )
            
            SELECT COALESCE(bal, 0) as balance FROM customerBalance;
        
        ";

        // echo $query;exit;

        $data = DB::select($query);
        $balance = $data[0]->balance;

        return response()->json(['data' => $balance, 'success' => true], 200);
    }

    public function getVendorPurchaseList(Request $request, $customerId)
    {
        $startDate = $request->start_date;
        $endDate = $request->end_date;
        $user = $request->user;
        $locationId = $request->location;
        $location_id = Auth::user()->location_id;

        $search = !empty($request->search) ? "AND il.token = '$request->token'" : '';

        // $date = " AND txn_date BETWEEN '$from' AND '$to'";
        $date = !empty($startDate) && !empty($endDate) ? "AND il.txn_date BETWEEN '$startDate' AND '$endDate'" : '';
        $userSort = !empty($user) ? "AND il.user_id='$user'" : '';
        $locationSort = !empty($locationId) ? "AND il.location_id='$locationId'" : " AND il.location_id='$location_id'";
        $client = Auth::user()->client_id;
        $limit = "LIMIT $request->count, 50";

        $result = "
            SELECT il.txn_date, il.tm,
                ROUND(SUM(il.amount), 0) AS amount,
                il.token, i.name as item, l.name as location, u.name as user, u.name as staff, t.comp as authoriser, t.effect, t.prefix, t.pmt_action, t.pmt, t.chart_id, t.being, t.pmt_split,
                0 as discount,
                c.name as customer,
                0 as tender,
                0 as vat,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'item', i.name,
                        'amount', round(p.amount, 2),
                        'qty', p.qty,
                        'abr', m.abr,
                        'price', round(p.price, 2),
                        'discount', 0,
                        'description', '',
                        'vat_value', 0,
                        'vat_status', 0,
                        'tender', 0
                    ))
    
                    FROM " . TBL_STOCK . " p
    
                    LEFT JOIN " . TBL_ITEMS . " i
                    ON i.id=p.item_id AND i.status = " . STATUS . " AND i.client_id='$client'
    
                    LEFT JOIN " .TBL_MEASUREMENT . " m
                    ON m.m_category_id=p.m_category_id AND m.status = " . STATUS . " AND m.client_id='$client' AND m.hierarchy=p.m_metric
    
                    WHERE p.token = il.token
                ) as receipt,
                (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'pmethod', i.pmethod,
                            'amount', i.jvalue
                        ))

                        FROM " . TBL_LOG . " i

                        WHERE i.token = il.token AND i.chart_id = " . CHART_STOCK . " AND i.pmethod != ''
                    ) as pmethod

            FROM  ".ONLINE_RETAILS.".stock il
                JOIN  ".ONLINE_RETAILS.".items i ON il.item_id = i.id
                JOIN  ".ONLINE_RETAILS.".location l ON il.location_id = l.id
                JOIN  ".ONLINE_ACCOUNT.".user u ON il.user_id = u.id
                LEFT JOIN " . TBL_CUSTOMERS . " c ON c.id = il.supplier_id
                LEFT JOIN " . TBL_LOG . " t ON t.token = il.token AND t.pmethod != '' AND t.chart_id = " . CHART_STOCK . "

            WHERE il.supplier_id = '$customerId' and il.client_id = '$client' and il.control_id != 1 $locationSort $userSort $date $search   
            GROUP BY il.token
            $limit;
        ";

        // echo $result;exit;

        $data = DB::select($result);

        return response()->json(['data' => $data, 'success' => true], 200);
    }
}


/**
 * 
 * SELECT il.txn_date, il.tm,
                            ROUND(SUM(il.amount), 0) AS amount,
                            il.token, i.name as item, l.name as location, u.name as user, u.name as staff, t.comp as authoriser, t.effect, t.prefix, t.pmt_action, t.pmt, t.chart_id, t.being, t.pmt_split,
                            0 as discount,
                            c.name,
                            0 as tender,
                            0 as vat,
                            (
                                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                                    'item', i.name,
                                    'amount', round(p.amount, 2),
                                    'qty', p.qty,
                                    'abr', m.abr,
                                    'price', round(p.price, 2),
                                    'discount', 0,
                                    'description', '',
                                    'vat_value', 0,
                                    'vat_status', 0,
                                    'tender', 0
                                ))
                
                                FROM " . TBL_STOCK . " p
                
                                LEFT JOIN " . TBL_ITEMS . " i
                                ON i.id=p.item_id AND i.status = " . STATUS . " AND i.client_id='$client'
                
                                LEFT JOIN " .TBL_MEASUREMENT . " m
                                ON m.m_category_id=p.m_category_id AND m.status = " . STATUS . " AND m.client_id='$client' AND m.hierarchy=p.m_metric
                
                                WHERE p.token = il.token
                            ) as receipt,
                            (
                                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                                        'pmethod', i.pmethod,
                                        'amount', i.jvalue
                                    ))
            
                                    FROM " . TBL_LOG . " i
            
                                    WHERE i.token = il.token AND i.pmethod != ''
                                ) as pmethod
            
                        FROM  ".ONLINE_RETAILS.".stock il
                            JOIN  ".ONLINE_RETAILS.".items i ON il.item_id = i.id
                            JOIN  ".ONLINE_RETAILS.".location l ON il.location_id = l.id
                            JOIN  ".ONLINE_ACCOUNT.".user u ON il.user_id = u.id
                            LEFT JOIN " . TBL_CUSTOMERS . " c ON c.id = il.supplier_id
                            LEFT JOIN " . TBL_LOG . " t ON t.token = il.token AND t.pmethod != ''
            
                        WHERE il.supplier_id = '$customerId' and il.client_id = '$client' and il.control_id != 1 $locationSort $userSort $date $search   
                        GROUP BY il.token
                        $limit
 */