<?php

namespace App\Http\Controllers\Folio\Location;

use App\Http\Controllers\Controller;
use App\Models\Retails\Location;
use Auth;
use Carbon\Carbon;
use DB;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Intervention\Image\Facades\Image;
use Validator;
use Illuminate\Database\QueryException;
use Exception;
use App\Models\Retails\QueueLog;
use App\Models\Retails\Countries
;
class LocationController extends Controller
{
    public function locationDropdown(Request $request)
    {
        $search = $request->input('search');
        $query = Location::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id);
        if (!empty($search)) {
            $query->where('name', 'like', '%'.$search.'%');
        }

        $aalLocation = $query->get();

        $formattedLocation = [];
        foreach ($aalLocation as $loc) {
            $formattedLocation[] = [
                'value' => $loc->id,
                'title' => $loc->name,
            ];
        }

        return response()->json(['message' => 'Location fetched successfully', 'data' => $formattedLocation, 'success' => true], 200);
    }


    public function createLocation($request, $currentDate, $licensee, $gst_value)
{
    try {
        $location = Location::create([
            'name' => $request->name,
            'app_plan_id' => ONE_STATUS,
            'status' => STATUS,
            'date' => $currentDate,
            'email' => $request->email,
            'address' => $request->address,
            'logo' => 'camera.png',
            'tm' => Carbon::now()->toTimeString(),
            'timezone' => $request->timezone,
            'vat_value' => $gst_value,
            'phone' => $request->phone,
            'country' => $request->country,
            'client_id' => $licensee,
            'allowed_users' => 1,
            'allowed_store' => ONE_STATUS,
            'multi_store' => ONE_STATUS,
            'allowed_items' => 50,
            'subscription_status' => STATUS,
            'update_date' => $currentDate,
            'start_date' => $currentDate,
            // 'end_date' => Carbon::parse($currentDate)->addYear(), // Set end_date to one year from currentDate
            'end_date' => $currentDate
        ]);

        if (!$location) {
            throw new Exception("Failed to create location.");
        }

        return $location;
    } catch (QueryException $e) {
        QueueLog::where('identifier', $request->name)->delete();
        throw new Exception("Database error on creating location: " . $e->getMessage());
    } catch (Exception $e) {
        QueueLog::where('identifier', $request->name)->delete();
        throw new Exception("Error creating location: " . $e->getMessage());
    }
}



// public function createLocation($request, $currentDate, $licensee, $gst_value)
// {
//     try {
//         $location = Location::create([
//             'name' => $request->name,
//             'app_plan_id' => ONE_STATUS,
//             'status' => STATUS,
//             'date' => $currentDate,
//             'email' => $request->email,
//             'address' => $request->address,
//             'logo' => 'camera.png',
//             'tm' => Carbon::now()->toTimeString(),
//             'timezone' => $request->timezone,
//             'vat_value' => $gst_value,
//             'phone' => $request->phone,
//             'country' => $request->country,
//             'client_id' => $licensee,
//             'allowed_users' => ONE_STATUS,
//             'allowed_store' => ONE_STATUS,
//             'multi_store' => ONE_STATUS,
//             'allowed_items' => 50,
//             'subscription_status' => STATUS,
//             'update_date' => $currentDate,
//             'start_date' => $currentDate,
//             'end_date' => $currentDate,
//         ]);

        
//         if (!$location) {
//             throw new Exception("Failed to create location.");
//         }

//         return $location;
//     } catch (QueryException $e) {
//         throw new Exception("Database error on creating location: " . $e->getMessage());
//     } catch (Exception $e) {
//         throw new Exception("Error creating location: " . $e->getMessage());
//     }
// }


    public function allowedUsers()
    {
        $paymentVerification = $this->getPaymentVerification(Auth::user()->client_id, Auth::user()->location_id);
        $allowedUsers = Location::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('id', Auth::user()->location_id)
        ->value('allowed_users');

        return $paymentVerification == STATUS ? $allowedUsers : ONE_STATUS;
    }

    public function logo($id)
    {
        return Location::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('id', $id)
        ->value('logo');
    }

    public function locationEmail($id, $client_id)
    {
        return Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $id)
        ->value('email');
    }

    public function locationPhone($id, $client_id)
    {
        return Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $id)
        ->value('phone');
    }

    public function getlocationRow($id, $client_id)
    {
        return Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $id)
        ->get();
    }

    public function allowedItems($client_id, $location_id)
    {
        $paymentVerification = $this->getPaymentVerification($client_id, $location_id);
        $allowedItems = Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $location_id)
        ->value('allowed_items');

        return $paymentVerification == STATUS ? $allowedItems : 50;
    }

    public function templateName()
    {

        return Location::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('id', Auth::user()->location_id)
        ->value('template');

    }

    public function locationLogo($id, $client_id)
    {
        return Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $id)
        ->value('logo');
    }

    public function locationAppPlan($id, $client_id) {
        return Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $id)
        ->value('app_plan_id');
    }

    public function allowedStore($client_id, $location_id)
    {
        $paymentVerification = $this->getPaymentVerification($client_id, $location_id);
        $allowedStore = Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $location_id)
        ->value('allowed_store');

        return $paymentVerification == STATUS ? $allowedStore : ONE_STATUS;
    }

    // public function isExpired($client_id, $location_id)
    // {
    //     echo $status = Location::where('status', STATUS)
    //     ->where('client_id', $client_id)
    //     ->where('id', $location_id)
    //     ->value('end_date'); 

    //     echo "<br>";
    //     echo Carbon::now()->format('Y-m-d');
        
    //     exit;

    //     return $status <= Carbon::now()->format('Y-m-d') ? 'expired' : 'active';
    // }



    public function isExpired($client_id, $location_id)
    {
        $endDate = Location::where('status', STATUS)
            ->where('client_id', $client_id)
            ->where('id', $location_id)
            ->value('end_date');

        // Convert $endDate to a Carbon instance for comparison
        $endDate = Carbon::createFromFormat('Y-m-d', $endDate);

        // Now compare using Carbon instances
        return $endDate->lessThanOrEqualTo(Carbon::now()) ? 'expired' : 'active';
    }


    public function getPaymentVerification($client_id, $location_id) {
        return Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $location_id)
        ->value('payment_confirmation');
    }

    public function getPlanId($client_id, $location_id) {
        $planId = Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $location_id)
        ->value('app_plan_id');

        $paymentVerification = $this->getPaymentVerification($client_id, $location_id);

        return $paymentVerification == STATUS ? $planId : ONE_STATUS;
    }

    public function activateExpiration($client_id, $location_id) {
        Location::where('status', STATUS)
        ->where('client_id', $client_id)
        ->where('id', $location_id)
        ->update([
            'app_plan_id' => ONE_STATUS,
            'allowed_users' => ONE_STATUS,
            'allowed_store' => ONE_STATUS,
            'allowed_items' => 50
        ]);

        return ONE_STATUS;
    }

    public function getLocations(Request $request) {
        
        $btxn_date = !empty($request->from) && !empty($request->to) ? "b.txn_date BETWEEN '$request->from' AND '$request->to' AND" : "";
        $atxn_date = !empty($request->from) && !empty($request->to) ? "AND a.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? "AND txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $client_id = Auth::user()->client_id;
        $search = !empty($request->name) ? "AND name LIKE '%$request->name%'" : "";

        $query = "with loc as (SELECT id, name, address, logo, email, phone, vat_status FROM " . TBL_LOCATION . " 

        WHERE client_id = '$client_id' and status = 0 $search),
        
        sales as (SELECT 
            
            b.location_id,
            COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) AS revenue,
            round(COALESCE(((SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END) / 
            (LAG(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END)) 
                OVER (ORDER BY b.txn_date ASC))) - 1) * 100, 0),2) AS growth
            FROM ".TBL_LOG." b 
            WHERE $btxn_date client_id = '$client_id' AND b.control_id != 1
            GROUP BY location_id
            ),
            
            stockValue as (SELECT 
           m.location_id,
            CASE WHEN cr.CreditBalance IS NULL
            THEN round(dr.DebitBalance,2)
            ELSE round((dr.DebitBalance - cr.CreditBalance),2)
            END as InventoryBalance,
        
            CASE WHEN cr.RetailValue IS NULL
            THEN round(dr.RetailValue,2)
            ELSE round((dr.RetailValue - cr.RetailValue),2) 
            END as Retailvalue
        
            FROM ".TBL_STOCK." m 
        
            LEFT JOIN ( 
                 SELECT l.id as location_id,
                 COALESCE(sum(a.uqty * a.price), 0) as CreditBalance,
                 COALESCE(sum(a.uqty * b.rate), 0) as RetailValue 
                 from 
                 ".TBL_LOCATION." l 
                 LEFT JOIN
                 ".TBL_STOCK." a on l.id = a.location_id and a.effect = 'cr' and a.control_id!=1 
                 LEFT JOIN ".TBL_ITEM_PRICE." b ON b.item_id = a.item_id and b.location_id=a.location_id and b.m_metric = 1 and b.client_id='$client_id' $atxn_date
        
                GROUP by l.id
            ) as cr on m.location_id = cr.location_id
                
            LEFT JOIN( 
                  SELECT l.id as location_id,
                     COALESCE(sum(a.uqty * a.price), 0) as DebitBalance,
                    COALESCE(sum(a.uqty * b.rate), 0) as RetailValue 
                    from 
                     ".TBL_LOCATION." l 
                     LEFT JOIN
                     ".TBL_STOCK." a on l.id = a.location_id and a.effect = 'dr' and a.control_id!=1 
                     LEFT JOIN ".TBL_ITEM_PRICE." b ON b.item_id = a.item_id and b.location_id=a.location_id and b.m_metric = 1 and b.client_id='$client_id' 
                    $atxn_date
        
                    GROUP by l.id
            ) dr on m.location_id = dr.location_id
            
            WHERE control_id!=1 and client_id='$client_id' $txn_date
            GROUP BY m.location_id
            ),
            
            inventory as (
            
            SELECT 
            inventoryBalance, Retailvalue, location_id,
            
            round((Retailvalue - inventoryBalance) * 100 / Retailvalue) as profitMargin,
            round((Retailvalue - inventoryBalance) * 100 / inventoryBalance) as markUp
            
            from stockValue),
            
            debtors AS (
            SELECT location_id, coy, chart_id, SUM(jvdr) AS dr, SUM(jvcr) AS cr , SUM(jvdr-jvcr)  AS 'sum' FROM ".TBL_LOG."
            WHERE  client_id = '$client_id' AND control_id != 1 $txn_date
            AND (chart_id = '20003' or chart_id = '40000') AND chart_id != '50042'
            AND coy != ''
            GROUP BY coy, location_id
            HAVING SUM(jvdr) - SUM(jvcr) > 0
            ORDER BY coy),
            
            creditors AS (
            SELECT location_id, coy, chart_id, SUM(jvdr) AS dr, SUM(jvcr) AS cr,
            SUM(jvcr-jvdr)  AS 'sum' FROM ".TBL_LOG."
            WHERE  client_id = '$client_id' AND control_id != 1 $txn_date
            AND (chart_id = '20003' or chart_id = '40000' ) AND chart_id != '50042'
            AND coy != ''
            GROUP BY coy, location_id
            HAVING SUM(jvcr) - SUM(jvdr) > 0
            ORDER BY coy),
            
            customer as (
            SELECT SUM(d.sum) AS debtors, 
            SUM(c.sum) AS creditors, 
            d.location_id
            FROM debtors d
            INNER JOIN creditors c ON d.location_id = c.location_id),
            
            cashflow as (
            SELECT 
            location_id,
            COALESCE(SUM(CASE WHEN b.chart_id = '20000' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS cash,
            COALESCE(SUM(CASE WHEN b.chart_id = '20001' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS bank
            FROM ".TBL_LOG." b 
            WHERE $btxn_date client_id = '$client_id' AND b.control_id != 1 GROUP by location_id
            ),
            
            overview as (
            SELECT 
            location_id,
            COALESCE(SUM(CASE WHEN b.account_type = '005' AND b.chart_id != '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS expense,
            COALESCE(SUM(CASE WHEN b.account_type = '005' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS totalExpenditure,
            COALESCE(SUM(CASE WHEN b.chart_id = '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS cog,
            COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) - COALESCE(SUM(CASE WHEN b.chart_id = '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) as grossProfit,
            COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) - COALESCE(SUM(CASE WHEN b.account_type = '005' THEN b.jvdr - b.jvcr ELSE 0 END), 0) as profit
        FROM ".TBL_LOG." b 
        WHERE $btxn_date b.control_id != 1
        GROUP by location_id
            ),
            realGrowth as 
            (SELECT 
                location_id,
                ROUND(
                COALESCE(
                    (
                        (SUM(CASE WHEN account_type = '004' THEN jvcr - jvdr ELSE 0 END) - 
                        LAG(SUM(CASE WHEN account_type = '004' THEN jvcr - jvdr ELSE 0 END)) 
                        OVER (PARTITION BY location_id ORDER BY txn_date ASC)) 
                        / COALESCE(LAG(SUM(CASE WHEN account_type = '004' THEN jvcr - jvdr ELSE 0 END)) 
                        OVER (PARTITION BY location_id ORDER BY txn_date ASC), 0)) * 100, 0), 2) AS growth
                        FROM ".TBL_LOG." WHERE control_id != 1 AND client_id = '$client_id' $txn_date 
                        GROUP BY location_id, txn_date
                        ORDER BY  txn_date DESC limit 1
        ),
        
        lba as (
            select location_id, count(id) as totalCustomers from ".TBL_CUSTOMERS."
            
            where cate='lba' and client_id='$client_id' 
            group by location_id
        ),

        vendors as (
            select location_id, count(id) as totalVendors from ".TBL_CUSTOMERS."
            
            where cate='cust' and client_id='$client_id' 
          group by location_id
        ),

        cashGrowth as 
            (SELECT 
                location_id,
                ROUND(
                COALESCE(
                    (
                        (SUM(CASE WHEN (chart_id = '20000' OR chart_id='20001') THEN jvcr - jvdr ELSE 0 END) - 
                        LAG(SUM(CASE WHEN (chart_id = '20000' OR chart_id='20001') THEN jvcr - jvdr ELSE 0 END)) 
                        OVER (PARTITION BY location_id ORDER BY txn_date ASC)) 
                        / COALESCE(LAG(SUM(CASE WHEN (chart_id = '20000' OR chart_id='20001') THEN jvcr - jvdr ELSE 0 END)) 
                        OVER (PARTITION BY location_id ORDER BY txn_date ASC), 0)) * 100, 0), 2) AS cashGrowth
                        FROM ".TBL_LOG." WHERE control_id != 1 AND client_id = '$client_id' $txn_date 
                        GROUP BY location_id, txn_date
                        ORDER BY  txn_date DESC limit 1
        ),

        locations as (
            SELECT l.*, s.revenue, rg.growth, ch.cashGrowth, sv.InventoryBalance, sv.Retailvalue, COALESCE(sv.profitMargin,0) as profitMargin, sv.markUp,
            COALESCE(c.debtors, 0) as debtors, COALESCE(c.creditors, 0) as creditors,
            COALESCE(cf.cash, 0) as cash, COALESCE(cf.bank, 0) as bank, (COALESCE(cf.cash, 0) +  COALESCE(cf.bank, 0)) as totalCashflow,
            ov.expense, ov.totalExpenditure, ov.cog, ov.grossProfit, COALESCE(ov.profit,0) as profit, lb.totalCustomers, ven.totalVendors
            
            from loc l
            LEFT JOIN sales s on l.id = s.location_id
            LEFT JOIN inventory sv on l.id = sv.location_id
            LEFT JOIN customer c on l.id = c.location_id
            LEFT JOIN cashflow cf on l.id = cf.location_id
            LEFT JOIN overview ov on l.id = ov.location_id
            LEFT JOIN realGrowth rg on l.id = rg.location_id
            LEFT JOIN lba lb on l.id = lb.location_id
            LEFT JOIN vendors ven on l.id = ven.location_id
            LEFT JOIN cashGrowth ch on l.id = ch.location_id)
            
        SELECT * from locations";
        
        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function getOneLocation(Request $request)
    {
        $btxn_date = !empty($request->from) && !empty($request->to) ? "b.txn_date BETWEEN '$request->from' AND '$request->to' AND" : "";
        $atxn_date = !empty($request->from) && !empty($request->to) ? "AND a.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? "AND txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $location = $request->id;
        $client_id = Auth::user()->client_id;

        $query = 'with loc as (SELECT id, name, address, logo, email, vat_status, phone FROM '.TBL_LOCATION." 

        WHERE client_id = '$client_id' and status = 0),

        sales as (SELECT 
            b.location_id,
            COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) AS revenue,
            ROUND(COALESCE(((SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END) / 
            (LAG(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END)) 
                OVER (ORDER BY b.txn_date ASC))) - 1) * 100, 0), 2) AS growth
            FROM ".TBL_LOG." b 
            WHERE $btxn_date client_id = '$client_id' and control_id != 1
            GROUP BY location_id
        ),
	
        stockValue as (SELECT 
        m.location_id,
        CASE WHEN cr.CreditBalance IS NULL
        THEN round(dr.DebitBalance, 2)
        ELSE round((dr.DebitBalance - cr.CreditBalance),2)
        END as InventoryBalance,

        CASE WHEN cr.RetailValue IS NULL
        THEN round(dr.RetailValue,2)
        ELSE round((dr.RetailValue - cr.RetailValue) ,2)
        END as Retailvalue

        FROM ".TBL_STOCK.' m 

        LEFT JOIN ( 
            SELECT l.id as location_id,
            COALESCE(sum(a.uqty * a.price), 0) as CreditBalance,
            COALESCE(sum(a.uqty * b.rate), 0) as RetailValue 
            from 
            '.TBL_LOCATION.' l 
            LEFT JOIN
            '.TBL_STOCK." a on l.id = a.location_id and a.effect = 'cr' and a.control_id!=1 
            LEFT JOIN ".TBL_ITEM_PRICE." b ON b.item_id = a.item_id and b.location_id=a.location_id and b.m_metric = 1 and b.client_id='$client_id' and b.status = 0
            $atxn_date

            GROUP by l.id
        ) as cr on m.location_id = cr.location_id
        
        LEFT JOIN( 
            SELECT l.id as location_id,
			COALESCE(sum(a.uqty * a.price), 0) as DebitBalance,
			COALESCE(sum(a.uqty * b.rate), 0) as RetailValue 
			from 
            ".TBL_LOCATION.' l 
            LEFT JOIN
            '.TBL_STOCK." a on l.id = a.location_id and a.effect = 'dr' and a.control_id!=1 
            LEFT JOIN ".TBL_ITEM_PRICE." b ON b.item_id = a.item_id and b.location_id=a.location_id and b.m_metric = 1 and b.client_id='$client_id' and b.status=0
			$atxn_date

			GROUP by l.id
        ) dr on m.location_id = dr.location_id
    
        WHERE control_id!=1 and client_id='$client_id' $txn_date
        GROUP BY m.location_id
        ),
	
            inventory as (
            
            SELECT 
            inventoryBalance, Retailvalue, location_id,
            
            round((Retailvalue - inventoryBalance) * 100 / Retailvalue, 2) as profitMargin
            
            from stockValue),
            
            debtors AS (
            SELECT location_id, coy, chart_id, SUM(jvdr) AS dr, SUM(jvcr) AS cr , SUM(jvdr-jvcr)  AS 'sum' FROM ".TBL_LOG."
            WHERE  client_id = '$client_id' AND control_id != 1 $txn_date
            AND (chart_id = '20003' or chart_id = '40000' ) AND chart_id != '50042'
            AND status = 0 AND coy != 0
            GROUP BY coy, location_id
            HAVING SUM(jvdr) - SUM(jvcr) > 0
            ORDER BY coy),
            
            creditors AS (
            SELECT location_id, coy, chart_id, SUM(jvdr) AS dr, SUM(jvcr) AS cr,
            SUM(jvcr-jvdr)  AS 'sum' FROM ".TBL_LOG."
            WHERE  client_id = '$client_id' AND control_id != 1 $txn_date
            AND (chart_id = '20003' or chart_id = '40000' ) AND chart_id != '50042'
            AND status = 0 AND coy != 0
            GROUP BY coy, location_id
            HAVING SUM(jvcr) - SUM(jvdr) > 0
            ORDER BY coy),
            
            customer as (SELECT SUM(d.sum) AS debtors, SUM(c.sum) AS creditors, d.location_id
            FROM debtors d  INNER JOIN creditors c ON d.location_id = c.location_id),
            
            cashflow as (
            SELECT 
            location_id,
            COALESCE(SUM(CASE WHEN b.chart_id = '20000' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS cash,
            COALESCE(SUM(CASE WHEN b.chart_id = '20001' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS bank
            FROM ".TBL_LOG." b 
            WHERE $btxn_date client_id = '$client_id' and control_id != 1 GROUP by location_id
            ),
            
            overview as (
            SELECT 
            location_id,
            COALESCE(SUM(CASE WHEN b.account_type = '005' AND b.chart_id != '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS expense,
            COALESCE(SUM(CASE WHEN b.account_type = '005' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS totalExpenditure,
            COALESCE(SUM(CASE WHEN b.chart_id = '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS cog,

            COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) - COALESCE(SUM(CASE WHEN b.chart_id = '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) as grossProfit,

            COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) - COALESCE(SUM(CASE WHEN b.account_type = '005' THEN b.jvdr - b.jvcr ELSE 0 END), 0) as profit
            FROM ".TBL_LOG." b 
            WHERE $btxn_date client_id = '$client_id' and control_id != 1
            GROUP by location_id
            ),
            realGrowth as 
            (SELECT 
                location_id,
                ROUND(
                COALESCE(
                    (
                        (SUM(CASE WHEN account_type = '004' THEN jvcr - jvdr ELSE 0 END) - 
                        LAG(SUM(CASE WHEN account_type = '004' THEN jvcr - jvdr ELSE 0 END)) 
                        OVER (PARTITION BY location_id ORDER BY txn_date ASC)) 
                        / COALESCE(LAG(SUM(CASE WHEN account_type = '004' THEN jvcr - jvdr ELSE 0 END)) 
                        OVER (PARTITION BY location_id ORDER BY txn_date ASC), 0)) * 100, 0), 2) AS growth
                        FROM ".TBL_LOG." WHERE control_id != 1 AND client_id = '$client_id' $txn_date 
                        GROUP BY location_id, txn_date
                        ORDER BY  txn_date DESC limit 1
        ),
        cashGrowth as 
            (SELECT 
                location_id,
                ROUND(
                COALESCE(
                    (
                        (SUM(CASE WHEN (chart_id = '20000' OR chart_id='20001') THEN jvcr - jvdr ELSE 0 END) - 
                        LAG(SUM(CASE WHEN (chart_id = '20000' OR chart_id='20001') THEN jvcr - jvdr ELSE 0 END)) 
                        OVER (PARTITION BY location_id ORDER BY txn_date ASC)) 
                        / COALESCE(LAG(SUM(CASE WHEN (chart_id = '20000' OR chart_id='20001') THEN jvcr - jvdr ELSE 0 END)) 
                        OVER (PARTITION BY location_id ORDER BY txn_date ASC), 0)) * 100, 0), 2) AS cashGrowth
                        FROM ".TBL_LOG." WHERE control_id != 1 AND client_id = '$client_id' $txn_date 
                        GROUP BY location_id, txn_date
                        ORDER BY  txn_date DESC limit 1
        ),
        locations as (
            SELECT l.*, s.revenue, rg.growth, ch.cashGrowth, sv.InventoryBalance, sv.Retailvalue, sv.profitMargin, 
            COALESCE(c.debtors, 0) as debtors, COALESCE(c.creditors, 0) as creditors,
            COALESCE(cf.cash, 0) as cash, COALESCE(cf.bank, 0) as bank, (COALESCE(cf.cash, 0) +  COALESCE(cf.bank, 0)) as totalCashflow,
            ov.expense, ov.totalExpenditure, ov.cog, ov.grossProfit, ov.profit
            
            from loc l
            LEFT JOIN sales s on l.id = s.location_id
            LEFT JOIN inventory sv on l.id = sv.location_id
            LEFT JOIN customer c on l.id = c.location_id
            LEFT JOIN cashflow cf on l.id = cf.location_id
            LEFT JOIN overview ov on l.id = ov.location_id
            LEFT JOIN realGrowth rg on l.id = rg.location_id
            LEFT JOIN cashGrowth ch on l.id = ch.location_id)
            
        
        SELECT * from locations WHERE id = '$location'";

        $data = DB::select($query);

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function getLocationSummary(Request $request)
    {
        $btxn_date = !empty($request->from) && !empty($request->to) ? "b.txn_date BETWEEN '$request->from' AND '$request->to' AND" : "";
        $atxn_date = !empty($request->from) && !empty($request->to) ? "AND a.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? "AND txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        
        $client_id = Auth::user()->client_id;

        $query = 'with loc as (SELECT id, name, address, logo, email, phone FROM '.TBL_LOCATION." 
    
        WHERE client_id = '$client_id' and status = 0),
    
        sales as (SELECT 
        
        b.location_id,
        COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) AS revenue,
        ROUND(COALESCE(((SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END) / 
        (LAG(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END)) 
            OVER (ORDER BY b.txn_date ASC))) - 1) * 100, 0),2) AS growth
        FROM ".TBL_LOG." b 
        WHERE $btxn_date client_id = '$client_id' and b.control_id != 1
        GROUP BY location_id
        ),
        
        stockValue as (SELECT 
       m.location_id,
        CASE WHEN cr.CreditBalance IS NULL
        THEN ROUND(dr.DebitBalance,2)
        ELSE ROUND((dr.DebitBalance - cr.CreditBalance) ,2)
        END as InventoryBalance,
    
        CASE WHEN cr.RetailValue IS NULL
        THEN dr.RetailValue
        ELSE (dr.RetailValue - cr.RetailValue) 
        END as Retailvalue
    
        FROM ".TBL_STOCK.' m 
    
        LEFT JOIN ( 
             SELECT l.id as location_id,
             COALESCE(sum(a.uqty * a.price), 0) as CreditBalance,
             COALESCE(sum(a.uqty * b.rate), 0) as RetailValue 
             from 
             '.TBL_LOCATION.' l 
             LEFT JOIN
             '.TBL_STOCK." a on l.id = a.location_id and a.effect = 'cr' and a.control_id!=1 
             LEFT JOIN ".TBL_ITEM_PRICE." b ON b.item_id = a.item_id and b.location_id=a.location_id and b.m_metric = 1 and b.client_id='$client_id' and b.status=0
            $atxn_date
    
            GROUP by l.id
        ) as cr on m.location_id = cr.location_id
            
        LEFT JOIN( 
              SELECT l.id as location_id,
                 COALESCE(sum(a.uqty * a.price), 0) as DebitBalance,
                COALESCE(sum(a.uqty * b.rate), 0) as RetailValue 
                from 
                ".TBL_LOCATION.' l 
                 LEFT JOIN
                 '.TBL_STOCK." a on l.id = a.location_id and a.effect = 'dr' and a.control_id!=1 
                 LEFT JOIN ".TBL_ITEM_PRICE." b ON b.item_id = a.item_id and b.location_id=a.location_id and b.m_metric = 1 and b.client_id='$client_id' and b.status=0
                $atxn_date
    
                GROUP by l.id
        ) dr on m.location_id = dr.location_id
        
        WHERE control_id!=1 and client_id='$client_id' $txn_date 
        GROUP BY m.location_id
        ),
        
        inventory as (
        
        SELECT 
        inventoryBalance, Retailvalue, location_id,
        
        round((Retailvalue - inventoryBalance) * 100 / Retailvalue,2) as profitMargin
        
        from stockValue),
        
        debtors AS (
        SELECT location_id, coy, chart_id, SUM(jvdr) AS dr, SUM(jvcr) AS cr , SUM(jvdr-jvcr)  AS 'sum' FROM ".TBL_LOG."
        WHERE  client_id = '$client_id' AND control_id != 1 $txn_date
        AND (chart_id = '20003' or chart_id = '40000' ) AND chart_id != '50042'
        AND status = 0 AND coy != 0
        GROUP BY coy, location_id
        HAVING SUM(jvdr) - SUM(jvcr) > 0
        ORDER BY coy),
        
        creditors AS (
        SELECT location_id, coy, chart_id, SUM(jvdr) AS dr, SUM(jvcr) AS cr,
        SUM(jvcr-jvdr)  AS 'sum' FROM ".TBL_LOG."
        WHERE  client_id = '$client_id' AND control_id != 1 $txn_date
        AND (chart_id = '20003' or chart_id = '40000' ) AND chart_id != '50042'
        AND status = 0 AND coy != 0
        GROUP BY coy, location_id
        HAVING SUM(jvcr) - SUM(jvdr) > 0
        ORDER BY coy),
        
        customer as (SELECT SUM(d.sum) AS debtors, SUM(c.sum) AS creditors, d.location_id
        FROM debtors d  INNER JOIN creditors c ON d.location_id = c.location_id),
        
        cashflow as (
        SELECT 
        location_id,
        COALESCE(SUM(CASE WHEN b.chart_id = '20000' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS cash,
        COALESCE(SUM(CASE WHEN b.chart_id = '20001' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS bank
        FROM ".TBL_LOG." b 
        WHERE $btxn_date client_id = '$client_id' and b.control_id != 1 GROUP by location_id
        ),
        
        overview as (
        SELECT 
        location_id,
        COALESCE(SUM(CASE WHEN b.account_type = '005' AND b.chart_id != '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS expense,
        COALESCE(SUM(CASE WHEN b.account_type = '005' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS totalExpenditure,
        COALESCE(SUM(CASE WHEN b.chart_id = '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) AS cog,
        COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) - COALESCE(SUM(CASE WHEN b.chart_id = '50047' THEN b.jvdr - b.jvcr ELSE 0 END), 0) as grossProfit,
        COALESCE(SUM(CASE WHEN b.account_type = '004' THEN b.jvcr - b.jvdr ELSE 0 END), 0) - COALESCE(SUM(CASE WHEN b.account_type = '005' THEN b.jvdr - b.jvcr ELSE 0 END), 0) as profit
        FROM ".TBL_LOG." b 
        WHERE $btxn_date client_id = '$client_id' and b.control_id != 1
        GROUP by location_id
        ),
        locations as (
            SELECT l.*, s.revenue, s.growth, sv.InventoryBalance, sv.Retailvalue, sv.profitMargin, 
            COALESCE(c.debtors, 0) as debtors, COALESCE(c.creditors, 0) as creditors,
            COALESCE(cf.cash, 0) as cash, COALESCE(cf.bank, 0) as bank, (COALESCE(cf.cash, 0) +  COALESCE(cf.bank, 0)) as totalCashflow,
            ov.expense, ov.totalExpenditure, ov.cog, ov.grossProfit, ov.profit
            
            from loc l
            LEFT JOIN sales s on l.id = s.location_id
            LEFT JOIN inventory sv on l.id = sv.location_id
            LEFT JOIN customer c on l.id = c.location_id
            LEFT JOIN cashflow cf on l.id = cf.location_id
            LEFT JOIN overview ov on l.id = ov.location_id)
            
        SELECT  count(*) as numOfLocations, sum(revenue) as totalRevenue, sum(inventoryBalance) as totalInventoryBalance, round(avg(profitMargin),2) as totalProfitMargin from locations";

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    private function validateRequest(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|unique:location',
            'phone' => 'required|size:11|unique:location',
            'email' => 'required|email|unique:location|regex:/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/|max:255',
            'address' => 'required|unique:location',
            // 'fileUri' => [
            //     function ($attribute, $value, $fail) {
            //         // Check if the value is a file and validate it
            //         if ($request->hasFile('fileUri')) {
            //             $file = $request->file('fileUri');
            //             $allowedMimes = ['jpeg', 'png', 'jpg', 'gif', 'svg'];
            //             $maxSize = 2048; // Maximum file size in kilobytes

            //             if (!in_array($file->getClientOriginalExtension(), $allowedMimes)) {
            //                 return $fail($attribute.' must be a valid image file.');
            //             }

            //             if ($file->getSize() > $maxSize * 1024) {
            //                 return $fail($attribute.' must not be greater than '.$maxSize.' KB.');
            //             }
            //         } else {
            //             // Check if the value is a valid base64 string
            //             if (!preg_match('/^data:image\/(jpeg|png|jpg|gif|svg)\b[;base64,]/', $value)) {
            //                 return $fail($attribute.' must be a valid base64 encoded image or a valid image file.');
            //             }
            //         }
            //     },
            // ],
        ]);

        // return response()->json(['request' => $request->all()], 200);

        if($this->getPlanId(Auth::user()->client_id, Auth::user()->location_id)==ONE_STATUS) {
            return response()->json(['message' => 'You cannot create another location with a basic plan', 'errors' => $validator->errors()->first()], 400);
        }

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 400);
        }
        
        return null;
    }

    private function validateEditRequest(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => [
                'required',
                function ($attribute, $value, $fail) use ($request) {
                    $location = Location::where('name', $value)
                        ->where('client_id', Auth::user()->client_id)
                        ->where('id', '!=', $request->input('id'))
                        ->first();

                    if ($location) {
                        // If a row exists with this name and it's not the row with id 34, fail the validation
                        $fail('The '.$attribute.' has already been taken.');
                    }
                },
            ],
            'phone' => [
                'required',
                'size:11',
                function ($attribute, $value, $fail) use ($request) {
                    $location = Location::where('phone', $value)
                        ->where('client_id', Auth::user()->client_id)
                        ->where('id', '!=', $request->input('id'))
                        ->first();

                    if ($location) {
                        // If a row exists with this name and it's not the row with id 34, fail the validation
                        $fail('The '.$attribute.' has already been taken.');
                    }
                },
            ],
            'email' => [
                'required',
                'email',
                'regex:/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/',
                'max:255',
                function ($attribute, $value, $fail) use ($request) {
                    $location = Location::where('email', $value)
                        ->where('client_id', Auth::user()->client_id)
                        ->where('id', '!=', $request->input('id'))
                        ->first();

                    if ($location) {
                        // If a row exists with this name and it's not the row with id 34, fail the validation
                        $fail('The '.$attribute.' has already been taken.');
                    }
                },
            ],
            'address' => [
                'required',
                function ($attribute, $value, $fail) use ($request) {
                    $location = Location::where('address', $value)
                        ->where('client_id', Auth::user()->client_id)
                        ->where('id', '!=', $request->input('id'))
                        ->first();

                    if ($location) {
                        // If a row exists with this name and it's not the row with id 34, fail the validation
                        $fail('The '.$attribute.' has already been taken.');
                    }
                },
            ],
            // 'fileUri' => 'file|mimes:jpeg,png,jpg,gif,svg|max:2048'
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 400);
        }

        return null;
    }

    public function addLocation(Request $request)
    {
        $validationResult = $this->validateRequest($request);
        if ($validationResult) {
            return $validationResult;
        }

        if ($request->hasFile('fileUri')) {

            $image = $request->file('fileUri');
            $filenameWithExt = $image->getClientOriginalName();

            // Get Filename
            $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);

            // Get just Extension
            $extension = $image->getClientOriginalExtension();

            // Filename To store
            $fileNameToStore = $filename.'_'.time().'.'.$extension;

            // $image->storeAs('uploads/logo', $fileNameToStore, 'public');
            $img = Image::make($image)->resize(800, null, function ($constraint) {
                $constraint->aspectRatio();
            });

            // Ensure the directory exists
            $directory = storage_path('app/public/uploads/logo');
            if (!file_exists($directory)) {
                mkdir($directory, 0775, true); // Creates the directory if it doesn't exist
            }

            // Save the resized image
            $img->save($directory.'/'.$fileNameToStore);
        }

        $file = $request->input('fileUri');

        if ($file) {
            list($type, $data) = explode(';', $request->fileUri);
            list(, $data) = explode(',', $data);

            // Decode the base64 data to get the binary data
            $data = base64_decode($data);

            // Generating file name with extension
            $fileNameToStore = 'image_'.time().'.jpeg';

            // Ensure the directory exists
            $path = storage_path('app/public/uploads/logo');
            if (!file_exists($path)) {
                mkdir($path, 0775, true); // Creates the directory if it doesn't exist
            }

            $img = $path.'/'.$fileNameToStore;
            // Save the decoded data to a file
            file_put_contents($img, $data);
        }

        // Insert other form data into the database
        $loc = new Location();
        $loc->name = $request->input('name');
        $loc->phone = $request->input('phone');
        $loc->email = $request->input('email');
        $loc->address = $request->input('address');
        $loc->vat_status = $request->input('vat');
        $loc->allowed_users = 1;
        $loc->app_plan_id = 1;
        $loc->allowed_items = 50;
        $loc->allowed_store = 1;
        $loc->client_id = Auth::user()->client_id;
        $loc->user_id = Auth::user()->id;
        $loc->subscription_status = STATUS;
        $loc->date = Carbon::now()->format('Y-m-d');
        $loc->start_date = Carbon::now()->format('Y-m-d');
        $loc->end_date = Carbon::now()->format('Y-m-d');
        $loc->tm = Carbon::now()->toTimeString();
        $loc->update_date = Carbon::now()->format('Y-m-d H:i:s');
        $loc->status = STATUS;
        $loc->logo = $fileNameToStore ?? null; // Save file path if file was uploaded
        $loc->save();

        return response()->json(['message' => 'Location created successfully', 'success' => true], 201);
    }

    public function editLocation(Request $request)
    {
        $validationResult = $this->validateEditRequest($request);
        if ($validationResult) {
            return $validationResult;
        }

        if ($request->hasFile('fileUri')) {

            $image = $request->file('fileUri');
            $filenameWithExt = $image->getClientOriginalName();
            // Get Filename
            $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);
            // Get just Extension
            $extension = $image->getClientOriginalExtension();
            // Filename To store
            $fileNameToStore = $filename.'_'.time().'.'.$extension;

            // $image->storeAs('uploads/logo', $fileNameToStore, 'public');
            $img = Image::make($image)->resize(800, null, function ($constraint) {
                $constraint->aspectRatio();
            });

            // Ensure the directory exists
            $directory = storage_path('app/public/uploads/logo');
            if (!file_exists($directory)) {
                mkdir($directory, 0775, true); // Creates the directory if it doesn't exist
            }

            // Save the resized image
            $img->save($directory.'/'.$fileNameToStore);

            // Deleting former logo
            Storage::disk('public')->delete("uploads/logo/".$request->input('filename'));
        }

        $file = $request->input('fileUri');

        if ($file) {
            list($type, $data) = explode(';', $request->fileUri);
            list(, $data) = explode(',', $data);

            // Decode the base64 data to get the binary data
            $data = base64_decode($data);

            // Generating file name with extension
            $fileNameToStore = 'image_'.time().'.jpeg';

            // Ensure the directory exists
            $path = storage_path('app/public/uploads/logo');
            if (!file_exists($path)) {
                mkdir($path, 0775, true); // Creates the directory if it doesn't exist
            }

            $img = $path.'/'.$fileNameToStore;
            // Save the decoded data to a file
            file_put_contents($img, $data);

            $location_log = $this->logo($request->input('id'));

            // Deleting former logo
            Storage::disk('public')->delete('uploads/logo/'.$location_log);
        }

        // Update other form data into the database

        Location::where('id', $request->input('id'))
        ->where('client_id', Auth::user()->client_id)
        ->update([
            'name' => $request->input('name'),
            'phone' => $request->input('phone'),
            'email' => $request->input('email'),
            'address' => $request->input('address'),
            'vat_status' => $request->input('vat'),
            'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            'logo' => $fileNameToStore ?? $request->logoName,
        ]);

        return response()->json(['message' => 'Location updated successfully', 'success' => true], 200);
    }

    public function getLocationByName($client_id, $email) {
        return Location::where('email', $email)
        ->where('client_id', $client_id)
        ->where('status', STATUS)
        ->value('id');
    }

    public function getTimeZone() {
        return Location::where('client_id', Auth::user()->client_id)
        ->where('id', Auth::user()->location_id)
        ->where('status', STATUS)
        ->value('timezone');
    }

    public function getTimeFormatZone(Request $request=null) {
        $timezone = !empty($request->timezone) ? $request->timezone : $this->getTimeZone();
        $time = Carbon::now($timezone);
        return $time->format('H:i:s');
    }

    public function getDateFormatZone(Request $request=null) {
        $timezone = !empty($request->timezone) ? $request->timezone : $this->getTimeZone();
        $time = Carbon::now($timezone);
        return $time->format('Y-m-d');
    }

    public function getDateTimeFormatZone(Request $request=null) {
        $timezone = !empty($request->timezone) ? $request->timezone : $this->getTimeZone();
        $time = Carbon::now($timezone);
        return $time->format('Y-m-d H:i:s');
    }

    public function getCountryTimezone() {
        return Countries::select('name as title', 'timezones as value')
        ->get();
    }

    public function setInventoryEvaluation(Request $request) {
        Location::where('id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->update([
            'inventory_evaluation' => $request->value
        ]);

        return response()->json(['message' => 'Inventory evaluation set successfully, you have to logout and login to effect this change', 'success' => true], 200);
    }

    public function getInventoryEvaluation($id, $client_id) {
        return Location::where('id', $id)
        ->where('client_id', $client_id)
        ->value('inventory_evaluation');
    }
}
