<?php

namespace App\Http\Controllers\General;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Account\Wallet;
use Unicodeveloper\Paystack\Facades\Paystack;
use Illuminate\Support\Facades\Auth;
use App\Models\Account\DebitCard;
use App\Models\Account\TransactionLog;
use Illuminate\Support\Str;
use App\Models\Account\CouponCode;
use App\Models\Account\CouponUser;
use Validator;
use App\Models\Retails\Location;
use DB;
use Illuminate\Support\Facades\Log;

class BillingController extends Controller
{
    //


    public function getTransactionHistory(Request $request)
    {
        $clientId = Auth::user()->client_id;
        $searchParams = $request->input('search') ?? "";
        $startDate = $request->start_date;
        $endDate = $request->end_date;
    
        // Prepare the SQL query with dynamic search and date range conditions
        $sql = "WITH transHistory AS (
                    SELECT 
                        biller_name AS name,
                        biller_description AS description,
                        payment_type,
                        amount,
                        payment_status AS status,
                        biller_reference,
                        date,
                        tm
                    FROM `transaction_log`
                    WHERE client_id = :clientId
                    " . (!empty($searchParams) ? "AND (biller_name LIKE '%{$searchParams}%' OR biller_description LIKE '%{$searchParams}%' OR biller_reference LIKE '%{$searchParams}%')" : "") . "
                    " . (!empty($startDate) && !empty($endDate) ? "AND date BETWEEN '{$startDate}' AND '{$endDate}'" : "") . "
                ),
                walletBalance AS (
                    SELECT 
                        (SUM(CASE WHEN effect = 'cr' THEN amount ELSE 0 END) -
                        SUM(CASE WHEN effect = 'dr' THEN amount ELSE 0 END)) AS balance
                    FROM `wallet`
                    WHERE client_id = :clientIdWallet
                ),
                transactionHistory AS (
                    SELECT 
                        JSON_ARRAYAGG(JSON_OBJECT(
                            'name', name,
                            'description', description,
                            'payment_type', payment_type,
                            'amount', amount,
                            'status', status,
                            'biller_reference', biller_reference,
                            'date', date,
                            'tm', tm
                        )) AS transHistory
                    FROM transHistory
                )
                SELECT 
                    transHistory,
                    (SELECT balance FROM walletBalance) AS walletBalance
                FROM transactionHistory;";
    
        // Execute the query
        $results = DB::select($sql, [
            'clientId' => $clientId,
            'clientIdWallet' => $clientId
        ]);
    
      
        if (count($results) > 0) {
            return response()->json([
                'walletBalance' => $results[0]->walletBalance ?? 0,
                'transHistory' => json_decode($results[0]->transHistory) ?? [],
            ]);
        }
    
        return response()->json(['message' => 'No data found', 'data' => []], 404);
    }
    

    
    







    public function getTransactionHistoryKeep()
    {
        $clientId = Auth::user()->client_id;

        // Use placeholders for parameters to prevent SQL injection
        $sql = "WITH transHistory AS (
                    SELECT 
                        biller_name AS name,
                        biller_description AS description,
                        payment_type,
                        amount,
                        payment_status AS status,
                        biller_reference,
                        date,
                        tm
                    FROM `transaction_log`
                    WHERE client_id = :clientId
                ),
                walletBalance AS (
                    SELECT 
                        (SUM(CASE WHEN effect = 'cr' THEN amount ELSE 0 END) -
                        SUM(CASE WHEN effect = 'dr' THEN amount ELSE 0 END)) AS balance
                    FROM `wallet`
                    WHERE client_id = :clientIdWallet
                ),
                transactionHistory AS (
                    SELECT 
                        JSON_ARRAYAGG(JSON_OBJECT(
                            'name', name,
                            'description', description,
                            'payment_type', payment_type,
                            'amount', amount,
                            'status', status,
                            'biller_reference', biller_reference,
                            'date', date,
                            'tm', tm
                        )) AS transHistory
                    FROM transHistory
                )
                SELECT 
                    transHistory,
                    (SELECT balance FROM walletBalance) AS walletBalance
                FROM transactionHistory;";

        // Use an array to safely pass the parameter to the query
        $results = DB::select($sql, ['clientId' => $clientId, 'clientIdWallet' => $clientId]);

        // Assuming the query returns one row with walletBalance and transHistory as JSON
        if (count($results) > 0) {
            return response()->json([
                'walletBalance' => $results[0]->walletBalance ?? 0,
                'transHistory' => json_decode($results[0]->transHistory) ?? 0,
            ]);
        }

        return response()->json(['message' => 'No data found', 'data' => []], 404);
    }


    public function getWalletBalance($clientId)
    {
        $totalCR = Wallet::where('client_id', $clientId)
                    ->where('effect', 'cr')
                    ->where('status', STATUS)
                    ->sum('amount');

        // Sum of debit transactions
        $totalDR = Wallet::where('client_id', $clientId)
                    ->where('effect', 'dr')
                    ->where('status', STATUS)
                    ->sum('amount');

        $totalWalletBalance = $totalCR - $totalDR;

        return $totalWalletBalance;
    }


    public function resolveBankAccount(Request $request)
    {
        $accountNumber = $request->acc_number;
        $bankCode = $request->bank_code;
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.paystack.co/bank/resolve?account_number={$accountNumber}&bank_code={$bankCode}",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => array(
                "Authorization: Bearer ".PAYSTACK_LIVE_KEY."",
                "Cache-Control: no-cache",
            ),
        ));
        
        $response = curl_exec($curl);
        $err = curl_error($curl);
        
        curl_close($curl);
        
        if ($err) {
            return response()->json(['error' => $err], 500);
        } else {
            return response()->json(json_decode($response, true));
        }
    }


    public function verifyPaystackTransaction($ref){
        $curl = curl_init();
          curl_setopt_array($curl, array(
            CURLOPT_URL => "https://api.paystack.co/transaction/verify/$ref",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => array(
              "Authorization: Bearer ".PAYSTACK_LIVE_KEY."",
              "Cache-Control: no-cache",
            ),
          ));
          $response = curl_exec($curl);
          $err = curl_error($curl);
        
          curl_close($curl);
          
          if ($err) {
            echo "cURL Error #:" . $err;
          } else {
            
          $ress = json_decode($response, true);
          return $ress;
            
          }
    }

 
    

    public function topupWallet(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'amount' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 400);
        }

        $email = $request->email;
        $amount = $request->amount;
        $ref = $request->reference_id;
        $saveCard = $request->save_card;
        $tokenUid = Str::uuid()->toString();
    
        $card_type = $request->card_type; // 'new_card', 'saved_card'
        if ($card_type == 'saved_card') {
            $auth_code = $request->auth_code;
            $cardData = $this->chargeFromCard($auth_code, $email, $amount);
            $ref = $cardData['data']['reference'];
        }
    
        $verify = $this->verifyPaystackTransaction($ref);
        $paymentStatus = $verify['data']['status'];
        

    
        if ($paymentStatus == 'success') {
          if($saveCard == ONE_STATUS){
            $this->saveCard($saveCard, $verify);
          }
           
            $signature = $verify['data']['authorization']['signature'];
    
            Wallet::create([
                'client_id' => Auth::user()->client_id,
                'location_id' => Auth::user()->location_id,
                'user_id' => Auth::user()->id,
                'amount' => $amount,
                'biller_name' => 'wallet toptup',
                'biller_description' => 'Money added through Bank Card',
                'biller_reference' => $ref,
                'token' => $tokenUid,
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => ONE_STATUS,
                'effect' => EFFECT_CREDIT
            ]);
    
            TransactionLog::create([
                'client_id' => Auth::user()->client_id,
                'user_id' => Auth::user()->id,
                'biller_name' => 'wallet topup',
                'biller_description' => 'Money added through Bank Card',
                'payment_type' => 'card',
                'card_signature' => $signature,
                'payment_source' => 'paystack',
                'token' => $tokenUid,
                'location_id' => Auth::user()->location_id,
                'amount' => $amount,
                'payment_status' => $paymentStatus,
                'biller_reference' => $ref,
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => ONE_STATUS
            ]);
    
            return response()->json(['success' => true, 'message' => 'Wallet topped up successfully']);
        } else {
            return response()->json(['success' => false, 'message' => 'Failed to top up wallet']);
        }
    }
    

    public function saveCard($saveCard, $verify = [])
    {
        if ($saveCard == 1 && !empty($verify)) {
            $authorization = $verify['data']['authorization'];
            
            $authCode = $authorization['authorization_code'];
            $bin = $authorization['bin'];
            $lastFour = $authorization['last4'];
            $expMonth = $authorization['exp_month'];
            $expYear = $authorization['exp_year'];
            
            // Check if the card already exists
            $exists = DebitCard::where('client_id', Auth::user()->client_id)
                            ->where('bin', $bin)
                            ->where('last_four', $lastFour)
                            ->where('exp_year', $expYear)
                            ->where('exp_month', $expMonth)
                            ->exists();

            if (!$exists) {
                DebitCard::create([
                    'authorization_code' => $authCode,
                    'bin' => $bin,
                    'last_four' => $lastFour,
                    'exp_month' => $expMonth,
                    'exp_year' => $expYear,
                    'channel' => $authorization['channel'],
                    'card_type' => trim($authorization['card_type']),
                    'bank' => $authorization['bank'],
                    'country_code' => $authorization['country_code'],
                    'brand' => $authorization['brand'],
                    'reusable' => $authorization['reusable'],
                    'signature' => $authorization['signature'],
                    'client_id' => Auth::user()->client_id,
                    'user_id' => Auth::user()->id,
                    'location_id' => Auth::user()->location_id,
                    'account_name' => $authorization['account_name'],
                    'date' => now()->toDateString(), 
                    'tm' => now()->toTimeString(), 
                    'status' => 0,
                ]);
            }
        }
    }

     public function chargeFromCard($auth_code, $email, $amount)
     {
        $url = "https://api.paystack.co/transaction/charge_authorization";
          $fields = [
            'authorization_code' => $auth_code,
            'email' => $email,
            'amount' => $amount
          ];
        
          $fields_string = http_build_query($fields);
        
          //open connection
          $ch = curl_init();
          
          //set the url, number of POST vars, POST data
          curl_setopt($ch,CURLOPT_URL, $url);
          curl_setopt($ch,CURLOPT_POST, true);
          curl_setopt($ch,CURLOPT_POSTFIELDS, $fields_string);
          curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: Bearer ".PAYSTACK_LIVE_KEY."",
            "Cache-Control: no-cache",
          ));
          
          //So that curl_exec returns the contents of the cURL; rather than echoing it
          curl_setopt($ch,CURLOPT_RETURNTRANSFER, true); 
          
          //execute post
          //execute post
          $result = curl_exec($ch);
        //   return $result;
          $ress = json_decode($result, true);
          return $ress;
     }


     public function confirmCoupon(Request $request)
     {
         $validator = Validator::make($request->all(), [
             'amount' => 'required',
             'coupon' => 'required',
         ]);
 
         if ($validator->fails()) {
             return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 400);
         }
 
         $amount = $request->amount;
         $coupon = $request->coupon;
 
         $checkCoupon = CouponCode::where(['code' => $coupon, 'status' => 0])->first();
 
         if (!$checkCoupon) {
             return response()->json(['message' => 'The coupon is not available.', 'error' => true], 400);
         }
 
         $checkCouponUsage = CouponUser::where(['code' => $coupon, 'status' => 0, 'client_id' => Auth::user()->id])->first();
 
         if ($checkCouponUsage) {
             return response()->json(['message' => 'The coupon has already been used by you.', 'error' => true], 400);
         }
 
         $couponPercentage = $checkCoupon->percentage;
         $totalPayment = $amount - ($amount * $couponPercentage / 100);
         $amountDiscounted = $amount - $totalPayment;
 
         $responseArray = [
             'id' => $checkCoupon->id,
             'code' => $checkCoupon->code,
             'percentage' => $couponPercentage,
             'amount_charged' => $amount,
             'amount_discounted' => $amountDiscounted,
             'newAmount' => $totalPayment,
         ];
 
         return response()->json(['message' => 'Coupon validated successfully', 'success' => true, 'data' => $responseArray], 200);
     }


    public function initialisePaystackPayment(Request $request){
        $amount = $request->amount;
        $email = $request->email;
        $url = "https://api.paystack.co/transaction/initialize";
        $refId = date('YmdHi').rand(10000, 99999).Auth::user()->id.Auth::user()->location_id . Auth::user()->client_id;
          $fields = [
            'email' => $email,
            'amount' => $amount,
            'reference' => $refId
          ];
        
          $fields_string = http_build_query($fields);
        
          //open connection
          $ch = curl_init();
          
          //set the url, number of POST vars, POST data
          curl_setopt($ch,CURLOPT_URL, $url);
          curl_setopt($ch,CURLOPT_POST, true);
          curl_setopt($ch,CURLOPT_POSTFIELDS, $fields_string);
          curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: Bearer ".PAYSTACK_LIVE_KEY."",
            "Cache-Control: no-cache",
          ));
          
          //So that curl_exec returns the contents of the cURL; rather than echoing it
          curl_setopt($ch,CURLOPT_RETURNTRANSFER, true); 
          
          //execute post
          $result = curl_exec($ch);
          echo $result;
    }


    



    public function handlePaystackWebhook(Request $request)
    {
       // \Log::channel('slack')->info('Dev Info', ['Webhook' => 'Webhook Started']); 

        // Retrieve the request's body and parse it as JSON
        $payload = json_decode($request->getContent(), true);

        // Optional: Verify event by checking the signature
        if(!$this->verifyPaystackEvent($request)) {
        //    \Log::channel('slack')->error('Signature Verification Failed', ['IP' => $request->ip()]); 
            return response()->json(['message' => 'Signature verification failed'], 403);
        }

        // Handle the event
        if ($payload['event'] === 'charge.success') {
        //    \Log::channel('slack')->info('Dev Info', ['Webhook' => 'Before Charge.Success']); 
            $this->handleSuccessfulPayment($payload['data']);
        //    \Log::channel('slack')->info('Dev Info', ['Webhook' => 'After Charge.Success']); 
        }

        // \Log::channel('slack')->info('Dev Info', [
        //     'Webhook' => 'Webhook Completed with Payload Data',
        //     'Payload' => json_encode($payload) 
        // ]);

        // Respond to Paystack
        return response()->json(['message' => 'Webhook handled']);
    }

    protected function verifyPaystackEvent(Request $request)
    {
        $signature = $request->header('x-paystack-signature');
        // Replace 'secret' with your Paystack secret key
        $secret = PAYSTACK_LIVE_KEY ;

        // Confirm the event's signature
        if ($signature) {
            $computedSignature = hash_hmac('sha512', $request->getContent(), $secret);
            return hash_equals($signature, $computedSignature);
        }

        return false;
    }


    public function handleSuccessfulPayment($data)
    {
      //  \Log::channel('slack')->info('handleSuccessfulPayment called', ['data' => $data]);
    
        $reference = $data['reference'];
    
        // Implementing retry logic for robust database updates
        $maxAttempts = 3;
        $attempt = 0;
        $success = false;
        $delayBetweenAttempts = 5; // seconds
    
        while (!$success && $attempt < $maxAttempts) {
            // Increment attempt counter
            $attempt++;
    
            // Update the Wallet model and log the result
            $walletUpdateCount = Wallet::where('biller_reference', $reference)->update(['status' => 0]);
            if ($walletUpdateCount > 0) {
             //   \Log::channel('slack')->info('Updated Wallet successfully', ['reference' => $reference, 'count' => $walletUpdateCount]);
                $success = true;
            } else {
             //   \Log::channel('slack')->error('Failed to update Wallet', ['reference' => $reference, 'attempt' => $attempt]);
                // Wait before retrying
                if ($attempt < $maxAttempts) {
                    sleep($delayBetweenAttempts);
                }
            }
    
            // Similar retry logic for TransactionLog and Location
            $transactionLogUpdateCount = TransactionLog::where('biller_reference', $reference)->update(['status' => 0]);
            if ($transactionLogUpdateCount > 0) {
            //    \Log::channel('slack')->info('Updated TransactionLog successfully', ['reference' => $reference, 'count' => $transactionLogUpdateCount]);
            } else {
           //     \Log::channel('slack')->error('Failed to update TransactionLog', ['reference' => $reference, 'attempt' => $attempt]);
            }
    
            $locationUpdateCount = Location::where('biller_reference', $reference)->update(['payment_confirmation' => 0]);
            if ($locationUpdateCount > 0) {
           //     \Log::channel('slack')->info('Updated Location successfully', ['reference' => $reference, 'count' => $locationUpdateCount]);
            } else {
           //     \Log::channel('slack')->error('Failed to update Location', ['reference' => $reference, 'attempt' => $attempt]);
            }
        }
    }
    



    public function handleSuccessfulPaymentOld($data)
    {
      ///  \Log::info('Starting to handle successful payment', ['data' => $data]);

        // Assuming $data['reference'] contains the biller reference
        $reference = $data['reference'];

        // Update the Wallet model and log the result
        $walletUpdateCount = Wallet::where('biller_reference', $reference)->update(['status' => 0]);
        if ($walletUpdateCount > 0) {
         //   \Log::info('Updated Wallet successfully', ['reference' => $reference, 'count' => $walletUpdateCount]);
        } else {
          //  \Log::error('Failed to update Wallet', ['reference' => $reference]);
        }

        // Update the TransactionLog model
        $transactionUpdateCount = TransactionLog::where('biller_reference', $reference)->update(['status' => 0]);
        if ($transactionUpdateCount > 0) {
       //     \Log::info('Updated TransactionLog successfully', ['reference' => $reference, 'count' => $transactionUpdateCount]);
        } else {
        //    \Log::error('Failed to update TransactionLog', ['reference' => $reference]);
        }

        // Update the Location model
        $locationUpdateCount = Location::where('biller_reference', $reference)->update(['payment_confirmation' => 0]);
        if ($locationUpdateCount > 0) {
       //     \Log::info('Updated Location successfully', ['reference' => $reference, 'count' => $locationUpdateCount]);
        } else {
       //     \Log::error('Failed to update Location', ['reference' => $reference]);
        }
    }


   

   


}
