<?php

namespace App\Http\Controllers\General;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Models\Retails\Customer;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use DB;
use App\Models\Account\DashboardIllustration;

class DashboardController extends Controller
{
    



    public function webDashboardData1()
    {
        // Hardcoded response for now
        $data = [
            'accountName' => 'Oyindamola',
            'klakpadApps' => [
                [
                    'name' => 'Klakpad Sella',
                    'location' => '5 Locations',
                    'subscriptions' => [
                        'active' => 3,
                        'inactive' => 2
                    ],
                ],
                [
                    'name' => 'Klakpad MenuPad',
                    'location' => '5 Locations',
                    'subscriptions' => [
                        'active' => 3,
                        'inactive' => 2
                    ],
                ],
                [
                    'name' => 'Klakpad Otel',
                    'location' => '5 Locations',
                    'subscriptions' => [
                        'active' => 3,
                        'inactive' => 2
                    ],
                ],
            ],
            'userManagement' => [
                'newlyAddedUsers' => 3,
                'totalUsers' => 6,
                'disabledUsers' => 4
            ],
            'wallet' => [
                'availableBalance' => '₦0.00',
            ]
        ];

        return response()->json($data);
    }

 

    public function webDashboardData()
    {
        $accountDbName = env('DB_DATABASE');
        $retailDbName = env('RETAIL_DB_DATABASE');
        
        // Get client_id from the authenticated user
        $clientId = Auth::user()->client_id;
        
        // Query to gather all necessary data using CTEs
        $query = "
        WITH LocationCounts AS (
            SELECT 
                COUNT(*) AS total_locations,
                COUNT(CASE WHEN end_date IS NULL OR end_date > NOW() THEN 1 END) AS active_locations,
                COUNT(CASE WHEN end_date IS NOT NULL AND end_date <= NOW() THEN 1 END) AS inactive_locations
            FROM {$retailDbName}.location
            WHERE client_id = ?
        ),
        NewlyAddedUsers AS (
            SELECT 
                COUNT(*) AS newly_added_users
            FROM {$accountDbName}.user
            WHERE created_at >= NOW() - INTERVAL 2 WEEK
              AND client_id = ?
        ),
        TotalUsers AS (
            SELECT 
                COUNT(*) AS total_users
            FROM {$accountDbName}.user
            WHERE client_id = ?
        ),
        DisabledUsers AS (
            SELECT 
                COUNT(*) AS disabled_users
            FROM {$accountDbName}.user
            WHERE status = 1
              AND client_id = ?
        ),
        WalletBalance AS (
            SELECT 
                (SUM(CASE WHEN effect = 'cr' THEN amount ELSE 0 END) -
                SUM(CASE WHEN effect = 'dr' THEN amount ELSE 0 END)) AS balance
            FROM {$accountDbName}.wallet
            WHERE client_id = ?
        )
    
        SELECT 
            (SELECT total_locations FROM LocationCounts) AS total_locations,
            (SELECT active_locations FROM LocationCounts) AS active_locations,
            (SELECT inactive_locations FROM LocationCounts) AS inactive_locations,
            (SELECT newly_added_users FROM NewlyAddedUsers) AS newly_added_users,
            (SELECT total_users FROM TotalUsers) AS total_users,
            (SELECT disabled_users FROM DisabledUsers) AS disabled_users,
            (SELECT balance FROM WalletBalance) AS available_balance
        ";
    
        // Execute the query using the client_id
        $results = DB::select($query, [$clientId, $clientId, $clientId, $clientId, $clientId]);
    
        // Constructing the response
        $data = [
            'accountName' => Auth::user()->name,
            'klakpadApps' => [
                [
                    'name' => 'Klakpad Sella',
                    'location' => $results[0]->total_locations . ' Locations',
                    'subscriptions' => [
                        'active' => $results[0]->active_locations,
                        'inactive' => $results[0]->inactive_locations,
                    ],
                ],
                [
                    'name' => 'Klakpad MenuPad',
                    'location' => $results[0]->total_locations . ' Locations',
                    'subscriptions' => [
                        'active' => $results[0]->active_locations,
                        'inactive' => $results[0]->inactive_locations,
                    ],
                ],
                [
                    'name' => 'Klakpad Otel',
                    'location' => $results[0]->total_locations . ' Locations',
                    'subscriptions' => [
                        'active' => $results[0]->active_locations,
                        'inactive' => $results[0]->inactive_locations,
                    ],
                ],
            ],
            'userManagement' => [
                'newlyAddedUsers' => $results[0]->newly_added_users,
                'totalUsers' => $results[0]->total_users,
                'disabledUsers' => $results[0]->disabled_users,
            ],
            'wallet' => [
                'availableBalance' => '₦' . number_format($results[0]->available_balance, 2),
            ],
        ];
    
        return response()->json($data);
    }
    
    public function dashboard(Request $request)
    {
        $cusName = $request->input('search') ?? '';
        $search = !empty($cusName) ? "AND c.coy_name like '%$cusName%'" : '';
        $location_id = Auth::user()->location_id;
        $locationSort = !empty($request->location_id) ? "AND location_id='" . $request->location_id . "'" : "AND location_id='$location_id'";
        $locationSortOthers = !empty($request->location_id) ? "AND s.location_id='" . $request->location_id . "'" : "AND s.location_id='$location_id'";
        $client = Auth::user()->client_id;
        $user_id = Auth::user()->id;
        $startDate = $request->start_date;
        $endDate = $request->end_date;
        $today = Carbon::now()->format('Y-m-d');
        $date = !empty($startDate) && !empty($endDate) ? "AND txn_date BETWEEN '$startDate' AND '$endDate'" : " AND txn_date = '$today'";
        $dateSummary = !empty($startDate) && !empty($endDate) ? "WHERE c.date BETWEEN '$startDate' AND '$endDate'" : "";
       
        $accountDbName = env('DB_DATABASE');
        $retailDbName = env('RETAIL_DB_DATABASE');

        $allCustomersQuery = "
        WITH payment_method AS(
            SELECT 'bank transfer' AS pmethod
            UNION
            SELECT 'cash' AS pmethod
            UNION
            SELECT 'charged to customer account' AS pmethod
            UNION
            SELECT 'complimentary' AS pmethod
            UNION
            SELECT 'pos' AS pmethod
            UNION
            SELECT 'loyalty' AS pmethod),
            
            txn as (
            SELECT * 
            FROM {$retailDbName}.txn_log 
            WHERE  control_id !=1 AND client_id = '$client'  $locationSort),
            
            purchases as ( 
             SELECT * 
             FROM {$retailDbName}.item_log
             WHERE control_id != 1  AND client_id = '$client'  $locationSort),

             business_detail as (
                SELECT  u.id as user_id, u.location_id, u.name as username , 
                l.name as businessName FROM {$accountDbName}.user u 
                LEFT JOIN {$retailDbName}.location l ON u.location_id = l.id
                WHERE u.client_id = '$client' $locationSort AND u.id ='$user_id' ),
            
        
                today_txn as (
                    SELECT * , (todaySales - todayExpense - cgs) as todayProfit FROM
                    (SELECT coalesce(SUM(amount-discount) ,0)as todaySales
                        FROM purchases
                        WHERE txn_date = CURRENT_DATE ) as sales
                        JOIN
                    (SELECT  
                        COALESCE(SUM(CASE WHEN account_type = '005' AND chart_id != '50047' THEN jvalue ELSE 0 END), 0) 
                        AS todayExpense,
                        COALESCE(SUM(CASE WHEN chart_id = '50047' THEN jvalue ELSE 0 END), 0) 
                        AS cgs
                        FROM txn 
                        WHERE txn_date = CURRENT_DATE ) as expenses
                        ON 1 = 1
                    ),
          
            sales AS (
                SELECT pmethod,  SUM(jvalue) as sales
                FROM txn
                WHERE  prefix = 's' and effect ='dr' and pmethod != ' ' and txn_date = CURRENT_DATE
                GROUP BY pmethod ),
               
            sales_analysis AS (
                SELECT a.pmethod, COALESCE(b.sales,0) as sales , COALESCE(SUM(c.sales),0)as today,
                (COALESCE((b.sales *100)/SUM(c.sales),0)) as percent
                FROM payment_method a
                LEFT JOIN sales b  ON a.pmethod = b.pmethod
                LEFT JOIN sales c  ON 1 = 1
                GROUP BY a.pmethod),
                
           revExpData AS (SELECT
              MONTH(txn_date) as month,
              SUM(CASE WHEN account_type = '004' THEN jvcr-jvdr ELSE 0 END) AS revenue,
              SUM(CASE WHEN account_type = '005' THEN jvdr-jvcr ELSE 0 END) AS expense,
              SUM(CASE WHEN account_type = '004' THEN jvcr-jvdr ELSE 0 END) +
              SUM(CASE WHEN account_type = '005' THEN jvdr-jvcr ELSE 0 END)  AS revExpSum
              FROM txn
              WHERE MONTH(txn_date) = MONTH(NOW())
              GROUP BY month
              ORDER BY month),
            
            profit_loss AS (
              SELECT month, revenue, expense,
               COALESCE(SUM(revenue-expense),0) AS profit,
                (revenue*100)/revExpSum AS rev_percent,
                (expense*100)/revExpSum AS exp_percent
               FROM revExpData),
             
             cash_flow AS (
               SELECT 
                 COALESCE(SUM(CASE WHEN effect= 'cr' AND chart_id=" . CHART_CASH . " THEN jvcr ELSE 0 END), 0) as cashSpent,
                 COALESCE(SUM(CASE WHEN effect= 'dr' AND chart_id=" . CHART_CASH . " THEN jvdr ELSE 0 END), 0) as cashReceived,
                 COALESCE(SUM(CASE WHEN effect= 'cr' AND chart_id=" . CHART_BANK . " THEN jvcr ELSE 0 END), 0) as bankSpent,
                 COALESCE(SUM(CASE WHEN effect= 'dr' AND chart_id=" . CHART_BANK . " THEN jvdr ELSE 0 END), 0) as bankReceived,
                 COALESCE(SUM(CASE WHEN chart_id=" . CHART_BANK . " THEN jvdr-jvcr ELSE 0 END), 0) as bankBalance,
                 COALESCE(SUM(CASE WHEN chart_id=" . CHART_CASH . " THEN jvdr-jvcr  ELSE 0 END), 0) as cashBalance,
                 COALESCE(SUM(jvdr-jvcr), 0) as totalBalance
               FROM txn
               WHERE chart_id=" . CHART_CASH . " OR chart_id=" . CHART_BANK . " $date
                ),

            debt_cred AS(
                SELECT coy, chart_id,token,
                SUM(jvcr) as jvcr,
                SUM(jvdr) as jvdr
            FROM txn
            WHERE
               (chart_id = '20003' OR chart_id = '40000') 
            AND status = 0
            GROUP BY coy
        ),
                 
        debtor_creditor_data AS (
            SELECT 
                SUM(CASE WHEN (jvdr - jvcr) < 0 THEN (jvcr - jvdr) ELSE 0 END) AS creditor_sum,
                SUM(CASE WHEN (jvdr - jvcr) > 0 THEN (jvdr - jvcr) ELSE 0 END) AS debtor_sum
            FROM debt_cred
        ),
                
              debtor_creditor AS (
                  SELECT creditor_sum, debtor_sum, 
                  SUM((creditor_sum*100)/(creditor_sum + debtor_sum)) AS creditor_percent,
                  SUM((debtor_sum*100)/(creditor_sum + debtor_sum)) AS debtor_percent
                  FROM debtor_creditor_data),
                  
               pmQty As (
                    SELECT cal_qty as pQty,item_id, rate FROM {$retailDbName}.item_price WHERE primary_measurement = 1 and status=0 ),
                  

               inventory as 
                (SELECT a.id as item_id,a.name,s.qty, c.cal_qty, b.cal_qty as bcal ,  s.qty* c.cal_qty/b.cal_qty as pQty, s.m_metric,m.name as msm,
                 s.effect,  b.rate, m.m_category_id,a.item_category_id,
                  CASE WHEN s.effect = 'dr' THEN (s.amount* b.cal_qty)/(c.cal_qty*s.qty) ELSE 0 END  AS price, 
                  SUM( CASE WHEN s.effect ='dr' THEN (s.amount) ELSE NULL END)
                  OVER ( PARTITION BY s.item_id ORDER BY s.txn_date,s.tm ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) /
                  SUM( CASE WHEN s.effect ='dr' THEN (c.cal_qty* s.qty) ELSE NULL END)
                  OVER ( PARTITION BY s.item_id ORDER BY s.txn_date,s.tm ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS running_cp,
                 ROW_NUMBER() OVER (
                            PARTITION BY s.item_id 
                            ORDER BY s.txn_date DESC, s.tm DESC
                        ) AS rn
                FROM {$retailDbName}.stock as s
                LEFT JOIN {$retailDbName}.items a ON s.item_id = a.id
                LEFT JOIN {$retailDbName}.item_price  c ON s.item_id = c.item_id AND c.m_metric = s.m_metric AND c.location_id = '$location_id'
                 AND (c.outlet_id  = s.store_id OR c.default_price = 1 ) 
                LEFT JOIN {$retailDbName}.item_price  b ON s.item_id = b.item_id AND b.primary_measurement = 1 AND b.location_id = '$location_id' 
                 AND (b.outlet_id  = s.store_id OR b.default_price = 1 )
                LEFT JOIN {$retailDbName}.measurement m ON m.m_category_id = s.m_category_id AND m.hierarchy = s.m_metric AND m.location_id = '$location_id'
                WHERE a.id IS NOT NULL  AND s.control_id != 1   AND s.client_id =  '$client' AND s.location_id = '$location_id'  
                GROUP BY s.id,s.item_id, s.effect,s.m_metric
                ORDER BY  a.name, s.txn_date,s.tm),
                
                inventoryDetails AS (
                    SELECT item_id, name, pQty as uQty, msm, effect, rate,m_category_id,item_category_id,
                    running_cp as price, 
                    running_cp * pQty as  amount,
                    (SELECT running_cp AS cp FROM inventory WHERE rn = 1 and item_id = a.item_id) as cp
                    FROM inventory a
                 ),

               stockQty AS (
                  SELECT
                      item_id as id, name,msm ,rate,cp,m_category_id,item_category_id,
                      SUM(CASE WHEN effect = 'cr' THEN (uQty) ELSE 0 END) as dispensed,
                      SUM(CASE WHEN effect = 'dr' THEN (uQty) ELSE 0 END) as qtyStock,
                      SUM(CASE WHEN effect = 'cr' THEN amount ELSE 0 END) as crAmount,
                      SUM(CASE WHEN effect = 'dr' THEN amount ELSE 0 END) as drAmount  
                  FROM inventoryDetails 
                GROUP BY item_id
                ORDER BY name
      			),
               
               stock_balance AS (
                   SELECT name, rate,
                   SUM(qtyStock - dispensed) AS closingBal ,
                   SUM(drAmount  - crAmount) AS stockBal,
                   SUM(rate * (qtyStock -dispensed)) AS retail_value
                   FROM stockQty
                   ),
                   
                  inventory_summary AS (
                    SELECT  stockBal, retail_value,
                    ((retail_value -stockBal) * 100)/retail_value AS pm,
                    ((retail_value -stockBal) * 100)/stockBal AS mkup
                    FROM stock_balance
                    ),
                    
                 sales_analysis_arr AS (
                    SELECT JSON_ARRAYAGG( JSON_OBJECT(
                        'pmethod', pmethod,
                        'sales', sales,
                        'today', today,
                        'percent', percent
                    )) AS sales_analysis
                    FROM sales_analysis
                ),
                
                profit_loss_arr AS (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'month', month,
                        'revenue', revenue,
                        'expense', expense,
                        'profit', profit,
                        'rev_percent', rev_percent,
                        'exp_percent', exp_percent
                    )) AS profit_loss
                FROM profit_loss
                ),
                
              today_txn_arr AS (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'todaySales', todaySales,
                        'todayExpense', todayExpense,
                        'cgs', cgs,
                        'todayProfit', todayProfit
                    )) AS today_txn
                    FROM today_txn
                ),
                inventory_summary_arr AS (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'stockBal', stockBal,
                        'retail_value', retail_value,
                        'pm', pm,
                        'mkup', mkup
                    )) AS inventory_summary
                    FROM inventory_summary
                ),
                
                debtor_creditor_arr AS(
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'creditor_sum', creditor_sum,
                        'debtor_sum', debtor_sum,
                        'creditor_percent', creditor_percent,
                        'debtor_percent', debtor_percent
                    )) AS debtor_creditor
                 FROM debtor_creditor ),
               
               cash_flow_arr AS(
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'cashSpent', cashSpent,
                        'cashReceived', cashReceived,
                        'bankSpent', bankSpent,
                        'bankReceived', bankReceived,
                        'bankBalance', bankBalance,
                        'cashBalance', cashBalance,
                        'totalBalance', totalBalance
                    )) AS cash_flow
                 FROM cash_flow ),

                 business_detail_arr AS (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                         'user_id', user_id,
                         'location_id', location_id,
                         'username', username,
                         'businessName', businessName
                     )) AS business_details
                  FROM business_detail )
                 
            
         
         SELECT * from sales_analysis_arr
         CROSS JOIN profit_loss_arr
         CROSS JOIN today_txn_arr
         CROSS JOIN inventory_summary_arr
         CROSS JOIN debtor_creditor_arr
         CROSS JOIN business_detail_arr
         CROSS JOIN cash_flow_arr;
         ";
    
        $jsonString = DB::select($allCustomersQuery);
        // var_dump($jsonString); exit;
    
    // Decode the nested JSON strings within "customer" and "bank_account" fields
        foreach ($jsonString as &$row) {
            if (property_exists($row, 'sales_analysis')) {
                $row->sales_analysis = json_decode(stripslashes($row->sales_analysis), true) ?? [];
            }
    
            if (property_exists($row, 'profit_loss')) {
                $row->profit_loss = json_decode(stripslashes($row->profit_loss), true);
            }

            if (property_exists($row, 'today_txn')) {
                $row->today_txn = json_decode(stripslashes($row->today_txn), true);
            }
    
            if (property_exists($row, 'inventory_summary')) {
                $row->inventory_summary = json_decode(stripslashes($row->inventory_summary), true);
            }

            if (property_exists($row, 'debtor_creditor')) {
                $row->debtor_creditor = json_decode(stripslashes($row->debtor_creditor), true);
            }

            if (property_exists($row, 'business_details')) {
                $row->business_details = json_decode(stripslashes($row->business_details), true);
            }

            if (property_exists($row, 'cash_flow')) {
                $row->cash_flow = json_decode(stripslashes($row->cash_flow), true);
            }
            
        }
    
        return response()->json([
            'data' => $jsonString,
            'success' => true
        ], 200);
        }




        public function dashboardIllustration()
        {
            // Fetch a random illustration
            $illustration = DashboardIllustration::inRandomOrder()->first();
        
            // Check if an illustration is found
            if ($illustration) {
                $illustrationData = [
                    'id' => $illustration->id,
                    'img' => asset('public/dashboardIllustration/' . $illustration->img),
                    'img_txt' => $illustration->img_txt,
                ];
        
                return response()->json(['illustration' => $illustrationData]);
            } else {
                // Handle the case where no illustration is found
                return response()->json(['message' => 'No illustrations found'], 404);
            }
        }
}




