<?php

namespace App\Http\Controllers\General;
use App\Http\Controllers\Controller;
use App\Models\Account\App;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response; 
use App\Models\Retails\Countries;
use App\Models\Retails\States;
use Illuminate\Support\Facades\DB;
use JWTAuth;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Account\ChartOfAccount\AccountCategoryController;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Settings\Store\StoreController;
use App\Http\Controllers\Folio\Location\LocationController;
use App\Http\Controllers\Settings\LoyaltyManagement\LoyaltyManagementController;
use App\Http\Controllers\Inventory\Product\ProductController;
use App\Http\Controllers\Settings\Measurement\MeasurementController;
use App\Http\Controllers\Folio\CustomerVendor\CustomerVendorController;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use App\Models\Retails\ChartOfAccount;
use App\Models\Retails\ChartCate;
use App\Models\Retails\Currency;
use App\Models\Retails\Store;
use App\Models\Retails\LoyaltyManagement;
use App\Models\Retails\ItemCategory;
use App\Models\Retails\MeasurementCategory;
use App\Models\Retails\Measurement;
use App\Models\Retails\Customer;
use App\Models\Retails\LoyaltyLog;
use App\Models\Retails\ItemLog;
use App\Models\Retails\StockTake;
use App\Models\Retails\CreditLog;
use App\Models\Retails\TxnLog;
use App\Models\Retails\StockAdjustment;
use App\Models\Retails\Stock;
use App\Models\Retails\RequestModel;
use App\Models\Retails\SynSetting;
use Illuminate\Support\Facades\Auth;
use App\Services\ZeptoMailService;

class GeneralController extends Controller
{
    //
    protected $locationController;
    protected $zeptoMailService;

    public function __construct(
        LocationController $locationController, 
        ZeptoMailService $zeptoMailService) {
        $this->locationController = $locationController;
        $this->zeptoMailService = $zeptoMailService;     
    }   



    public function sendEmailMessage(Request $request)
    {
        // Validate incoming request
        $validatedData = $request->validate([
            'to' => 'required|email',
            'subject' => 'required|string|max:255',
            'message' => 'required|string',
        ]);

        // Capture the full response from sendEmail
        $response = $this->zeptoMailService->sendEmail(
            $validatedData['to'], 
            $validatedData['subject'], 
            $validatedData['message']
        );

        // Handle the response more granularly
        if ($response->successful()) {
            return response()->json(['message' => 'Email sent successfully'], 200);
        } else {
            // Provide a failure response with appropriate status
            return response()->json([
                'message' => 'Email sending failed. Status: ' . $response->status(),
                'error' => true,
            ], $response->status());
        }
    }


    public function sendEmail($to, $subject, $message)
    {
        // Capture the full response from sendEmail
        $response = $this->zeptoMailService->sendEmail($to, $subject, $message);

        // Instead of just checking for a boolean, handle the response more granularly
        if ($response->successful()) {
            return 'Email sent successfully';
        } else {
            // Log the status code and response body for debugging
            \Log::error('Email sending failed', [
                'status' => $response->status(),
                'response' => $response->body(),
            ]);

            // Provide a generic failure message or include some response details if appropriate
            return 'Email sending failed. Status: ' . $response->status();
        }
    }
   

    public function generateUniqueNumber($table, $column) {
        $generatedNumber = $this->generateRandomNumber();
      
        $query = DB::table($table)->where($column, $generatedNumber)->first();
      
        while ($query != null) {
          $generatedNumber = $this->generateRandomNumber();
          $query = DB::table($table)->where($column, $generatedNumber)->first();
        }
      
        return $generatedNumber;
      }

      public function generateRandomNumber() {
        $number = rand(1000000000, 9999999999);
        return $number;
      }


    public function sendSuccess($message)
    {
        return Response::json([
            'success' => true,
            'message' => $message,
        ], 200);
    }

    public static function arrayPrinter($array)
    {
        echo '<pre>';
        print_r($array);
        echo '</pre>';
    }

    public function generateToken($prefix = '', $user)
    {
        return $prefix.str_shuffle(time().$user);
    }

    public function hashPass($password, $salt) {
		$hash_password = hash("SHA512", base64_encode(str_rot13(hash("SHA512", str_rot13($salt . $password)))));
		return $hash_password;
	}

    public function salt() {
		$length = 45;
		return base64_encode(random_bytes(ceil(0.75 * $length)));
	}

    public static function checkPhone($phone)
    {
        if (!is_numeric($phone)) {
            return true;
        }
        if (
            strlen($phone) !== 11 &&
            strlen($phone) !== 14 &&
            strlen($phone) !== 13
        ) {
            return true;
        }
        return false;
    }

    public static function checkEmail($email)
    {
        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
            return true;
        }
        return false;
    }

    public function getCountry()
    {
        $countries = Countries::get();
        $json = [];
        foreach ($countries as $datas) {
            $json['id'] = (string) $datas->id;
            $json['name'] = $datas->name;
            $json['iso3'] = $datas->iso3;
            $json['phonecode'] = $datas->phonecode;
            $json['flag'] = $datas->currency;
            $json['currency'] = 'https://flagcdn.com/224x168/'.strtolower($datas->iso2).'.png';

            $data[] = $json;
        }
        if (count($json) == 0) {
            $data_r = $json;
        } else {
            $data_r = $data;
        }

        return Response::json(['message' => '', 'data' => $data_r, 'success' => true], 200);
    }

    public function testAuth()
    {
        echo 'here';
    }

    public function getStates(Request $request)
    {
        $data = States::where('country_id', $request->country_id)->get();

        return Response::json(['message' => '', 'data' => $data, 'success' => true], 200);
    }

    public function getApp(Request $request)
    {
        
        $data = App::where('status', STATUS)->get();

        return Response::json(['message' => '', 'data' => $data, 'success' => true], 200);
    }

    public function standardPhone($recipients)
    {
        $gsm = [];
        $country_code = '234';
        $arr_recipient = explode(',', $recipients);
        //$mobilenumber = trim($arr_recipient);

        foreach ($arr_recipient as $recipient) {
            $mobilenumber = trim($recipient);
            if (substr($mobilenumber, 0, 1) == '0') {
                $mobilenumber = $country_code.substr($mobilenumber, 1);
            } elseif (substr($mobilenumber, 0, 1) == '+') {
                $mobilenumber = substr($mobilenumber, 1);
            }
            $generated_id = uniqid('int_', false);
            $generated_id = substr($generated_id, 0, 30);
            $gsm['gsm'][] = ['msidn' => $mobilenumber, 'msgid' => $generated_id];
        }

        return  $mobilenumber;
    }

    public function undoTransaction(Request $request) {
        $tables = [ItemLog::class, TxnLog::class, Stock::class, RequestModel::class, StockAdjustment::class, LoyaltyLog::class, CreditLog::class, StockTake::class];

        foreach($tables as $table) {
            $this->undoTransactionTables($table, $request->token);
        } 

        return response()->json(['message' => 'Transaction archived successfully', 'success' => true], 200);
    }

    private function undoTransactionTables($table, $token) {
        
       $table::where('token', $token)
        ->where('control_id', STATUS)
        ->where('location_id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->update([
            'control_id' => ONE_STATUS,
            'control_user_id' => Auth::user()->id,
            'up_date' => Carbon::now()->format('Y-m-d')
        ]);

    }

    public function getUserRights() {
        $client_id=Auth::user()->client_id;
        $location_id=Auth::user()->location_id;
        $user_id=Auth::user()->id;
        $plan_id = $this->locationController->getPlanId($client_id, $location_id);

        $query = "
            WITH modules AS (
                SELECT DISTINCT r.module_id, m.module_name, m.mobile_module_icon, m.module_icon, sub_mobile_module_icon
                FROM " . TBL_RIGHTS . " r

                LEFT JOIN " . TBL_MODULE . " m
                ON m.id=r.module_id

                WHERE r.module_id != 11 AND r.module_id != 12 AND r.client_id='$client_id' AND r.location_id='$location_id' AND r.user='$user_id' AND r.status = " . STATUS . "
            ),

            userRight AS (
                SELECT 
                m.*,
                u.name,
                loc.name as location,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'id', r.menu_id,
                        'name', l.menu_name,
                        'screen', l.screen,
                        'module_id', r.module_id,
                        'menu_icon',l.menu_icon,
                        'sub_menu_icon',l.sub_menu_icon
                    ))

                    FROM " . TBL_RIGHTS ." r 

                    LEFT JOIN " . TBL_SUB_MENU . " l
                    ON l.module_id = r.module_id AND l.is_menu = 1 AND l.app_id=" . APP_ID . " AND l.id = r.menu_id AND l.screen != '' AND l.mobile = 1

                    WHERE m.module_id = r.module_id AND r.client_id='$client_id' AND r.location_id='$location_id' AND r.user='$user_id' AND r.status = " . STATUS . " AND r.menu_id=l.id AND r.plan_id='$plan_id'

                    GROUP BY r.module_id
                ) as rights

                FROM modules m

                LEFT JOIN " . TBL_USER . " u
                ON u.id='$user_id'

                LEFT JOIN " . TBL_LOCATION . " loc
                ON loc.id='$location_id'
            ),

            location AS (
               SELECT
                        id,
                        name
                    FROM " . TBL_LOCATION ." 

                    WHERE client_id='$client_id' AND status = 0
                    ),
                
               store AS (
                    SELECT 
                    id,
                    name
                    
                    FROM " . TBL_STORE ." 

                    WHERE client_id='$client_id' AND location_id='$location_id' AND status = 0
               ),
               
                userRightsArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'name',u.name,
                        'location',u.location,
                        'rights',u.rights,
                        'module_id', u.module_id,
                        'module_name', u.module_name,
                        'mobile_module_icon', u.mobile_module_icon,
                        'module_icon', u.module_icon,
                        'sub_mobile_module_icon',u.sub_mobile_module_icon
                        
                    )) as userRights
                FROM userRight u 
                WHERE u.rights IS NOT NULL),
            
                locationArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'id', l.id,
                        'name', l.name
                        
                    )) as locations
                FROM location l),

                storeArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'id', s.id,
                        'name', s.name
                    )) as stores
                FROM store s)
            
                SELECT * FROM userRightsArr, locationArr, storeArr
        ";

        $data = DB::select($query);

        foreach ($data as $row) {
            if (property_exists($row, 'userRights')) {
                $row->userRights = $row->userRights ? json_decode(stripslashes($row->userRights), true) : [];
            }
            if (property_exists($row, 'locations')) {
                $row->locations = $row->locations ? json_decode(stripslashes($row->locations), true) : [];
            }
            if (property_exists($row, 'stores')) {
                $row->stores = $row->stores ? json_decode(stripslashes($row->stores), true) : [];
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function getDefaultCurrency()
    {
        
        $query = "
            SELECT 
            *
            FROM " . TBL_CURRENCY_CATEGORY . "
            WHERE status = 0
        ";

        $data = DB::select($query);

        foreach ($data as $row) {
            if ($row->name == 'naira') {
                $row->symbol = '₦';
            } elseif ($row->name == 'dollar') {
                $row->symbol = '$';
            } elseif ($row->name == 'pounds') {
                $row->symbol = '€';
            } elseif ($row->name == 'euro') {
                $row->symbol = '£';
            } elseif ($row->name == 'japanese yen') {
                $row->symbol = '¥';
            } elseif ($row->name == 'chinese yuan') {
                $row->symbol = '元';
            }

        }

        // return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
        return $data;
    }

    public function getDefaultVendor($client_id)
    {
        global $db;

        $query = "
            SELECT * FROM
            " . TBL_CUSTOMERS . "
            WHERE client_id='$client_id' AND coy_name ='default vendor' AND cate='cust' AND status = 0
        ";
        
        $data = DB::select($query);

        return $data;
    }

    public function getDefaultWalkInCustomer($client_id)
    {
        $query = "
            SELECT * FROM
            " . TBL_CUSTOMERS . "
            WHERE client_id='$client_id' AND coy_name ='walk-in customer' AND cate='lba' AND status = 0
        ";

        $data = DB::select($query);

        return $data;
    }

    public function getDefaultAppBanks()
    {

        $query = "
            SELECT * FROM
            " . TBL_APP_BANK . "
            WHERE status = 0 ORDER BY id ASC
        ";

        $data = DB::select($query);

        return $data;
        
    }

    public function getDefaultLocations($client_id, $location_id, $user_id, $location_overview)
    {

        // $locationID = $location_overview == 1 ? '' : "AND id='$location_id'";

        $query="
            SELECT * FROM
            " . TBL_LOCATION . "
            WHERE client_id='$client_id' AND status = 0 ORDER BY id ASC
        ";

        $data = DB::select($query);

        return $data;
    }


    public function getDefaultUsers($client_id, $overview, $userId, $location_id) {
	    
	    $overviewRight = $overview == 0 ? " AND id='$userId'" : "";
	    $query = "
            SELECT * FROM
            " . TBL_USER . "
            WHERE client_id='$client_id' AND location_id='$location_id' $overviewRight
        ";
	    $data = DB::select($query);

        return $data;
	}

    public function getDefaultUsersPinCode($client_id, $location_id) {
	    
	    $query = "
            SELECT * FROM
            " . TBL_USER . "
            WHERE client_id='$client_id' AND location_id='$location_id'
        ";
	    $data = DB::select($query);

        return $data;
	}

    public function getDefaultAccountClass() {
        $query = "
            SELECT * FROM
            " . TBL_ACCOUNT_CLASS . "
            WHERE status = 0 ORDER BY id ASC
        ";
        $data = DB::select($query);

        return $data;
    }

    public function getDefaultMeasurement($client_id, $location_id) {

        $query = "
            SELECT * FROM
            " . TBL_MEASUREMENT . "
            WHERE client_id = '$client_id' AND location_id='$location_id' AND status = 0 ORDER BY id ASC       
        ";

        $data = DB::select($query);

        return $data;

        
    }

    public function getChartCateInfo($client_id, $location_id) {
        return ChartCate::where('client_id', $client_id)
        ->where('location_id', $location_id)
        ->where('status', STATUS)
        ->get();
    }

    public function getChartOfAccountInfo($client_id, $location_id) {
        return ChartOfAccount::where('client_id', $client_id)
        ->where('location_id', $location_id)
        ->where('status', STATUS)
        ->get();
    }

    public function getDefaultMeasurementCategory($client_id) {

        $query = "
            SELECT * FROM
            " . TBL_MEASUREMENT_CATEGORY . "
            WHERE client_id = '$client_id' AND status = 0 ORDER BY id ASC
        ";

        $data = DB::select($query);

        return $data;

    }

    public function getSyncSetting($client_id, $location_id) {

        $rlt = SynSetting::where('client_id', $client_id)
        ->where('location_id', $location_id)
        ->get();
        
        $response = [
            "login" => null,
            "action" => null,
            // "inactive" => null,
        ];
        
        // Check if the results exist and assign accordingly
        if ($rlt->isNotEmpty()) {
            if (isset($rlt[0])) $response["login"] = $rlt[0]->status;
            if (isset($rlt[1])) $response["action"] = $rlt[1]->status;
            // if (isset($rlt[2])) $response["inactive"] = $rlt[2]->status;
        }
        
        return $response;
    }

    public function removeComma($arg)
    {
        return str_replace(',', '', $arg);
    }

    public function removeComma2($arg)
    {
        return str_replace(',', ' ', $arg);
    }

    public function removeApostrophe($arg)
    {
        return str_replace("'", '', $arg);
    }
}