<?php

namespace App\Http\Controllers\General;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Account\ChartOfAccount\AccountCategoryController;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Settings\Store\StoreController;
use App\Http\Controllers\Settings\LoyaltyManagement\LoyaltyManagementController;
use App\Http\Controllers\Settings\Device\DeviceController;
use App\Http\Controllers\Settings\User\UserController;
use App\Http\Controllers\Inventory\Product\ProductController;
use App\Http\Controllers\Settings\Measurement\MeasurementController;
use App\Http\Controllers\Folio\CustomerVendor\CustomerVendorController;
use App\Http\Controllers\General\SyncController;
use App\Http\Controllers\CustomerEngagement\CampaignController;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use Auth;
use App\Models\Retails\ChartOfAccount;
use App\Models\Retails\ChartCate;
use App\Models\Retails\Currency;
use App\Models\Retails\Store;
use App\Models\Retails\LoyaltyManagement;
use App\Models\Retails\ItemCategory;
use App\Models\Retails\MeasurementCategory;
use App\Models\Retails\Measurement;
use App\Models\Retails\Customer;
use App\Models\Account\DeviceLog;
use App\Models\Retails\SynSetting;
use App\Models\Retails\Segment;


class LoadDefaultController extends Controller
{
    //

    protected $chartOfAccountController;
    protected $accountCategoryController;
    protected $currencyController;
    protected $storeController;
    protected $loyaltyManagementController;
    protected $productController;
    protected $measurementController;
    protected $customerVendorController;
    protected $deviceController;
    protected $userController;
    protected $syncController;

    public function __construct(
        ChartOfAccountController $chartOfAccountController, 
        AccountCategoryController $accountCategoryController,
        CurrencyController $currencyController,
        StoreController $storeController,
        LoyaltyManagementController $loyaltyManagementController,
        ProductController $productController,
        MeasurementController $measurementController,
        CustomerVendorController $customerVendorController,
        DeviceController $deviceController,
        UserController $userController,
        SyncController $syncController,
        CampaignController $campaignController
        
    ) {
        $this->chartOfAccountController = $chartOfAccountController;
        $this->accountCategoryController = $accountCategoryController;
        $this->currencyController = $currencyController;
        $this->storeController = $storeController;
        $this->loyaltyManagementController = $loyaltyManagementController;
        $this->productController = $productController;
        $this->measurementController = $measurementController;
        $this->customerVendorController = $customerVendorController;
        $this->deviceController = $deviceController;
        $this->userController = $userController;
        $this->syncController = $syncController;
        $this->campaignController = $campaignController;
    }

    public function loadDefault($user, $request, $plan_id) {
        $this->loadDefaultChartOfAccount($user);
        $this->loadDefaultChartCate($user);
        $this->loadDefaultCurrency($user);
        $this->loadDefaultStore($user);
        $this->loadDefaultLoyaltyManagement($user);
        $this->loadDefaultProductGrouping($user);
        $this->loadDefaultProductCategory($user);
        $this->loadDefaultMeasurementCategory($user);
        $this->loadDefaultMeasurement($user);
        $this->loadDefaultCustomer($user);
        $this->loadDefaultVendor($user);
        if($request->origin != 3){
            $this->loadDefaultDevice($user, $request);
        }
        $this->loadDefaultSyncSetting($user);
        $this->userController->masterUser($user, $plan_id);
        $this->loadDefaultCustomerSegment($user);
        $this->campaignController->loadCustomerCampaign($user);
    }

    public function loadDefaultCustomerSegment($user) {
        $segments = Segment::where('client_id', STATUS)
        ->where('status', STATUS)
        ->get();    
    
        foreach($segments as $segment) {    
            $customerSegment = Segment::where('client_id', $user->client_id)
            ->where('location_id', $user->location_id)
            ->where('status', STATUS)
            ->where('name', $segment->name)
            ->first();

            if($customerSegment < ONE_STATUS){
                $fields = [
                    'name' => $segment->name,
                    'description' => $segment->description,
                    'visit_type' => $segment->visit_type,
                    'visit_frequency' => $segment->visit_frequency,
                    'visit_threshold' => $segment->visit_threshold,
                    'purchase_type' => $segment->purchase_type,
                    'purchase_frequency' => $segment->purchase_frequency,
                    'purchase_threshold' => $segment->purchase_threshold,
                    'default' => 1,
                    'date' => Carbon::now()->format('Y-m-d'),
                    'tm' => Carbon::now()->toTimeString(),
                    'status' => STATUS,
                    'user_id' => $user->id,
                    'location_id' => $user->location_id,
                    'client_id' => $user->client_id,
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    'control_user_id' => STATUS,
                    'control_id' => STATUS,
                ];
        
                Segment::create($fields);
            }
        }
    }

    public function loadDefaultSegment() {
        $names = [
            'most valuable',
            'least valuable',
            'most frequent',
            'least frequent'
        ];
        $descriptions = [
            'spotlight your top spenders. Send personalized emails and SMS to show appreciation for their valued support.',
            'identify customers who may need more attention. Reach out with friendly messages to re-ignite their interest.',
            'celebrate your frequent visitors. Send enticing emails and SMS with exclusive offers to keep them coming back.',
            're-engage occasional visitors. Send appealing messages to encourage more frequent visits and connection.'
        ];
    
        for ($i = 0; $i < sizeof($names); $i++) {
            $name = $names[$i];
            $description = $descriptions[$i];
            $visitType = null;
            $purchaseType = null;
            $visitFrequency = null;
            $visitThreshold = null;
            $purchaseFrequency = null;
            $purchaseThreshold = null;
    
            // Set visitType, purchaseType, and related thresholds
            if ($name === 'most frequent') {
                $visitType = 'high visit';
                $visitFrequency = 10;
                $visitThreshold = 1000;
            } elseif ($name === 'least frequent') {
                $visitType = 'low visit';
                $visitFrequency = 10;
                $visitThreshold = 1000;
            } elseif ($name === 'most valuable') {
                $purchaseType = 'most purchase amount';
                $purchaseFrequency = 10;
                $purchaseThreshold = 1000;
            } elseif ($name === 'least valuable') {
                $purchaseType = 'least purchase amount';
                $purchaseFrequency = 10;
                $purchaseThreshold = 1000;
            }
    
            $fields = [
                'name' => $name,
                'description' => $description,
                'visit_type' => $visitType,
                'visit_frequency' => $visitFrequency,
                'visit_threshold' => $visitThreshold,
                'purchase_type' => $purchaseType,
                'purchase_frequency' => $purchaseFrequency,
                'purchase_threshold' => $purchaseThreshold,
                'default' => 1,
                'date' => Carbon::now()->format('Y-m-d'),
                'tm' => Carbon::now()->toTimeString(),
                'status' => STATUS,
                'user_id' => 0,
                'location_id' => 0,
                'client_id' => 0,
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                'control_user_id' => STATUS,
                'control_id' => STATUS,
            ];
    
            Segment::create($fields);
        }
    }
    

    public function loadDefaultDevice($user, $request) {
        $isExist = $this->deviceController->getDefaultDeviceLog($user->location_id, $user->client_id);
        $status = $isExist ? ONE_STATUS : STATUS;

        $response = $this->deviceController->isDeviceAddress($user->location_id, $user->client_id, $request);

        // $useRID = Auth::user()->client_id;  
        
        if(!$response || $response == null || empty($response)) {
            $fields = [
                'mac_address' => $request->macAddress1,
                'mac_address_2' => $request->macAddress2,
                'email' => $user->username,
                'timezone' => $request->timezone,
                'user_id' => $user->id,
                'location_id' => $user->location_id,
                'client_id' => $user->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                'status' => $status,
            ];
            
            DeviceLog::create($fields);
            
        }
    }


    public function loadDefaultCustomer($user) {
        $response = $this->customerVendorController->isDefaultCustomer($user->client_id);

        if(!$response) {
            $uuid = Uuid::uuid4()->toString();
            $uuidWithoutDashes = str_replace('-', '', $uuid);
            $fields = [
                'id' => $uuidWithoutDashes,
                'name' => 'walk-in customer',
                'coy_name' => 'walk-in customer',
                'status' => STATUS,
                'cate' => 'lba',
                'year' => Carbon::now()->year,
                'month' => Carbon::now()->month,
                'day' => Carbon::now()->day,
                'user_id' => $user->id,
                'location_id' => $user->location_id,
                'client_id' => $user->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            ];
            
            Customer::create($fields);
        }
    }

    public function loadDefaultVendor($user) {
        $response = $this->customerVendorController->isDefaultVendor($user->client_id);

        if(!$response) {
            $uuid = Uuid::uuid4()->toString();
            $uuidWithoutDashes = str_replace('-', '', $uuid);
            $fields = [
                'id' => $uuidWithoutDashes,
                'name' => 'default vendor',
                'coy_name' => 'default vendor',
                'status' => STATUS,
                'cate' => 'cust',
                'year' => Carbon::now()->year,
                'month' => Carbon::now()->month,
                'day' => Carbon::now()->day,
                'user_id' => $user->id,
                'location_id' => $user->location_id,
                'client_id' => $user->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            ];

            Customer::create($fields);
        }
    }

    public function loadDefaultMeasurementCategory($user) {
        $response = $this->measurementController->getDefaultMeasurementCategory();

        foreach($response as $res){
            if($this->measurementController->isMeasurementCategory($user->client_id, $res->name) < ONE_STATUS){
                $fields = [
                    'name' => $res->name,
                    'default_measurement' => $res->default_measurement,
                    'mc_id' => $res->id,
                    'abr' => $res->abr,
                    'status' => STATUS,
                    'user_id' => $user->id,
                    'location_id' => $user->location_id,
                    'client_id' => $user->client_id,
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s')
                ];
    
                MeasurementCategory::create($fields);
            }
        }
    }

    public function loadDefaultMeasurement($user) {
        $response = $this->measurementController->getDefaultMeasurement();

        foreach($response as $res){
            if($this->measurementController->isMeasurement($user->client_id, $user->location_id, $res->name) < ONE_STATUS){
                $fields = [
                    'name' => $res->name,
                    'default_measurement' => $res->default_measurement,
                    'm_category_id' => $res->m_category_id,
                    'abr' => $res->abr,
                    'status' => STATUS,
                    'user_id' => $user->id,
                    'location_id' => $user->location_id,
                    'client_id' => $user->client_id,
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    'convertion_rate' => $res->convertion_rate,
                    'hierarchy' => $res->hierarchy,
                    'mstatus' => $res->mstatus
                ];
    
                
                Measurement::create($fields);
            }
        }
    }

    public function loadDefaultProductCategory($user) {
        $response = $this->productController->isDefaultProductCategory($user->client_id);

        if(!$response) {
            $uuid = Uuid::uuid4()->toString();
            $uuidWithoutDashes = str_replace('-', '', $uuid);
            $fields = [
                'id' => $uuidWithoutDashes,
                'name' => 'default product category',
                'income_account' => CHART_SALES,
                'status' => STATUS,
                'user_id' => $user->id,
                'location_id' => $user->location_id,
                'client_id' => $user->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s')
            ];
    
            ItemCategory::create($fields);
        }
    }

    public function loadDefaultProductGrouping($user) {
        $response = $this->productController->isProductGrouping($user->client_id);
        if(!$response) {
            $uuid = Uuid::uuid4()->toString();
            $uuidWithoutDashes = str_replace('-', '', $uuid);
            $fields = [
                'id' => $uuidWithoutDashes,
                'name' => 'item grouping',
                'income_account' => CHART_SALES,
                'status' => STATUS,
                'user_id' => $user->id,
                'location_id' => $user->location_id,
                'client_id' => $user->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                'item_class_id' => FOUR_STATUS
            ];

            ItemCategory::create($fields);
        }
    }

    public function loadDefaultLoyaltyManagement($user) {
        $response = $this->loyaltyManagementController->getDefaultLoyaltyManagement();

        foreach($response as $res) {
            if($this->loyaltyManagementController->isLoyaltyManagement($user->client_id, $user->location_id, $res->name) < ONE_STATUS){
                $fields = [
                    'id' => Uuid::uuid4()->toString(),
                    'user_id' => $user->id,
                    'location_id' => $user->location_id,
                    'client_id' => $user->client_id,
                    'tm' => Carbon::now()->toTimeString(),
                    'date' => Carbon::now()->format('Y-m-d'),
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    'status' => STATUS,
                    'name' => $res->name,
                    'loyalty_id' => $res->loyalty_id,
                ];
            
                LoyaltyManagement::create($fields);
            }
        }
    }

    public function loadDefaultStore($user) {
        $res = $this->storeController->isPrimaryStore($user->client_id, $user->location_id);

        if(!$res) {
            $uuid = Uuid::uuid4()->toString();
            $uuidWithoutDashes = str_replace('-', '', $uuid);
            $fields = [
                'id' => $uuidWithoutDashes,
                'user_id' => $user->id,
                'location_id' => $user->location_id,
                'client_id' => $user->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                'status' => STATUS,
                'name' => 'front outlet',
                'type' => 'sales point',
                'primary_status' => ONE_STATUS,
            ];
        
            Store::create($fields);
        }
    }

    public function loadDefaultCurrency($user) {

        $res = $this->currencyController->isPrimaryCurrency($user->client_id, $user->location_id);

        if(!$res) {
            $uuid = Uuid::uuid4()->toString();
            $uuidWithoutDashes = str_replace('-', '', $uuid);
            
            $fields = [
                'id' => $uuidWithoutDashes,
                'user_id' => $user->id,
                'location_id' => $user->location_id,
                'client_id' => $user->client_id,
                'tm' => Carbon::now()->toTimeString(),
                'date' => Carbon::now()->format('Y-m-d'),
                'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                'status' => STATUS,
                'currency_cate_id' => ONE_STATUS,
                'primary_currency' => ONE_STATUS,
                'unit' => ONE_STATUS
            ];
        
            Currency::create($fields);
        }
    }

    private function loadDefaultChartCate($user) {
        $response = $this->accountCategoryController->getDefaultAccountCategory();
        
        foreach($response as $res) {
            if($this->accountCategoryController->isChartCate($res->name, $user->location_id, $user->client_id) < ONE_STATUS){
                $fields = [
                    'id' => Uuid::uuid4()->toString(),
                    'user_id' => $user->id,
                    'name' => $res->name,
                    'location_id' => $user->location_id,
                    'client_id' => $user->client_id,
                    'tm' => Carbon::now()->toTimeString(),
                    'date' => Carbon::now()->format('Y-m-d'),
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    'status' => STATUS,
                    'cate_id' => $res->cate_id,
                    'acct_type_id' => $res->acct_type_id,
                ];
            
                ChartCate::create($fields);
            }
        }
    }

    private function loadDefaultChartOfAccount($user) {
        $response = $this->chartOfAccountController->getDefaultAccount();

        foreach($response as $res) {
            if($this->chartOfAccountController->isChartId($res->code, $user->location_id, $user->client_id) < ONE_STATUS){
                $fields = [
                    'id' => Uuid::uuid4()->toString(),
                    'user_id' => $user->id,
                    'name' => $res->name,
                    'location_id' => $user->location_id,
                    'client_id' => $user->client_id,
                    'tm' => Carbon::now()->toTimeString(),
                    'date' => Carbon::now()->format('Y-m-d'),
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    'status' => STATUS,
                    'sort_code' => $res->sort_code,
                    'effect' => $res->effect,
                    'acct_cate' => $res->acct_cate,
                    'code' => $res->code,
                ];
            
                ChartOfAccount::create($fields);
            }
        }
    }

    private function loadDefaultSyncSetting($user) {
        $response = $this->syncController->getDefaultSyncSetting();

        foreach($response as $res) {
            if($this->syncController->isSyncSetting($res->name, $user->location_id, $user->client_id) < ONE_STATUS){
                $fields = [
                    'user_id' => $user->id,
                    'name' => $res->name,
                    'location_id' => $user->location_id,
                    'client_id' => $user->client_id,
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    'status' => STATUS,
                    'opt_status' => $res->opt_status
                ];
            
                SynSetting::create($fields);
            }
        }
    }

}
