<?php

namespace App\Http\Controllers\General;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use Illuminate\Support\Facades\Auth;
use App\Services\ExpoNotificationService;
use App\Models\Account\User;

class NotificationController extends Controller
{

    protected $expoNotificationService;

    public function __construct(ExpoNotificationService $expoNotificationService)
    {
        $this->expoNotificationService = $expoNotificationService;
    }

    public function sendExpoNotification(Request $request)
    {
        $request->validate([
            'to' => 'required|string',
            'title' => 'required|string',
            'body' => 'required|string',
        ]);

        $to = $request->input('to');
        $title = $request->input('title');
        $body = $request->input('body');
        $data = ['type' => 'broadcast'];
        $sound = $request->input('sound', 'default'); // Default to 'default' sound


        $result = $this->expoNotificationService->sendNotification($to, $title, $body, $data, $sound);

        return response()->json($result);
    }


    public function broadcastExpoNotification(Request $request)
    {
        $request->validate([
            'title' => 'required|string',
            'body' => 'required|string'
        ]);

        $title = $request->input('title');
        $body = $request->input('body');
        $data = ['type' => 'broadcast'];

        // Retrieve all users with a non-null expo_token
        $users = User::whereNotNull('expo_token')->get();

        // Iterate over each user and send a personalized notification
        foreach ($users as $user) {
            $to = $user->expo_token;
            $personalizedBody = "Hi {$user->name}, " . $body;
            $this->expoNotificationService->sendNotification($to, $title, $personalizedBody, $data);
        }

        return response()->json(['message' => 'Notification broadcasted to all users'], 200);
    }



    // public function broadcastExpoNotification(Request $request)
    // {
    //     $request->validate([
    //         'title' => 'required|string',
    //         'body' => 'required|string'
    //     ]);

    //     $title = $request->input('title');
    //     $body = $request->input('body');
    //     $data = ['type' => 'broadcast'];

    //     // Retrieve all users with a non-null expo_token
    //     $users = User::whereNotNull('expo_token')->get();

    //     // Iterate over each user and send a notification
    //     foreach ($users as $user) {
    //         $to = $user->expo_token;
    //         $this->expoNotificationService->sendNotification($to, $title, $body, $data);
    //     }

    //     return response()->json(['message' => 'Notification broadcasted to all users'], 200);
    // }


    public function index()
{
    $user = Auth::user();

    // Retrieve the last 10 notifications for the user, ordered by the latest first
    $notifications = $user->notifications()->orderBy('created_at', 'desc')->take(10)->get();

    // Return notifications as JSON
    return response()->json($notifications);
}


    public function markAsRead($notificationId)
    {
        $user = Auth::user();

        // Find the notification by its id and mark it as read
        $notification = $user->notifications()->where('id', $notificationId)->first();
        if ($notification) {
            $notification->markAsRead();
            return response()->json(['message' => 'Notification marked as read.']);
        }

        return response()->json(['message' => 'Notification not found.'], 404);
    }

    public function markAllAsRead()
    {
        $user = Auth::user();

        // Mark all notifications as read
        $user->unreadNotifications->markAsRead();

        return response()->json(['message' => 'All notifications marked as read.']);
    }
}
