<?php

namespace App\Http\Controllers\General;

use App\Http\Controllers\Controller;
use App\Models\Account\SmsTransaction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Models\Account\Wallet;
use Unicodeveloper\Paystack\Facades\Paystack;
use App\Models\Account\DebitCard;
use App\Models\Account\TransactionLog;
use Illuminate\Support\Str;
use App\Models\Account\CouponCode;
use App\Models\Account\CouponUser;
use Validator;
use App\Models\Retails\Location;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\General\BillingController;
use App\Services\TermiiSMSService;

class SmsController extends BillingController
{

    protected $termiiSMSService;

    public function __construct(TermiiSMSService $termiiSMSService)
    {
        $this->termiiSMSService = $termiiSMSService;
    }

    public function sendSMSMessage(Request $request)
    {
        // Validate the request

        $validator = Validator::make($request->all(), [
            'to' => 'required|numeric',
            'message' => 'required|string|max:160',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation failed', 'errors' => $validator->errors()], 400);
        }

        // Use the TermiiSMSService to send SMS with standardized phone number
        $response = $this->termiiSMSService->sendSMS($request->to, $request->message);

        // Return the response to the user
        if ($response['success']) {
            return response()->json([
                'message' => $response['message'],
                'data' => $response['response']
            ], 200);
        } else {
            return response()->json([
                'message' => $response['message'],
                'status_code' => $response['status_code'],
                'data' => $response['response'] ?? null
            ], $response['status_code'] ?? 500);
        }
    }

    public function sendSMS($phone, $message)
    {
        // Validate the inputs
        $validator = Validator::make(compact('phone', 'message'), [
            'phone' => 'required|numeric',
            'message' => 'required|string|max:160',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->errors()->first(),
                'error' => true
            ], 400);
        }

        // Use the TermiiSMSService to send SMS with standardized phone number
        $response = $this->termiiSMSService->sendSMS($phone, $message);

        // Return the response to the user
        if ($response['success']) {
            return response()->json([
                'message' => $response['message'],
                'data' => $response['response']
            ], 200);
        } else {
            return response()->json([
                'message' => $response['message'],
                'status_code' => $response['status_code'],
                'data' => $response['response'] ?? null
            ], $response['status_code'] ?? 500);
        }
    }

    public function buySms(Request $request)
    {
  
        $amount = $request->amount;
        $unit = $amount / SMS_AMOUNT_PER_UNIT;
        $ref = date('YmdHi').rand(10000, 99999).Auth::user()->id.Auth::user()->location_id . Auth::user()->client_id;
      
        $tokenUid = Str::uuid()->toString();

        if($this->getWalletBalance(Auth::user()->client_id) < $amount){
            return response()->json(['message' => 'Insufficient Wallet Balance', 'error' => true], 400);
        }


        SmsTransaction::create([
            'client_id' => Auth::user()->client_id,
            'location_id' => Auth::user()->location_id,
            'amount' => $amount,
            'unit' => $unit,
            'reference' => $ref,
            'effect' => EFFECT_CREDIT,
            'date' => now()->toDateString(),
            'tm' => now()->toTimeString(),
            'status' => STATUS
        ]);
    
    
        Wallet::create([
                'client_id' => Auth::user()->client_id,
                'location_id' => Auth::user()->location_id,
                'user_id' => Auth::user()->id,
                'amount' => $amount,
                'biller_name' => 'SMS Purchased',
                'biller_description' => 'SMS purchased through wallet',
                'biller_reference' => $ref,
                'token' => $tokenUid,
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => ONE_STATUS,
                'effect' => EFFECT_DEBIT
            ]);

           

    
            TransactionLog::create([
                'client_id' => Auth::user()->client_id,
                'user_id' => Auth::user()->id,
                'biller_name' => 'SMS Purchased',
                'biller_description' => 'SMS purchased through wallet',
                'payment_type' => 'SMS',
                'payment_source' => 'wallet',
                'location_id' => Auth::user()->location_id,
                'amount' => $amount,
                'payment_status' => 'success',
                'biller_reference' => $ref,
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => ONE_STATUS
            ]);
    
            return response()->json(['success' => true, 'message' => 'SMS purchased successfully']);
        } 
    
    
        public function SmsHistory(Request $request)
{
    $clientId = Auth::user()->client_id;

    // Summary Query
    $summaryQuery = "
        SELECT
            SUM(CASE WHEN effect = 'cr' THEN unit ELSE 0 END) as total_sms,
            SUM(CASE WHEN effect = 'dr' THEN unit ELSE 0 END) as used_sms,
            (SUM(CASE WHEN effect = 'cr' THEN unit ELSE 0 END) - SUM(CASE WHEN effect = 'dr' THEN unit ELSE 0 END)) as available_sms
        FROM
            sms_transaction
        WHERE
            client_id = :client_id
            AND effect IN ('dr', 'cr')
    ";

    $summaryData = DB::select($summaryQuery, ['client_id' => $clientId]);

    if (empty($summaryData)) {
        return response()->json(['error' => 'No data found'], 404);
    }

    $summary = [
        'total_sms' => $summaryData[0]->total_sms,
        'used_sms' => $summaryData[0]->used_sms,
        'available_sms' => $summaryData[0]->available_sms
    ];

    // History Query
    $historyQuery = "
        SELECT
            id,
            unit as sms_unit,
            amount,
            reference,
            DATE_FORMAT(date, '%b. %D, %l:%i%p, %Y') as date
        FROM
            sms_transaction
        WHERE
            client_id = :client_id
            AND effect IN ('dr', 'cr')
    ";

    $historyData = DB::select($historyQuery, ['client_id' => $clientId]);

    $history = array_map(function($item) {
        return [
            'sms_unit' => $item->sms_unit,
            'amount' => $item->amount,
            'reference' => $item->reference,
            'date' => $item->date,
        ];
    }, $historyData);

    // Format the data as needed
    $formattedData = [
        'total_sms' => $summary['total_sms'],
        'used_sms' => $summary['used_sms'],
        'available_sms' => $summary['available_sms'],
        'sms_history' => $history
    ];

    return response()->json($formattedData);
}

}
