<?php

namespace App\Http\Controllers\General;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Folio\Location\LocationController;
use App\Http\Controllers\Settings\License\LicenseController;
use App\Models\Account\CouponCode;
use App\Models\Account\CouponUser;
use App\Models\Account\SubscriptionPlanDetails;
use App\Models\Account\Wallet;
use Illuminate\Support\Facades\Auth;
use DB;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use App\Http\Controllers\General\BillingController;
use Unicodeveloper\Paystack\Facades\Paystack;
use App\Models\Account\Subsciption;
use Validator;
use App\Models\Account\TransactionLog;
use App\Models\Retails\Location;
use App\Models\Account\Licensee;
use Carbon\Carbon;
class SubscriptionController extends BillingController
{
    protected $locationController;
    protected $licenseController;

    public function __construct(LocationController $locationController, LicenseController $licenseController)
    {
        $this->locationController = $locationController;
        $this->licenseController = $licenseController;
    }

    public function getSubscriptionPlans()
    {
        $sql = "
        WITH plans AS (
            SELECT 
                id AS id,
                core_name AS name,
                amount_month AS amountMonth,
                amount_year AS amountYear,
                year_discount AS discount
            FROM `subscription_plans`
            WHERE status = 0
        ),
        features AS (
            SELECT 
                head_features,
                features,
                description,
                SUM(CASE WHEN plan_id = 1 THEN 1 ELSE 0 END) AS basic,
                SUM(CASE WHEN plan_id = 1 THEN value ELSE 0 END) AS basic_value,
                SUM(CASE WHEN plan_id = 2 THEN 1 ELSE 0 END) AS plus,
                SUM(CASE WHEN plan_id = 2 THEN value ELSE 0 END) AS plus_value,
                SUM(CASE WHEN plan_id = 3 THEN 1 ELSE 0 END) AS enterprise,
                SUM(CASE WHEN plan_id = 3 THEN value ELSE 0 END) AS enterprise_value
            FROM `subscription_plan_details` WHERE status = 0 AND is_addon != 1 AND head_features != ''
            GROUP BY head_features, features, description
            ORDER BY CASE 
            WHEN head_features = 'Essentials' THEN 1
            ELSE 2
            END,features ASC
        ),
        head_features AS (
            SELECT 
                head_features,
                JSON_ARRAYAGG(JSON_OBJECT(
                    'features', features,
                    'description', description,
                    'basic', basic,
                    'basic_value', basic_value,
                    'plus', plus,
                    'plus_value', plus_value,
                    'enterprise', enterprise,
                    'enterprise_value', enterprise_value
                )) AS features
            FROM features
            GROUP BY head_features
            ORDER BY CASE 
            WHEN head_features = 'Essentials' THEN 1
            ELSE 2 END
        )
        SELECT 
            p.headers,
            h.head_features
        FROM 
            (SELECT 
                JSON_ARRAYAGG(JSON_OBJECT(
                    'id', id,
                    'name', name,
                    'amountMonth', amountMonth,
                    'amountYear', amountYear,
                    'discount', discount
                )) AS headers
            FROM plans) p
        CROSS JOIN
            (SELECT 
                JSON_ARRAYAGG(JSON_OBJECT(
                    'name', head_features,
                    'features', features
                )) AS head_features
            FROM head_features) h;
        ;
        ";

        $results = DB::select(DB::raw($sql));

        // Assuming the query returns one row with headers and head_features as JSON
        if (count($results) > 0) {
            return response()->json([
                'headers' => json_decode($results[0]->headers),
                'head_features' => json_decode($results[0]->head_features),
            ]);
        }

        return response()->json(['message' => 'No subscription details found'], 404);
    }



    public function subscriptionSummary(Request $request)
{
    $client_id = Auth::user()->client_id;
    $planId = $request->plan_id;
    $planDuration = $request->plan_duration;
    $subscriptionMonth = $planDuration === '1' ? 'Monthly' : 'Yearly';
    $walletBalance = $this->getWalletBalance($client_id);

    // Directly incorporate dynamic values into the query as they are not user inputs
    // Ensure to escape any potential user inputs properly to avoid SQL injection
    $query = "
        WITH payment_card AS (
            SELECT 
                * 
                FROM debit_card 
                WHERE client_id = '$client_id'
        ),
        
        plan_details AS (
            SELECT 
                id AS plan_id,
                core_name AS plan_name,
                amount_month AS amountMonth,
                amount_year AS amountYear,
                '$subscriptionMonth' AS subscriptionMonth,
                $walletBalance AS walletBalance,
                year_discount AS discount
            FROM subscription_plans
            WHERE id = ?
        ),
        
        addons AS (
            SELECT 
                features AS name,
                description,
                0 AS selected, 
                menu_icon AS image,
                JSON_ARRAYAGG(JSON_OBJECT(
                    'id', id,
                    'name', features,
                    'qty', value,
                    'amount', amount
                )) AS options
            FROM subscription_plan_details
            WHERE is_addon = 1 AND plan_id = ? AND duration = ?
            GROUP BY features, description, menu_icon
        ), 
        
        onboarding AS (
            SELECT 
                id AS id,
                features AS name,
                amount AS amount,
                description,
                0 AS selected, 
                menu_icon AS image
            FROM subscription_plan_details
            WHERE is_addon = 1 AND addon_type = 'onboarding'
            GROUP BY id, features, description, menu_icon
        ), 
        
        plande AS (
            SELECT JSON_ARRAYAGG(JSON_OBJECT(
                'plan_id', p.plan_id,
                'plan_name', p.plan_name,
                'amountMonth', p.amountMonth,
                'amountYear', p.amountYear,
                'subscriptionMonth', p.subscriptionMonth,
                'walletBalance', p.walletBalance,
                'discount', p.discount
            )) AS planDetails 
            FROM plan_details p
        ),
        
        ads AS (
            SELECT JSON_ARRAYAGG(JSON_OBJECT(
                'name', a.name,
                'description', a.description,
                'selected', a.selected,
                'image', a.image,
                'options', a.options
            )) AS addons 
            FROM addons a
        ),
        
        pcards AS (
            SELECT JSON_ARRAYAGG(JSON_OBJECT(
                'id', pc.id,
                'authorization_code', pc.authorization_code,
                'bin', pc.bin,
                'last_four', pc.last_four,
                'exp_month', pc.exp_month,
                'exp_year', pc.exp_year,
                'card_type', pc.card_type
            )) AS pCardDetails 
            FROM payment_card pc
        ),
        
        onb AS (
            SELECT JSON_ARRAYAGG(JSON_OBJECT(
                'id', o.id,
                'name', o.name,
                'amount', o.amount,
                'description', o.description,
                'selected', o.selected,
                'image', o.image
            )) AS onboarding
            FROM onboarding o
        )
        
        SELECT * FROM plande, ads, pcards, onb;
    ";

    // Execute the query with direct bindings for the planId and planDuration
    $result = DB::select($query, [$planId, $planId, $planDuration]);

    // Decode JSON data in the result set
    foreach ($result as $row) {
        if (property_exists($row, 'planDetails')) {
            $row->planDetails = json_decode($row->planDetails, true);
        }
        if (property_exists($row, 'addons')) {
            $row->addons = json_decode($row->addons, true);
        }
        if (property_exists($row, 'onboarding')) {
            $row->onboarding = json_decode($row->onboarding, true);
        }
        if (property_exists($row, 'pCardDetails')) {
            $row->pCardDetails = json_decode($row->pCardDetails, true);
        }
    }

    return response()->json(['success' => true, 'data' => $result], 200);
}


public function calculatePaystackCharges($amount, $currency)
{
    $fee = 0;
    $vat = 0;

    if ($currency === 'NGN') {
        // Local transaction fee calculation
        if ($amount <= 2500) {
            $fee = 0.015 * $amount;
            $fee = ($fee > 100) ? 100 : $fee; // Cap the fee at NGN 100 for transactions below NGN 2500
        } else {
            $fee = (0.015 * $amount) + 100; // Add NGN 100 flat fee for transactions above NGN 2500
        }

        // Add VAT of 7.5% to the fee
        $vat = 0.075 * $fee;
    } else {
        // International transaction fee calculation
        $fee = (0.039 * $amount) + 0.06; // 3.9% + NGN 100 for international transactions
    }

    // Total charges
    $totalCharges = $fee + $vat;

    // Log the calculated charges
    Log::channel('slack')->info('Paystack Charges', [
        'currency' => $currency,
        'transaction_amount' => $amount,
        'fee' => $fee,
        'vat' => $vat,
        'total_charges' => $totalCharges
    ]);

    return $totalCharges;
}

public function addonSummary(Request $request)
{
    $client_id = Auth::user()->client_id;
    $planId = $request->plan_id;
    $planDuration = $request->plan_duration;

    $query = "
        WITH addons AS (
            SELECT 
                features AS name,
                description,
                0 AS selected, 
                menu_icon AS image,
                JSON_ARRAYAGG(JSON_OBJECT(
                    'id', id,
                    'name', features,
                    'qty', value,
                    'amount', amount
                )) AS options
            FROM subscription_plan_details
            WHERE is_addon = 1 AND plan_id = ? AND duration = ?
            GROUP BY features, description, menu_icon
        )
        
        SELECT * FROM addons;
    ";

    // Execute the query with direct bindings for the planId and planDuration
    $result = DB::select($query, [$planId, $planDuration]);

    // Decode JSON data for the options field
    foreach ($result as $row) {
        if (property_exists($row, 'options')) {
            $row->options = json_decode($row->options, true);
        }
    }

    // Return the result
    return response()->json(['success' => true, 'addons' => $result ?? []], 200);
}



  
    // public function subscriptionSummary(Request $request)
    // {
    //     $client_id = Auth::user()->client_id;
    //     $planId = $request->plan_id;
    //     $planDuration = $request->plan_duration;
    //     $subscriptionMonth = $planDuration === '1' ? 'Monthly' : 'Yearly';
    //     $walletBalance = $this->getWalletBalance(Auth::user()->client_id);
    
    //     // Directly incorporate dynamic values into the query as they are not user inputs
    //     // Ensure to escape any potential user inputs properly to avoid SQL injection
    //     $query = "
    //         WITH  payment_card AS (
    //             SELECT 
    //                 * 
    //                 FROM debit_card 
    //                 WHERE client_id = '$client_id'
    //         ),
            
    //         plan_details AS (
    //             SELECT 
    //                 id AS plan_id,
    //                 core_name AS plan_name,
    //                 amount_month AS amountMonth,
    //                 amount_year AS amountYear,
    //                 '$subscriptionMonth' AS subscriptionMonth,
    //                 $walletBalance AS walletBalance,
    //                 year_discount as discount
    //             FROM subscription_plans
    //             WHERE id = ?
    //         ),
    //         addons AS (
    //             SELECT 
    //                 features AS name,
    //                 description,
    //                 0 AS selected, 
    //                 menu_icon AS image,
    //                 JSON_ARRAYAGG(JSON_OBJECT(
    //                     'id', id,
    //                     'name', features,
    //                     'qty', value,
    //                     'amount', amount
    //                 )) AS options
    //             FROM subscription_plan_details
    //             WHERE is_addon = 1 AND plan_id = ? AND duration = ?
    //             GROUP BY features, description, menu_icon
    //         ), 
    //         onboarding AS (
    //             SELECT 
    //                 id AS id,
    //                 features AS name,
    //                 description,
    //                 0 AS selected, 
    //                 menu_icon AS image
    //             FROM subscription_plan_details
    //             WHERE is_addon = 1 AND addon_type = 'onboarding'
    //             GROUP BY features, description, menu_icon
    //         ), 
    //         plande AS (
    //             SELECT JSON_ARRAYAGG(JSON_OBJECT(
    //                 'plan_id', p.plan_id,
    //                 'plan_name', p.plan_name,
    //                 'amountMonth', p.amountMonth,
    //                 'amountYear', p.amountYear,
    //                 'subscriptionMonth', p.subscriptionMonth,
    //                 'walletBalance', p.walletBalance,
    //                 'discount', p.discount
    //             )) AS planDetails 
    //             FROM plan_details p
    //         ),
    //         ads AS (
    //             SELECT JSON_ARRAYAGG(JSON_OBJECT(
    //                 'name', a.name,
    //                 'description', a.description,
    //                 'selected', a.selected,
    //                 'image', a.image,
    //                 'options', a.options
    //             )) AS addons 
    //             FROM addons a
    //             ),
    //             pcards AS (
    //                 SELECT JSON_ARRAYAGG(JSON_OBJECT(
    //                     'id', pc.id,
    //                     'authorization_code', pc.authorization_code,
    //                     'bin', pc.bin,
    //                     'last_four', pc.last_four,
    //                     'exp_month', pc.exp_month,
    //                     'exp_year', pc.exp_year,
    //                     'card_type', pc.card_type)
    //                     ) AS pCardDetails 
    //                 FROM payment_card pc
    //             )
    //         SELECT * FROM plande, ads, pcards;
    //     ";
    
    //     // Execute the query with direct bindings for the planId and planDuration
    //     $result = DB::select($query, [$planId, $planId, $planDuration]);
    
    //     // Decode JSON data in the result set
    //     foreach ($result as $row) {
    //         if (property_exists($row, 'planDetails')) {
    //             $row->planDetails = json_decode($row->planDetails, true);
    //         }
    //         if (property_exists($row, 'addons')) {
    //             $row->addons = json_decode($row->addons, true);
    //         }
    //         if (property_exists($row, 'onboarding')) {
    //             $row->onboarding = json_decode($row->onboarding, true);
    //         }
    //         if (property_exists($row, 'pCardDetails')) {
    //             $row->pCardDetails = json_decode($row->pCardDetails, true);
    //         }
    //     }
    
    //     return response()->json(['success' => true, 'data' => $result], 200);
    // }
    
    
    
    
    public function validatePaymentWithCards(Request $request)
    {
        $rules = [
            'plan_id' => 'required|integer',
            'plan_amount' => 'required|numeric',
            'total_amount' => 'required|numeric',
            'discount_percentage' => 'nullable|numeric',
            'amount_paid' => 'required|numeric',
            'coupon' => 'nullable|string|max:255',
            'payment_method' => 'required|string|max:255',
            'duration' => 'required|integer',
            'location_id' => 'required|integer',
            'additional_products' => 'nullable|integer',
            'additional_users' => 'nullable|integer',
            'additional_warehouse' => 'nullable|integer',
            'card_type' => 'required|string',
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 400);
        }

        // Variables specific to wallet payment method
        $coupon = $request->coupon;

        // Check for active subscription
        if ($this->locationController->isExpired(Auth::user()->client_id, Auth::user()->location_id) == 'active') {
            return response()->json(['message' => 'You have an active subscription', 'error' => true], 400);
        }

        // Coupon validation
        if (!empty($coupon)) {
                $checkCoupon = CouponCode::where(['code' => $coupon , 'status' => 0])->first();
 
                if (!$checkCoupon) {
                    return response()->json(['message' => 'The coupon is not available.', 'error' => true], 400);
                }
        
                $checkCouponUsage = CouponUser::where(['code' => $coupon , 'status' => 0, 'client_id' => Auth::user()->id])->first();
        
                if ($checkCouponUsage) {
                    return response()->json(['message' => 'The coupon has already been used by you.', 'error' => true], 400);
                }     
        }

        // Return a success response
        return response()->json(['message' => 'Proceed to payment', 'error' => false], 200);
    }



    public function subscribeWithWallet(Request $request)
    {
        $rules = [
            'plan_id' => 'required|integer',
            'plan_amount' => 'required|numeric',
            'total_amount' => 'required|numeric',
            'discount_percentage' => 'nullable|numeric',
            'amount_paid' => 'required|numeric',
            'coupon' => 'nullable|string|max:255',
            'payment_method' => 'required|string|max:255',
            'duration' => 'required|integer',
            'additional_products' => 'nullable|integer',
            'additional_users' => 'nullable|integer',
            'additional_warehouse' => 'nullable|integer',
            'onboarding' => 'nullable|integer'
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        // Variables specific to wallet payment method
        $amountPaid = $request->amount_paid;
        $coupon = $request->coupon;
        $locationId = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        $planId = $request->plan_id;
        $duration = $request->duration;

        // Unique token for this subscription
        $tokenUid = Str::uuid()->toString();

        $timestamp = Carbon::now()->format('YmdHis');  
        $ref = Auth::user()->client_id . $timestamp . Auth::user()->id . rand(10000, 99999);
        

        // Check for active subscription
        if ($this->locationController->isExpired(Auth::user()->client_id, Auth::user()->location_id) == 'active') {
            return response()->json(['message' => 'You have an active subscription', 'error' => true], 400);
        }

        // Check wallet balance
        if ($this->getWalletBalance(Auth::user()->client_id) < $amountPaid) {
            return response()->json(['message' => 'Insufficient Wallet Balance', 'error' => true], 400);
        }

        // Coupon validation
        if (!empty($coupon)) {
                $checkCoupon = CouponCode::where(['code' => $coupon , 'status' => 0])->first();
 
                if (!$checkCoupon) {
                    return response()->json(['message' => 'The coupon is not available.', 'error' => true], 400);
                }
        
                $checkCouponUsage = CouponUser::where(['code' => $coupon , 'status' => 0, 'client_id' => Auth::user()->id])->first();
        
                if ($checkCouponUsage) {
                    return response()->json(['message' => 'The coupon has already been used by you.', 'error' => true], 400);
                }     
        }

        // Deduct from wallet
        Wallet::create([
            'client_id' => Auth::user()->client_id,
            'location_id' =>  $locationId,
            'user_id' => Auth::user()->id,
            'amount' =>  $amountPaid,
            'biller_name' => 'plan subscription',
            'biller_description' => 'Plan subscription through wallet',
            'biller_reference' => '',
            'token' => $tokenUid,
            'date' => now()->toDateString(),
            'tm' => now()->toTimeString(),
            'status' => STATUS,
            'effect' => EFFECT_DEBIT
        ]);

      

        TransactionLog::create([
            'client_id' => Auth::user()->client_id,
            'user_id' => Auth::user()->id,
            'biller_name' => 'plan subscription',
            'biller_id' => $request->plan_id,
            'biller_reference' => $ref,
            'biller_description' => 'Plan subscription through wallet',
            'payment_type' => 'wallet',
            'card_signature' => '',
            'payment_source' => '',
            'token' => $tokenUid,
            'location_id' =>  $locationId, 
            'amount' => $request->amount_paid,
            'plan_amount' => $request->plan_amount,
            'total_amount' => $request->total_amount,
            'discount_percentage' => $request->discount_percentage,
            'additional_products' => $request->additional_products,
            'additional_users' => $request->additional_users,
            'additional_warehouse'  => $request->additional_warehouse,
            'onboarding'  => $request->onboarding,
            'payment_status' => 'success',
            'date' => now()->toDateString(),
            'tm' => now()->toTimeString(),
            'status' => STATUS
        ]);

        Location::where('id',  $locationId)->update([
            'status' => 0,
            'start_date' => now()->toDateString(), 
            'end_date' =>  $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
            'biller_reference' => $ref,
            'payment_confirmation' => STATUS,
            'allowed_users' => $this->getPlanDetailValue($planId, 'allowed_users', $request->additional_users),
            'allowed_items' => $this->getPlanDetailValue($planId, 'allowed_items', $request->additional_products),
            'allowed_store' => $this->getPlanDetailValue($planId, 'allowed_store', $request->additional_warehouse),
            'app_plan_id' => $planId
        ]);

        if (!empty($coupon)) {
            CouponUser::create([
                'client_id' => Auth::user()->client_id,
                'location_id' =>  $locationId,
                'user_id' => Auth::user()->id,
                'code' =>  $coupon,
                'percentage' => $request->discount_percentage,
                'amount_charged' => $request->total_amount,
                'amount_paid' => $request->amount_paid,
                'token' => $tokenUid,
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => STATUS,
            ]);
         }

     //to agent portal
          $arrayData = [
            'user_id' => Auth::user()->id,
            'page' => 'subscribe',
            'client_id' => Auth::user()->client_id,
            'location_id' => $locationId,
            'proratedAmount' => $request->total_amount,
            'nextChargeDate' => $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
            'wallet_balance' => $this->getWalletBalance(Auth::user()->client_id),
            'no_of_products' => $this->getPlanDetailValue($planId, 'allowed_items', $request->additional_products),
            'no_of_users' =>  $this->getPlanDetailValue($planId, 'allowed_users', $request->additional_users),
            'payment_method' => 'wallet',
            'duration' => '',
            'email' => '',
            'branch_id' => '',
            'plan' => $planId,
    ];
    $postApi = $this->postData(
        $arrayData,
        'https://www.klakpad.org/agents_api/l/getRegistration.php'
    );

        // Return a success response
        return response()->json(['message' => 'Subscription created successfully', 'error' => false], 200);
    }


   


    public function postData($arrayData = [], $url)
    {
        $headers = ['Content-Type: application/json'];
        $payload = json_encode($arrayData);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        //curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        //curl_getinfo($ch, CURLINFO_HTTP_CODE );

        $response_string = curl_exec($ch);
        curl_close($ch);

        echo $response_string;
    }

    public function subscribeWithCard(Request $request)
    {
        $rules = [
            'plan_id' => 'required|integer',
            'plan_amount' => 'required|numeric',
            'total_amount' => 'required|numeric',
            'discount_percentage' => 'nullable|numeric',
            'amount_paid' => 'required|numeric',
            'coupon' => 'nullable|string|max:255',
            'payment_method' => 'required|string|max:255',
            'duration' => 'required|integer',
            'location_id' => 'required|integer',
            'additional_products' => 'nullable|integer',
            'additional_users' => 'nullable|integer',
            'additional_warehouse' => 'nullable|integer',
            'card_type' => 'nullable|string',
            'save_card' => 'nullable',
            'referenc_id' => 'nullable',
            'email' => 'required',
            'onboarding' => 'nullable|integer'
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 422);
        } 

        // Variables specific to wallet payment method
        $amountPaid = $request->amount_paid;
        $coupon = $request->coupon;
        $locationId = $request->location_id;
        $planId = $request->plan_id;
        $duration = $request->duration;
        $email = $request->email;
        $saveCard = $request->save_card;
        $ref = $request->reference_id;

        // Unique token for this subscription
        $tokenUid = Str::uuid()->toString();


        $card_type = $request->card_type; // 'new_card', 'saved_card'
        if ($card_type == 'saved_card') {
            $auth_code = $request->auth_code;
            $cardData = $this->chargeFromCard($auth_code, $email, $amountPaid);
            $ref = $cardData['data']['reference'];
        }
    
        $verify = $this->verifyPaystackTransaction($ref);
        $paymentStatus = $verify['data']['status'];

        if ($paymentStatus == 'success') {
            if($saveCard == ONE_STATUS){
              $this->saveCard($saveCard, $verify);
            }
             
        $signature = $verify['data']['authorization']['signature'];
    
                  
        TransactionLog::create([
            'client_id' => Auth::user()->client_id,
            'user_id' => Auth::user()->id,
            'biller_name' => 'plan subscription',
            'biller_id' => $request->plan_id,
            'biller_description' => 'Plan subscription through CARD',
            'biller_reference' => $ref,
            'payment_type' => 'card',
            'card_signature' => $signature,
            'payment_source' => 'paystack',
            'token' => $tokenUid,
            'location_id' =>  $locationId, 
            'amount' => $request->amount_paid,
            'plan_amount' => $request->plan_amount,
            'total_amount' => $request->total_amount,
            'discount_percentage' => $request->discount_percentage,
            'additional_products' => $request->additional_products,
            'additional_users' => $request->additional_users,
            'additional_warehouse'  => $request->additional_warehouse,
            'onboarding'  => $request->onboarding,
            'payment_status' => $paymentStatus,
            'date' => now()->toDateString(),
            'tm' => now()->toTimeString(),
            'status' => ONE_STATUS
        ]);

        Location::where('id',  $locationId)->update([
            'status' => 0,
            'start_date' => now()->toDateString(), 
            'end_date' =>  $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
            'payment_confirmation' => ONE_STATUS,
            'biller_reference' => $ref,
            'allowed_users' => $this->getPlanDetailValue($planId, 'allowed_users', $request->additional_users),
            'allowed_items' => $this->getPlanDetailValue($planId, 'allowed_items', $request->additional_products),
            'allowed_store' => $this->getPlanDetailValue($planId, 'allowed_store', $request->additional_warehouse),
            'app_plan_id' => $planId
        ]);


        if (!empty($coupon)) {
            CouponUser::create([
                'client_id' => Auth::user()->client_id,
                'location_id' =>  $locationId,
                'user_id' => Auth::user()->id,
                'code' =>  $coupon,
                'percentage' => $request->discount_percentage,
                'amount_charged' => $request->total_amount,
                'amount_paid' => $request->amount_paid,
                'token' => $tokenUid,
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => STATUS,
            ]);
         }


          //to agent portal
          $arrayData = [
            'user_id' => Auth::user()->id,
            'page' => 'subscribe',
            'client_id' => Auth::user()->client_id,
            'location_id' => $locationId,
            'proratedAmount' => $request->total_amount,
            'nextChargeDate' => $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
            'wallet_balance' => $this->getWalletBalance(Auth::user()->client_id),
            'no_of_products' => $this->getPlanDetailValue($planId, 'allowed_items', $request->additional_products),
            'no_of_users' =>  $this->getPlanDetailValue($planId, 'allowed_users', $request->additional_users),
            'payment_method' => 'wallet',
            'duration' => '',
            'email' => '',
            'branch_id' => '',
            'plan' => $planId,
    ];
    $postApi = $this->postData(
        $arrayData,
        'https://www.klakpad.org/agents_api/l/getRegistration.php'
    );

      
              return response()->json(['success' => true, 'message' => 'Subscription created successfully']);
          } else {
              return response()->json(['success' => false, 'message' => 'Failed to top up wallet']);
          }

        // Return a success response
    }


     public function subscribeForFree(Request $request)
    {
        $rules = [
            'plan_id' => 'required|integer',
            'plan_amount' => 'required|numeric',
            'total_amount' => 'required|numeric',
            'discount_percentage' => 'nullable|numeric',
            'amount_paid' => 'required|numeric',
            'coupon' => 'nullable|string|max:255',
            'payment_method' => 'required|string|max:255',
            'duration' => 'required|integer',
            'location_id' => 'required|integer',
            'additional_products' => 'nullable|integer',
            'additional_users' => 'nullable|integer',
            'additional_warehouse' => 'nullable|integer',
            'card_type' => 'nullable|string',
            'save_card' => 'nullable',
            'referenc_id' => 'nullable',
            'email' => 'required',
            'onboarding' => 'required|integer'
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 422);
        } 

        // Variables specific to wallet payment method
        $amountPaid = $request->amount_paid;
        $coupon = $request->coupon;
        $locationId = $request->location_id;
        $planId = $request->plan_id;
        $duration = $request->duration;
        $email = $request->email;
        $saveCard = $request->save_card;
        $ref = $request->reference_id;

        // Unique token for this subscription
        $tokenUid = Str::uuid()->toString();


       
    
             
       
    
                  
        TransactionLog::create([
            'client_id' => Auth::user()->client_id,
            'user_id' => Auth::user()->id,
            'biller_name' => 'plan subscription',
            'biller_id' => $request->plan_id,
            'biller_description' => 'Plan subscription through CARD',
            'biller_reference' => $ref,
            'payment_type' => 'free',
            'card_signature' => '',
            'payment_source' => 'paystack',
            'token' => $tokenUid,
            'location_id' =>  $locationId, 
            'amount' => $request->amount_paid,
            'plan_amount' => $request->plan_amount,
            'total_amount' => $request->total_amount,
            'discount_percentage' => $request->discount_percentage,
            'additional_products' => $request->additional_products,
            'additional_users' => $request->additional_users,
            'additional_warehouse'  => $request->additional_warehouse,
            'onboarding'  => $request->onboarding,
            'payment_status' => 'success',
            'date' => now()->toDateString(),
            'tm' => now()->toTimeString(),
            'status' => STATUS
        ]);

        Location::where('id',  $locationId)->update([
            'status' => 0,
            'start_date' => now()->toDateString(), 
            'end_date' =>  $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
            'payment_confirmation' => STATUS,
            'biller_reference' => $ref,
            'allowed_users' => $this->getPlanDetailValue($planId, 'allowed_users', $request->additional_users),
            'allowed_items' => $this->getPlanDetailValue($planId, 'allowed_items', $request->additional_products),
            'allowed_store' => $this->getPlanDetailValue($planId, 'allowed_store', $request->additional_warehouse),
            'app_plan_id' => $planId
        ]);


        if (!empty($coupon)) {
            CouponUser::create([
                'client_id' => Auth::user()->client_id,
                'location_id' =>  $locationId,
                'user_id' => Auth::user()->id,
                'code' =>  $coupon,
                'percentage' => $request->discount_percentage,
                'amount_charged' => $request->total_amount,
                'amount_paid' => $request->amount_paid,
                'token' => $tokenUid,
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => STATUS,
            ]);
         }


          //to agent portal
          $arrayData = [
            'user_id' => Auth::user()->id,
            'page' => 'subscribe',
            'client_id' => Auth::user()->client_id,
            'location_id' => $locationId,
            'proratedAmount' => $request->total_amount,
            'nextChargeDate' => $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
            'wallet_balance' => $this->getWalletBalance(Auth::user()->client_id),
            'no_of_products' => $this->getPlanDetailValue($planId, 'allowed_items', $request->additional_products),
            'no_of_users' =>  $this->getPlanDetailValue($planId, 'allowed_users', $request->additional_users),
            'payment_method' => 'wallet',
            'duration' => '',
            'email' => '',
            'branch_id' => '',
            'plan' => $planId,
    ];
    $postApi = $this->postData(
        $arrayData,
        'https://www.klakpad.org/agents_api/l/getRegistration.php'
    );

      
              return response()->json(['success' => true, 'message' => 'Subscription created successfully']);
 } 

        // Return a success response
    

    private function getSubscriptionPlanValue($id){

    }


    public function getPlanDetailValue($plan_id, $type, $id = null)
    {
        $featureMap = [
            'allowed_users' => 'Active Users',
            'allowed_items' => 'Products',
            'allowed_store' => 'Location',
        ];

        // Ensure that the requested type is valid
        if (!isset($featureMap[$type])) {
            return 0; // Or throw an exception
        }

        // Fetch the essential value for the given type
        $essentialValue = SubscriptionPlanDetails::where('plan_id', $plan_id)
                            ->where('features', $featureMap[$type])
                            ->value('value') ?? 0;

      
        $addonValue = !empty($id) ? SubscriptionPlanDetails::where('id', $id)->value('value') : 0;

        
        $totalValue = $essentialValue + $addonValue;

        return $totalValue;
    }


    // public function getPlanDetailValue($plan_id, $type, $id = null)
    // {
    //     $totalValue = 0; // Initialize totalValue to ensure it's always defined
    
    //     if ($type == 'allowed_users') {
    //         $essentialValue = SubscriptionPlanDetails::where('plan_id', $plan_id)->where('features', 'Active Users')->value('value');
    //         $addonValue = !empty($id) ? SubscriptionPlanDetails::where('id', $id)->value('value') : 0;
    //         $totalValue = $essentialValue + $addonValue;
    //     } elseif ($type == 'allowed_items') {
    //         $essentialValue = SubscriptionPlanDetails::where('plan_id', $plan_id)->where('features', 'Products')->value('value');
    //         $addonValue = !empty($id) ? SubscriptionPlanDetails::where('id', $id)->value('value') : 0;
    //         $totalValue = $essentialValue + $addonValue;
    //     } elseif ($type == 'allowed_store') {
    //         $essentialValue = SubscriptionPlanDetails::where('plan_id', $plan_id)->where('features', 'Location')->value('value');
    //         $addonValue = !empty($id) ? SubscriptionPlanDetails::where('id', $id)->value('value') : 0;
    //         $totalValue = $essentialValue + $addonValue;
    //     }
    
    //     return $totalValue;
    // }
    



    
    public function subscribe(Request $request)
    {
        $rules = [
            'plan_id' => 'required|integer',
            'plan_amount' => 'required|numeric',
            'total_amount' => 'required|numeric',
            'discount_percentage' => 'required|numeric',
            'amount_paid' => 'required|numeric',
            'coupon' => 'nullable|string|max:255',
            'sub_type' => 'required|string|max:255',
            'payment_method' => 'required|string|max:255',
            'duration' => 'required|integer|min:1',
            'reference_id' => 'required|string|max:255',
            'additional_products' => 'nullable|integer',
            'additional_users' => 'nullable|integer',
            'additional_warehouse' => 'nullable|integer',
            'location_id' => 'required|integer',
            'new_card' => 'required|integer',
        ];

    
        $validator = Validator::make($request->all(), $rules);
    
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        $planId = $request->plan_id;
        $planAmount = $request->plan_amount;
        $totalAmount = $request->total_amount;
        $discountPercentage = $request->discount_percentage;
        $amountPaid = $request->amount_paid;
        $coupon = $request->coupon;
        $subType = $request->sub_type;
        $paymentMethod = $request->payment_method;
        $duration = $request->duration;
        $referenceId = $request->reference_id;
        $additionalProducts = $request->additional_products;
        $additionalUsers = $request->additional_users;
        $additionalWarehouse = $request->additional_warehouse;
        $locationId = $request->location_id;
        $saveCard = $request->save_card;
        $authCode = $request->auth_code;
        $email = $request->email;
       

    
        // Unique token for this subscription
        $tokenUid = Str::uuid()->toString();
    
        $subscriptionStatus = $this->locationController->isExpired(Auth::user()->client_id, Auth::user()->location_id);
    
        if ($subscriptionStatus == 'active') {
            return response()->json(['message' => 'You have an active subscription', 'error' => true], 400);
        }
    
       

        if($paymentMethod == 'wallet'){
           
            if($this->getWalletBalance(Auth::user()->client_id) < $amountPaid){
                return response()->json(['message' => 'Insufficient Wallet Balance', 'error' => true], 400);
            }

            if(!empty($coupon )){
                $checkCoupon = CouponCode::where(['code' => $coupon , 'status' => 0])->first();
 
                if (!$checkCoupon) {
                    return response()->json(['message' => 'The coupon is not available.', 'error' => true], 400);
                }
        
                $checkCouponUsage = CouponUser::where(['code' => $coupon , 'status' => 0, 'client_id' => Auth::user()->id])->first();
        
                if ($checkCouponUsage) {
                    return response()->json(['message' => 'The coupon has already been used by you.', 'error' => true], 400);
                }     
            }

            Wallet::create([
                'client_id' => Auth::user()->client_id,
                'location_id' =>  $locationId,
                'user_id' => Auth::user()->id,
                'amount' =>  $amountPaid,
                'biller_name' => 'plan subscription',
                'biller_description' => 'Plan subscription through wallet',
                'biller_reference' => '',
                'token' => $tokenUid,
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => ONE_STATUS,
                'effect' => EFFECT_DEBIT
            ]);


            TransactionLog::create([
                'client_id' => Auth::user()->client_id,
                'user_id' => Auth::user()->id,
                'biller_name' => 'plan subscription',
                'biller_id' => $request->plan_id,
                'biller_description' => 'Plan subscription through wallet',
                'payment_type' => 'wallet',
                'card_signature' => '',
                'payment_source' => '',
                'token' => $tokenUid,
                'location_id' =>  $locationId, 
                'amount' => $request->amount_paid,
                'payment_status' => 'success',
                'biller_reference' => '',
                'date' => now()->toDateString(),
                'tm' => now()->toTimeString(),
                'status' => ONE_STATUS
            ]);

            Location::where('location_id',  $locationId)->update([
                'status' => 0,
                'start_date' => now()->toDateString(), 
                'end_date' =>  $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
                'payment_confirmation' => STATUS,
                'allowed_users' => ONE_STATUS,
                'allowed_items' => 50,
                'allowed_store' => ONE_STATUS,
                'app_plan_id' => $planId
            ]);

             // Return a success response
         return response()->json(['message' => 'Subscription created successfully', 'error' => false], 200);

        }

        if($paymentMethod== 'new_card'){
            $verify = $this->verifyPaystackTransaction($referenceId);
            $paymentStatus = $verify['data']['status'];
            if ($paymentStatus == 'success') {
                if($saveCard == ONE_STATUS){
                  $this->saveCard($saveCard, $verify);
                }
                 
                  $signature = $verify['data']['authorization']['signature'];
          
          
                  TransactionLog::create([
                      'client_id' => Auth::user()->client_id,
                      'user_id' => Auth::user()->id,
                      'biller_name' => 'plan subscription',
                      'biller_id' => $request->plan_id,
                      'biller_description' => 'Plan subscription through debit card',
                      'payment_type' => 'card',
                      'card_signature' => $signature,
                      'payment_source' => 'paystack',
                      'token' => $tokenUid,
                      'location_id' => $locationId,
                      'amount' => $amountPaid,
                      'payment_status' => $paymentStatus,
                      'biller_reference' => $referenceId,
                      'date' => now()->toDateString(),
                      'tm' => now()->toTimeString(),
                      'status' => ONE_STATUS
                  ]);

                  Location::where('location_id',  $locationId)->update([
                    'status' => 0,
                    'start_date' => now()->toDateString(), 
                    'end_date' =>  $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
                    'payment_confirmation' => ONE_STATUS,
                    'allowed_users' => ONE_STATUS,
                    'allowed_items' => 50,
                    'allowed_store' => ONE_STATUS,
                    'app_plan_id' => $planId
                ]);
          
                return response()->json(['message' => 'Subscription created successfully', 'error' => false], 200);              }
        }

        if($paymentMethod== 'saved_card'){
            $cardData = $this->chargeFromCard($authCode, $email, $amountPaid);
            $referenceId = $cardData['data']['reference'];

            $verify = $this->verifyPaystackTransaction($referenceId);
            $paymentStatus = $verify['data']['status'];
            if ($paymentStatus == 'success') {            
                  $signature = $verify['data']['authorization']['signature'];         
                  TransactionLog::create([
                      'client_id' => Auth::user()->client_id,
                      'user_id' => Auth::user()->id,
                      'biller_name' => 'plan subscription',
                      'biller_id' => $request->plan_id,
                      'biller_description' => 'Plan subscription through debit card',
                      'payment_type' => 'card',
                      'card_signature' => $signature,
                      'payment_source' => 'paystack',
                      'token' => $tokenUid,
                      'location_id' => $locationId,
                      'amount' => $amountPaid,
                      'payment_status' => $paymentStatus,
                      'biller_reference' => $referenceId,
                      'date' => now()->toDateString(),
                      'tm' => now()->toTimeString(),
                      'status' => ONE_STATUS
                  ]);

                  Location::where('location_id',  $locationId)->update([
                    'status' => 0,
                    'start_date' => now()->toDateString(), 
                    'end_date' =>  $duration == 1 ? Carbon::now()->addMonth() : Carbon::now()->addYear(),
                    'payment_confirmation' => ONE_STATUS,
                    'allowed_users' => ONE_STATUS,
                    'allowed_items' => 50,
                    'allowed_store' => ONE_STATUS,
                    'app_plan_id' => $planId
                ]);
          
                return response()->json(['message' => 'Subscription created successfully', 'error' => false], 200);              
            }
        }
    
       
    }



    public function getClientLocations()
    {
        $locations = Location::select('name', 'address', 'app_plan_id', 'status', 'start_date', 'end_date')
            ->where('client_id', Auth::user()->client_id)
            ->get()
            ->map(function ($location) {
                // Convert app_plan_id to meaningful names
                switch ($location->app_plan_id) {
                    case 1:
                        $location->plan = 'Basic';
                        break;
                    case 2:
                        $location->plan = 'Plus';
                        break;
                    case 3:
                        $location->plan = 'Enterprise';
                        break;
                }
                $location->status = $location->status == 0 ? 'Active' : 'Inactive';
    
                // Add new keys here
                return [
                    'name' => $location->name,
                    'address' => $location->address,
                    'plan' => $location->plan,
                    'status' => $location->status,
                    'start_date' => $location->start_date,
                    'end_date' => $location->end_date
                ];
            });
    
        return response()->json(['message' => 'Location Retrieved Successfully', 'data' => $locations], 404);

    }
    

   
}
