<?php

namespace App\Http\Controllers\Inventory\Inventory;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Settings\Measurement\MeasurementController;
use App\Http\Controllers\Inventory\Product\PriceController;
use App\Http\Controllers\Folio\CustomerVendor\CustomerVendorController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use App\Models\Retails\Stock;
use App\Models\Retails\Location;
use App\Models\Retails\StockCsvTemp;
use App\Models\Retails\Items;
use App\Models\Retails\ItemPrice;
use App\Models\Retails\ItemCategory;
use App\Http\Controllers\Folio\Bank\BankAccountController;
use App\Http\Controllers\Folio\Location\LocationController;
use App\Http\Controllers\Settings\Store\StoreController;
use Validator;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use Exception;
use Illuminate\Support\Facades\Log;

class InventoryController extends Controller
{
    //

    protected $currencyController;
    protected $productController;
    protected $priceController;
    protected $txnLogController;
    protected $chartOfAccountController;
    protected $customerVendorController;
    protected $bankAccountController;
    protected $storeController;
    protected $measurementController;
    protected $locationController;
    

    public function __construct(
        CurrencyController $currencyController,
        PriceController $priceController,
        TxnLogController $txnLogController,
        ChartOfAccountController $chartOfAccountController,
        CustomerVendorController $customerVendorController,
        BankAccountController $bankAccountController,
        StoreController $storeController,
        MeasurementController $measurementController,
        LocationController $locationController
    ) {
        $this->currencyController = $currencyController;
        $this->priceController = $priceController;
        $this->txnLogController = $txnLogController;
        $this->chartOfAccountController = $chartOfAccountController;
        $this->customerVendorController = $customerVendorController;
        $this->bankAccountController = $bankAccountController;
        $this->storeController = $storeController;
        $this->measurementController = $measurementController;
        $this->locationController = $locationController;
        
    }
 
    public function insertStock($token, $txn_date, $item_id, $supplier_id="", $item_category_id, $qty, $cp="", $currency_id, $m_metric, $m_category_id, $store_id, $effect, $expiry_date=null, $amt, $location_id=""){

        try{
        $location_id = !empty($location_id) ? $location_id : Auth::user()->location_id;
        $calQty = $this->priceController->itemPriceCalQty($item_id, $m_metric, $m_category_id, $location_id);
        $uqty = $qty * $calQty;
        $cp = !empty($cp) ? $cp : $amt / $uqty;
        $amount = $cp * $uqty;

        $price = $this->priceController->itemPrice($item_id, $m_metric, $m_category_id, $location_id);

        if($price < $cp) {
            return false;
        }
        
        $currencyUnit = $this->currencyController->getCurrencyUnit(Auth::user()->client_id, $location_id , $currency_id);
        $currencyValue = $amount * $currencyUnit;

        $fields = [
            'id' => Str::uuid()->toString(),
            'txn_date' => $txn_date,
            'token' => $token,
            'expiry_date' => $expiry_date,
            'supplier_id' => $supplier_id,
            'item_category_id' => $item_category_id,
            'item_id' => $item_id,
            'qty' => $qty,
            'price' => $cp,
            'currency_id' => $currency_id,
            'effect' => $effect,
            'uqty' => $uqty,
            'store_id' => $store_id,
            'm_category_id' => $m_category_id,
            'm_metric' => $m_metric,
            'date' => Carbon::now()->format('Y-m-d'),
            'tm' => Carbon::now()->toTimeString(),
            "day_name" => Carbon::now()->format('l'),
            "week_no" => Carbon::now()->weekOfYear,
            "month_no" => Carbon::now()->month,
            'status' => STATUS,
            'user_id' => Auth::user()->id,
            'location_id' => $location_id,
            'client_id' => Auth::user()->client_id,
            'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            'ampm' => date('A'),
            'control_user_id' => STATUS,
            'control_id' => STATUS,
            'amount' => $amount,
            'currency_unit' => $currencyUnit,
            'currency_value' => $currencyValue,
        ];

        $stock = Stock::create($fields);

        return $stock;
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error inserting stock', 'success' => false, 'error' => $e->getMessage()], 500);
        }

    }

    public function saveInventoryValidation($request) {
        $vendorName = $this->customerVendorController->customerName(Auth::user()->client_id, $request->supplier_id);

        $amtReq= '';

        if($request->pmethod == 'charged to customer account') {
            $amtReq = '';
        } else {
            if($vendorName == DEFAULT_VENDOR) {
                $amtReq = '';
            } else {
                $amtReq = 'required';
            }
        }
        
        $validationRules = [
            'bank_account_id' => $request->pmethod === POS || $request->pmethod === BANK_TRANSFER ? 'required' : '',
            'qty' => 'required',
            'amt' =>  $amtReq,
            'cp' => 'required',
            'item_id' => 'required',
            'currency_id' => 'required',
            'm_metric' => 'required',
            'm_category_id' => 'required',
            'txn_date' => 'required',
            'supplier_id' => 'required',
            'item_category_id' => 'required',
            'store_id' => 'required',
            'pmethod' => 'required',
        ];
        // Create a validator with the defined rules and the request data
        $validator = Validator::make($request->all(), $validationRules);
        // Check if validation fails and return an error response if it does
        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }
    }

    // saving single inventory
    public function saveSingleInventory(Request $request) {
        $validation = $this->saveInventoryValidation($request);
        if($validation){
            return $validation;
        }
        $token = Uuid::uuid4()->toString();
        // inserting into stock table
        $amount = $request->cp * $request->qty;
        $stock = $this->insertStock($token, $request->txn_date, $request->item_id, $request->supplier_id, $request->item_category_id, $request->qty, "", $request->currency_id, $request->m_metric, $request->m_category_id, $request->store_id, EFFECT_DEBIT, $request->expiry_date, $amount);

        if($stock === false) {
            $itemName = Items::where('status', STATUS)
            ->where('client_id', Auth::user()->client_id)
            ->where('id', $request->item_id)
            ->value('name');

            return response()->json(['message' => "The unit cost price of $itemName is higher than its selling price", 'error' => true], 400);
        }

        ItemPrice::where('item_id', $request->item_id)
            ->where('client_id', Auth::user()->client_id)
            ->where('location_id', Auth::user()->location_id)
            ->where('primary_measurement', ONE_STATUS)
            ->update([
                'cp' => $request->cp,
                'update_date' => Carbon::now()->format('Y-m-d H:i:s')
            ]);
        
        $supplierName = $this->customerVendorController->customerName(Auth::user()->client_id, $request->supplier_id);
        // saving transaction for inventory
        $this->saveInventoryTxnLog($request->pmethod, $stock->amount, $token, $stock->currency_unit, $request->txn_date, $request->supplier_id, "being added stock", $supplierName, "grn", $request->currency_id, $request->bank_account_id, $request->amt);
        
        return response()->json(['message' => 'Inventory saved successfully', 'data' => $stock, 'success' => true], 201);
    }

    public function saveInventoryTxnLog($pmethod, $amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id, $bank_account_id="", $tender=null) { 
        
        if($pmethod === CASH){
            $this->saveInventoryCashTxnLog($tender, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id);
        } else if($pmethod === POS || $pmethod === BANK_TRANSFER) {
            $bank_id = $this->bankAccountController->getBankIdByBankAccountId($bank_account_id);
            $this->saveInventoryBankTxnLog($tender, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id, $bank_account_id, $bank_id);
        } else if($pmethod === SUSPENSE) {
            $this->saveInventorySuspenseTxnLog($amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id);
        } 
        // else if($pmethod === CREDIT) {
        //     $this->saveInventoryCreditTxnLog($amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id);
        // }
        
        if($pmethod != SUSPENSE) {
            $accLegs = $this->chartOfAccountController->getAccountLegs(CHART_STOCK, CHART_RECEIVABLE, Auth::user()->client_id);
            $this->txnLogController->createTxnLog($accLegs, $amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id);
        }

    }

    public function saveInventorySuspenseTxnLog($amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id) {
        $accLegs = $this->chartOfAccountController->getAccountLegs(CHART_STOCK, CHART_SUSPENSE, Auth::user()->client_id);
        $this->txnLogController->createTxnLog($accLegs, $amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id);
    }

    public function saveInventoryCashTxnLog($amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id) {
        $this->savePmethodTxnLog(CHART_CASH, $amount, $token, CASH, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id);
    }

    public function savePmethodTxnLog($chart_id, $amount, $token, $pmethod, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id, $bankAccount="", $bank_id="") {
        $pmethodLeg = $this->chartOfAccountController->getSingleAccountLeg($chart_id, Auth::user()->client_id);
        $currencyValue = $currency_unit * $amount;

        $this->txnLogController->createSingleTxnLog($pmethodLeg, EFFECT_CREDIT, $amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id, $currencyValue, $pmethod, $bankAccount, $bank_id);

        $receivableLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_RECEIVABLE, Auth::user()->client_id);
        $this->txnLogController->createSingleTxnLog($receivableLeg, EFFECT_DEBIT, $amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id, $currencyValue);
    }

    public function saveInventoryBankTxnLog($amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id, $bank_account_id, $bank_id) {
        $this->savePmethodTxnLog(CHART_BANK, $amount, $token, BANK_TRANSFER, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id, $bank_account_id, $bank_id);
    }

    // public function saveInventoryCreditTxnLog($amount, $token, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id) {
    //     $this->savePmethodTxnLog(CHART_PAYABLE, $amount, $token, CREDIT, $currency_unit, $txn_date, $supplier_id, $being, $supplier, $prefix, $currency_id);
    // }

    public function saveInventory(Request $request) {
        $validation = $this->saveInventoryValidation($request);
        if($validation){
            return $validation;
        }
        $token = Uuid::uuid4()->toString();
        $data=[];
        $amount = STATUS;

        for($i=0; $i < sizeof($request->item_id); $i++){
            $amt = $request->cp[$i] * $request->qty[$i];
            $stock = $this->insertStock($token, $request->txn_date, $request->item_id[$i], $request->supplier_id, $request->item_category_id[$i], $request->qty[$i], "", $request->currency_id, $request->m_metric[$i], $request->m_category_id[$i], $request->store_id, EFFECT_DEBIT, $request->expiry_date[$i], $amt);

            if($stock === false) {
                $itemName = Items::where('status', STATUS)
                ->where('client_id', Auth::user()->client_id)
                ->where('id', $request->item_id)
                ->value('name');
                
                return response()->json(['message' => "The unit cost price of $itemName is higher than its selling price", 'error' => true], 400);
            }

            ItemPrice::where('item_id', $request->item_id[$i])
                ->where('client_id', Auth::user()->client_id)
                ->where('location_id', Auth::user()->location_id)
                ->where('primary_measurement', ONE_STATUS)
                ->update([
                    'cp' => $request->cp[$i],
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s')
                ]);

            $amount += $stock->amount;
            array_push($data, $stock);
        }

        $supplier = $this->customerVendorController->customerName(Auth::user()->client_id, $request->supplier_id);

        $this->saveInventoryTxnLog($request->pmethod, $amount, $token, $stock->currency_unit, $request->txn_date, $request->supplier_id, "being added stock", $supplier, "grn", $request->currency_id, $request->bank_account_id, $request->amt);

        return response()->json(['message' => 'Inventory saved successfully', 'data' => $data, 'success' => true], 201);
    }

    public function getQtyBalance($itemId){
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        
        $query="
            WITH uqty As (
                SELECT 
                cal_qty, 
                m_metric, 
                item_id
                FROM " . TBL_ITEM_PRICE . "
                WHERE client_id='$client_id' AND location_id='$location_id' AND item_id='$itemId' AND status = " . STATUS . "
            ),
            
            crStockQty As (
                SELECT
                s.item_id,
                s.client_id,
                s.location_id,
                s.control_id,
                round(SUM(s.qty * u.cal_qty),2) as dispensed
                FROM " . TBL_STOCK . " s
                LEFT JOIN uqty u
                ON u.item_id = s.item_id AND s.m_metric=u.m_metric
                WHERE s.effect = 'cr' AND s.item_id='$itemId' AND s.location_id='$location_id' AND s.control_id != 1
                group by s.item_id
            ),

            drStockQty As (
                SELECT 
                s.item_id,
                s.client_id,
                s.location_id,
                s.control_id,
                ROUND(SUM(coalesce(s.qty * u.cal_qty,0)), 2) as received
                FROM " . TBL_STOCK . " s
                LEFT JOIN uqty u
                ON u.item_id = s.item_id AND s.m_metric=u.m_metric
                WHERE s.effect = 'dr' AND s.item_id='$itemId' AND s.location_id='$location_id' AND s.control_id != 1
                group by s.item_id
            )

            SELECT
            CASE WHEN cr.dispensed IS NOT NULL 
            THEN
            SUM(dr.received - cr.dispensed)
            ELSE
            SUM(dr.received)
            END as qty
            FROM

            drStockQty dr

            LEFT JOIN crStockQty cr
            ON cr.item_id='$itemId'

            WHERE 
            dr.client_id='$client_id' AND dr.location_id='$location_id' AND dr.item_id='$itemId' AND 
            dr.control_id != " . ONE_STATUS . "
        ";

        $data = DB::select($query);

        return $data[0]->qty;
    }


    public function getCOGByToken($token) {
        return Stock::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('token', $token)
        ->selectRaw('SUM(amount) as cog')
        ->value('cog');
    }

    public function itemAverageCostPrice($item_id, $storeId="", $location="") {
        $client_id = Auth::user()->client_id;
        $location_id = empty($location) ? Auth::user()->location_id : $location;
        $store_id = !empty($storeId) ? $storeId : $this->storeController->getPrimaryStore(Auth::user()->client_id, $location_id);

        $query="
            SELECT 
            
            round(sum((m.qty * b.cal_qty) * m.price) / sum(m.qty * b.cal_qty),2) as costPrice 
            from " . TBL_STOCK . " m 

            LEFT JOIN " . TBL_ITEM_PRICE . " b
            ON b.item_id=m.item_id AND b.m_metric=m.m_metric AND b.client_id='$client_id' AND b.location_id='$location_id' AND b.status = 0
                     
            WHERE m.control_id!=1 and m.store_id = '$store_id' and m.client_id = '$client_id' and m.location_id='$location_id' and m.item_id = '$item_id' and m.effect = 'dr' LIMIT 1
        ";
        
        $data = DB::select($query);

        $cp = count($data) > 0 ? $data[0]->costPrice : 0;

        return $cp;
      
    }

    public function currentMarketingPrice($item_id, $storeId="", $location="") {
        $client_id = Auth::user()->client_id;
        $location_id = empty($location) ? Auth::user()->location_id : $location;
        $store_id = !empty($storeId) ? $storeId : $this->storeController->getPrimaryStore(Auth::user()->client_id, $location_id);

        $query="
            SELECT 
            cp * cal_qty
            from " . TBL_ITEM_PRICE . " 
            WHERE item_id= '$item_id' AND primary_measurement=". ONE_STATUS ." AND client_id='$client_id' AND location_id='$location_id' AND status = 0
        ";
        
        $data = DB::select($query);

        $cp = count($data) > 0 ? $data[0]->cp : 0;

        return $cp;
      
    }

    public function inventoryEvaluationPrice($item_id, $storeId="", $location="") {
        $client_id = Auth::user()->client_id;
        $location_id = empty($location) ? Auth::user()->location_id : $location;
        $inventoryEvaluation = Location::where("client_id", $client_id)
        ->where("id", $location_id)
        ->value("inventory_evaluation");

        // weighted average cost price
        if($inventoryEvaluation==STATUS) {
            return $this->itemAverageCostPrice($item_id, $storeId, $location);
        }

        // current market pricing
        if($inventoryEvaluation==ONE_STATUS) {
            return $this->currentMarketingPrice($item_id, $storeId, $location);
        }

        // FIFO cost price
        if($inventoryEvaluation==TWO_STATUS) {
            // ...
        }
    }

    public function getStockMovement(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $limit = "LIMIT $request->count, 50";
        $primaryStore = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $storeId = !empty($request->store) ? "AND s.store_id='$request->store'" : "AND s.store_id='$primaryStore'";
        $token = !empty($request->token) ? " AND s.token LIKE '%$request->token%'" : "";

        $validationRules = [
            'from' => 'required',
            'to' => 'required',
            'item_id' => 'required',
        ];
        // Create a validator with the defined rules and the request data
        $validator = Validator::make($request->all(), $validationRules);
        // Check if validation fails and return an error response if it does
        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }

        $query = "
        WITH stockMovement AS (
            SELECT 
                ROW_NUMBER() OVER  (ORDER BY s.txn_date, s.tm) as rowNum, 
                s.txn_date,
                s.tm,
                s.price,
                s.m_metric,
                s.token,
                s.qty * i.cal_qty as qty,
                CASE 
                    when s.effect ='dr' THEN s.qty * i.cal_qty
                    ELSE -s.qty * i.cal_qty
                END AS bal,
                s.effect,
                m.name
            FROM " . TBL_STOCK . " s

            LEFT JOIN " . TBL_ITEM_PRICE . " i
            ON i.item_id = '$request->item_id' AND i.m_metric=s.m_metric AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.status=" . STATUS . "

            LEFT JOIN " . TBL_MEASUREMENT . " m
            ON m.m_category_id = s.m_category_id AND m.hierarchy=1 AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = " . STATUS . "

            WHERE s.item_id = '$request->item_id' AND s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 $storeId $token
            
            $limit
        ), 

        runningBal as (
            SELECT s1.*, sum(s2.bal) as runningBal FROM stockMovement s1 
            LEFT JOIN stockMovement s2 ON
            s1.rowNum  >= s2.rowNum
            group by s1.rowNum
        ),

        openingBal as(
            Select coalesce(sum(bal),0) as openingBal from runningBal
            where txn_date < '$request->from'
        ),
       
        
        records as(
        SELECT r.*, b.openingBal FROM 
            runningBal r 
            JOIN 
            openingBal b
        where r.txn_date between '$request->from' AND '$request->to')

        SELECT a.*, b.total_balance, b.total_dispensed, b.total_received 
        FROM records a
        JOIN (
            SELECT 
                SUM(bal) AS total_balance, 
                SUM(CASE WHEN effect = 'cr' THEN qty ELSE 0 END) AS total_dispensed,
                SUM(CASE WHEN effect = 'dr' THEN qty ELSE 0 END) AS total_received
            FROM records
        ) b ON 1 = 1;

        ";

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function getPurchaseLog(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;

        $primaryStore = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $storeId = !empty($request->store) ? "AND s.store_id='$request->store'" : "AND s.store_id='$primaryStore'";
        
        $date = !empty($request->from) && !empty($request->to) ? "AND s.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $sortUser = !empty($request->user) ? "AND s.user_id='$request->user'" : "";
        $category = !empty($request->category) ? "AND s.item_category_id='$request->category'" : "";

        $searchName = !empty($request->text) ? " AND i.name LIKE '%$request->text%'" : "";
        $limit = "LIMIT $request->count, 50";

        $query = "
            SELECT 
                s.token,
                s.txn_date,
                s.expiry_date,
                s.tm,
                s.price as cp,
                s.qty,
                s.amount,
                s.id,
                s.item_id,
                i.name,
                m.abr,
            
            FROM " . TBL_STOCK . " s
            
            LEFT JOIN  " . TBL_ITEMS . " i 
            ON i.id=s.item_id AND i.status = 0 AND i.client_id='$client_id' $searchName

            LEFT JOIN " . TBL_MEASUREMENT . " m
            ON m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0 AND m.hierarchy=s.m_metric AND m.m_category_id=s.m_category_id
            
            WHERE s.effect='dr' AND s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND i.id IS NOT NULL
            $date $sortUser $category $storeId
            ORDER BY s.tm, s.txn_date DESC
            $limit
        ";

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function deleteSinglePurchaseLog(Request $request) {

        if(!$request->id) {
            return response()->json(['message' => 'You have not picked a record to delete', 'error' => true], 400);
        }
    
        $stock = Stock::findOrFail($request->id);

        $stock->control_id = ONE_STATUS;
        $stock->control_user_id = Auth::user()->id;
        
        $stock->save();

        $txn_date = Carbon::now()->format('Y-m-d');
        $token = Uuid::uuid4()->toString();
        $supplierName = $this->customerVendorController->customerName(Auth::user()->client_id, $stock->supplier_id);
        
        $this->saveInventorySuspenseTxnLog($stock->amount, $token, $stock->currency_unit, $txn_date, $stock->supplier_id, "being stock removed", $supplierName, "grn", $stock->currency_id);

        return response()->json(['message' => 'deleted successfully', 'success' => true], 200);
    }

    
    public function cogList(Request $request) {
       
        $client_id = Auth::user()->client_id;
        $token = !empty($request->token) ? " AND l.token LIKE '%$request->token%'" : "";
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $user_id = !empty($request->user_id) ? " AND l.user_id='$request->user_id'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? " AND l.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $store_id = !empty($request->store_id) ? " AND l.store_id='$request->store_id'" : "";
        $item_id = !empty($request->item_id) ? " AND l.item_id = '$request->item_id'" : "";
        $limit = "LIMIT $request->count, 50";
        
        $query = "
            WITH salesList AS ( 
                SELECT
                    l.token,
                    u.name,
                    l.txn_date,
                    l.tm,
                    l.customer,
                    sum((l.cp * ip.cal_qty) * l.qty) as amount,
                    

                    (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'item', i.name,
                            'amount', round((p.qty * ip.cal_qty) * p.cp, 2),
                            'qty', p.qty,
                            'abr', m.abr,
                            'price', round(p.cp * ip.cal_qty, 2)
                        ))

                        FROM " . TBL_PRODUCT_LOG . " p

                        LEFT JOIN " . TBL_ITEMS . " i
                        ON i.id=p.item_id AND i.status = " . STATUS . " AND i.client_id='$client_id'

                        LEFT JOIN " . TBL_ITEM_PRICE . " ip
                        ON ip.item_id=p.item_id AND ip.status = " . STATUS . " AND ip.client_id='$client_id' AND ip.location_id='$location_id' AND ip.m_metric=p.m_metric

                        LEFT JOIN " .TBL_MEASUREMENT . " m
                        ON m.m_category_id=p.m_category_id AND m.status = " . STATUS . " AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.hierarchy=p.m_metric

                        WHERE p.token = l.token
                    ) as receipt

                FROM " . TBL_PRODUCT_LOG . " l

                LEFT JOIN " . TBL_USER . " u
                ON u.id=l.user_id AND u.client_id='$client_id' AND u.location_id='$location_id'

                LEFT JOIN " . TBL_ITEM_PRICE . " ip
                ON ip.item_id=l.item_id AND ip.status = " . STATUS . " AND ip.client_id='$client_id' AND ip.location_id='$location_id' AND ip.m_metric=l.m_metric

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " $user_id $txn_date $store_id $token $item_id

                GROUP BY l.token
                ORDER BY l.txn_date DESC, l.tm DESC
                $limit
            ),

            amounts AS (
                SELECT
                l.token,
                sum((l.qty * l.cp)*b.cal_qty) as amount

                FROM " . TBL_PRODUCT_LOG . " l

                LEFT JOIN " . TBL_ITEM_PRICE . " b
                ON b.item_id=l.item_id AND b.m_metric=l.m_metric AND b.client_id='$client_id' AND b.location_id='$location_id' AND b.status = 0

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " $user_id $txn_date $store_id $token

                GROUP BY l.token
                ORDER BY l.txn_date DESC, l.tm DESC
                $limit
            ),

            summary AS (
                SELECT

                COUNT(DISTINCT l.token) as total_products,
                SUM((l.qty * l.cp) * b.cal_qty) as total_amount,
                SUM((l.qty * l.price) - l.discount) as total_revenue,
                ROUND(SUM((l.qty * l.price) - l.discount) - SUM((l.qty * l.cp)*b.cal_qty), 2) as total_profit

                FROM " . TBL_PRODUCT_LOG . " l

                LEFT JOIN " . TBL_ITEM_PRICE . " b
                ON b.item_id=l.item_id AND b.m_metric=l.m_metric AND b.client_id='$client_id' AND b.location_id='$location_id' AND b.status = 0

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " $user_id $txn_date $store_id $token
            )
 
            SELECT *
            FROM salesList
            JOIN amounts ON salesList.token = amounts.token
            CROSS JOIN summary

        ";

        // echo $query;exit;

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }


    public function cogSummary(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $user_id = !empty($request->user_id) ? " AND l.user_id='$request->user_id'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? " AND l.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $store_id = !empty($request->store_id) ? " AND l.store_id='$request->store_id'" : "";
        $name = !empty($request->name) ? " AND i.name LIKE '%$request->name%'" : "";
        $limit = "LIMIT $request->count, 50";

        $query="
            WITH salesSummary as (
                SELECT
                i.name,
                l.item_id,
                SUM((l.qty * l.cp) * p.cal_qty) AS amount,
                SUM(l.qty * p.cal_qty) as qty,
                ROUND(SUM((l.qty * p.cal_qty) * l.cp) / SUM(l.qty * p.cal_qty), 2) AS avg_cost,
                ROUND(SUM(l.amount-l.discount) / SUM(l.qty * p.cal_qty), 2) AS avg_revenue,
                
                round(
                    ((
                        sum(((l.qty * p.cal_qty) * l.price) - l.discount) / sum(l.qty * p.cal_qty) - 
                        sum((l.qty * p.cal_qty) * l.cp) / sum(l.qty * p.cal_qty)
                    ) / sum(((l.qty * p.cal_qty) * l.price)) - l.discount) * 100,
                    2
                ) as pm,

                round(
                (
                sum(((l.qty * p.cal_qty) * l.price) - l.discount) / sum(l.qty * p.cal_qty) -
                sum((l.qty * p.cal_qty) * l.cp) / sum(l.qty * p.cal_qty))
                /  (sum((l.qty * p.cal_qty) * l.cp) / sum(l.qty * p.cal_qty)) * 100
                ,2)
                as mk
            
                FROM " . TBL_PRODUCT_LOG . " l
                
                LEFT JOIN " . TBL_ITEMS . " i
                ON i.id=l.item_id AND i.status = " . STATUS . " AND i.client_id='$client_id' $name

                LEFT JOIN " . TBL_ITEM_PRICE . " p
                ON l.item_id=p.item_id AND p.m_metric=l.m_metric AND p.status = " . STATUS . " AND p.location_id='$location_id'

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " AND i.id IS NOT NULL $user_id $txn_date $store_id
                GROUP BY l.item_id
                $limit
            ),

            summary AS (
                SELECT

                COUNT(DISTINCT l.item_id) as total_products,
                SUM((l.qty * l.cp) * p.cal_qty) as total_cost,
                SUM((l.qty * l.price) - l.discount) as total_revenue,
                SUM((l.qty * l.price) - l.discount) - SUM((l.qty * l.cp) * p.cal_qty) as total_profit

                FROM " . TBL_PRODUCT_LOG . " l

                LEFT JOIN " . TBL_ITEM_PRICE . " p
                ON l.item_id=p.item_id AND p.m_metric=l.m_metric AND p.status = " . STATUS . "

                LEFT JOIN " . TBL_ITEMS . " i
                ON i.id=l.item_id AND i.status = " . STATUS . " AND i.client_id='$client_id' $name

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " AND i.id IS NOT NULL $user_id $txn_date $store_id
            )

            SELECT * FROM salesSummary CROSS JOIN summary;
        ";
        // echo $query;exit;
        $data = DB::select($query);
 
        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function createInventoryTemplate(Request $request) {
        $m_category_id = $request->m_category_id;

        $measurement = $this->measurementController->getMeausrementInfoByCategory($m_category_id);
        $token = Uuid::uuid4()->toString();

        $filename = $token . '.csv';

        $fields = [
            'Product Category',
            'Product Name',
            'Stock Qty (must be in lowest measurement)',
            'Cost Price (must be in lowest measurement)',
            'Low Quantity Alert',
            'Bar code',
        ];
    
        foreach ($measurement as $mst) {
            $name = ucfirst($mst['name']);
            if ($mst['hierarchy'] > 1) {
                $hierarchy = $mst['hierarchy'] - 1;
                // $prevName = $this->measurementController->getMeasurementNameByHierarchy(
                //     $hierarchy,
                //     $m_category_id
                // );
                // $prevName = ucfirst($prevName);
                array_push($fields, "$name Qty (Unit Qty in $name)");
                array_push($fields, "$name Price");
            } 
            else {
                array_push($fields, "$name Qty");
                array_push($fields, "$name Price");
            }
        }
        
        $string = [];

        foreach ($fields as $fl) {
            array_push($string, $fl);
        }

        $path = storage_path('app/public/uploads/template');
        
        if (!file_exists($path)) {
            mkdir($path, 0775, true); // Creates the directory if it doesn't exist
        }

        $filepath = $path.'/'.$filename;

        $csvfile = fopen($filepath, 'w'); // Open or create the file for writing

        // Write the headers to the CSV file
        fputcsv($csvfile, $string);
        
        // file_put_contents($path.'/'.$filename, $string);

        $location_template = $this->locationController->templateName();

        if(!empty($location_template)){
            Storage::disk('public')->delete('uploads/template/'.$location_template);
        }

        Location::where('id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->where('status', STATUS)
        ->update([
            'template' => $filename
        ]);

        return response()->json(['data' => $filename, 'message' => 'Template created successfully, you can now get the template by clicking the download button', 'success' => true], 201);
    }


    private function validateRequest($request)
    {
        $validator = Validator::make($request->all(), [
            'date' => 'required',
            'm_category_id' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'errors' => $validator->errors()->first()], 400);
        }

        if (!$request->hasFile('fileUri')) {
            return response()->json(['message' => 'You have to upload the csv template', 'success' => false], 400);
        }
        
        $exts = explode('.', $request->fileName);
        $n = count($exts) - 1;
        
        if ($exts[$n] != 'csv') {
            return response()->json(['message' => 'Only csv file is allowed', 'success' => false], 400);
        }

        $isMeasurementCategory = $this->measurementController->getMeausrementInfoByCategory($request->m_category_id);

        if (empty($isMeasurementCategory)) {
            return response()->json(['message' => 'Please be sure you picked the correct measurement category', 'success' => false], 400);
        }

        return null;
    }

    
    public function uploadInventory(Request $request) {
        DB::beginTransaction(); // Start the transaction
    
        try {
            $validationResult = $this->validateRequest($request);
            if ($validationResult) {
                return $validationResult;
            }
    
            $file = $request->file('fileUri');
            $m_category_count = $this->measurementController->countMeausrementByCategory($request->m_category_id);
    
            if (($handle = fopen($file->getRealPath(), 'r')) !== false) {
                $data = [];
    
                while (($row = fgetcsv($handle, 1000, ',')) !== false) {
                    $row = str_replace("'", '', $row);
                    for ($i = 1; $i <= $m_category_count; $i++) {
                        $rows = [];
                        if ($row[1] != 'Product Name') {
                            $name = str_replace(',', ' ', $row[1]);
                            array_push($rows, str_replace(',', ' ', $row[0]), $name);
    
                            $qty = $i > 1 ? 0 : $row[2];
                            $cp = $i > 1 ? 0 : $row[3];
                            $reorderLevel = $i > 1 ? 0 : $row[4];
    
                            array_push($rows, str_replace(',', '', $qty));
                            array_push($rows, str_replace(',', '', $cp));
                            array_push($rows, str_replace(',', '', $reorderLevel));
                            array_push($rows, str_replace(',', '', $row[5]));
    
                            $mmetric = 0 + $i;
                            array_push($rows, $mmetric);
    
                            $x = 5 - 1 + ($i + $i);
                            $z = 6 - 1 + ($i + $i);
    
                            array_push($rows, str_replace(',', '', $row[$x]));
                            array_push($rows, str_replace(',', '', $row[$z]));
                            $data[] = $rows;
                        }    
                    }
                }
                fclose($handle);
            }
    
            // Save the CSV data to the database
            $this->saveStockCsv($data, $m_category_count);
            
            // Get the stock CSV information
            $getStockCsv = $this->getStockCsvInfo();
    
            // Upload the inventory using the stock CSV data
            $this->uploadedInventory($getStockCsv, $request->m_category_id, $request->store);
    
            DB::commit(); // Commit the transaction if all steps are successful
    
            return response()->json(['message' => 'Stock Uploaded Successfully', 'success' => true], 201);
        } catch (\Exception $e) {
            DB::rollBack(); // Roll back the transaction in case of an error
          //  \Log::channel('slack')->error('Error uploading inventory', ['exception' => $e]);
            return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }
    
    private function saveStockCsv($data, $m_category_count) {
        
        try {
            for ($x = 0; $x < sizeof($data); $x++) {
                $row =  $data[$x];
                $itemCount = $this->countStockCsvRows($row[1]);
    
                if ($itemCount < $m_category_count) {
                    StockCsvTemp::create([
                        'p_category' => preg_replace('/[^A-Za-z0-9 ]/', '', $row[0]),
                        'p_name' => preg_replace('/[^A-Za-z0-9 ]/', '', $row[1]),
                        'qty' => preg_replace('/[^A-Za-z0-9. ]/', '', $row[2]),
                        'c_price' => preg_replace('/[^A-Za-z0-9. ]/', '', $row[3]),
                        'reorder_level' => preg_replace('/[^A-Za-z0-9. ]/', '', $row[4]),
                        'bar_code' => preg_replace('/[^A-Za-z0-9. ]/', '', $row[5]),
                        'mmetric' => preg_replace('/[^A-Za-z0-9 ]/', '', $row[6]),
                        'm_qty' => preg_replace('/[^A-Za-z0-9. ]/', '', $row[7]),
                        's_price' => preg_replace('/[^A-Za-z0-9. ]/', '', $row[8]),
                        'user_id' => Auth::user()->id,
                        'location_id' => Auth::user()->location_id,
                        'client_id' => Auth::user()->client_id,
                        'date' => Carbon::now()->format('Y-m-d')
                    ]);
                }
            }
        } catch (\Exception $e) {
            throw new Exception("Error saving stock CSV: " . $e->getMessage());
        }
    }
    
    private function uploadedInventory($stockCsvData, $m_category_id, $store="") {
        try {
            $allowedItems = $this->locationController->allowedItems(Auth::user()->client_id, Auth::user()->location_id);
            $m_qty = $stockCsvData[0]['m_qty'];
            $token = Uuid::uuid4()->toString();
            $stockNum = 0;
            $currency_id = $this->currencyController->getPrimaryCurrency(Auth::user()->client_id, Auth::user()->location_id);
            $currencyUnit = $this->currencyController->getCurrencyUnit(Auth::user()->client_id, Auth::user()->location_id , $currency_id);
            $store_id = !empty($store) ? $store : $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
            $txn_date = Carbon::now()->format('Y-m-d');
    
            for ($i = 0; $i < sizeof($stockCsvData); $i++) {
                $Y = $stockCsvData[$i];
                $itemCategory = $Y['p_category'];
                $itemName = $Y['p_name'];
                $costPrice = $Y['c_price'];
                $reorderLevel = $Y['reorder_level'];
                $barCode = $Y['bar_code'];
                $price = $Y['s_price'];
                $qty = $Y['qty'];
                $calQty = $Y['m_qty'];
                $mmetric = $Y['mmetric'];
                $primaryMeasurement = $mmetric == 1 ? 1 : 0;
    
                if ($mmetric == 1) {
                    $m_qty = $stockCsvData[$i]['m_qty'];
                } elseif ($mmetric > 1) {
                    if ($calQty > 0) {
                        $m_qty = $calQty / $stockCsvData[$i - 1]['m_qty']; 
                    } else {
                        $m_qty = 0;
                    }
                }
    
                $isCategory = ItemCategory::where('name', strtolower($itemCategory))
                    ->where('client_id', Auth::user()->client_id)
                    ->where('status', STATUS)
                    ->count();
    
                if ($isCategory < ONE_STATUS) {
                    $uuid = Uuid::uuid4()->toString();
                    $uuidWithoutDashes = str_replace('-', '', $uuid);
                    ItemCategory::create([
                        'id' => $uuidWithoutDashes,
                        'item_class_id' => STATUS,
                        'name' => $itemCategory,
                        'income_account' => CHART_SALES,
                        'status' => STATUS,
                        'user_id' => Auth::user()->id,
                        'location_id' => Auth::user()->location_id,
                        'client_id' => Auth::user()->client_id,
                        'tm' => Carbon::now()->toTimeString(),
                        'date' => Carbon::now()->format('Y-m-d')
                    ]);
                }
    
                $isItemId = Items::where('status', STATUS)
                    ->where('client_id', Auth::user()->client_id)
                    ->where('name', strtolower($itemName))
                    ->count();
    
                $itemCategoryId = ItemCategory::where('name', strtolower($itemCategory))
                    ->where('client_id', Auth::user()->client_id)
                    ->where('status', STATUS)
                    ->value('id');
    
                if ($isItemId < ONE_STATUS) {
                    $uuid = Uuid::uuid4()->toString();
                    $uuidWithoutDashes = str_replace('-', '', $uuid);
                    Items::create([
                        'id' => $uuidWithoutDashes,
                        'item_class_id' => 0,
                        'item_category_id' => $itemCategoryId,
                        'name' => $itemName,
                        'reorder_level' => $reorderLevel,
                        'barcode' => $barCode,
                        'vat_status' => 0,
                        'sku' => $this->createSku($itemName),
                        'sales_type' => ONE_STATUS,
                        'income_account' => CHART_SALES,
                        'm_category_id' => $m_category_id,
                        'status' => STATUS,
                        'disabled_status' => STATUS,
                        'user_id' => Auth::user()->id,
                        'location_id' => Auth::user()->location_id,
                        'client_id' => Auth::user()->client_id,
                        'tm' => Carbon::now()->toTimeString(),
                        'date' => Carbon::now()->format('Y-m-d'),
                        'update_date' => Carbon::now()->format('Y-m-d'),
                    ]);
                }
    
                $itemId = Items::where('name', strtolower($itemName))
                    ->where('client_id', Auth::user()->client_id)
                    ->where('status', STATUS)
                    ->value('id');
    
                $countPrice = $this->priceController->countPriceByItemIdAndMetric($itemId, $mmetric);
    
                if ($countPrice < 1) {
                    $uuid = Uuid::uuid4()->toString();
                    $uuidWithoutDashes = str_replace('-', '', $uuid);
                    ItemPrice::create([
                        'id' => $uuidWithoutDashes,
                        'rate' => $price,
                        'item_id' => $itemId,
                        'm_metric' => $mmetric,
                        'primary_measurement' => $primaryMeasurement,
                        'qty' => $m_qty,
                        'cal_qty' => $calQty,
                        'control_id' => STATUS,
                        'item_category_id' => $itemCategoryId,
                        'item_class_id' => STATUS,
                        'm_category_id' => $m_category_id,
                        'default_price' => 1,
                        'status' => STATUS,
                        'cp' => $primaryMeasurement == ONE_STATUS ? $costPrice : STATUS,
                        'user_id' => Auth::user()->id,
                        'location_id' => Auth::user()->location_id,
                        'client_id' => Auth::user()->client_id,
                        'tm' => Carbon::now()->toTimeString(),
                        'date' => Carbon::now()->format('Y-m-d'),
                        'update_date' => Carbon::now()->format('Y-m-d'),
                    ]);
                }
    
                // $isStockExist = Stock::where('control_id', '!=', ONE_STATUS)
                //     ->where('item_id', $itemId)
                //     ->where('client_id', Auth::user()->client_id)
                //     ->where('location_id', Auth::user()->location_id)
                //     ->first();

                $stockNum = Stock::where('control_id', '!=', ONE_STATUS)
                ->where('client_id', Auth::user()->client_id)
                ->where('location_id', Auth::user()->location_id)
                ->distinct('item_id')
                ->count();
    
                // if (empty($isStockExist)) {
                    if ($costPrice > STATUS && $qty > STATUS) {
                        // if ($stockNum < $allowedItems || (!empty($isStockExist) && $stockNum >= $allowedItems)) {
                        if($stockNum < $allowedItems){
                            $amount = $costPrice * $qty;
                            $this->insertStock($token, $txn_date, $itemId, "", $itemCategoryId, $qty, $costPrice, $currency_id, $mmetric, $m_category_id, $store_id, EFFECT_DEBIT, null, $amount);
                        }
                        // }
                    }
                // }
            }
    
            $stockValue = $this->getStockValue($token);
            if ($stockValue > STATUS) {
                $accLegs = $this->chartOfAccountController->getAccountLegs(CHART_STOCK, CHART_SUSPENSE, Auth::user()->client_id);
                $this->txnLogController->createTxnLog($accLegs, $stockValue, $token, $currencyUnit, $txn_date, "", 'Being Previous Balance', "", 'pb', $currency_id);
            }
    
            StockCsvTemp::where('client_id', Auth::user()->client_id)
                ->where('location_id', Auth::user()->location_id)
                ->delete();
        } catch (\Exception $e) {
            throw new Exception("Error uploading inventory: " . $e->getMessage());
        }
    }
    
    public function getStockValue($token) {
        return Stock::where('control_id', '!=', ONE_STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('token', $token)
        ->sum('amount');
    }

    protected function createSku($name)
    {
        $randomNumber = mt_rand(100, 999);
        $prefix = substr($name, 0, 2);

        return $prefix . $randomNumber . now()->timestamp;
    }

    private function getStockCsvInfo() {
        return StockCsvTemp::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('user_id', Auth::user()->id)
        ->orderBy('id', 'asc')
        ->get();
    }

    private function countStockCsvRows($pname) {
        return StockCsvTemp::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('p_name', $pname)
        ->count();
    }

    public function getInventory(Request $request) {
        $client_id = Auth::user()->client_id;
        
        $item_id_for_products = !empty($request->item_id) ? " AND a.id='$request->item_id'" : '';
        $item_id_for_inventory = !empty($request->item_id) ? " AND item_id='$request->item_id'" : '';
        $item_id_for_primary = !empty($request->item_id) ? " AND p.item_id='$request->item_id'" : '';
        $item_id_for_item_price = !empty($request->item_id) ? " AND b.item_id='$request->item_id'" : '';

        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        $primaryStore = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $storeId = !empty($request->store) ? "AND store_id='$request->store'" : "AND store_id='$primaryStore'";
        $search = !empty($request->text) ? "AND p.name LIKE '%$request->text%'" : "";
        $date = !empty($request->from) && !empty($request->to) ? "AND txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $sortUser = !empty($request->user) ? "AND user_id='$request->user'" : "";
        $category = !empty($request->category) ? "AND a.item_category_id='$request->category'" : "";
        $limit = "LIMIT $request->count, 50";
        $limit2 = "LIMIT $request->count_customer, 50";
        $searchCustomer = !empty($request->search_customer) ? "AND name LIKE '%$request->search_customer%'" : "";

        $query = " 
            WITH inventory AS (
                SELECT * FROM " . ONLINE_RETAILS .".stock
                WHERE control_id != 1  and client_id = '$client_id' and location_id = '$location_id' $sortUser $date $storeId $item_id_for_inventory
            ),
            customerNumber As (
                SELECT count(distinct coy) as buyers, item_id 
                FROM " . ONLINE_RETAILS .".item_log 
                WHERE control_id != 1
                GROUP BY item_id
            ),
            products AS (
                SELECT a.id, a.name,b.m_category_id, b.m_metric ,a.item_category_id ,b.cal_qty, b.rate, a.sku, a.disabled_status, c.buyers, c.name as category, a.status, a.date as dateCreated, mc.m_category_id as mcat

                FROM " . ONLINE_RETAILS .".items a 

                LEFT JOIN " . ONLINE_RETAILS .".item_price as b ON a.id = b.item_id and b.client_id = '$client_id' and b.location_id = '$location_id' and b.status=0 $item_id_for_item_price

                LEFT JOIN " . ONLINE_RETAILS .".item_category as c ON c.id = a.item_category_id and c.client_id = '$client_id'

                LEFT JOIN " . ONLINE_RETAILS .".measurement as m ON m.m_category_id = b.m_category_id AND m.hierarchy = b.m_metric and m.client_id = '$client_id' and m.location_id = '$location_id'
                
                LEFT JOIN " . ONLINE_RETAILS .".measurement as mc ON mc.m_category_id = a.m_category_id AND mc.client_id = '$client_id' and mc.location_id = '$location_id' AND mc.hierarchy = 1

                LEFT JOIN customerNumber c
                ON c.item_id=a.id
                WHERE a.client_id = '$client_id' $item_id_for_products and a.status = 0 and a.disabled_status = 0 $category 
            ),

            pmQty As (
                SELECT p.cal_qty as pQty, p.item_id, p.rate, m.abr, m.name as mname
                FROM " . ONLINE_RETAILS .".item_price p
                LEFT JOIN " . ONLINE_RETAILS .".measurement as m ON m.m_category_id = p.m_category_id AND  m.hierarchy = p.m_metric and m.client_id = '$client_id' and m.location_id = '$location_id' AND m.mstatus = 1
                WHERE p.primary_measurement = 1  and p.client_id = '$client_id' and p.status=0 and p.location_id = '$location_id' $item_id_for_primary
            ),

            inventoryDetail As (
                SELECT s.id, p.id as item_id, s.token,p.name ,s.m_metric ,s.effect, s.qty, p.m_category_id, p.item_category_id,p.cal_qty,s.price,s.amount, p.rate, p.sku, p.disabled_status, p.buyers, p.category, p.dateCreated, p.mcat

                FROM products p
                LEFT JOIN inventory s ON s.item_id = p.id AND s.m_metric = p.m_metric
                
                WHERE p.status = 0
                $search
                order by name
            ),	
       
            newStockQty As (
                SELECT a.*, b.rate as pr, b.pQty, a.cal_qty * a.qty as uQty, b.abr, b.mname,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'value', t.m_metric,
                        'title', m.name
                    ))

                    FROM " . TBL_ITEM_PRICE . " t
                    
                    LEFT JOIN " . TBL_MEASUREMENT . " m
                    ON m.m_category_id = a.m_category_id AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0 AND t.m_metric IS NOT NULL AND t.m_metric=m.hierarchy

                    WHERE t.item_id=a.item_id AND t.client_id='$client_id' AND t.location_id='$location_id' AND t.status = 0 AND a.item_id IS NOT NULL
                    GROUP BY t.item_id
                ) AS itemMeasurement
                
                FROM inventoryDetail a
                LEFT JOIN pmQty b ON b.item_id = a.item_id
            ),

            crStockQty  As (
                SELECT item_id, name, m_category_id,item_category_id, effect, pr as rate,
                round(SUM(uQty/pQty),2) as dispensed, 
                round(SUM(amount),2) as crAmount 
                FROM newStockQty
                WHERE effect = 'cr'
                group by item_id
            ),

            drStockQty  As (
                SELECT item_id, name, cal_qty, m_category_id,item_category_id, effect, pr as rate, 
                SUM(coalesce(uQty/pQty,0)) as received, mname, abr,
                SUM(amount) as drAmount ,
                coalesce((SUM(amount) / SUM(uQty)),0) as cp,
                sku, disabled_status, buyers, category, itemMeasurement, dateCreated, mcat

                FROM newStockQty
                WHERE effect = 'dr' OR effect IS NULL
                group by item_id
            ),

            inventoryBal As (
                SELECT dr.item_id, dr.name, dr.rate, dr.m_category_id, dr.item_category_id, received, dr.cp, 
                coalesce(dispensed,0) as dispensed , dr.sku, dr.disabled_status, dr.buyers, dr.mname, dr.abr,
                round(received - coalesce(dispensed,0) ,2) as closingBal ,
                drAmount, coalesce(crAmount,0) as crAmount, 
                round(((received - coalesce(dispensed,0)) * dr.cp) ,2) as amount, 
                round(((received - coalesce(dispensed,0)) * dr.rate) ,2) as RetailValue,
                dr.category, itemMeasurement, dr.dateCreated, dr.mcat
                
                FROM drStockQty dr
                LEFT JOIN  crStockQty cr
                ON  cr.item_id = dr.item_id
                
                ORDER BY dr.name
                $limit
            ),

            inventory_analysis as(
                SELECT *,coalesce(round(((RetailValue - amount)/RetailValue)*100,2),0) as pm ,
                coalesce(round(((RetailValue - amount)/amount)*100,2),0) as mkup
                FROM inventoryBal 
            ),

            overall_invBal as (
                SELECT item_id as id, name,m_category_id,item_category_id,round(received,2) as qty, dispensed,rate,
                drAmount as DebitAMT, crAmount as CreditAMT,round(cp,2) as cp,closingBal as closingBalance, amount, RetailValue,
                pm, mkup, sku, disabled_status, buyers, category, mname, abr, itemMeasurement, dateCreated, mcat
                FROM inventory_analysis
            ),
                
            summary as (
                SELECT count(*) as items, sum(RetailValue)as totalRetailValue, sum(amount) as totalStockBalance 
                FROM overall_invBal
            ),
                
            summary_analysis as(
                SELECT *,  
                round(((totalRetailValue - totalStockBalance)/totalRetailValue)*100,2) as totalPm,
                round(((totalRetailValue - totalStockBalance)/totalStockBalance)*100,2) as totalMkup 
                FROM Summary
            ),
            
            combined as (
            SELECT * FROM overall_invBal CROSS JOIN summary_analysis),
            
            combinedArr as (
                SELECT  JSON_ARRAYAGG(JSON_OBJECT(
                    'id', c.id,
                    'name', c.name,
                    'm_category_id', c.m_category_id,
                    'mcat',c.mcat,
                    'item_category_id', c.item_category_id,
                    'qty', c.qty,
                    'dispensed', c.dispensed,
                    'rate', c.rate,
                    'DebitAMT', c.DebitAMT,
                    'CreditAMT', c.CreditAMT,
                    'cp', c.cp,
                    'closingBalance', c.closingBalance,
                    'amount', c.amount,
                    'RetailValue', c.RetailValue,
                    'pm', c.pm,
                    'mkup', c.mkup,
                    'sku', c.sku,
                    'disabled_status', c.disabled_status,
                    'buyers', c.buyers,
                    'category', c.category,
                    'mname', c.mname,
                    'abr', c.abr,
                    'totalPm', c.totalPm,
                    'totalMkup', c.totalMkup,
                    'items', items,
                    'totalRetailValue',totalRetailValue,
                    'totalStockBalance',totalStockBalance,
                    'dateCreated',c.dateCreated
                    
                )) as inventories 
            FROM combined c), 

            store as (
                SELECT 
                id as value,
                name as title,
                primary_status
                FROM 
                " . TBL_STORE . "
                WHERE client_id='$client_id' AND location_id='$location_id' AND status = " . STATUS . "
            ),

            storeArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', s.value,
                    'title', s.title,
                    'primary_store', s.primary_status
                )) as stores
                FROM store s),

            currency as (
                SELECT
                c.id as value,
                x.name as title,
                c.primary_currency
                FROM
                " . TBL_CURRENCY . " c
                LEFT JOIN " . TBL_CURRENCY_CATEGORY . " x 
                ON c.currency_cate_id=x.id AND x.status=" . STATUS . "
                WHERE c.client_id='$client_id' AND c.location_id='$location_id' AND c.status=" . STATUS . "
            ),

            currencyArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', c.value,
                    'title', c.title,
                    'primary_currency', c.primary_currency
                )) as currencies
                FROM currency c),

            category as (
                SELECT
                id as value,
                name as title
                FROM
                " . TBL_ITEM_CATEGORY . "

                WHERE client_id='$client_id' AND status=" . STATUS . "
            ),

            categoryArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', c.value,
                    'title', c.title
                )) as categories
                FROM category c),

            adjustmentAccount as (
                SELECT
                code as value,
                name as title
                FROM
                " . TBL_CHART_OF_ACCOUNTS . "

                WHERE client_id='$client_id' AND location_id='$location_id' AND status=" . STATUS . "
            ),

            adjustmentAccountArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', c.value,
                    'title', c.title
                )) as adjustmentAccounts
                FROM adjustmentAccount c),

            incomeAccount as (
                SELECT
                code as value,
                name as title
                FROM
                " . TBL_CHART_OF_ACCOUNTS . "

                WHERE client_id='$client_id' AND location_id='$location_id' AND sort_code=" . SORT_REVENUE . " AND status=" . STATUS . "
            ),

            incomeAccountArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', c.value,
                    'title', c.title
                )) as incomeAccounts
                FROM incomeAccount c),

            measurement as (
                SELECT
                *
                FROM
                " . TBL_MEASUREMENT . "
                WHERE client_id='$client_id' AND location_id='$location_id' AND status=" . STATUS . "
            ),

            measurementArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'title', c.name,
                    'value', c.hierarchy,
                    'abr',c.abr,
                    'mc',c.m_category_id
                )) as measurements
            FROM measurement c),

            normalMeasurement as (
                SELECT
                name as title,
                CONCAT(hierarchy, '-', m_category_id) as value
                FROM
                " . TBL_MEASUREMENT . "
                WHERE client_id='$client_id' AND location_id='$location_id' AND status=" . STATUS . "
            ),

            normalMeasurementArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'title', c.title,
                        'value', c.value
                    )) as normalMeasurements
                    FROM normalMeasurement c),

            customer as (
                SELECT
                name as title,
                id as value

                FROM " . TBL_CUSTOMERS . "

                WHERE client_id='$client_id' AND name != 'walk-in customer' AND status=" . STATUS ." $searchCustomer
                $limit
            ),

            customerArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'title', c.title,
                    'value', c.value
                    
                )) as customers
                FROM customer c),

                bankAccount as (
                    SELECT i.id as value, CONCAT(i.name, ' - ', b.name) as title FROM " . TBL_BANK_ACCOUNT . " i
                    LEFT JOIN " . TBL_APP_BANK . " b
                    ON b.id=i.bank_id AND b.status=" . STATUS . " AND b.status = " . STATUS . "
                    WHERE i.client_id = '$client_id' AND i.status = " . STATUS . "
                ),

                bankAccountArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'title', c.title,
                        'value', c.value
                        
                    )) as bankAccounts
                    FROM bankAccount c)

            SELECT * FROM 
            combinedArr,storeArr,currencyArr,categoryArr, adjustmentAccountArr, incomeAccountArr, measurementArr, customerArr, normalMeasurementArr,bankAccountArr   
        ";
        // echo $query; exit;
        $data = DB::select($query);

        foreach ($data as $row) {
            if (property_exists($row, 'inventories')) {
                $row->inventories = $row->inventories ? json_decode(stripslashes($row->inventories), true) : [];
            }
    
            if (property_exists($row, 'stores')) {
                $row->stores = json_decode(stripslashes($row->stores), true);
            }

            if (property_exists($row, 'currencies')) {
                $row->currencies = json_decode(stripslashes($row->currencies), true);
            }

            if (property_exists($row, 'categories')) {
                $row->categories = json_decode(stripslashes($row->categories), true);
            }

            if (property_exists($row, 'adjustmentAccounts')) {
                $row->adjustmentAccounts = json_decode(stripslashes($row->adjustmentAccounts), true);
            }

            if (property_exists($row, 'incomeAccounts')) {
                $row->incomeAccounts = json_decode(stripslashes($row->incomeAccounts), true);
            }

            if (property_exists($row, 'measurements')) {
                $row->measurements = json_decode(stripslashes($row->measurements), true);
            }

            if (property_exists($row, 'normalMeasurements')) {
                $row->normalMeasurements = json_decode(stripslashes($row->normalMeasurements), true);
            }

            if (property_exists($row, 'customers')) {
                $row->customers = json_decode(stripslashes($row->customers), true);
            }

            if (property_exists($row, 'bankAccounts')) {
                $row->bankAccounts = json_decode(stripslashes($row->bankAccounts), true);
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function getOptimisedInventory(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        
        $item_id_for_products = !empty($request->item_id) ? " AND p.id='$request->item_id'" : '';
        $item_id_for_inventory = !empty($request->item_id) ? " AND s.item_id='$request->item_id'" : '';
        $item_id_for_primary = !empty($request->item_id) ? " AND p.item_id='$request->item_id'" : '';
        $item_id_for_item_price = !empty($request->item_id) ? " AND b.item_id='$request->item_id'" : '';

        $primaryStore = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $storeId = !empty($request->store) ? "AND s.store_id='$request->store'" : "AND s.store_id='$primaryStore'";
        $search = !empty($request->text) ? "AND p.name LIKE '%$request->text%'" : "";
        $category = !empty($request->category) ? "AND p.item_category_id='$request->category'" : "";
        $date = !empty($request->from) && !empty($request->to) ? "AND s.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $sortUser = !empty($request->user) ? "AND s.user_id='$request->user'" : "";
        $limit = "LIMIT $request->count, 50";
        // $limit2 = "LIMIT $request->count_customer, 50";
        $searchCustomer = !empty($request->search_customer) ? "AND name LIKE '%$request->search_customer%'" : "";
        
        $query = "
            WITH customerNumber As (
                SELECT count(distinct coy) as buyers, item_id 
                FROM " . ONLINE_RETAILS .".item_log 
                WHERE control_id != 1
                GROUP BY item_id
            ),
            
            products AS (
                SELECT 
                    p.id, 
                    p.name,
                    c.buyers, 
                    p.m_category_id,
                    c.name as category,
                    p.item_category_id, 
                    p.sku, 
                    p.disabled_status,
                    p.status, 
                    p.date as dateCreated, 
                    mc.m_category_id as mcat
                    
                    
                FROM ".TBL_ITEMS." p

                LEFT JOIN customerNumber c
                    ON c.item_id=p.id

                LEFT JOIN " . TBL_ITEM_CATEGORY ." c 
                    ON c.id = p.item_category_id AND c.client_id = '$client_id'
                
                LEFT JOIN " . TBL_MEASUREMENT ." mc ON mc.m_category_id = p.m_category_id AND mc.client_id = '$client_id' and mc.location_id = '$location_id' AND mc.hierarchy = 1

                WHERE p.client_id='$client_id' AND p.status=0 $item_id_for_products $category $search

                GROUP BY 
                    p.id, p.name
                ORDER BY 
                    p.id
                $limit
            ),

            drInventory as (
                SELECT 
                    p.*,
                    SUM(s.amount) as drAmount,
                    COALESCE(ROUND(SUM(s.qty * i.cal_qty), 2),0) as received,
                    COALESCE(ROUND(SUM(s.amount) / SUM(s.qty * i.cal_qty), 2),0) as cp,
                    (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'value', t.m_metric,
                            'title', m.name
                        ))
    
                        FROM " . TBL_ITEM_PRICE . " t
                        
                        LEFT JOIN " . TBL_MEASUREMENT . " m
                        ON m.m_category_id = p.m_category_id AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0 AND t.m_metric IS NOT NULL AND t.m_metric=m.hierarchy
    
                        WHERE t.item_id=p.id AND t.client_id='$client_id' AND t.location_id='$location_id' AND t.status = 0 AND p.id IS NOT NULL
                        GROUP BY t.item_id
                    ) AS itemMeasurement
                
                FROM products p
                
                LEFT JOIN ".TBL_STOCK." s 
                    ON p.id = s.item_id AND s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND (s.effect = 'dr' OR s.effect IS NULL) $sortUser $date $storeId $item_id_for_inventory

                LEFT JOIN ".TBL_ITEM_PRICE." i 
                    ON p.id = i.item_id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.m_metric=s.m_metric AND i.status = 0
                GROUP by p.id
            ),

            crInventory as (
                SELECT 
                    p.*,
                    COALESCE(ROUND(SUM(s.qty * i.cal_qty), 0)) as dispensed,
                    SUM(s.amount) as crAmount
                    
                FROM products p
                
                LEFT JOIN ".TBL_STOCK." s 
                    ON p.id = s.item_id AND s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND s.effect = 'cr' $sortUser $date $storeId $item_id_for_inventory

                LEFT JOIN ".TBL_ITEM_PRICE." i 
                    ON p.id = i.item_id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.m_metric=s.m_metric AND i.status = 0
                GROUP by p.id
            ),

            pmQty As (
                SELECT p.cal_qty as pQty, p.item_id, p.rate, m.abr, m.name as mname
                FROM " . ONLINE_RETAILS .".item_price p
                LEFT JOIN " . ONLINE_RETAILS .".measurement as m ON m.m_category_id = p.m_category_id AND  m.hierarchy = p.m_metric and m.client_id = '$client_id' and m.location_id = '$location_id' AND m.mstatus = 1
                WHERE p.primary_measurement = 1  and p.client_id = '$client_id' and p.status=0 and p.location_id = '$location_id' $item_id_for_primary
            ),

            totalInventory as (
                SELECT 
                dr.*,
                pr.*,
                cr.dispensed,
                cr.crAmount,
                ROUND(((dr.received - coalesce(cr.dispensed,0)) * dr.cp) ,2) as amount, 
                ROUND(((dr.received - coalesce(cr.dispensed,0)) * pr.rate) ,2) as RetailValue,
                ROUND(COALESCE(ROUND(dr.received,0),0) - COALESCE(ROUND(cr.dispensed,0),0) / COALESCE(ROUND(pr.pQty,0)),2) as closingBal
                FROM drInventory dr

                LEFT JOIN crInventory cr
                ON cr.id=dr.id

                LEFT JOIN pmQty pr
                ON pr.item_id=dr.id
            ),

            inventory_analysis as(
                SELECT *,coalesce(round(((RetailValue - amount)/RetailValue)*100,2),0) as pm ,
                coalesce(round(((RetailValue - amount)/amount)*100,2),0) as mkup
                FROM totalInventory 
            ),
            
            overall_invBal as (
                SELECT id, name, m_category_id, item_category_id, round(received,2) as qty, dispensed,rate,
                drAmount as DebitAMT, crAmount as CreditAMT,round(cp,2) as cp,closingBal as closingBalance, amount, RetailValue,
                pm, mkup, sku, disabled_status, buyers, category, mname, abr, itemMeasurement, dateCreated, mcat
                FROM inventory_analysis
            ),
            
            
            combined as (
            SELECT * FROM overall_invBal),
            
            
            combinedArr as (
                SELECT  JSON_ARRAYAGG(JSON_OBJECT(
                    'id', c.id,
                    'name', c.name,
                    'm_category_id', c.m_category_id,
                    'mcat',c.mcat,
                    'item_category_id', c.item_category_id,
                    'qty', c.qty,
                    'dispensed', c.dispensed,
                    'rate', c.rate,
                    'DebitAMT', c.DebitAMT,
                    'CreditAMT', c.CreditAMT,
                    'cp', round(c.cp, 2),
                    'closingBalance', round(c.closingBalance, 2),
                    'amount', round(c.amount, 2),
                    'RetailValue', c.RetailValue,
                    'pm', round(c.pm,2),
                    'mkup', round(c.mkup,2),
                    'sku', c.sku,
                    'disabled_status', c.disabled_status,
                    'buyers', c.buyers,
                    'category', c.category,
                    'mname', c.mname,
                    'abr', c.abr,
                    'dateCreated',c.dateCreated
                    
                )) as inventories 
            FROM combined c)

            SELECT * FROM 
            combinedArr;

        ";

        // echo $query;exit;

        $data = DB::select($query);

        foreach ($data as $row) {
            if (property_exists($row, 'inventories')) {
                $row->inventories = $row->inventories ? json_decode(stripslashes($row->inventories), true) : [];
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function getInventoryDependencies(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        $primaryStore = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $storeId = !empty($request->store) ? "AND s.store_id='$request->store'" : "AND s.store_id='$primaryStore'";
        $date = !empty($request->from) && !empty($request->to) ? "AND s.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $sortUser = !empty($request->user) ? "AND s.user_id='$request->user'" : "";
        $item_id_for_inventory = !empty($request->item_id) ? " AND s.item_id='$request->item_id'" : '';
        $searchCustomer = !empty($request->search_customer) ? "AND name LIKE '%$request->search_customer%'" : "";
        $limit = "LIMIT $request->count, 50";


        $query="
            WITH stockNumber as (
                SELECT count(DISTINCT item_id) as items
                FROM ".TBL_STOCK."
                WHERE control_id != 1 AND client_id='$client_id' AND location_id='$location_id'
            ),

            drAmounts as (
                SELECT
                s.item_id,
                sum(i.cal_qty * s.qty) as qty,
                
                s.amount / (s.qty * i.cal_qty) as cp,
                i.rate
                    
                FROM ".TBL_STOCK." s 
                    
                LEFT JOIN ".TBL_ITEM_PRICE." i 
                    ON s.item_id = i.item_id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.m_metric=s.m_metric AND i.status = 0

                WHERE s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND s.effect = 'dr' $sortUser $date $storeId $item_id_for_inventory
                
                GROUP BY i.item_id
                
            ),

            crAmounts as (
                SELECT
                s.item_id,
                sum(i.cal_qty * s.qty) as qty
                    
                FROM ".TBL_STOCK." s 
                    
                LEFT JOIN ".TBL_ITEM_PRICE." i 
                    ON s.item_id = i.item_id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.m_metric=s.m_metric AND i.status = 0

                WHERE s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND s.effect = 'cr' $sortUser $date $storeId $item_id_for_inventory
                
                GROUP BY i.item_id
                
            ),

            stocklist as (
                SELECT
                sum(coalesce(dr.qty - cr.qty, dr.qty)) as qty,

                sum(coalesce((dr.qty - cr.qty) * dr.cp,dr.qty*dr.cp)) as amount,

                sum(coalesce((dr.qty-cr.qty) * dr.rate,dr.qty*dr.rate)) as retailAmount

                from drAmounts dr
                left join crAmounts cr
                on dr.item_id=cr.item_id
            ),

            Summary as (
                SELECT
                
                COALESCE(ROUND(SUM(l.amount),2),0) as totalStockBalance,
                COALESCE(ROUND(SUM(l.retailAmount),2),0) as totalRetailValue
                FROM stocklist l
                
                ),
                
                summary_analysis as(
                SELECT 
                *,
                COALESCE(round(((totalRetailValue - totalStockBalance)/totalRetailValue)*100,2),0) as totalPm,
                COALESCE(round(((totalRetailValue - totalStockBalance)/totalStockBalance)*100,2),0) as totalMkup
                FROM Summary
                CROSS JOIN stockNumber
                ),
                
            summaryAnalysisArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'totalRetailValue', s.totalRetailValue,
                    'totalStockBalance', s.totalStockBalance,
                    'totalPm', s.totalPm,
                    'totalMkup',s.totalMkup,
                    'items',s.items
                )) as summary_analysis
                FROM summary_analysis s),
            
            store as (
                SELECT 
                id as value,
                name as title,
                primary_status
                FROM 
                " . TBL_STORE . "
                WHERE client_id='$client_id' AND location_id='$location_id' AND status = " . STATUS . "
            ),

            storeArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', s.value,
                    'title', s.title,
                    'primary_store', s.primary_status
                )) as stores
                FROM store s),

            currency as (
                SELECT
                c.id as value,
                x.name as title,
                c.primary_currency
                FROM
                " . TBL_CURRENCY . " c
                LEFT JOIN " . TBL_CURRENCY_CATEGORY . " x 
                ON c.currency_cate_id=x.id AND x.status=" . STATUS . "
                WHERE c.client_id='$client_id' AND c.location_id='$location_id' AND c.status=" . STATUS . "
            ),

            currencyArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', c.value,
                    'title', c.title,
                    'primary_currency', c.primary_currency
                )) as currencies
                FROM currency c),

            category as (
                SELECT
                id as value,
                name as title
                FROM
                " . TBL_ITEM_CATEGORY . "

                WHERE client_id='$client_id' AND status=" . STATUS . "
            ),

            categoryArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', c.value,
                    'title', c.title
                )) as categories
                FROM category c),

            adjustmentAccount as (
                SELECT
                code as value,
                name as title
                FROM
                " . TBL_CHART_OF_ACCOUNTS . "

                WHERE client_id='$client_id' AND location_id='$location_id' AND status=" . STATUS . "
            ),

            adjustmentAccountArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', c.value,
                    'title', c.title
                )) as adjustmentAccounts
                FROM adjustmentAccount c),

            incomeAccount as (
                SELECT
                code as value,
                name as title
                FROM
                " . TBL_CHART_OF_ACCOUNTS . "

                WHERE client_id='$client_id' AND location_id='$location_id' AND sort_code=" . SORT_REVENUE . " AND status=" . STATUS . "
            ),

            incomeAccountArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'value', c.value,
                    'title', c.title
                )) as incomeAccounts
                FROM incomeAccount c),

            measurement as (
                SELECT
                *
                FROM
                " . TBL_MEASUREMENT . "
                WHERE client_id='$client_id' AND location_id='$location_id' AND status=" . STATUS . "
            ),

            measurementArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'title', c.name,
                    'value', c.hierarchy,
                    'abr',c.abr,
                    'mc',c.m_category_id
                )) as measurements
            FROM measurement c),

            normalMeasurement as (
                SELECT
                name as title,
                CONCAT(hierarchy, '-', m_category_id) as value
                FROM
                " . TBL_MEASUREMENT . "
                WHERE client_id='$client_id' AND location_id='$location_id' AND status=" . STATUS . "
            ),

            normalMeasurementArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'title', c.title,
                        'value', c.value
                    )) as normalMeasurements
                    FROM normalMeasurement c),

            customer as (
                SELECT
                name as title,
                id as value

                FROM " . TBL_CUSTOMERS . "

                WHERE client_id='$client_id' AND name != 'walk-in customer' AND status=" . STATUS ." $searchCustomer
                $limit
            ),

            customerArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'title', c.title,
                    'value', c.value
                    
                )) as customers
                FROM customer c),

                bankAccount as (
                    SELECT i.id as value, CONCAT(i.name, ' - ', b.name) as title FROM " . TBL_BANK_ACCOUNT . " i
                    LEFT JOIN " . TBL_APP_BANK . " b
                    ON b.id=i.bank_id AND b.status=" . STATUS . " AND b.status = " . STATUS . "
                    WHERE i.client_id = '$client_id' AND i.status = " . STATUS . "
                ),

                bankAccountArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'title', c.title,
                        'value', c.value
                        
                    )) as bankAccounts
                    FROM bankAccount c)

                    SELECT * FROM 
                    summaryAnalysisArr, storeArr,currencyArr,categoryArr, adjustmentAccountArr, incomeAccountArr, measurementArr, customerArr, normalMeasurementArr,bankAccountArr;
        ";

        $data = DB::select($query);

        foreach ($data as $row) {
            if (property_exists($row, 'summary_analysis')) {
                $row->summary_analysis = json_decode(stripslashes($row->summary_analysis), true);
            }

            if (property_exists($row, 'stores')) {
                $row->stores = json_decode(stripslashes($row->stores), true);
            }

            if (property_exists($row, 'currencies')) {
                $row->currencies = json_decode(stripslashes($row->currencies), true);
            }

            if (property_exists($row, 'categories')) {
                $row->categories = json_decode(stripslashes($row->categories), true);
            }

            if (property_exists($row, 'adjustmentAccounts')) {
                $row->adjustmentAccounts = json_decode(stripslashes($row->adjustmentAccounts), true);
            }

            if (property_exists($row, 'incomeAccounts')) {
                $row->incomeAccounts = json_decode(stripslashes($row->incomeAccounts), true);
            }

            if (property_exists($row, 'measurements')) {
                $row->measurements = json_decode(stripslashes($row->measurements), true);
            }

            if (property_exists($row, 'normalMeasurements')) {
                $row->normalMeasurements = json_decode(stripslashes($row->normalMeasurements), true);
            }

            if (property_exists($row, 'customers')) {
                $row->customers = json_decode(stripslashes($row->customers), true);
            }

            if (property_exists($row, 'bankAccounts')) {
                $row->bankAccounts = json_decode(stripslashes($row->bankAccounts), true);
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function inventorySummary($request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
            
        $query = "
            WITH inventory as 
            (SELECT 
            a.id as item_id,a.name,  s.qty* c.cal_qty *1.0/b.cal_qty as pQty,s.effect,c.rate,
            CASE 
                    WHEN s.effect = 'dr' THEN (s.amount * b.cal_qty * 1.0) / (c.cal_qty * s.qty) 
                    ELSE s.price 
                END AS price, 
            SUM( CASE WHEN s.effect ='dr' THEN (s.amount*1.0) ELSE NULL END)
            OVER ( PARTITION BY s.item_id ORDER BY s.txn_date,s.tm ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) /
            SUM( CASE WHEN s.effect ='dr' THEN (c.cal_qty* s.qty) ELSE NULL END)
            OVER ( PARTITION BY s.item_id ORDER BY s.txn_date,s.tm ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS running_cp
            FROM " . TBL_STOCK . " s
            INNER JOIN ".TBL_ITEMS." a ON a.id =s.item_id AND a.location_id = '$location_id'AND a.client_id = '$client_id' AND a.status = 0
            INNER JOIN ".TBL_ITEM_PRICE." c ON s.item_id = c.item_id AND c.m_metric = s.m_metric AND c.location_id = '$location_id' AND c.status = 0 AND (c.outlet_id  = s.store_id OR c.default_price = 1 or c.default_price = '' or c.default_price is NULL)
            LEFT JOIN ".TBL_ITEM_PRICE." b ON s.item_id = b.item_id AND b.primary_measurement = 1 AND b.location_id = '$location_id' AND b.status = 0 AND (b.outlet_id  = s.store_id OR b.default_price = 1 or b.default_price = '' or b.default_price is NULL )
            WHERE s.control_id != 1 AND s.client_id = '$client_id' AND s.location_id = '$location_id'
            GROUP BY  s.id, s.item_id, s.effect,s.m_metric
            ORDER BY  a.name, s.txn_date,s.tm),
        
            inventory_balances AS (
                SELECT name,
                    SUM(CASE WHEN d.effect = 'dr' THEN (d.price * d.pQty) ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN (d.running_cp * d.pQty) ELSE 0 END) AS amount,
                    (SUM(CASE WHEN d.effect = 'dr' THEN d.pQty ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN d.pQty ELSE 0 END))* d.rate AS RetailValue
                FROM inventory d
                Group by item_id
            )
            SELECT 
            SUM(amount) as amounts,
            SUM(RetailValue) as retailValues,
            ROUND(COALESCE(((SUM(RetailValue) - SUM(amount)) * 1.0 / SUM(RetailValue)) * 100, 0),2) as pm,
            ROUND(COALESCE(((SUM(RetailValue) - SUM(amount)) * 1.0 / SUM(amount)) * 100, 0),2) as mkup
            FROM inventory_balances
        ";

        $data = DB::select($query);
        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function cmpInventorySummary($request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
            
        $query = "
            WITH inventory as 
            (SELECT 
            a.id as item_id,a.name,  s.qty* c.cal_qty *1.0/b.cal_qty as pQty,s.effect,c.rate,
            b.cp AS price, 
            b.cp AS running_cp
            FROM " . TBL_STOCK . " s
            INNER JOIN ".TBL_ITEMS." a ON a.id =s.item_id AND a.location_id = '$location_id'AND a.client_id = '$client_id' AND a.status = 0
            INNER JOIN ".TBL_ITEM_PRICE." c ON s.item_id = c.item_id AND c.m_metric = s.m_metric AND c.location_id = '$location_id' AND c.status = 0 AND (c.outlet_id  = s.store_id OR c.default_price = 1 or c.default_price = '' or c.default_price is NULL)
            LEFT JOIN ".TBL_ITEM_PRICE." b ON s.item_id = b.item_id AND b.m_metric = 1 AND b.location_id = '$location_id' AND b.status = 0 AND (b.outlet_id  = s.store_id OR b.default_price = 1 or b.default_price = '' or b.default_price is NULL )
            WHERE s.control_id != 1 AND s.client_id = '$client_id' AND s.location_id = '$location_id'
            GROUP BY  s.id, s.item_id, s.effect,s.m_metric
            ORDER BY  a.name, s.txn_date,s.tm),
        
            inventory_balances AS (
                SELECT name,
                    SUM(CASE WHEN d.effect = 'dr' THEN (d.price * d.pQty) ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN (d.running_cp * d.pQty) ELSE 0 END) AS amount,
                    (SUM(CASE WHEN d.effect = 'dr' THEN d.pQty ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN d.pQty ELSE 0 END))* d.rate AS RetailValue
                FROM inventory d
                Group by item_id
            )
            SELECT 
            SUM(amount) as amounts,
            SUM(RetailValue) as retailValues,
            ROUND(COALESCE(((SUM(RetailValue) - SUM(amount)) * 1.0 / SUM(RetailValue)) * 100, 0),2) as pm,
            ROUND(COALESCE(((SUM(RetailValue) - SUM(amount)) * 1.0 / SUM(amount)) * 100, 0),2) as mkup
            FROM inventory_balances
        ";

        $data = DB::select($query);
        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function stockSummary(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;

        $inventoryEvaluation = Location::where("client_id", $client_id)
        ->where("id", $location_id)
        ->value("inventory_evaluation");

        if($inventoryEvaluation == STATUS) {
            return $this->inventorySummary($request);
        }

        if($inventoryEvaluation == ONE_STATUS) {
            return $this->cmpInventorySummary($request);
        }

        if($inventoryEvaluation == TWO_STATUS) {
            
        }
    }

    public function inventoryData($request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        $store_id = !empty($request->store_id) ? $request->store_id : $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $text = !empty($request->text) ? "AND a.name LIKE '%$request->text%'" : "";
        $category = !empty($request->category) ? "AND a.item_category_id = '$request->category'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? "AND s.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $user_id = !empty($request->user_id) ? "AND s.user_id='$request->user_id'" : "";
        $limit = "LIMIT $request->count, 50";
        
        $query="
            WITH products AS (
                SELECT DISTINCT a.id, a.name, a.status, a.item_category_id
                FROM ".TBL_ITEMS." a
                INNER JOIN ".TBL_STOCK." b 
                    ON b.item_id = a.id 
                    AND b.control_id != 1 
                    AND b.client_id = '$client_id' 
                    AND b.location_id = '$location_id'
                WHERE a.location_id = '$location_id'
                AND a.client_id = '$client_id'
                AND a.status = 0
                AND EXISTS (
                    SELECT 1
                    FROM ".TBL_ITEM_PRICE." c
                    WHERE c.item_id = a.id
                        AND c.primary_measurement = 1
                        AND c.status = 0
                ) $text $category
                GROUP BY a.id
                ORDER BY a.name
                $limit
            ),
            inventory AS (
                SELECT 
                    a.id AS item_id, 
                    a.name, 
                    s.qty * c.cal_qty * 1.0 / b.cal_qty AS pQty, 
                    s.m_metric, 
                    s.effect, 
                    b.rate, 
                    a.item_category_id, 
                    s.m_category_id, 
                    CASE 
                        WHEN s.effect = 'dr' THEN (s.amount * b.cal_qty * 1.0) / (c.cal_qty * s.qty) 
                        ELSE 0 
                    END AS price,
                    SUM(CASE WHEN s.effect = 'dr' THEN (s.amount * 1.0) ELSE NULL END)
                        OVER (PARTITION BY s.item_id ORDER BY s.txn_date, s.tm ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) /
                    SUM(CASE WHEN s.effect = 'dr' THEN (c.cal_qty * s.qty) ELSE NULL END)
                        OVER (PARTITION BY s.item_id ORDER BY s.txn_date, s.tm ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS running_cp,
                    ROW_NUMBER() OVER (
                        PARTITION BY s.item_id 
                        ORDER BY s.txn_date DESC, s.tm DESC
                    ) AS rn
                FROM products a
                LEFT JOIN ".TBL_STOCK." s 
                    ON a.id = s.item_id 
                    AND s.control_id != 1 
                    AND s.client_id = '$client_id'
                    AND s.location_id = '$location_id'
                    AND s.store_id = '$store_id' $txn_date $user_id

                LEFT JOIN ".TBL_ITEM_PRICE." c 
                    ON s.item_id = c.item_id 
                    AND s.m_metric = c.m_metric 
                    AND c.location_id = '$location_id' 
                    AND (c.outlet_id = s.store_id OR c.default_price = 1 OR c.default_price IS NULL)

                LEFT JOIN ".TBL_ITEM_PRICE." b 
                    ON s.item_id = b.item_id 
                    AND b.primary_measurement = 1 
                    AND b.location_id = '$location_id' 
                    AND (b.outlet_id = s.store_id OR b.default_price = 1 OR b.default_price IS NULL)
            ),
            inventory_balances AS (
                SELECT 
                    d.item_id AS id, 
                    d.name, 
                    d.rate, 
                    d.m_category_id, 
                    m.name AS msm, 
                    d.item_category_id, 
                    a.running_cp AS cp,
                    SUM(CASE WHEN d.effect = 'cr' THEN d.pQty ELSE 0 END) AS dispensed,
                    SUM(CASE WHEN d.effect = 'dr' THEN d.pQty ELSE 0 END) AS received,
                    SUM(CASE WHEN d.effect = 'dr' THEN d.pQty ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN d.pQty ELSE 0 END) AS closingBalance,
                    SUM(CASE WHEN d.effect = 'cr' THEN (d.running_cp * d.pQty) ELSE 0 END) AS CreditAMT,
                    SUM(CASE WHEN d.effect = 'dr' THEN (d.price * d.pQty) ELSE 0 END) AS DebitAMT,
                    SUM(CASE WHEN d.effect = 'dr' THEN (d.price * d.pQty) ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN (d.running_cp * d.pQty) ELSE 0 END) AS amount,
                    (SUM(CASE WHEN d.effect = 'dr' THEN d.pQty ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN d.pQty ELSE 0 END))* d.rate AS RetailValue
                
                FROM inventory d
                LEFT JOIN inventory a 
                    ON a.item_id = d.item_id AND a.rn = 1
                LEFT JOIN ".TBL_MEASUREMENT." m 
                    ON m.m_category_id = d.m_category_id 
                    AND m.hierarchy = d.m_metric  
                    AND m.location_id = '$location_id'
                GROUP BY d.item_id, d.name, d.rate, d.m_category_id, m.name, d.item_category_id, a.running_cp
            )
            SELECT *,
            ROUND(COALESCE(((RetailValue - amount) * 1.0 / RetailValue) * 100, 0),2) as pm,
            ROUND(COALESCE(((RetailValue - amount) * 1.0 / amount) * 100, 0),2) as mkup
            FROM inventory_balances
            GROUP BY name
        ";

        $data = DB::select($query);
        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function cmpInventoryData($request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        $store_id = !empty($request->store_id) ? $request->store_id : $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $text = !empty($request->text) ? "AND a.name LIKE '%$request->text%'" : "";
        $category = !empty($request->category) ? "AND a.item_category_id = '$request->category'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? "AND s.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $user_id = !empty($request->user_id) ? "AND s.user_id='$request->user_id'" : "";
        $limit = "LIMIT $request->count, 50";
        
        $query="
            WITH products AS (
                SELECT DISTINCT a.id, a.name, a.status, a.item_category_id
                FROM ".TBL_ITEMS." a
                INNER JOIN ".TBL_STOCK." b 
                    ON b.item_id = a.id 
                    AND b.control_id != 1 
                    AND b.client_id = '$client_id' 
                    AND b.location_id = '$location_id'
                WHERE a.location_id = '$location_id'
                AND a.client_id = '$client_id'
                AND a.status = 0
                AND EXISTS (
                    SELECT 1
                    FROM ".TBL_ITEM_PRICE." c
                    WHERE c.item_id = a.id
                        AND c.primary_measurement = 1
                        AND c.status = 0
                ) $text $category
                GROUP BY a.id
                ORDER BY a.name
                $limit
            ),
            inventory AS (
                SELECT 
                    a.id AS item_id, 
                    a.name, 
                    s.qty * c.cal_qty * 1.0 / b.cal_qty AS pQty, 
                    s.m_metric, 
                    s.effect, 
                    b.rate, 
                    a.item_category_id, 
                    s.m_category_id, 
                    b.cp AS price,
                    b.cp AS running_cp,
                    ROW_NUMBER() OVER (
                        PARTITION BY s.item_id 
                        ORDER BY s.txn_date DESC, s.tm DESC
                    ) AS rn
                FROM products a
                LEFT JOIN ".TBL_STOCK." s 
                    ON a.id = s.item_id 
                    AND s.control_id != 1 
                    AND s.client_id = '$client_id'
                    AND s.location_id = '$location_id'
                    AND s.store_id = '$store_id' $txn_date $user_id

                LEFT JOIN ".TBL_ITEM_PRICE." c 
                    ON s.item_id = c.item_id 
                    AND s.m_metric = c.m_metric 
                    AND c.location_id = '$location_id' 
                    AND (c.outlet_id = s.store_id OR c.default_price = 1 OR c.default_price IS NULL)

                LEFT JOIN ".TBL_ITEM_PRICE." b 
                    ON s.item_id = b.item_id 
                    AND b.m_metric = 1 
                    AND b.location_id = '$location_id' 
                    AND (b.outlet_id = s.store_id OR b.default_price = 1 OR b.default_price IS NULL)
            ),
            inventory_balances AS (
                SELECT 
                    d.item_id AS id, 
                    d.name, 
                    d.rate, 
                    d.m_category_id, 
                    m.name AS msm, 
                    d.item_category_id, 
                    a.running_cp AS cp,
                    SUM(CASE WHEN d.effect = 'cr' THEN d.pQty ELSE 0 END) AS dispensed,
                    SUM(CASE WHEN d.effect = 'dr' THEN d.pQty ELSE 0 END) AS received,
                    SUM(CASE WHEN d.effect = 'dr' THEN d.pQty ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN d.pQty ELSE 0 END) AS closingBalance,
                    SUM(CASE WHEN d.effect = 'cr' THEN (d.running_cp * d.pQty) ELSE 0 END) AS CreditAMT,
                    SUM(CASE WHEN d.effect = 'dr' THEN (d.price * d.pQty) ELSE 0 END) AS DebitAMT,
                    SUM(CASE WHEN d.effect = 'dr' THEN (d.price * d.pQty) ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN (d.running_cp * d.pQty) ELSE 0 END) AS amount,
                    (SUM(CASE WHEN d.effect = 'dr' THEN d.pQty ELSE 0 END) -
                    SUM(CASE WHEN d.effect = 'cr' THEN d.pQty ELSE 0 END))* d.rate AS RetailValue
                
                FROM inventory d
                LEFT JOIN inventory a 
                    ON a.item_id = d.item_id AND a.rn = 1
                LEFT JOIN ".TBL_MEASUREMENT." m 
                    ON m.m_category_id = d.m_category_id 
                    AND m.hierarchy = d.m_metric  
                    AND m.location_id = '$location_id'
                GROUP BY d.item_id, d.name, d.rate, d.m_category_id, m.name, d.item_category_id, a.running_cp
            )
            SELECT *,
            ROUND(COALESCE(((RetailValue - amount) * 1.0 / RetailValue) * 100, 0),2) as pm,
            ROUND(COALESCE(((RetailValue - amount) * 1.0 / amount) * 100, 0),2) as mkup
            FROM inventory_balances
            GROUP BY name
        ";

        $data = DB::select($query);
        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function stockData(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;

        $inventoryEvaluation = Location::where("client_id", $client_id)
        ->where("id", $location_id)
        ->value("inventory_evaluation");

        if($inventoryEvaluation == STATUS) {
            return $this->inventoryData($request);
        }

        if($inventoryEvaluation == ONE_STATUS) {
            return $this->cmpInventoryData($request);
        }

        if($inventoryEvaluation == TWO_STATUS) {
            
        }
    }
}

