<?php

namespace App\Http\Controllers\Inventory\Inventory;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Inventory\Inventory\InventoryController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use Illuminate\Http\Request;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use App\Models\Retails\Stock;
use App\Models\Retails\StockAdjustment;
use Validator;
use DB;

class StockAdjustmentController extends Controller
{
    //

    protected $inventoryController;
    protected $txnLogController;
    protected $chartOfAccountController;
    protected $currencyController;

    public function __construct(
        InventoryController $inventoryController,
        TxnLogController $txnLogController,
        ChartOfAccountController $chartOfAccountController,
        CurrencyController $currencyController,
    ) {
        $this->inventoryController = $inventoryController;
        $this->chartOfAccountController = $chartOfAccountController;
        $this->txnLogController = $txnLogController;
        $this->currencyController = $currencyController;
    }

    public function insertStockAdjustment($token, $txn_date, $item_id, $cp, $reason, $chart_id, $store_id, $m_category_id, $available_qty, $new_qty, $v, $jv, $effect, $m_metric) {
        
        $fields = [
            'id' => Uuid::uuid4()->toString(),
            'stock_date' => $txn_date,
            'token' => $token,
            'item_id' => $item_id,
            'adj_qty' => $new_qty,
            'available_qty' => $available_qty,
            'variance' => $v,
            'reason' => $reason,
            'effect' => $effect,
            'm_category_id' => $m_category_id,
            'm_metric' => $m_metric,
            'date' => Carbon::now()->format('Y-m-d'),
            'tm' => Carbon::now()->toTimeString(),
            'status' => STATUS,
            'user_id' => Auth::user()->id,
            'location_id' => Auth::user()->location_id,
            'client_id' => Auth::user()->client_id,
            'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            'control_user_id' => STATUS,
            'control_id' => STATUS,
            'adj_value' => $jv,
        ];

        $stock_adjustment = StockAdjustment::create($fields);

        return $stock_adjustment;
    }

    public function validation($request) {
        $validationRules = [
            'store_id' => 'required',
            'txn_date' => 'required',
            'new_qty' => 'required',
            'chart_id' => 'required'
        ];
        // Create a validator with the defined rules and the request data
        $validator = Validator::make($request->all(), $validationRules);
        // Check if validation fails and return an error response if it does
        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }
    }

    public function saveStockAdjustment(Request $request){

      
        $validation = $this->validation($request);
        if($validation){
            return $validation;
        }
        
        $txn_date = $request->txn_date;
        $item_id = $request->item_id;
        $reason = $request->reason;
        $chart_id = $request->chart_id == "suspense" ? CHART_SUSPENSE : $request->chart_id;
        $store_id = $request->store_id;
        $m_category_id = $request->m_category_id;
        $available_qty = $request->available_qty;
        $new_qty = $request->new_qty;
        $item_category_id = $request->item_category_id;

        $token = Uuid::uuid4()->toString();
        $cp = $this->inventoryController->inventoryEvaluationPrice($item_id, $store_id);
        
        if(!$cp || $cp == STATUS) {
            return response()->json(['message' => 'You cannot adjust an item that never had a stock, recreate the item and use previous balance instead', 'error' => true], 400);
        }

        $variance = $available_qty - $new_qty;
        $effect = $variance < STATUS ? EFFECT_DEBIT : EFFECT_CREDIT;
        $v = $variance < STATUS ? abs($variance) : $variance;
        $jv = $v * $cp;
        
        $stock_adjustment = $this->insertStockAdjustment($token, $txn_date, $item_id, $cp, $reason, $chart_id, $store_id, $m_category_id, $available_qty, $new_qty, $v, $jv, $effect, ONE_STATUS);

        $currency_id = $this->currencyController->getPrimaryCurrency(Auth::user()->client_id, Auth::user()->location_id);
        $currency_unit = $this->currencyController->getCurrencyUnit(Auth::user()->client_id, Auth::user()->location_id, $currency_id);

        $this->inventoryController->insertStock($token, $txn_date, $item_id, "", $item_category_id, $v, $cp, $currency_id, ONE_STATUS, $m_category_id, $store_id, $effect, null, $jv);

        $accLegs = $variance < STATUS ?
        $this->chartOfAccountController->getAccountLegs(CHART_STOCK, $chart_id, Auth::user()->client_id) :
        $this->chartOfAccountController->getAccountLegs($chart_id, CHART_STOCK, Auth::user()->client_id);

        $this->txnLogController->createTxnLog($accLegs, $jv, $token, $currency_unit, $txn_date, "", "being stock adjustment", "", "adj", $currency_id);

        return response()->json(['message' => 'Stock Adjustment saved successfully', 'data' => $stock_adjustment, 'success' => true], 201);
        
    }

    public function getStockAdjustment(Request $request) {
        $client_id=Auth::user()->client_id;
        $location_id =Auth::user()->location_id;

        $searchName = !empty($request->search) ? " AND i.name LIKE '%$request->search%'" : "";
        $limit = "LIMIT $request->count, 50";

        $query = "
            SELECT

            s.stock_date,
            s.token,
            s.date,
            s.tm,
            s.id,
            s.variance,
            s.adj_value,
            s.effect,
            s.reason,
            i.name,
            s.item_id,
            m.abr

            FROM

            " . TBL_STOCK_ADJUSTMENT . " s

            LEFT JOIN " . TBL_ITEMS . " i
            ON i.id=s.item_id AND i.client_id='$client_id' AND i.status = 0 $searchName

            LEFT JOIN " . TBL_MEASUREMENT . " m
            ON m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0 AND m.hierarchy=s.m_metric AND m.m_category_id=s.m_category_id

            WHERE s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND i.id IS NOT NULL
            ORDER BY s.stock_date, s.tm DESC
            $limit
        ";

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }
}