<?php

namespace App\Http\Controllers\Inventory\Inventory;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Controllers\Inventory\Inventory\InventoryController;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Inventory\Product\ProductController;
use App\Http\Controllers\Inventory\Product\PriceController;
use Ramsey\Uuid\Uuid;
use App\Models\Retails\RequestModel;
use Carbon\Carbon;
use DB;
use App\Http\Controllers\Settings\Currency\CurrencyController;

class StockRequisitionController extends Controller
{
    //
    protected $inventoryController;
    protected $productController;
    protected $priceController;
    protected $currencyController;

    public function __construct(
        InventoryController $inventoryController, ProductController $productController, PriceController $priceController, CurrencyController $currencyController
    ){
        $this->inventoryController = $inventoryController;
        $this->productController = $productController;
        $this->priceController = $priceController;
        $this->currencyController = $currencyController;
    }

    public function sendRequest(Request $request) {
 
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        $user_id = Auth::user()->id;

        $items = $request->items;
        $requestingStore = $request->requesting_store;
        $requestingLocation = $request->requesting_location;
        $comment = $request->comment;
        $receivingStore = $request->receiving_store;
        $receivingLocation = $request->receiving_location;

        $token= Uuid::uuid4()->toString();

        foreach($items as $itm) {
            if (!empty($itm['id'])) {
                $item = $itm['id'];
                $metric = $itm['measurement'];
                $qty = $itm['requestedQty'];
                $itmCat = $this->productController->getItemCategoryByItemId($item);
                $itmClass = $this->productController->getItemClassByItemId($item);
                $mCat = $this->productController->getItemMeasurementCategoryByItemId($item);
                $itemName = $this->productController->getItemName($item);

                $uuid = Uuid::uuid4()->toString();
                $id = str_replace('-', '', $uuid);

                $requestFields = [
                    'id' => $id,
                    'user_id' => $user_id,
                    'store_id' => $receivingStore,
                    'sender_store_id' => $requestingStore,
                    'item_category_id' => $itmCat,
                    'rq_token' => $token,
                    'sender_id' => $requestingLocation,
                    'recipient_id' => $receivingLocation,
                    // 'item_class_id' => $itmClass ?? STATUS,
                    'qty' => $qty,
                    'm_metric' => $metric,
                    'm_category_id' => $mCat,
                    'comment' => $comment,
                    'status' => STATUS,
                    'tm' => Carbon::now()->toTimeString(),
                    'date' => Carbon::now()->format('Y-m-d'),
                    'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    'item_id' => $item,
                    'control_id' => STATUS,
                    'client_id' => $client_id,
                    'location_id' => $location_id
                ];
            }

            RequestModel::create($requestFields);
        }

        return response()->json(['message' => 'Request sent successfully', 'success' => true], 201);
        
    } 

    public function fetchRequest(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        
        $from = $request->from;
        $to = $request->to;

        $txn_date = !empty($from) && !empty($to) ? " AND r.date BETWEEN '$from' AND '$to'" : "";
   
        $query = "
            with req as (
                SELECT  r.id as requestId, r.date, r.tm, r.sender_id, r.item_id, r.recipient_id, r.qty, r.rq_token, r.m_metric, r.store_id, r.sender_store_id, r.m_category_id, r.status, r.comment
                FROM " . TBL_REQUEST . " r 
                WHERE r.status != 1 AND r.control_id != 1 AND r.sender_id='$location_id' AND r.client_id='$client_id' $txn_date
            ),

            stockBal as (
                SELECT s.item_id, i.cal_qty, s.qty, s.effect, SUM(i.cal_qty * s.qty) as availableQty from " . TBL_STOCK . " s
                LEFT JOIN ".TBL_ITEM_PRICE." i ON i.item_id=s.item_id AND i.m_metric=s.m_metric AND i.status=0
                WHERE s.location_id='$location_id' AND s.control_id != 1
                GROUP BY s.item_id, effect
            ),

            closingBal as (
                SELECT d.*, d.availableQty as dr, c.availableQty as cr, coalesce(d.availableQty, 0) - coalesce(c.availableQty, 0) as clBal FROM stockBal d
                LEFT JOIN (SELECT * FROM stockBal WHERE effect = 'cr' GROUP BY item_id) c
                ON d.item_id=c.item_id 
                WHERE d.effect = 'dr'
                GROUP BY d.item_id
            ),

            requestMeasurement as (
                SELECT i.id, m.name as msm, m.hierarchy, m.m_category_id FROM ".TBL_ITEMS." i
                LEFT JOIN ".TBL_MEASUREMENT." m ON i.m_category_id = m.m_category_id AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0
                GROUP BY i.id, m.hierarchy
            ),

            requestItems as (
                SELECT i.name, c.clBal, rm.msm, p.cal_qty, coalesce((c.clBal / p.cal_qty), 0) as avaQty, r.* from req r
                LEFT JOIN ".TBL_ITEMS." i ON r.item_id = i.id AND i.status = 0 AND i.disabled_status = 0
                LEFT JOIN closingBal c ON r.item_id = c.item_id
                LEFT JOIN ".TBL_ITEM_PRICE." p ON r.item_id=p.item_id AND p.m_metric=r.m_metric	
                LEFT JOIN requestMeasurement rm ON r.item_id=rm.id AND r.m_metric=rm.hierarchy 
                GROUP BY r.requestId
            ),

            requests as (
                SELECT 
                r.sender_id, 
                r.recipient_id, 
                r.rq_token, 
                r.status, 
                sl.name as requestingBranch, 
                l.name as fulfillingBranch, 
                st.name as requestingStore, 
                s.name as fulfillingStore, 
                r.date, 
                r.sender_store_id, 
                r.store_id, 
                r.tm, 
                r.comment,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'requestId', i.requestId,
                        'item_id', i.item_id, 
                        'name', i.name, 
                        'avaQty', i.avaQty, 
                        'msm', i.msm, 
                        'measurement', i.m_metric, 
                        'm_category_id', i.m_category_id, 
                        'qty', i.qty,
                        'status',i.status
                    ))
                    FROM requestItems i
                    WHERE i.rq_token = r.rq_token
                    GROUP BY i.rq_token
                ) AS items 
            FROM req r
            LEFT JOIN ".TBL_LOCATION." l ON l.id = r.recipient_id AND l.client_id = '$client_id'
            LEFT JOIN ".TBL_LOCATION." sl ON sl.id = r.sender_id AND sl.client_id = '$client_id'
            LEFT JOIN ".TBL_STORE." s ON s.id = r.store_id AND s.client_id = '$client_id'
            LEFT JOIN ".TBL_STORE." st ON st.id = r.sender_store_id AND st.client_id = '$client_id'

                GROUP BY r.rq_token
            )

            SELECT * from requests
        ";

        $data = DB::select($query);
        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function cancelRequest(Request $request) {
        
        RequestModel::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('rq_token', $request->token)
        ->update([
            'status' => 5,
            'control_user_id' => Auth::user()->id
        ]);
        return response()->json(['message' => 'Request canceled successfully', 'success' => true], 200);
    }

  

    public function declineRequest(Request $request) {
        
        RequestModel::where('client_id', Auth::user()->client_id)
        ->where('recipient_id', Auth::user()->location_id)
        ->where('rq_token', $request->token)
        ->update([
            'status' => 3,
            'control_user_id' => Auth::user()->id
        ]);
        return response()->json(['message' => 'Request declined successfully', 'success' => true], 200);
    }

    private function isExistRequest($id) {
        return RequestModel::where('control_id', STATUS)
        ->where('id', $id)
        ->value('id');
    }
    
    public function editRequest(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        $user_id = Auth::user()->id;

        $items = $request->items;
        $requestingStore = $request->requesting_store;
        $requestingLocation = $request->requesting_location;
        $comment = $request->comment;
        $receivingStore = $request->receiving_store;
        $receivingLocation = $request->receiving_location;
        $editToken = $request->token;

        foreach($items as $itm) {
            if (!empty($itm['item_id'])) {
                $id = $itm['requestId'];
                $item = $itm['item_id'];
                $metric = $itm['measurement'];
                $qty = $itm['qty'];
                $itmCat = $this->productController->getItemCategoryByItemId($item);
                $itmClass = $this->productController->getItemClassByItemId($item);
                $mCat = $this->productController->getItemMeasurementCategoryByItemId($item);
                $itemName = $this->productController->getItemName($item);

                if ($itm['status'] == 0) {
                    
                    $exist = $this->isExistRequest($id);

                    if($exist) {
                        RequestModel::where('item_id', $item)
                        ->where('client_id', $client_id)
                        ->where('location_id', $location_id)
                        ->where('rq_token', $editToken)
                        ->update([
                            'store_id' => $receivingStore,
                            'sender_store_id' => $requestingStore,
                            'recipient_id' => $receivingLocation,
                            'qty' => $qty,
                            'm_metric' => $metric,
                            'comment' => $comment,
                            'sender_id' => $requestingLocation,
                        ]);
                    } else {
                        $uuid = Uuid::uuid4()->toString();
                        $id = str_replace('-', '', $uuid);
                        
                        $requestFields = [
                            'id' => $id,
                            'user_id' => $user_id,
                            'store_id' => $receivingStore,
                            'sender_store_id' => $requestingStore,
                            'item_category_id' => $itmCat,
                            'rq_token' => $editToken,
                            'sender_id' => $requestingLocation,
                            'recipient_id' => $receivingLocation,
                            'item_class_id' => $itmClass,
                            'qty' => $qty,
                            'm_metric' => $metric,
                            'm_category_id' => $mCat,
                            'comment' => $comment,
                            'status' => STATUS,
                            'tm' => Carbon::now()->toTimeString(),
                            'date' => Carbon::now()->format('Y-m-d'),
                            'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
                            'item_id' => $item,
                            'control_id' => STATUS,
                            'client_id' => $client_id,
                            'location_id' => $location_id
                        ];

                        RequestModel::create($requestFields);
                    }
                } else {
                    RequestModel::where('item_id', $item)
                    ->where('client_id', $client_id)
                    ->where('sender_id', $requestingLocation)
                    ->where('rq_token', $editToken)
                    ->where('m_metric', $metric)
                    ->update([
                        'control_id' => ONE_STATUS,
                        'control_user_id' => $user_id,
                    ]);
                }
            }
        }

        return response()->json(['message' => 'Request edited successfully', 'success' => true], 200);
        
    }

    public function releaseRequest(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        $user_id = Auth::user()->id;

        $requestToken = $request->token;
        $items = $request->items;
        $store = $request->store; 
        $comment = $request->releaseComment;  

        $sentToken= Uuid::uuid4()->toString();
        $txn_date = Carbon::now()->format('Y-m-d');

        foreach($items as $item) {
            $id = $item['requestId'];

            if(!empty($id)) {
                $item_id = $item['item_id'];
                $metric = $item['measurement'];
                $qty = $item['releasedQty'];
                
                $item_category_id = $this->productController->getItemCategoryByItemId($item_id);
                $m_category_id = $this->productController->getItemMeasurementCategoryByItemId($item_id);
                $availableQty = $this->inventoryController->getQtyBalance($item_id);
                $cp = $this->inventoryController->inventoryEvaluationPrice($item_id);
                
                $calQty = $this->priceController->itemPriceCalQty($item_id, $metric, $m_category_id);
                $uqty = $qty * $calQty;
                $amount = $uqty * $cp;
                
                if($uqty <= $availableQty) {
                    RequestModel::where('item_id', $item_id)
                    ->where('id', $id)
                    ->where('client_id', $client_id)
                    ->where('recipient_id', $location_id)
                    ->where('rq_token', $requestToken)
                    ->update([
                        'release_comment' => $comment,
                        'qty_sent' => $qty,
                        'status' => TWO_STATUS,
                        'sent_token' => $sentToken,
                        'control_user_id' => $user_id
                    ]);

                    $currency = $this->currencyController->getPrimaryCurrency($client_id, $location_id);
                    
                    $this->inventoryController->insertStock($sentToken, $txn_date, $item_id, "", $item_category_id, $qty, $cp, $currency, $metric, $m_category_id, $store, EFFECT_CREDIT, null, $amount);
                }
            }
        }
        return response()->json(['message' => 'Request released successfully', 'success' => true], 200);
    }

    public function acceptRequest(Request $request) {
        $requestToken = $request->rq_token;
        $items = $request->items;
        $store = $request->store_id;
        $recipientId = $request->recipient_id;
        $comment = $request->acceptComment;
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        $user_id = Auth::user()->id;

        $acceptToken= Uuid::uuid4()->toString();
        $txn_date = Carbon::now()->format('Y-m-d');

        foreach($items as $item) {
            $id = $item['requestId'];
            
            if(!empty($id)) {
                $item_id = $item['item_id'];
                $metric = $item['measurement'];
                $qty = $item['acceptQty'];
                $releasedQty = $this->requisitionReleasedQty($item_id, $requestToken, $location_id, $client_id);
                $item_category_id = $this->productController->getItemCategoryByItemId($item_id);
                $m_category_id = $this->productController->getItemMeasurementCategoryByItemId($item_id);
                $requestedQty = $this->requisitionRequestedQty($item_id, $requestToken, $location_id, $client_id);
                $recipientStore = $this->requisitionRecipientStore($item_id, $requestToken, $recipientId, $client_id);
                $cp = $this->inventoryController->inventoryEvaluationPrice($item_id, $recipientStore, $recipientId);
                $calQty = $this->priceController->itemPriceCalQty($item_id, $metric, $m_category_id);

                $itemName = $this->productController->getItemName($item_id);

                if($releasedQty){
                    
                    if($calQty) {
                        $uqty = $qty * $calQty;
                        $amount = $uqty * $cp;
                        $status = $releasedQty < $requestedQty ? 7 : 6;
                        $rejectedQty = $releasedQty - $qty;
                        
                        RequestModel::where('item_id', $item_id)
                        ->where('id', $id)
                        ->where('client_id', $client_id)
                        ->where('recipient_id', $recipientId)
                        ->where('sender_id', $location_id)
                        ->where('rq_token', $requestToken)
                        ->update([
                            'accept_comment' => $comment,
                            'qty_accepted' => $qty,
                            'status' => $status,
                            'accepted_token' => $acceptToken,
                            'control_user_id' => $user_id
                        ]);

                        $currency = $this->currencyController->getPrimaryCurrency($client_id, $location_id);
                        
                        $this->inventoryController->insertStock($acceptToken, $txn_date, $item_id, "", $item_category_id, $qty, $cp, $currency, $metric, $m_category_id, $store, EFFECT_DEBIT, null, $amount);
                        
                        if($rejectedQty > STATUS) {
                            $rejectedCalQty = $this->priceController->itemPriceCalQty($item_id, $metric, $m_category_id, $recipientId);
                            $rejectedUQty = $rejectedQty * $rejectedCalQty;
                            $rejectedAmount = $rejectedUQty * $cp;

                            $this->inventoryController->insertStock($acceptToken, $txn_date, $item_id, "", $item_category_id, $rejectedQty, $cp, $currency, $metric, $m_category_id, $recipientStore, EFFECT_DEBIT, null, $rejectedAmount, $recipientId);
                        }

                    } else {
                        return response()->json(['message' => "To accept quantity for $itemName first set its price.", 'success' => false], 400);
                    }
                } else {
                    RequestModel::where('client_id', $client_id)
                    ->where('sender_id', $location_id)
                    ->where('recipient_id', $recipientId)
                    ->where('rq_token', $requestToken)
                    ->where('item_id', $item_id)
                    ->update([
                        'status' => 3,
                        'control_user_id' => $user_id
                    ]);
                }
            }
        }
        return response()->json(['message' => 'Request accepted successfully', 'success' => true], 200);
    }


    public function requisitionReleasedQty($itemId, $token, $location_id, $client_id) {
        return RequestModel::where('item_id', $itemId)
        ->where('rq_token', $token)
        ->where('sender_id', $location_id)
        ->where('client_id', $client_id)
        ->value('qty_sent');
    }

    public function requisitionRequestedQty($itemId, $token, $location_id, $client_id) {
        return RequestModel::where('item_id', $itemId)
        ->where('rq_token', $token)
        ->where('sender_id', $location_id)
        ->where('client_id', $client_id)
        ->value('qty');
    }

    public function requisitionRecipientStore($itemId, $token, $location_id, $client_id) {
        return RequestModel::where('item_id', $itemId)
        ->where('rq_token', $token)
        ->where('recipient_id', $location_id)
        ->where('client_id', $client_id)
        ->value('store_id');
    }

    public function fetchReleased(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        
        $from = $request->from;
        $to = $request->to;
        $txn_date = !empty($from) && !empty($to) ? " AND r.date BETWEEN '$from' AND '$to'" : "";
   
        $query = "
            with req as (
                SELECT  r.id as requestId, r.date, r.tm, r.sender_id, r.item_id, r.recipient_id, r.qty, r.rq_token, r.m_metric, r.store_id, r.sender_store_id, r.m_category_id, r.status, r.comment
	        FROM " . TBL_REQUEST . " r 
                WHERE r.status != 1 AND r.control_id != 1 AND r.recipient_id='$location_id' AND r.client_id='$client_id' $txn_date
            ),

            stockBal as (
                SELECT s.item_id, i.cal_qty, s.qty, s.effect, SUM(i.cal_qty * s.qty) as availableQty from " . TBL_STOCK . " s
                LEFT JOIN ".TBL_ITEM_PRICE." i ON i.item_id=s.item_id AND i.m_metric=s.m_metric AND i.status=0 AND i.location_id='$location_id'
                WHERE s.location_id='$location_id' AND s.control_id != 1
                GROUP BY s.item_id, effect
            ),

            closingBal as (
                SELECT d.*, d.availableQty as dr, c.availableQty as cr, coalesce(d.availableQty, 0) - coalesce(c.availableQty, 0) as clBal FROM stockBal d
                LEFT JOIN (SELECT * FROM stockBal WHERE effect = 'cr' GROUP BY item_id) c
                ON d.item_id=c.item_id 
                WHERE d.effect = 'dr'
                GROUP BY d.item_id
            ),

            requestMeasurement as (
                SELECT i.id, m.name as msm, m.hierarchy, m.m_category_id FROM ".TBL_ITEMS." i
                LEFT JOIN ".TBL_MEASUREMENT." m ON i.m_category_id = m.m_category_id AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0
                GROUP BY i.id, m.hierarchy
            ),

            requestItems as (
                SELECT i.name, c.clBal, rm.msm, p.cal_qty, coalesce((c.clBal / p.cal_qty), 0) as avaQty, r.* from req r
                LEFT JOIN ".TBL_ITEMS." i ON r.item_id = i.id AND i.status = 0 AND i.disabled_status = 0
                LEFT JOIN closingBal c ON r.item_id = c.item_id
                LEFT JOIN ".TBL_ITEM_PRICE." p ON r.item_id=p.item_id AND p.m_metric=r.m_metric	
                LEFT JOIN requestMeasurement rm ON r.item_id=rm.id AND r.m_metric=rm.hierarchy 
                GROUP BY r.requestId
            ),

            requests as (
                SELECT 
                r.sender_id, 
                r.recipient_id, 
                r.rq_token, 
                r.status, 
                sl.name as requestingBranch, 
                l.name as fulfillingBranch, 
                st.name as requestingStore, 
                s.name as fulfillingStore, 
                r.date, 
                r.sender_store_id, 
                r.store_id, 
                r.tm, 
                r.comment,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'requestId', i.requestId,
                        'item_id', i.item_id, 
                        'name', i.name, 
                        'avaQty', i.avaQty, 
                        'msm', i.msm, 
                        'measurement', i.m_metric, 
                        'm_category_id', i.m_category_id, 
                        'qty', i.qty
                    ))
                    FROM requestItems i
                    WHERE i.rq_token = r.rq_token
                    GROUP BY i.rq_token
                ) AS items 
            FROM req r
            LEFT JOIN ".TBL_LOCATION." l ON l.id = r.recipient_id AND l.client_id = '$client_id'
            LEFT JOIN ".TBL_LOCATION." sl ON sl.id = r.sender_id AND sl.client_id = '$client_id'
            LEFT JOIN ".TBL_STORE." s ON s.id = r.store_id AND s.client_id = '$client_id'
            LEFT JOIN ".TBL_STORE." st ON st.id = r.sender_store_id AND st.client_id = '$client_id'

                GROUP BY r.rq_token
            )

            SELECT * from requests
        ";

        $data = DB::select($query);
        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function fetchFulfilled(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        
        $from = $request->from;
        $to = $request->to;
        $txn_date = !empty($from) && !empty($to) ? " AND r.date BETWEEN '$from' AND '$to'" : "";
   
        $query = "
            with req as (
                SELECT r.id as requestId, r.date, r.tm, r.sender_id, r.item_id, r.recipient_id, r.qty, r.rq_token, r.m_metric, r.store_id, r.sender_store_id, r.m_category_id, r.status, r.release_comment, r.qty_sent
                FROM " . TBL_REQUEST . " r 
                WHERE r.status = 2 AND r.control_id != 1 AND r.sender_id='$location_id' AND r.client_id='$client_id' $txn_date
            ),

            stockBal as (
                SELECT s.item_id, i.cal_qty, s.qty, s.effect, SUM(i.cal_qty * s.qty) as availableQty  from " . TBL_STOCK . " s
                LEFT JOIN ".TBL_ITEM_PRICE." i ON i.item_id=s.item_id AND i.m_metric=s.m_metric AND i.status=0
                WHERE s.location_id='$location_id' AND s.control_id != 1
                GROUP BY s.item_id, effect
            ),

            closingBal as (
                SELECT d.*, d.availableQty as dr, c.availableQty as cr, coalesce(d.availableQty, 0) - coalesce(c.availableQty, 0) as clBal FROM stockBal d
                LEFT JOIN (SELECT * FROM stockBal WHERE effect = 'cr' GROUP BY item_id) c
                ON d.item_id=c.item_id 
                WHERE d.effect = 'dr'
                GROUP BY d.item_id
            ),

            requestMeasurement as (
                SELECT i.id, m.name as msm, m.hierarchy, m.m_category_id FROM ".TBL_ITEMS." i
                LEFT JOIN ".TBL_MEASUREMENT." m ON i.m_category_id = m.m_category_id AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0
                GROUP BY i.id, m.hierarchy
            ),

            requestItems as (
                SELECT i.name, c.clBal, rm.msm, p.cal_qty, coalesce((c.clBal / p.cal_qty), 0) as avaQty, r.* from req r
                LEFT JOIN ".TBL_ITEMS." i ON r.item_id = i.id AND i.status = 0 AND i.disabled_status = 0
                LEFT JOIN closingBal c ON r.item_id = c.item_id
                LEFT JOIN ".TBL_ITEM_PRICE." p ON r.item_id=p.item_id AND p.m_metric=r.m_metric	
                LEFT JOIN requestMeasurement rm ON r.item_id=rm.id AND r.m_metric=rm.hierarchy 
                GROUP BY r.requestId
            ),

            requests as (
                SELECT 
                r.sender_id, 
                r.recipient_id, 
                r.rq_token, 
                r.status, 
                sl.name as requestingBranch, 
                l.name as fulfillingBranch, 
                st.name as requestingStore, 
                s.name as fulfillingStore, 
                r.date, 
                r.sender_store_id, 
                r.store_id, 
                r.tm, 
                r.release_comment,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'requestId', i.requestId,
                        'item_id', i.item_id, 
                        'name', i.name, 
                        'avaQty', i.avaQty, 
                        'msm', i.msm, 
                        'measurement', i.m_metric, 
                        'm_category_id', i.m_category_id, 
                        'qty', i.qty,
                        'sentQty', i.qty_sent
                    ))
                    FROM requestItems i
                    WHERE i.rq_token = r.rq_token
                    GROUP BY i.rq_token
                ) AS items 
            FROM req r
            LEFT JOIN ".TBL_LOCATION." l ON l.id = r.recipient_id AND l.client_id = '$client_id'
            LEFT JOIN ".TBL_LOCATION." sl ON sl.id = r.sender_id AND sl.client_id = '$client_id'
            LEFT JOIN ".TBL_STORE." s ON s.id = r.store_id AND s.client_id = '$client_id'
            LEFT JOIN ".TBL_STORE." st ON st.id = r.sender_store_id AND st.client_id = '$client_id'

                GROUP BY r.rq_token
            )

            SELECT * from requests
        ";
        

        $data = DB::select($query);
        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function stockRequisitionSummary(Request $request) {

        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;

        $canceled = RequestModel::whereIn('status', [5, 3])
        ->where('sender_id', $location_id)
        ->where('client_id', $client_id)
        ->whereBetween('date', [$request->from, $request->to])
        ->distinct('rq_token')
        ->count('rq_token');

        $success = RequestModel::whereIn('status', [7, 6])
        ->where('sender_id', $location_id)
        ->where('client_id', $client_id)
        ->whereBetween('date', [$request->from, $request->to])
        ->distinct('rq_token')
        ->count('rq_token');

        $pending = RequestModel::where('status', 0)
        ->where('sender_id', $location_id)
        ->where('client_id', $client_id)
        ->whereBetween('date', [$request->from, $request->to])
        ->distinct('rq_token')
        ->count('rq_token');

        $total = $pending + $success + $canceled;

        $data = ["message" =>"Data fetched succesfully", "canceled" => $canceled, "success" => $success, "pending" => $pending, "total" => $total];

        return response()->json(['data' => $data, 'success' => true], 200);
    }
}