<?php

namespace App\Http\Controllers\Settings\Device;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Account\DeviceLog;
use Illuminate\Support\Str;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use DB;
use App\Http\Controllers\Folio\Location\LocationController;
use Illuminate\Database\QueryException;
use Exception;

class DeviceController extends Controller
{

    protected $locationController;

    public function __construct(LocationController $locationController)
    {
        $this->locationController = $locationController;
    }

    public function getDeviceId($client_id, $location_id, $request){

        return DeviceLog::where('client_id', $client_id)
        ->where('location_id', $location_id)
        ->where('status', STATUS)
        // ->where(function ($query) use ($request) {
        //     $query->where('mac_address', $request->macAddress1)
        //           ->orWhere('mac_address_2', $request->macAddress2);
        // })
        ->where(function ($query) use ($request) {
            $query->where('mac_address', $request->macAddress1)
            ->orWhere('mac_address_2', 'LIKE', '%' . $request->macAddress1 . '%');
        })
        ->value('id');
        
    }

    public function getDevices(){

        $response = DeviceLog::where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->get();

        return response()->json(['results' => count($response), 'data' => $response, 'success' => true], 200);
    }

    public function getDevice($client_id, $location_id, $request){

        return DeviceLog::where('client_id', $client_id)
        ->where('location_id', $location_id)
        ->where('status', STATUS)
        ->where(function ($query) use ($request) {
            $query->where('mac_address', $request->macAddress1)
            ->orWhere('mac_address_2', 'LIKE', '%' . $request->macAddress1 . '%');
        })
        ->get();
        
    }

    private function countDevice() {
        return DeviceLog::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->count();
    }

    public function getDefaultDeviceLog($location_id, $client_id) {
        return DeviceLog::where('status', STATUS)
            ->where('client_id', $client_id)
            ->where('location_id', $location_id)
            ->first(); 
    }

    public function isDeviceAddress($location_id, $client_id, $request) {
        
        return DeviceLog::where('client_id', $client_id)
        ->where('location_id', $location_id)
        ->where(function ($query) use ($request) {
            $query->where('mac_address', $request->macAddress1)
            ->orWhere('mac_address_2', 'LIKE', '%' . $request->macAddress1 . '%');
        })
        ->first();
    }



public function createDeviceLog($licensee, $location, $deviceName, $request, $user, $currentDate)
{
    try {
        $deviceLog = DeviceLog::create([
            'client_id' => $licensee,
            'location_id' => $location,
            'mac_address' => $deviceName,
            'mac_address_2' => $request->macAddress2,
            'user_id' => $user,
            'email' => $request->email,
            'timezone' => $request->timezone,
            'status' => STATUS,
            'date' => $currentDate,
        ]);

        if (!$deviceLog) {
            throw new Exception("Failed to create device log.");
        }

        return $deviceLog;
    } catch (QueryException $e) {
        throw new Exception("Database error on creating device log: " . $e->getMessage());
    } catch (Exception $e) {
        throw new Exception("Error creating device log: " . $e->getMessage());
    }
}


    public function setDevice(Request $request) {

        $status = DB::table(TBL_DEVICE)->where('id', $request->id)
        ->where('location_id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->value('status');

        if($status == STATUS) {
            $devices = $this->countDevice();
            $allowedUsers = $this->locationController->allowedUsers();
            
            if($devices >= $allowedUsers) {
                return response()->json(['message' => 'You need to upgrade your subscription plan or lock another device before you can unlock this device', 'success' => false], 400);
            }
        }

        DB::table(TBL_DEVICE)->where('id', $request->id)
        ->where('location_id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->update([
            'status' => $status == STATUS ? ONE_STATUS : STATUS,
            'update_date' => Carbon::now()->format('Y-m-d')
        ]);

        return response()->json(['message' => 'Device unlocked successfully', 'success' => true], 200);
    }

    public function setDeviceName(Request $request) {

        DB::table(TBL_DEVICE)->where('id', $request->id)
        ->where('location_id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->update([
            'name' => $request->deviceName,
            'outlet_id' => $request->outlet_id,
            'update_date' => Carbon::now()->format('Y-m-d')
        ]);

        return response()->json(['message' => 'Device set successfully', 'success' => true], 200);
    }
}
