<?php

namespace App\Http\Controllers\Settings\License;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Account\Licensee;
use Auth;
use App\Http\Controllers\General\GeneralController;
use Carbon\Carbon;
use Illuminate\Database\QueryException;
use Exception;
use DB;
use Response;

class LicenseController extends Controller
{

    protected $GeneralController;

    public function __construct(GeneralController $GeneralController)
    {
        $this->GeneralController = $GeneralController;
    }

    
    public function createLicensee($request, $currentDate, $gst_value, $salt, $pass)
    {
        try {
            $licensee = Licensee::create([
                'name' => $request->name,
                'status' => 0,
                'address' => $request->address,
                'start_date' => $currentDate,
                'end_date' => $currentDate,
                'email' => $request->email,
                'tm' => Carbon::now()->toTimeString(),
                'phone' => $request->phone,
                'multi' => '1 Location',
                'user_id' => STATUS,
                'source' => $request->where,
                'app' => APP_ID,
                'advisor_id' => $request->referral_code,
                'origin' => $request->origin,
                'currency_sign' => $request->currency_sign,
                'timezone' => $request->timezone,
                'device_name' => $request->macAddress1,
                'license_no' => $this->GeneralController->generateUniqueNumber('licensee', 'license_no')
            ]);
    
            // Assuming successful creation returns a model instance
            if (!$licensee) {
                throw new Exception("Failed to create licensee.");
            }
    
            return $licensee;
        } catch (QueryException $e) {
            // This will catch exceptions related to database integrity, such as unique constraint violations
            throw new Exception("Database error on creating licensee: " . $e->getMessage());
        } catch (Exception $e) {
            // This catches any other general exceptions
            throw new Exception("Error creating licensee: " . $e->getMessage());
        }
    }

    public function registrants() {
        try{
            $today = Carbon::now()->format('Y-m-d');
            $lastMonth = Carbon::now()->subMonth()->format('Y-m-d');

            $query = "
                SELECT 
                    ROW_NUMBER() OVER () AS serial_number, 
                    name, 
                    email, 
                    phone, 
                    address, 
                    start_date AS date  
                FROM " . TBL_LICENSEE . " 
                WHERE start_date BETWEEN '$lastMonth' AND '$today'
            ";

            $data = DB::select($query);

            return response()->json(['message' => 'Registrants fetched successfully', 'data' => $data, 'success' => true], 200);
        }catch (\Exception $e) {
            return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 400);
        }
    }
    

    public function licenseEmail($client_id) {
        return Licensee::where('id', $client_id)
        ->value('email');
    }

    public function licenseUser() {
        return Licensee::where('status', STATUS)
        ->where('id', Auth::user()->client_id)
        ->where('email', Auth::user()->username)
        ->count();
    }

    public function licensePhone($client_id) {
        return Licensee::where('id', $client_id)
        ->value('phone');
    }
}

