<?php

namespace App\Http\Controllers\Settings\Measurement;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Retails\Measurement;
use App\Models\Retails\MeasurementCategory;
use Illuminate\Support\Facades\Auth;
use DB;

class MeasurementController extends Controller
{
    /**
     * Get the base measurements for a specific client and location.
     *
     * Retrieves a list of base measurements that belong to a specified client and location.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\JsonResponse
     *
     * @author Idowu Adeyinka
     */
    public function getAllBaseMeasurement(Request $request)
    {
        $measurement = Measurement::where('client_id', STATUS)
            ->where('location_id', STATUS)
            ->where('hierarchy', ONE_STATUS)
            ->get();

            $res = [];
        foreach ($measurement as $data) {
            $res[] = [
                'value' => $data->id,
                'title' => $data->name,
                'm_category_id' => $data->m_category_id
            ];
        }

        return response()->json([
            'message' => 'Base measurement fetched successfully',
            'data' => $res,
            'success' => true
        ], 200);
    }

    public function getNonBaseMeasurement(Request $request, $id)
    {
        $measurement = Measurement::where('client_id', STATUS)
            ->where('location_id', STATUS)
            ->where('m_category_id', $id)
            ->where('hierarchy', '>', ONE_STATUS)
            ->orderBy('hierarchy', 'ASC')
            ->get();

            $res = [];
        foreach ($measurement as $data) {
            $res[] = [
                'value' => $data->id,
                'title' => $data->name,
            ];
        }

        return response()->json([
            'message' => 'Base measurement fetched successfully',
            'data' => $res,
            'success' => true
        ], 200);
    }

    public function getPrimaryMeasurement($id, $client_id, $location_id)
    {
        $measurement = Measurement::where('m_category_id', $id)
            ->where('client_id', $client_id)
            ->where('location_id', $location_id)
            ->where('status', STATUS)
            ->where('mstatus', 1)
            ->min('hierarchy');

        return $measurement;
    }

    public function getMeasurementInfo()
    {
        $data = Measurement::where('client_id', Auth::user()->client_id)
            ->where('location_id', Auth::user()->location_id)
            ->where('status', STATUS)
            ->get();

            return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function getDefaultMeasurementCategory()
    {
        return MeasurementCategory::where('client_id', 0)
            ->where('location_id', 0)
            ->get();
    }

    public function getDefaultMeasurement()
    {
        return Measurement::where('client_id', 0)
            ->where('location_id', 0)
            ->get();
    }

    public function isMeasurementCategory($client_id, $name)
    {
        return MeasurementCategory::where('status', STATUS)
            ->where('client_id', $client_id)
            ->where('name', $name)
            ->first();  
    }

    public function isMeasurement($client_id, $location_id, $name)
    {
        return Measurement::where('status', STATUS)
            ->where('client_id', $client_id)
            ->where('location_id', $location_id)
            ->where('name', $name)
            ->count();  
            
    }

    public function getAllMeausrementDropdown() {
        $data = Measurement::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->selectRaw('CONCAT(hierarchy, "-", m_category_id) as value, name as title')
        ->orderBy('hierarchy')
        ->get();  

        return response()->json([
            'message' => 'Fetched successfully',
            'data' => $data,
            'success' => true
        ], 200);
    }

    public function getMeausrementByCategoryDropdown(Request $request) {
        $data = Measurement::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('m_category_id', $request->m_category_id)
        ->select('hierarchy as value', 'name as title')
        ->orderBy('hierarchy')
        ->get();  

        return response()->json([
            'message' => 'Fetched successfully',
            'data' => $data,
            'success' => true
        ], 200);
    }

    public function getMeausrementByCategoryAndMetricDropdown(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;

        $query = "
            SELECT
                m.name as title,
                CONCAT(m.hierarchy, '-', m.abr) as value,
                i.rate as price

            FROM " . ONLINE_RETAILS . ".item_price i

            INNER JOIN " . ONLINE_RETAILS . ".measurement m 
            ON m.m_category_id='$request->m_category_id' AND m.hierarchy=i.m_metric AND m.client_id='$client_id' AND m.location_id='$location_id' AND i.item_id IS NOT NULL AND m.status = 0

            WHERE i.client_id='$client_id' AND i.location_id='$location_id' AND i.item_id='$request->item_id' AND i.status = 0 AND i.rate > 0 AND i.qty > 0 AND i.cal_qty > 0
            
            ORDER BY m.hierarchy ASC
        ";

        $data = DB::select($query);

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function setPrimaryMeasurement(Request $request) {

        DB::table(TBL_MEASUREMENT)->where('m_category_id', $request->m_category_id)
        ->where('hierarchy', $request->m_metric)
        ->where('location_id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->update([
            'mstatus' => 1
        ]);

        DB::table(TBL_MEASUREMENT)
        ->where('m_category_id', $request->m_category_id)
        ->where('hierarchy', '!=', $request->m_metric) // Corrected line
        ->where('location_id', Auth::user()->location_id)
        ->where('client_id', Auth::user()->client_id)
        ->update(['mstatus' => 0]);


        return response()->json(['message' => 'Primary measurement set successfully', 'success' => true], 200);

    }

    public function getMeausrementByCategory($m_category_id) {
        $data = Measurement::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('m_category_id', $m_category_id)
        ->select('hierarchy')
        ->get();  

        return $data;
    }

    public function getMeausrementInfoByCategory($m_category_id) {
        $data = Measurement::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('m_category_id', $m_category_id)
        ->orderBy('hierarchy', 'asc')
        ->get();  

        return $data;
    }

    public function getMeasurementNameByHierarchy(
        $hierarchy,
        $m_category_id
    ) {
        return Measurement::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('m_category_id', $m_category_id)
        ->where('hierarchy', $hierarchy)
        ->value('name');
    }

    public function getMeasurementCategoryInfo()
    {
        $data = MeasurementCategory::where('status', STATUS)
            ->where('client_id', Auth::user()->client_id)
            ->get();  

        return response()->json(['data' => $data, 'success' => true], 200);
    }

    public function countMeausrementByCategory($m_category_id) {
        return Measurement::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->where('m_category_id', $m_category_id)
        ->count();  

    }
}
