<?php

namespace App\Http\Controllers\Settings\User;

use App\Http\Controllers\Controller;
use App\Http\Controllers\General\GeneralController;
use App\Http\Controllers\Folio\Location\LocationController;
use App\Http\Controllers\Settings\License\LicenseController;
use App\Mail\UserSignUpMail;
use Illuminate\Support\Facades\Mail;
use App\Models\Retails\Location;
use Illuminate\Http\Request;
use App\Models\Retails\Menu;
use App\Models\Retails\Rights;
use App\Models\Account\User;
use App\Models\Account\SubscriptionPlanDetails;
use App\Models\Retails\MobileModule;
use Carbon\Carbon;
use DB;
use Validator;
use Auth;
use Illuminate\Database\QueryException;
use Exception;

class UserController extends GeneralController
{
    //
    protected $locationController;
    protected $licenseController;

    public function __construct(
        LocationController $locationController,
        LicenseController $licenseController
    ) {
        $this->locationController = $locationController;
        $this->licenseController = $licenseController;
    }

    public function masterShipStatus($user) {
        return User::where('status', STATUS)
        ->where('id', $user->id)
        ->where('location_id', $user->location_id)
        ->where('client_id', $user->client_id)
        ->value('master_status');
    }

    public function userDropdown() {
        $data = User::where('status', STATUS)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->select('name as title', 'id as value')
        ->get();

        return response()->json(['results' => count($data), 'message' => 'Users fetched successfully', 'data' => $data, 'success' => true], 200);
    }

    public function masterUser($user, $plan_id) {
        $masterRight = $this->masterShipStatus($user);

        if($masterRight == ONE_STATUS) {
            $menu = SubscriptionPlanDetails::where('status', STATUS)
            ->where('app_id', APP_ID)
            ->where('plan_id', $plan_id)
            ->where('is_menu', ONE_STATUS)
            ->get();

            foreach($menu as $m) {

                $rights = Rights::where('user_id', $user->username)
                ->where('module_id', $m->module_id)
                ->where('plan_id', $plan_id)
                ->where('menu_id', $m->id)
                ->where('client_id', $user->client_id)
                ->where('location_id', $user->location_id)
                ->where('user', $user->id)
                ->count();

                if($rights < ONE_STATUS) {
                    $fields = [
                        'user_id'=>$user->username,
                        'plan_id'=>$m->plan_id,
                        'menu_id'=> $m->id,
                        'sub_menu'=> $m->sub_menu,
                        'module_id' => $m->module_id,
                        'user' => $user->id,
                        'location_id' => $user->location_id,
                        'client_id' => $user->client_id,
                        'tm' => Carbon::now()->toTimeString(),
                        'date' => Carbon::now()->format('Y-m-d'),
                        'status' => STATUS,
                    ];
        
                    Rights::create($fields);
                }
            }
        } else {
            // saving for a regular user if subscription is new

            if($plan_id > ONE_STATUS){
                $menu = SubscriptionPlanDetails::where('status', STATUS)
                ->where('app_id', APP_ID)
                ->where('plan_id', $plan_id)
                ->where('is_menu', ONE_STATUS)
                ->get();

                foreach($menu as $m) {
                
                    $rights = Rights::where('user_id', $user->username)
                    ->where('module_id', $m->module_id)
                    ->where('plan_id', $plan_id)
                    ->where('menu_id', $m->id)
                    ->where('client_id', $user->client_id)
                    ->where('location_id', $user->location_id)
                    ->where('user', $user->id)
                    ->count();

                    $basicMenu = Rights::where('user_id', $user->username)
                    ->where('module_id', $m->module_id)
                    ->where('plan_id', ONE_STATUS)
                    ->where('menu_id', $m->id)
                    ->where('client_id', $user->client_id)
                    ->where('location_id', $user->location_id)
                    ->where('user', $user->id)
                    ->first();

                    if($rights < ONE_STATUS) {
                        $fields = [
                            'user_id'=>$user->username,
                            'plan_id'=>$plan_id,
                            'menu_id'=> $m->id,
                            'sub_menu'=> $m->sub_menu,
                            'module_id' => $m->module_id,
                            'user' => $user->id,
                            'location_id' => $user->location_id,
                            'client_id' => $user->client_id,
                            'tm' => Carbon::now()->toTimeString(),
                            'date' => Carbon::now()->format('Y-m-d'),
                            'status' => $basicMenu ? STATUS : ONE_STATUS,
                        ];
            
                        Rights::create($fields);
                    }
                }
            }
        }
    }
    
//     SELECT
//     mm.id,
//     mm.module_name,
//     CAST(JSON_ARRAYAGG(JSON_OBJECT('menu_name', m.menu_name, 'menu_id', m.id)) AS JSON) AS menu_items
// FROM
//     " . ONLINE_RETAILS . ".mobile_module mm
// JOIN
//     " . ONLINE_RETAILS . ".menu m ON mm.id = m.module_id
// GROUP BY
//     mm.id, mm.module_name;
   

    public function getModulesWithMenus()
    {
        $plan_id = $this->locationController->getPlanId(Auth::user()->client_id, Auth::user()->location_id);
        $result = DB::select("
         
                SELECT
                    mm.id,
                    mm.module_name,
                    mm.status,
                    CONCAT('[', GROUP_CONCAT(JSON_OBJECT(
                        'menu_name', m.menu_name, 
                        'menu_id', m.id,
                        'status',false,
                        'module_id',mm.id,
                        'display', CASE WHEN m.module_id = 5 THEN 0 ELSE 1 END
                    )), ']') AS menu_items
                FROM
                  " . ONLINE_RETAILS . ".mobile_module mm
                JOIN
                " . TBL_SUB_MENU . " m ON mm.id = m.module_id AND m.screen != '' AND m.is_menu=1 AND m.plan_id='$plan_id' AND m.mobile=1 AND m.status=0
                WHERE   mm.status = '0' 
                GROUP BY
                    mm.id, mm.module_name;

        "); 
    
        return response()->json(['data' => $result, 'success' => true], 200);
    }

    // public function registerUserProcess(){
    //     $client_id = Auth::user()->client_id;
    //     $location_id = Auth::user()->location_id;
    //     $result = DB::select("
         

    //     WITH usermodule AS (
    //         SELECT
    //             mm.id,
    //             mm.module_name,
    //             mm.status,
    //             CONCAT('[', GROUP_CONCAT(JSON_OBJECT('menu_name', m.menu_name, 'menu_id', m.id)), ']') AS menu_items
    //         FROM
    //           " . ONLINE_RETAILS . ".mobile_module mm
    //         JOIN
    //         " . ONLINE_RETAILS . ".menu m ON mm.id = m.module_id
    //         WHERE   mm.status = '0'
    //         GROUP BY
    //             mm.id, mm.module_name),

    //     WITH location_dropdown AS (
    //         SELECT 
    //             *
    //         FROM
    //         " . ONLINE_RETAILS . ".location 
    //         WHERE  client_id = '$client_id' AND status = '0' 
    //         ),

    //         WITH store_dropdown AS (
    //             SELECT 
    //             *
    //         FROM
    //         " . ONLINE_RETAILS . ".store 
    //         WHERE  client_id = '$client_id'  AND location_id = '$location_id' AND status = '0' 

    //         )


    // "); 

    // return response()->json(['data' => $result, 'success' => true], 200);

    // }

    public function registerUserProcess()
    {
        $clientId = Auth::user()->client_id;
        $locationId = Auth::user()->location_id;
        $onlineRetails = ONLINE_RETAILS; // Ensure this constant is defined
    
        $query = "
            WITH 
            usermodule AS (
                SELECT
                    mm.id,
                    mm.module_name AS name,
                    mm.status,
                    CONCAT('[', GROUP_CONCAT(JSON_OBJECT('menu_name', m.menu_name, 'menu_id', m.id)), ']') AS menu_items,
                    'module' AS type
                FROM
                    {$onlineRetails}.mobile_module mm
                JOIN
                    {$onlineRetails}.menu m ON mm.id = m.module_id
                WHERE   
                    mm.status = '0'
                GROUP BY
                    mm.id, mm.module_name, mm.status
            ),
            location_dropdown AS (
                SELECT 
                    id,
                    name,
                    status,
                    '[]' AS menu_items,
                    'location' AS type
                FROM
                    {$onlineRetails}.location 
                WHERE  
                    client_id = '$clientId' AND status = '0'
            ),
            store_dropdown AS (
                SELECT 
                    id,
                    name,
                    status,
                    '[]' AS menu_items,
                    'store' AS type
                FROM
                    {$onlineRetails}.store 
                WHERE  
                    client_id = '$clientId' AND location_id = '$locationId' AND status = '0'
            )
            SELECT * FROM usermodule
            UNION ALL
            SELECT * FROM location_dropdown
            UNION ALL
            SELECT * FROM store_dropdown
        ";
    
        $result = DB::select($query);
    
        return response()->json(['data' => $result, 'success' => true], 200);
    }
    

    private function sendEmailVerification($email, $name, $userId)
    {
        $verificationUrl = $this->generateVerificationUrl($userId);
        $arr = ['fullname' => $name, 'auth_code' => '1212122', 'verificationUrl' => $verificationUrl];
        Mail::to($email)->send(new UserSignUpMail($arr));
    }

    private function generateVerificationUrl($userId)
    {
      
        $url = env('APP_URL') . "/api/email-verification/" . $userId . "/" . sha1($userId);

        // $url = 'http://127.0.0.1:8000/email-verification/' . $userId . '/' . sha1($userId);
        return $url;
    }


    public function addUser(Request $request)
    {
        // try {
            $validationResult = $this->validateRequest($request);
            
            if ($validationResult) {
                return $validationResult;
            }

            $locationId = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;

            $currentUsersCount = User::where('client_id', Auth::user()->client_id)
                                ->where('location_id', $locationId)
                                ->where('status', STATUS)
                                ->count();

            $location = Location::find($locationId);

            if ($currentUsersCount >= $location->allowed_users) {
                return response()->json(['message' => 'You have exceeded the allowed number of users for this location', 'success' => false], 400);
            }

            $currentDate = Carbon::now()->format('Y-m-d');
            $salt = $this->salt();
            $pass = $this->hashPass($request->password, $salt);
            $overview = $request->overview;
            $deletion = $request->deletion;
            $masterStatus = $request->master_status;
            $salesPriviledge = $request->sales_priviledge;
            $upsell = $request->upsell;
            $storeId = $request->store_id;
            $locationOverview = $request->location_overview;
            $backDate = $request->backdate;
            $discount = $request->discount;
            $edit = $request->edit;
            $position = $request->position;
            $emailVerification = ONE_STATUS;
            $smsVerification = STATUS;
            $priceVisiiity = $request->visibility;
            $dashboard = $request->dashboard;
            
            $web = $request->allowedOnWeb;
            $mobile = $request->allowedOnMobile;
            $desktop = $request->allowedOnDesktop;
            $confirmBankPos = $request->confirmBankPos;
            $bankPrivilege = $request->bankPrivilege;
    
            // Create User
            $user = $this->createUser($request, $currentDate, Auth::user()->client_id, $locationId, $salt, $pass, $overview, $deletion, $masterStatus, $salesPriviledge, $upsell, $storeId, $locationOverview, $backDate, $discount, $edit, $position, $emailVerification, $smsVerification, $priceVisiiity, $dashboard, $web, $mobile, $desktop, $confirmBankPos, $bankPrivilege);
    
            // Create Rights

            $plan_id = $this->locationController->getPlanId(Auth::user()->client_id, $locationId);
            $modules = $request->modules;

            if ($user) {
                foreach ($modules as $module) {
                    $mod = $request->platform =='desktop' ? $module['child_routes'] : $module['menuItem'];
                    foreach ($mod as $menuItem) {
                        // Insert only if the status is true
                        // if ($menuItem['status'] === true) {
                            $status = 1;

                            if($menuItem['status'] == true){
                                $status = 0;
                            } else if($menuItem['status'] == 0) {
                                $status = 1;
                            } else {
                                $status = 0;
                            }

                            Rights::create([
                                'module_id' => $menuItem['module_id'],
                                'menu_id' => $menuItem['menu_id'],
                                'plan_id' => $plan_id,
                                'user_id' => $request->email,
                                'sub_menu' => STATUS, 
                                'status' => $status,
                                'location_id' => $locationId,
                                'client_id' => Auth::user()->client_id,
                                'user' => $user->id,
                                'date' => $currentDate, 
                                'tm' => Carbon::now()->toTimeString()
                            ]);
                        // }
                    }
                }

                $this->sendEmailVerification($user->username, $user->name, $user->id);
            }
    
            return response()->json(['message' => 'Success', 'success' => true, 'data' => $user], 200);
        // } catch (\Exception $e) {
        //     return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 500);
        // }
    }

    // public function getUserById($userId)
    // {
    //     try {
    //         $user = User::find($userId);

    //         if (!$user) {
    //             return response()->json(['message' => 'User not found', 'success' => false], 404);
    //         }

    //         // Retrieve rights associated with the user
    //         $rights = Rights::where('user', $userId)->get();

    //         return response()->json([
    //             'message' => 'Success',
    //             'success' => true,
    //             'data' => [
    //                 'user' => $user,
    //                 'rights' => $rights,
    //             ],
    //         ], 200);
    //     } catch (\Exception $e) {
    //         return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 500);
    //     }
    // }





    public function getUserById($userId)
    {
        try {
            $user = User::find($userId);
    
            if (!$user) {
                return response()->json(['message' => 'User not found', 'success' => false], 404);
            }
    
            // Retrieve rights associated with the user along with module names and menu names
            $rights = Rights::with(['module', 'menu']) // Assuming relationships are defined in Rights model with MobileModule and Menu models
                ->where('user', $userId)
                ->get();
    
            // Build an associative array with module names as objects containing an array of rights
            $modules = [];
            foreach ($rights as $right) {
                $moduleName = $right->module->module_name;
                $moduleId = $right->module->id;
                $menuName = $right->menu->menu_name;
    
                if ($moduleName) {
                    if (!isset($modules[$moduleName])) {
                        $modules[$moduleName] = (object)['module_name' => $moduleName, 'module_id' => $moduleId, 'rights' => []];
                    }
    
                    $rightData = [
                        'menu_id' => $right->menu_id,
                        'menu_name' => $menuName,
                        // Add other right attributes as needed
                    ];
    
                    $modules[$moduleName]->rights[] = $rightData;
                }
            }
    
            return response()->json([
                'message' => 'Success',
                'success' => true,
                'data' => [
                    'user' => $user,
                    'modules' => array_values($modules), // Convert associative array to indexed array
                ],
            ], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 500);
        }
    }
    

    public function editUser(Request $request, $userId)
    {
        // try {
            // Check for required fields
            $fieldsToCheck = ['name', 'email', 'username'];
        
            foreach ($fieldsToCheck as $field) {
                $result = $this->checkExistingRecord($field, $request->$field, $userId);
                if ($result) {
                    return response()->json(['message' => $result, 'error' => true], 400);
                }
            }

            $currentDate = Carbon::now()->format('Y-m-d');
            
            // Initialize the user's update data
            $updateData = [
                'name' => $request->name,
                'username' => $request->email,
                'username2' => $request->username,
                'over_view' => $request->overview,
                'deletion' => $request->deletion,
                'status' => $request->status,
                'discount' => $request->discount,
                'up_sell' => $request->upsell,
                'back_date' => $request->backdate,
                'sales_priviledge' => $request->sales_privilege,
                'sms_verification' => STATUS,
                'master_status' => $request->master_status,
                'edit' => $request->edit,
                'confirm_bank_pos' => $request->confirm_bank_pos,
                'sales_return' => ONE_STATUS,
                'allowed_on_desktop' => $request->allowedOnDesktop,
                'dashboard' => $request->dashboard,
                'stock_cp' => $request->visibility,
                'bank_priviledge' => $request->bankPrivilege,
                'allowed_on_web' =>  $request->allowedOnWeb,
                'allowed_on_mobile' => $request->allowedOnMobile,
                'position' => $request->position,
                'office' => 'control',
                'location_overview' => $request->location_overview,
                'store_master' => ONE_STATUS,
                'store_id' => $request->store_id,
                'update_date' => $currentDate,
                'email_verified_at' => now()
            ];

            // Update the password only if it is provided
            if (!empty($request->password)) {
                $salt = $this->salt();
                $pass = $this->hashPass($request->password, $salt);

                $updateData['salt'] = $salt;
                $updateData['password'] = $pass;
            }

            // Update User
            $user = User::find($userId);
            if (!$user) {
                return response()->json(['message' => 'User not found', 'success' => false], 404);
            }

            // echo json_encode($updateData);exit;

            $user->update($updateData);

            // Update Rights
            $modules = $request->modules;
            if ($user) {
                
                foreach ($modules as $module) {
                    $mod = $request->platform =='desktop' ? $module['child_routes'] : $module['menu'];
                    foreach ($mod as $menuItem) {

                        $status = 1;
                        if($request->platform =='desktop') {
                            if($menuItem['status'] === true || $menuItem['status'] === 1){
                                $status = 0;
                            } else {
                                $status = 1;
                            }
                        } else {
                            if($menuItem['status'] === 0){
                                $status = 0;
                            } else if($menuItem['status'] === 1) {
                                $status = 1;
                            } 
                        }

                        Rights::where('user', $user->id)
                            ->where('module_id', $menuItem['module_id'])
                            ->where('menu_id', $menuItem['menu_id'])
                            ->update([
                                'status' => $status,
                                'tm' => Carbon::now()->toTimeString()
                            ]);
                    }
                }
                
            }

            return response()->json(['message' => 'Success', 'success' => true, 'data' => $user], 200);
        // } catch (\Exception $e) {
        //     return response()->json(['message' => 'Error', 'success' => false, 'error' => $e->getMessage()], 500);
        // }
    }


    public function checkExistingRecord($field, $value, $userId = null) {
        $conditions = [
            'client_id' => Auth::user()->client_id,
            'status' => 0,
        ];

        switch ($field) {
            case 'name':
                $conditions['name'] = $value;
                $message = 'Name already exists.';
                break;

            case 'phone':
                $conditions['phone'] = $value;
                $message = 'Phone already exists.';
                break;

            case 'email':
                $conditions['email'] = $value;
                $message = 'Email already exists.';
                break;

            default:
                return false; // Invalid field
        }

        // If $customerId is provided, exclude that record from the check
        if ($userId) {
            $existingRecord = User::where($conditions)->where('id', '!=', $userId)->first();
        } else {
            $existingRecord = User::where($conditions)->first();
        }

        return $existingRecord ? $message : false;
    }

    

    private function validateRequest(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required',
            'password' => 'required|min:8|confirmed',
            'dashboard' => 'required',
            'sales_priviledge' => 'required',
            'location_id' => 'required',
            'store_id' => 'required',
            'email' => 'required|email|unique:user,username|regex:/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/|max:255',
            'username' => 'required|unique:user,username2'
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation failed '.$validator->errors()->first().'', 'errors' => $validator->errors()->first()], 400);
        }

        return null;
    }


    public function toggleUserStatus($userId)
    {
        $user = User::findOrFail($userId);

        $newStatus = $user->status == 1 ? 0 : 1;

        $user->update(['status' => $newStatus]);

        $action = $newStatus == 1 ? 'enabled' : 'disabled';

        return response()->json(['message' => "User $action successfully",  "data" => '', 'success' => true], 200);
    }


    
   

   
    
    public function createUser($request, $currentDate, $licensee, $location, $salt, $pass, $overview, $deletion, $masterStatus, $salesPriviledge, $upsell, $storeId, $locationOverview, $backDate, $discount, $edit, $position, $emailVerification, $smsVerification, $priceVisiiity, $dashboard, $web="0", $mobile="0", $desktop="0", $confirmBankPos="0", $bankPrivilege="0")
    {

        try {
            $user = User::create([
                'name' => $request->name,
                'location_id' => $location,
                'salt' => $salt,
                'password' => $pass,
                'username' => $request->email,
                'username2' => $request->username,
                'phone' => $request->phone,
                'over_view' =>  $overview,
                'deletion' =>  $deletion,
                'status' => STATUS,
                'discount' => $discount,
                'up_sell' =>  $upsell,
                'back_date' =>$backDate,
                'verification_status' => $smsVerification,
                'sales_priviledge' =>  $salesPriviledge,
                'email_verification' => $emailVerification,
                'master_status' =>  $masterStatus,
                'edit' =>  $edit,
                'confirm_bank_pos' => $confirmBankPos,
                'sales_return' => ONE_STATUS,
                'app' => APP_ID,
                'allowed_on_desktop' => $desktop,
                'dashboard' => $dashboard,
                'stock_cp' => $priceVisiiity,
                'bank_priviledge' => $bankPrivilege,
                'allowed_on_web' => $web,
                'allowed_on_mobile' => $mobile,
                'position' =>  $position,
                'office' => 'control',
                'location_overview' =>  $locationOverview,
                'store_master' => ONE_STATUS,
                'store_id' =>  $storeId,
                'date' => $currentDate,
                'tm' => $currentDate,
                'client_id' => $licensee,
                'update_date' => $currentDate,
                'expo_token' => $request->expo_token,
                'email_verified_at' => now()
            ]);
    
            
            if (!$user) {
                throw new Exception("Failed to create user.");
            }
    
            return $user;
        } catch (QueryException $e) {
          
            throw new Exception("Database error on creating user: " . $e->getMessage());
        } catch (Exception $e) {
            throw new Exception("Error creating user: " . $e->getMessage());
        }
    }
    

    public function registerUser(){
        
    }

   
    public function getAllUsersByClientId(Request $request)
    {
        $client_id = Auth::user()->client_id;
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $name = !empty($request->name) ? "AND u.name LIKE '%$request->name%'" : "";
        $plan_id = $this->locationController->getPlanId($client_id, $location_id);
        
        $query = "
            
            WITH rights as (
                SELECT r.id, r.plan_id, r.module_id, r.menu_id, r.status, m.module_name, x.menu_name, x.path, x.screen, x.menu_title, r.user 
            FROM ".TBL_RIGHTS." r
            LEFT JOIN ".TBL_MODULE." m ON r.module_id = m.id
            LEFT JOIN ".TBL_SUB_MENU." x ON x.id=r.menu_id AND x.status = 0 AND x.is_menu = 1 AND x.screen != ''
            WHERE r.plan_id = '$plan_id'
            GROUP By r.module_id, r.user,r.menu_id 
            HAVING x.menu_name is NOT NULL
        ),

            rightMenu as (
                SELECT module_id, module_name, user,
                JSON_ARRAYAGG(
                JSON_OBJECT(
                    'menu_id', menu_id,
                    'menu_name', menu_name,
                    'module_id',module_id,
                    'status', status,
                    'display', CASE WHEN module_id = 5 THEN 0 ELSE 1 END
                )) as menu
            FROM rights
            GROUP BY  user, module_id),

            users AS (
                SELECT
                    u.id, 
                    u.name, 
                    u.username,
                    u.username2,
                    u.sales_priviledge,
                    u.dashboard,
                    u.over_view,
                    u.location_overview,
                    u.location_id,
                    u.store_id,
                    u.deletion,
                    u.edit,
                    u.back_date,
                    u.discount,
                    u.up_sell,
                    u.stock_cp,
                    u.bank_priviledge,
                    u.master_status,
                    u.confirm_bank_pos,
                    u.allowed_on_web,
                    u.allowed_on_mobile,
                    u.allowed_on_desktop,
                    u.date,
                    u.status,
                    CASE WHEN s.name IS NULL THEN 'all' ELSE s.name END as store,
                    l.name as location,
                    (
                        
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'id', r.module_id,
                            'module_name', r.module_name,
                            'menu',r.menu
                        )) as menu
                       
                        FROM rightMenu r
                        
                        WHERE r.user=u.id
                     
                    ) as rights
            
                FROM " . TBL_USER . " u

                LEFT JOIN " . TBL_LOCATION . " l
                ON l.id= '$location_id' AND l.status=0 AND l.client_id='$client_id'

                LEFT JOIN " . TBL_STORE . " s
                ON s.location_id= '$location_id' AND s.status=0 AND s.client_id='$client_id' AND s.id=u.store_id
            
                WHERE 
                    u.client_id='$client_id' AND u.location_id='$location_id' $name
                ORDER BY 
                    u.id ASC
                                )
            SELECT 
                id, 
                name, 
                username, 
                username2,
                sales_priviledge,
                dashboard,
                over_view,
                location_overview,
                location_id,
                store_id,
                deletion,
                edit,
                back_date,
                discount,
                up_sell,
                stock_cp,
                bank_priviledge,
                master_status,
                confirm_bank_pos,
                allowed_on_web,
                allowed_on_mobile,
                allowed_on_desktop,
                store,
                location,
                date,
                status,
                rights
            FROM users
        ";

        // echo $query;exit;

        $data = DB::select($query);

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }


    public function getUserDesktopRights($client_id, $location_id, $user_id, $origin) {
        $plan_id = $this->locationController->getPlanId($client_id, $location_id);
        $platform1 = $origin == 2 ? "AND x.desktop=1" : "AND x.mobile=1";
        $platform2 = $origin == 2 ? "AND sub.desktop=1" : "AND sub.mobile=1";
        $query = "
            WITH rights AS (
                SELECT
                    x.menu_id AS subMenuId,
                    m.module_title,
                    m.module_icon,
                    r.plan_id,
                    m.id,
                    r.module_id,
                    r.menu_id,
                    r.status,
                    m.module_name,
                    x.menu_name,
                    x.path,
                    x.screen,
                    x.menu_title,
                    r.user,
                    x.sub_module,
                    x.sub_menu
                FROM ".TBL_RIGHTS." r
                LEFT JOIN ".TBL_MODULE." m ON m.module_icon != '' AND m.status=0
                LEFT JOIN ".TBL_SUB_MENU." x ON x.id = r.menu_id AND x.module_id = r.module_id AND x.is_menu = 1 $platform1 AND x.status=0

                WHERE r.module_id=m.id AND r.client_id = '$client_id' AND r.location_id = '$location_id' AND r.module_id IS NOT NULL AND r.plan_id = '$plan_id' and r.status = 0
                AND x.menu_name IS NOT NULL
                ORDER BY m.id ASC
            ),
            rightMenu AS (
                SELECT
                    JSON_OBJECT(
                        'module_id', r.module_id,
                        'menu_title', r.module_title,
                        'path', 'javascript:void(0)',
                        'icon', r.module_icon,
                        'category', 'general',
                        'isMenuOpen', false,
                        'menu', r.module_name,
                        'fullPageMenu', false,
                        'desc', true,
                        'content', '',
                        'status', 0,


                        'child_routes', JSON_ARRAYAGG(
                            JSON_OBJECT(
                                'sub_module', r.sub_module,
                                'module_id', r.module_id,
                                'menu_id', r.menu_id,
                                'id', r.subMenuId,
                                'sub_menu', r.sub_menu,
                                'menu_title', r.menu_title,
                                'menu_name', r.menu_name,
                                'path', r.path,
                                'child_routes', null,
                                'icon', 'policy',
                                'fullPageMenu', false,
                                'isMenuOpen', false,
                                'exact', true,
                                
                                'third_child_routes', (
                                    SELECT JSON_ARRAYAGG(JSON_OBJECT(  
                                        'sub_module', sub.sub_module,
                                        'module_id', sub.module_id,
                                        'id', sub.id,
                                        'menu_id', sub.id,
                                        'sub_menu', sub.sub_menu,
                                        'menu_title', sub.menu_title,
                                        'path', sub.path,
                                        'menu_name', sub.menu_name,
                                        'child_routes', null,
                                        'icon', 'policy',
                                        'fullPageMenu', false,
                                        'isMenuOpen', false,
                                        'third_child_routes', null,
                                        'exact', true
                                    ))
            
                                    FROM " . TBL_SUB_MENU . " sub
            
                                    WHERE sub.menu_id=r.id AND sub.is_menu=1 $platform2 AND sub.status=0
                                ),
                                'status', r.status
                                
                            )
                        )
                    ) AS menu,
                    r.user
                FROM rights r
                GROUP BY r.module_id, r.user
            ), 
            users AS (
                SELECT
                    
                    (SELECT JSON_ARRAYAGG(m.menu) FROM rightMenu m WHERE m.user=u.id) AS defaultRights
                FROM " . TBL_USER . " u

                WHERE 
                    u.client_id='$client_id' AND u.location_id='$location_id' AND u.id='$user_id'
            
            )
                        
            SELECT 
            
                *
            FROM users
   
        ";
                        
        // echo $query; exit;

        $data = DB::select($query);

        foreach ($data as $row) {
            if (property_exists($row, 'defaultRights')) {
                $row->defaultRights = $row->defaultRights ? json_decode(stripslashes($row->defaultRights), true) : [];
            }
        }

        return $data[0]->defaultRights;

    }
    

    public function getDesktopAllUsersByClientId(Request $request)
    {
        $client_id = Auth::user()->client_id;
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $name = !empty($request->name) ? "AND u.name LIKE '%$request->name%'" : "";
        $plan_id = $this->locationController->getPlanId($client_id, $location_id);
        $owner = $this->licenseController->licenseUser();
        $filterLocation = $owner > STATUS ? "" : "AND u.location_id='$location_id'";

        $query = "
            WITH rights AS (
                SELECT
                    x.menu_id AS subMenuId,
                    m.module_title,
                    m.module_icon,
                    r.plan_id,
                    m.id,
                    r.module_id,
                    r.menu_id,
                    r.status,
                    m.module_name,
                    x.menu_name,
                    x.path,
                    x.screen,
                    x.menu_title,
                    r.user,
                    x.sub_module,
                    x.sub_menu
                FROM ".TBL_RIGHTS." r
                LEFT JOIN ".TBL_MODULE." m ON m.module_icon != '' AND m.status=0
                LEFT JOIN ".TBL_SUB_MENU." x ON x.id = r.menu_id AND x.module_id = r.module_id AND x.is_menu = 1 AND x.desktop = 1 AND x.status=0

                WHERE r.module_id=m.id AND r.client_id = '$client_id' AND r.module_id IS NOT NULL AND r.plan_id = '$plan_id'
                AND x.menu_name IS NOT NULL
                ORDER BY m.id ASC
            ),
            rightMenu AS (
                SELECT
                    JSON_OBJECT(
                        'module_id', r.module_id,
                        'menu_title', r.module_title,
                        'path', 'javascript:void(0)',
                        'icon', r.module_icon,
                        'category', 'general',
                        'isMenuOpen', false,
                        'menu', r.module_name,
                        'fullPageMenu', false,
                        'desc', true,
                        'content', '',
                        'status', 0,

                        'child_routes', JSON_ARRAYAGG(
                            JSON_OBJECT(
                                'sub_module', r.sub_module,
                                'module_id', r.module_id,
                                'menu_id', r.menu_id,
                                'id', r.subMenuId,
                                'sub_menu', r.sub_menu,
                                'menu_title', r.menu_title,
                                'path', r.path,
                                'child_routes', null,
                                'icon', 'policy',
                                'fullPageMenu', false,
                                'isMenuOpen', false,
                                'exact', true,
                                
                                'third_child_routes', (
                                    SELECT JSON_ARRAYAGG(JSON_OBJECT(  
                                        'sub_module', sub.sub_module,
                                        'module_id', sub.module_id,
                                        'id', sub.id,
                                        'menu_id', sub.id,
                                        'sub_menu', sub.sub_menu,
                                        'menu_title', sub.menu_title,
                                        'path', sub.path,
                                        'child_routes', null,
                                        'icon', 'policy',
                                        'fullPageMenu', false,
                                        'isMenuOpen', false,
                                        'third_child_routes', null,
                                        'exact', true
                                    ))
            
                                    FROM " . TBL_SUB_MENU . " sub
            
                                    WHERE sub.menu_id=r.id AND sub.is_menu=1 AND sub.desktop = 1 AND sub.status=0
                                ),
                                'status', CASE WHEN r.status = 0 THEN 1 ELSE 0 END,
                                'display', CASE WHEN r.module_id = 5 THEN 0 ELSE 1 END
                                
                            )
                        )
                    ) AS menu,
                    r.user
                FROM rights r
                GROUP BY r.module_id, r.user
            ), 
            users AS (
                SELECT
                    u.id, 
                    u.name, 
                    u.username,
                    u.username2,
                    u.sales_priviledge,
                    u.dashboard,
                    u.over_view,
                    u.location_overview,
                    u.location_id,
                    u.store_id,
                    u.deletion,
                    u.edit,
                    u.back_date,
                    u.discount,
                    u.up_sell,
                    u.stock_cp,
                    u.bank_priviledge,
                    u.master_status,
                    u.confirm_bank_pos,
                    u.allowed_on_web,
                    u.allowed_on_mobile,
                    u.allowed_on_desktop,
                    u.date,
                    u.status,
                    CASE WHEN s.name IS NULL THEN 'all' ELSE s.name END as store,
                   
                    (SELECT JSON_ARRAYAGG(m.menu) FROM rightMenu m WHERE m.user=u.id) AS rights
                FROM " . TBL_USER . " u

                LEFT JOIN " . TBL_STORE . " s
                ON s.status=0 AND s.client_id='$client_id' AND s.id=u.store_id
            
                WHERE 
                    u.client_id='$client_id' $filterLocation $name
                ORDER BY 
                    u.id ASC
            )
                        
            SELECT 
                u.id, 
                u.name, 
                u.username,
                u.username2, 
                u.sales_priviledge,
                u.dashboard,
                u.over_view,
                u.location_overview,
                u.location_id,
                u.store_id,
                u.deletion,
                u.edit,
                u.back_date,
                u.discount,
                u.up_sell,
                u.stock_cp,
                u.bank_priviledge,
                u.master_status,
                u.confirm_bank_pos,
                u.allowed_on_web,
                u.allowed_on_mobile,
                u.allowed_on_desktop,
                u.store,
                u.date,
                u.status,
                u.rights,
                l.name as location
            FROM " . TBL_LOCATION . " l 
            LEFT JOIN users u
            ON l.status=0 AND l.client_id='$client_id' AND u.location_id = l.id AND l.id IS NOT NULL
            
            WHERE l.client_id = '$client_id' AND l.status = 0 AND u.id IS NOT NULL
        ";

        // echo $query;exit;

        $data = DB::select($query);

        foreach ($data as $row) {
            if (property_exists($row, 'rights')) {
                $row->rights = $row->rights ? json_decode(stripslashes($row->rights), true) : [];
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function getDesktopRights()
    {
        $client_id = Auth::user()->client_id;
        $user_id = Auth::user()->id;
        $location_id =  Auth::user()->location_id;
        $plan_id = $this->locationController->getPlanId($client_id, $location_id);

        $query = "
            WITH rights AS (
                SELECT
                    x.id AS menuId,
                    x.menu_id AS subMenuId,
                    m.module_title,
                    m.module_icon,
                    r.plan_id,
                    m.id,
                    r.module_id,
                    r.menu_id,
                    r.status,
                    m.module_name,
                    x.menu_name,
                    x.path,
                    x.screen,
                    x.menu_title,
                    r.user,
                    x.sub_module,
                    x.sub_menu
                FROM ".TBL_RIGHTS." r
                LEFT JOIN ".TBL_MODULE." m ON r.module_id = m.id
                LEFT JOIN ".TBL_SUB_MENU." x ON x.id = r.menu_id AND x.module_id = r.module_id AND x.is_menu = 1 AND x.desktop=1 AND x.status=0
                WHERE r.client_id = '$client_id' AND r.user='$user_id' AND r.location_id = '$location_id' AND r.module_id IS NOT NULL AND r.plan_id = '$plan_id'
                AND x.menu_name IS NOT NULL
            ),
            rightMenu AS (
                SELECT
                    JSON_OBJECT(
                        'module_id', r.module_id,
                        'menu_title', r.module_title,
                        'path', 'javascript:void(0)',
                        'icon', r.module_icon,
                        'category', 'general',
                        'isMenuOpen', false,
                        'menu', r.module_name,
                        'fullPageMenu', false,
                        'desc', true,
                        'content', '',
                        'status', 0,
                        'child_routes', JSON_ARRAYAGG(
                            JSON_OBJECT(
                                'sub_module', r.sub_module,
                                'module_id', r.module_id,
                                'menu_id', r.menuId,
                                'id', r.subMenuId,
                                'sub_menu', r.sub_menu,
                                'menu_title', r.menu_title,
                                'path', r.path,
                                'child_routes', null,
                                'icon', 'policy',
                                'fullPageMenu', false,
                                'isMenuOpen', false,
                                'status', 0,
                                'exact', true,
                                'display', CASE WHEN r.module_id = 5 THEN 0 ELSE 1 END
                            )
                        )
                    ) AS menu
                FROM rights r
                GROUP BY r.module_id
            ),
            users AS (
                SELECT
                    JSON_ARRAYAGG(
                        JSON_OBJECT(
                            'title', l.name,
                            'value', l.id
                        )
                    ) AS locations,
                    (SELECT JSON_ARRAYAGG(m.menu) FROM rightMenu m) AS rights
                FROM ".TBL_LOCATION." l
                WHERE l.client_id = '$client_id' AND l.status = 0
            )
            SELECT
                u.locations,
                u.rights
            FROM users u;
        ";
        // echo $query;exit;

        $data = DB::select($query);

        foreach ($data as $row) {
            if (property_exists($row, 'rights')) {
                $row->rights = $row->rights ? json_decode(stripslashes($row->rights), true) : [];
            }

            if (property_exists($row, 'locations')) {
                $row->locations = $row->locations ? json_decode(stripslashes($row->locations), true) : [];
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }
}
