<?php

namespace App\Http\Controllers\Transactions\Sales;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Retails\ItemLog;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Inventory\Product\PriceController;
use App\Http\Controllers\Inventory\Product\ProductController;
use App\Http\Controllers\Inventory\Inventory\InventoryController;
use App\Http\Controllers\Folio\CustomerVendor\CustomerVendorController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use App\Models\Retails\Stock;
use App\Http\Controllers\Folio\Bank\BankAccountController;
use App\Http\Controllers\Settings\Store\StoreController;
use Validator;
use DB;

class ItemLogController extends Controller
{
    //
    protected $storeController;
    protected $inventoryController;
    protected $customerVendorController;
    protected $itemLogController;
    protected $txnLogController;
    protected $chartOfAccountController;
    protected $currencyController;
    protected $bankAccountController;
    protected $productController;

    public function __construct(
        StoreController $storeController,
        InventoryController $inventoryController,
        CustomerVendorController $customerVendorController,
        TxnLogController $txnLogController,
        ChartOfAccountController $chartOfAccountController,
        CurrencyController $currencyController,
        BankAccountController $bankAccountController,
        ProductController $productController
    ) {
        $this->storeController = $storeController;
        $this->inventoryController = $inventoryController;
        $this->customerVendorController = $customerVendorController;
        $this->txnLogController = $txnLogController;
        $this->chartOfAccountController = $chartOfAccountController;
        $this->currencyController = $currencyController;
        $this->bankAccountController = $bankAccountController;
        $this->productController = $productController;
    }

    public function getDiscountedAmount($token){
        return ItemLog::where('status', STATUS)
        ->where('token', $token)
        ->where('client_id', Auth::user()->client_id)
        ->where('location_id', Auth::user()->location_id)
        ->selectRaw('SUM(amount - discount) as jvalue')
        ->value('jvalue');
    }

    public function insertItemLog($customer_id, $customer, $amount, $cp, $rate, $qty, $token, $item_id, $item_category_id, $store_id, $cal_qty, $m_metric, $m_category_id, $discount, $sales_type, $description, $item_class_id, $vat_status, $vat_value, $income_account, $queuer, $txn_date, $tender) {

        $uqty = $cal_qty * $qty;

        $fields = [
            'id' => Uuid::uuid4()->toString(),
            'txn_date' => $txn_date,
            'token' => $token,
            'item_category_id' => $item_category_id,
            'item_id' => $item_id,
            'item_class_id' => $item_class_id,
            'vat_status' => $vat_status,
            'vat_value' => $vat_value,
            'qty' => $qty,
            'tender' => $tender,
            'coy' => $customer_id,
            'customer' => $customer,
            'description' => $description,
            'discount' => $discount,
            'sales_type' => $sales_type,
            'price' => $rate,
            'cp' => $cp,
            'effect' => EFFECT_CREDIT,
            'uqty' => $uqty,
            'store_id' => $store_id,
            'm_category_id' => $m_category_id,
            'income_account' => $income_account,
            'm_metric' => $m_metric,
            'queuer' => $queuer,
            'date' => Carbon::now()->format('Y-m-d'),
            'tm' => Carbon::now()->toTimeString(),
            "day_name" => Carbon::now()->format('l'),
            "week_no" => Carbon::now()->weekOfYear,
            "month_no" => Carbon::now()->month,
            'ampm' => date('A'),
            'status' => STATUS,
            'user_id' => Auth::user()->id,
            'location_id' => Auth::user()->location_id,
            'client_id' => Auth::user()->client_id,
            'update_date' => Carbon::now()->format('Y-m-d H:i:s'),
            'control_user_id' => STATUS,
            'control_id' => STATUS,
            'amount' => $amount,
        ];

        ItemLog::create($fields);
    }

    public function salesList(Request $request) {
        $user = Auth::user()->id;
        $client_id = Auth::user()->client_id;
        $token = !empty($request->token) ? " AND l.token LIKE '%$request->token%'" : "";
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $user_id = !empty($request->user_id) ? " AND l.user_id='$request->user_id'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? " AND l.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $store_id = !empty($request->store_id) ? " AND l.store_id='$request->store_id'" : "";
        $item_id = !empty($request->item_id) ? " AND l.item_id = '$request->item_id'" : "";
        $limit = "LIMIT $request->count, 50";
        $overview = Auth::user()->over_view === STATUS ? "AND l.user_id='$user'" : "";
        
        $query = "
            WITH itemList AS (
                SELECT
                l.token,
                ROUND(SUM(l.amount), 0) as amount,
                ROUND(SUM(l.discount), 0) as discount,
                ROUND(SUM(l.vat_value), 0) as vat,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'id', l.id,
                        'item', i.name,
                        'item_id',l.item_id,
                        'amount', round(l.amount, 2),
                        'qty', l.qty,
                        'price', round(l.price, 2)
                    ))
                ) as items,

                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'item', i.name,
                        'amount', round(l.amount, 2),
                        'qty', l.qty,
                        'abr', m.abr,
                        'price', round(l.price, 2),
                        'discount', round(l.discount, 2),
                        'description', l.description,
                        'vat_value', round(l.vat_value, 2),
                        'vat_status', round(l.vat_status, 2)
                    ))
                ) as receipt

                FROM " . TBL_PRODUCT_LOG . " l

                INNER JOIN " . TBL_ITEMS . " i
                ON i.id=l.item_id AND i.status = " . STATUS . " AND i.client_id='$client_id'

                LEFT JOIN " .TBL_MEASUREMENT . " m
                ON m.m_category_id=l.m_category_id AND m.status = " . STATUS . " AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.hierarchy=l.m_metric

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " $user_id $txn_date $store_id $token 
                
                GROUP BY l.token
            ),
            
            salesList AS ( 
                SELECT
                    l.token,
                    u.name,
                   
                    l.txn_date,
                    l.tm,
                    l.customer,
                    l.tender,
                   
                    txn.comp as authoriser,
                    
                    li.items,
                    li.receipt,
                    li.amount,
                    li.discount,
                    li.vat,

                    (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'pmethod', p.pmethod,
                            'amount', p.jvalue
                        ))
                    
                        FROM " . TBL_LOG . " p
                        
                        WHERE p.token=l.token AND p.pmethod != '' AND p.control_id != " . ONE_STATUS . " AND p.client_id='$client_id' AND p.location_id='$location_id'
                    ) AS pmethod

                FROM " . TBL_PRODUCT_LOG . " l

                LEFT JOIN itemList li ON li.token=l.token

                LEFT JOIN " . TBL_USER . " u
                ON u.id=l.user_id AND u.client_id='$client_id' AND u.location_id='$location_id'

                LEFT JOIN " . TBL_LOG . " txn
                ON txn.token=l.token AND txn.control_id != " . ONE_STATUS . " AND txn.client_id='$client_id' AND txn.location_id='$location_id' AND txn.comp != ''

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " $user_id $txn_date $store_id $token $item_id $overview

                GROUP BY l.token
                ORDER BY l.txn_date DESC, l.tm DESC
                $limit
            ),

            amounts AS (
                SELECT
                l.token,
                ROUND(SUM(l.amount), 0) as itemAmount,
                ROUND(SUM(l.discount), 0) as itemDiscount

                FROM " . TBL_PRODUCT_LOG . " l

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " $user_id $txn_date $store_id $token $item_id

                GROUP BY l.token
                ORDER BY l.txn_date DESC, l.tm DESC
                $limit
            ),

            

            summary AS (
                SELECT
                COUNT(DISTINCT l.token) as total_products,
                ROUND(SUM(l.amount - l.discount), 0) as total_amount

                FROM " . TBL_PRODUCT_LOG . " l

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " $user_id $txn_date $store_id $token $item_id
            ),

            company AS (
                SELECT
                name as location,
                address,
                phone,
                email,
                logo
                FROM " . TBL_LOCATION . " 

                WHERE client_id='$client_id' AND id='$location_id'

            )
 
            SELECT s.*, com.*, a.itemAmount, a.itemDiscount, sm.total_products, sm.total_amount
            FROM salesList s
            JOIN amounts a ON s.token = a.token
            CROSS JOIN summary sm
            CROSS JOIN company com
        ";

    

        $data = DB::select($query);
        DB::disconnect();
        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function salesSummary(Request $request) {
        $user = Auth::user()->id;
        $overview = Auth::user()->over_view === STATUS ? "AND l.user_id='$user'" : "";
        $client_id = Auth::user()->client_id;
        $location_id = !empty($request->location_id) ? $request->location_id : Auth::user()->location_id;
        $user_id = !empty($request->user_id) ? " AND l.user_id='$request->user_id'" : "";
        $txn_date = !empty($request->from) && !empty($request->to) ? " AND l.txn_date BETWEEN '$request->from' AND '$request->to'" : "";
        $store_id = !empty($request->store_id) ? " AND l.store_id='$request->store_id'" : "";
        $name = !empty($request->name) ? " AND i.name LIKE '%$request->name%'" : "";
        $limit = "LIMIT $request->count, 50";

        $query="
            WITH salesSummary as (
                SELECT
                i.name,
                l.item_id,
                ROUND(SUM(l.qty * p.cal_qty),0) as qty,
                ROUND(SUM(l.amount - l.discount),0) AS amount
            
                FROM " . TBL_PRODUCT_LOG . " l
                
                LEFT JOIN " . TBL_ITEMS . " i
                ON i.id=l.item_id AND i.status = " . STATUS . " AND i.client_id='$client_id' $name

                LEFT JOIN " . TBL_ITEM_PRICE . " p
                ON i.id=p.item_id AND p.m_metric=l.m_metric AND p.client_id='$client_id' AND p.location_id='$location_id' AND p.status = " . STATUS . "

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " AND i.id IS NOT NULL $user_id $txn_date $store_id $overview
                GROUP BY l.item_id
                ORDER BY i.name DESC
                $limit
            ),

            summary AS (
                SELECT

                COUNT(DISTINCT l.token) as total_products,
                ROUND(SUM(l.amount - l.discount), 0) as total_amount

                FROM " . TBL_PRODUCT_LOG . " l

                LEFT JOIN " . TBL_ITEMS . " i
                ON i.id=l.item_id AND i.status = " . STATUS . " AND i.client_id='$client_id' $name

                WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " AND i.id IS NOT NULL $user_id $txn_date $store_id $overview
            )

            SELECT * FROM salesSummary CROSS JOIN summary;
        ";


        $data = DB::select($query);
 
        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

}