<?php

namespace App\Http\Controllers\Transactions\Sales;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Inventory\Product\PriceController;
use App\Http\Controllers\Inventory\Product\ProductController;
use App\Http\Controllers\Inventory\Inventory\InventoryController;
use App\Http\Controllers\Transactions\Sales\ItemLogController;
use App\Http\Controllers\Folio\CustomerVendor\CustomerVendorController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use Ramsey\Uuid\Uuid;
use Carbon\Carbon;
use App\Models\Retails\Stock;
use App\Models\Retails\Location;
use App\Http\Controllers\Folio\Bank\BankAccountController;
use App\Http\Controllers\Settings\Store\StoreController;
use Validator;
use DB;

class SalesController extends Controller
{
    //
    protected $storeController;
    protected $inventoryController;
    protected $customerVendorController;
    protected $itemLogController;
    protected $txnLogController;
    protected $chartOfAccountController;
    protected $currencyController;
    protected $bankAccountController;
    protected $productController;
    protected $priceController;
 
    public function __construct(
        StoreController $storeController,
        InventoryController $inventoryController,
        CustomerVendorController $customerVendorController,
        ItemLogController $itemLogController,
        TxnLogController $txnLogController,
        ChartOfAccountController $chartOfAccountController,
        CurrencyController $currencyController,
        BankAccountController $bankAccountController,
        ProductController $productController,
        PriceController $priceController
    ) {
        $this->storeController = $storeController;
        $this->inventoryController = $inventoryController;
        $this->customerVendorController = $customerVendorController;
        $this->itemLogController = $itemLogController;
        $this->txnLogController = $txnLogController;
        $this->chartOfAccountController = $chartOfAccountController;
        $this->currencyController = $currencyController;
        $this->bankAccountController = $bankAccountController;
        $this->productController = $productController;
        $this->priceController = $priceController;
    }

    public function getOptimisedSalesItems($request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        $store_id = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $search = !empty($request->text) ? " AND i.name LIKE '%$request->text%'" : "";
        $category = !empty($request->category) ? "AND i.item_category_id='$request->category'" : "";
        $customerName = !empty($request->customerName) ? "AND name LIKE '%$request->customerName%'" : "";
        $limit = "LIMIT $request->count, 50";
        $limitCustomer = "LIMIT $request->customerCount, 50";

        $query = "
            WITH products as (
                SELECT i.id, i.name, i.reorder_level, i.sales_type, p.m_metric, p.m_category_id, i.item_category_id, p.cal_qty, p.rate, c.name as category, i.income_account, i.vat_status, i.item_class_id
                FROM " . ONLINE_RETAILS .".items i

                LEFT JOIN " . ONLINE_RETAILS .".item_price p 
                ON p.item_id = i.id AND p.location_id = '$location_id' AND p.client_id = '$client_id' AND p.status = 0

                LEFT JOIN " . ONLINE_RETAILS .".measurement m
                ON m.m_category_id = p.m_category_id AND m.hierarchy = p.m_metric AND m.client_id = '$client_id' AND m.location_id = '$location_id'

                LEFT JOIN " . ONLINE_RETAILS .".item_category c
                ON c.id = i.item_category_id AND c.client_id = '$client_id' AND c.status = 0

                WHERE i.client_id = '$client_id' AND i.status = 0 AND i.disabled_status = 0 $search $category 
                GROUP BY 
                i.id, i.name
                ORDER BY 
                i.id
                $limit
            
            ),

            drInventory as (
                SELECT 
                    p.id, p.name, p.reorder_level, s.effect, p.m_category_id, p.item_category_id, p.cal_qty, p.category, p.sales_type, p.income_account, p.vat_status, p.item_class_id,
                    SUM(s.amount) as amount, 0 as discount, '' as description, 0 as up_sell,
                    COALESCE(ROUND(SUM(s.qty * i.cal_qty), 2),0) as received,
                    COALESCE(ROUND(SUM(s.amount) / SUM(s.qty * i.cal_qty), 2),0) as cp,
                    (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', m.abr,
                            'value', round(m.hierarchy,0), 
                            'cal_qty', round(i.cal_qty,0), 
                            'price', round(i.rate,0)
                        ))
    
                        FROM " . TBL_ITEM_PRICE . " i
                        
                        LEFT JOIN " . TBL_MEASUREMENT . " m
                        ON m.m_category_id = p.m_category_id AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0 AND i.m_metric IS NOT NULL AND i.m_metric=m.hierarchy
    
                        WHERE i.item_id=p.id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.status = 0 AND p.id IS NOT NULL
                        GROUP BY i.item_id
    
                    ) AS measurements
                
                FROM products p
                
                INNER JOIN ".TBL_STOCK." s 
                    ON p.id = s.item_id AND s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND (s.effect = 'dr' OR s.effect IS NULL) AND s.store_id = '$store_id'

                LEFT JOIN ".TBL_ITEM_PRICE." i 
                    ON p.id = i.item_id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.m_metric=s.m_metric AND i.status = 0
                GROUP by p.id
            ),

            crInventory as (
                SELECT 
                    p.*,
                    COALESCE(ROUND(SUM(s.qty * i.cal_qty), 0)) as dispensed,
                    SUM(s.amount) as crAmount
                    
                FROM products p
                
                INNER JOIN ".TBL_STOCK." s 
                    ON p.id = s.item_id AND s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND s.effect = 'cr'  AND s.store_id = '$store_id'

                LEFT JOIN ".TBL_ITEM_PRICE." i 
                    ON p.id = i.item_id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.m_metric=s.m_metric AND i.status = 0
                GROUP by p.id
            ),

            pmsm As (
                SELECT i.cal_qty as pQty, i.item_id, round(i.rate,2) as price, i.m_metric, m.abr as msm, m.name as measurement 
                FROM " . ONLINE_RETAILS .".item_price i
                LEFT JOIN " . ONLINE_RETAILS .".measurement m
                ON m.m_category_id = i.m_category_id AND m.hierarchy = i.m_metric AND m.client_id = '$client_id' AND m.location_id = '$location_id'
                WHERE i.primary_measurement = 1 AND i.client_id = '$client_id' AND i.location_id = '$location_id' AND i.status = 0
            ),

            totalInventory as (
                SELECT 
                
                dr.*,
                pr.*,
                ROUND(COALESCE(ROUND(dr.received,0),0) - COALESCE(ROUND(cr.dispensed,0),0) / COALESCE(ROUND(pr.pQty,0)),2) as qty

                FROM drInventory dr

                LEFT JOIN crInventory cr
                ON cr.id=dr.id

                LEFT JOIN pmsm pr
                ON pr.item_id=dr.id
            ),

            combinedArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'id', s.id,
                    'name', s.name, 
                    'cal_qty', s.pQty, 
                    'current_cal_qty', s.pQty,
                    'msm', s.msm,
                    'measurement', s.measurement,
                    'price', s.price,
                    'reorder_level', s.reorder_level,
                    'qty', s.qty,
                    'category', s.category,
                    'discount', s.discount,
                    'description', s.description,
                    'up_sell', s.up_sell,
                    'm_category_id', s.m_category_id,
                    'item_category_id', s.item_category_id,
                    'm_metric', s.m_metric,
                    'sales_type', s.sales_type,
                    'income_account', s.income_account,
                    'vat_status', s.vat_status,
                    'item_class_id', s.item_class_id,
                    'measurements', s.measurements,
                    'currentQty', 1,
                    'cp',s.cp
                )) as salesItems
                FROM totalInventory s
                ),

                currency as (
                    SELECT
                    c.id as value,
                    x.name as title,
                    c.primary_currency
                    FROM
                    " . TBL_CURRENCY . " c
                    LEFT JOIN " . TBL_CURRENCY_CATEGORY . " x 
                    ON c.currency_cate_id=x.id AND x.status=" . STATUS . "
                    WHERE c.client_id='$client_id' AND c.location_id='$location_id' AND c.status=" . STATUS . "
                ),

                currencyArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'value', c.value,
                        'title', c.title,
                        'primary_currency', c.primary_currency
                    )) as currencies
                    FROM currency c),
    
                category as (
                    SELECT
                    id as value,
                    name as title
                    FROM
                    " . TBL_ITEM_CATEGORY . "
    
                    WHERE client_id='$client_id' AND status=" . STATUS . "
                ),
    
                categoryArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'value', c.value,
                        'title', c.title
                    )) as categories
                    FROM category c),

                    customer as (
                        SELECT
                        name as title,
                        id as value,
                        CASE WHEN cate='lba' THEN '' ELSE '- Vendor' END AS customerCategory
        
                        FROM " . TBL_CUSTOMERS . "
        
                        WHERE client_id='$client_id' AND status=" . STATUS ." AND name !='default vendor' $customerName
                        ORDER BY date, tm ASC
                        $limitCustomer
                        
                    ),

                    walkInCustomer as (
                        SELECT
                        name as title,
                        id as value
        
                        FROM " . TBL_CUSTOMERS . "
        
                        WHERE client_id='$client_id' AND status=" . STATUS ." AND name = 'walk-in customer'
                        LIMIT 1
                    ),
        
                    customerArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', CONCAT(c.title, ' ', c.customerCategory),
                            'value', c.value
                            
                        )) as customers
                        FROM customer c),

                    walkInCustomerArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', c.title,
                            'value', c.value
                        )) as walkInCustomer
                        FROM walkInCustomer c
                    ),

                    bankAccount as (
                        SELECT i.id as value, i.name as title, b.logo as icon FROM " . TBL_BANK_ACCOUNT . " i
                        LEFT JOIN " . TBL_APP_BANK . " b
                        ON b.id=i.bank_id AND b.status=" . STATUS . " AND b.status = " . STATUS . "
                        WHERE i.client_id = '$client_id' AND i.status = " . STATUS . "
                    ),

                    bankAccountArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', c.title,
                            'value', c.value,
                            'icon', c.icon
                        )) as bankAccounts
                        FROM bankAccount c)

            SELECT * FROM combinedArr, currencyArr, categoryArr, customerArr, bankAccountArr, walkInCustomerArr
            
        ";

        // echo $query;exit;

        $data = DB::select($query);

        foreach ($data as $row) {
            // $row->salesItems = $row->salesItems ? $row->salesItems : [];
            if (property_exists($row, 'salesItems')) {
                $row->salesItems = $row->salesItems ? json_decode(stripslashes($row->salesItems), true) : [];
            }
            if (property_exists($row, 'currencies')) {
                $row->currencies = $row->currencies ? json_decode(stripslashes($row->currencies), true) : [];
            }
            if (property_exists($row, 'categories')) {
                $row->categories = $row->categories ? json_decode(stripslashes($row->categories), true) : [];
            }
            if (property_exists($row, 'customers')) {
                $row->customers = $row->customers ? json_decode(stripslashes($row->customers), true) : [];
            }
            if (property_exists($row, 'bankAccounts')) {
                $row->bankAccounts = $row->bankAccounts ? json_decode(stripslashes($row->bankAccounts), true) : [];
            }

            if (property_exists($row, 'walkInCustomer')) {
                $row->walkInCustomer = $row->walkInCustomer ? json_decode(stripslashes($row->walkInCustomer), true) : [];
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function getCmpOptimisedSalesItems($request) {
        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        $store_id = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $search = !empty($request->text) ? " AND i.name LIKE '%$request->text%'" : "";
        $category = !empty($request->category) ? "AND i.item_category_id='$request->category'" : "";
        $customerName = !empty($request->customerName) ? "AND name LIKE '%$request->customerName%'" : "";
        $limit = "LIMIT $request->count, 50";
        $limitCustomer = "LIMIT $request->customerCount, 50";

        $query = "
            WITH products as (
                SELECT i.id, i.name, i.reorder_level, i.sales_type, p.m_metric, p.m_category_id, i.item_category_id, p.cal_qty, p.rate, c.name as category, i.income_account, i.vat_status, i.item_class_id
                FROM " . ONLINE_RETAILS .".items i

                LEFT JOIN " . ONLINE_RETAILS .".item_price p 
                ON p.item_id = i.id AND p.location_id = '$location_id' AND p.client_id = '$client_id' AND p.status = 0

                LEFT JOIN " . ONLINE_RETAILS .".measurement m
                ON m.m_category_id = p.m_category_id AND m.hierarchy = p.m_metric AND m.client_id = '$client_id' AND m.location_id = '$location_id'

                LEFT JOIN " . ONLINE_RETAILS .".item_category c
                ON c.id = i.item_category_id AND c.client_id = '$client_id' AND c.status = 0

                WHERE i.client_id = '$client_id' AND i.status = 0 AND i.disabled_status = 0 $search $category 
                GROUP BY 
                i.id, i.name
                ORDER BY 
                i.id
                $limit
            
            ),

            drInventory as (
                SELECT 
                    p.id, p.name, p.reorder_level, s.effect, p.m_category_id, p.item_category_id, p.cal_qty, p.category, p.sales_type, p.income_account, p.vat_status, p.item_class_id,
                    SUM(s.amount) as amount, 0 as discount, '' as description, 0 as up_sell,
                    COALESCE(ROUND(SUM(s.qty * i.cal_qty), 2),0) as received,
                    cmp.cp as cp,
                    (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', m.abr,
                            'value', round(m.hierarchy,0), 
                            'cal_qty', round(i.cal_qty,0), 
                            'price', round(i.rate,0)
                        ))
    
                        FROM " . TBL_ITEM_PRICE . " i
                        
                        LEFT JOIN " . TBL_MEASUREMENT . " m
                        ON m.m_category_id = p.m_category_id AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.status = 0 AND i.m_metric IS NOT NULL AND i.m_metric=m.hierarchy
    
                        WHERE i.item_id=p.id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.status = 0 AND p.id IS NOT NULL
                        GROUP BY i.item_id
    
                    ) AS measurements
                
                FROM products p
                
                INNER JOIN ".TBL_STOCK." s 
                    ON p.id = s.item_id AND s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND (s.effect = 'dr' OR s.effect IS NULL) AND s.store_id = '$store_id'

                LEFT JOIN ".TBL_ITEM_PRICE." i 
                    ON p.id = i.item_id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.m_metric=s.m_metric AND i.status = 0

                LEFT JOIN " . TBL_ITEM_PRICE . " cmp ON cmp.item_id=p.id AND cmp.client_id='$client_id' AND cmp.location_id='$location_id' AND cmp.status = 0 AND cmp.m_metric = 1
                GROUP by p.id
            ),

            crInventory as (
                SELECT 
                    p.*,
                    COALESCE(ROUND(SUM(s.qty * i.cal_qty), 0)) as dispensed,
                    SUM(s.amount) as crAmount
                    
                FROM products p
                
                INNER JOIN ".TBL_STOCK." s 
                    ON p.id = s.item_id AND s.client_id='$client_id' AND s.location_id='$location_id' AND s.control_id != 1 AND s.effect = 'cr'  AND s.store_id = '$store_id'

                LEFT JOIN ".TBL_ITEM_PRICE." i 
                    ON p.id = i.item_id AND i.client_id='$client_id' AND i.location_id='$location_id' AND i.m_metric=s.m_metric AND i.status = 0
                GROUP by p.id
            ),

            pmsm As (
                SELECT i.cal_qty as pQty, i.item_id, round(i.rate,2) as price, i.m_metric, m.abr as msm, m.name as measurement 
                FROM " . ONLINE_RETAILS .".item_price i
                LEFT JOIN " . ONLINE_RETAILS .".measurement m
                ON m.m_category_id = i.m_category_id AND m.hierarchy = i.m_metric AND m.client_id = '$client_id' AND m.location_id = '$location_id'
                WHERE i.primary_measurement = 1 AND i.client_id = '$client_id' AND i.location_id = '$location_id' AND i.status = 0
            ),

            totalInventory as (
                SELECT 
                
                dr.*,
                pr.*,
                ROUND(COALESCE(ROUND(dr.received,0),0) - COALESCE(ROUND(cr.dispensed,0),0) / COALESCE(ROUND(pr.pQty,0)),2) as qty

                FROM drInventory dr

                LEFT JOIN crInventory cr
                ON cr.id=dr.id

                LEFT JOIN pmsm pr
                ON pr.item_id=dr.id
            ),

            combinedArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'id', s.id,
                    'name', s.name, 
                    'cal_qty', s.pQty, 
                    'current_cal_qty', s.pQty,
                    'msm', s.msm,
                    'measurement', s.measurement,
                    'price', s.price,
                    'reorder_level', s.reorder_level,
                    'qty', s.qty,
                    'category', s.category,
                    'discount', s.discount,
                    'description', s.description,
                    'up_sell', s.up_sell,
                    'm_category_id', s.m_category_id,
                    'item_category_id', s.item_category_id,
                    'm_metric', s.m_metric,
                    'sales_type', s.sales_type,
                    'income_account', s.income_account,
                    'vat_status', s.vat_status,
                    'item_class_id', s.item_class_id,
                    'measurements', s.measurements,
                    'currentQty', 1,
                    'cp',s.cp
                )) as salesItems
                FROM totalInventory s
                ),

                currency as (
                    SELECT
                    c.id as value,
                    x.name as title,
                    c.primary_currency
                    FROM
                    " . TBL_CURRENCY . " c
                    LEFT JOIN " . TBL_CURRENCY_CATEGORY . " x 
                    ON c.currency_cate_id=x.id AND x.status=" . STATUS . "
                    WHERE c.client_id='$client_id' AND c.location_id='$location_id' AND c.status=" . STATUS . "
                ),

                currencyArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'value', c.value,
                        'title', c.title,
                        'primary_currency', c.primary_currency
                    )) as currencies
                    FROM currency c),
    
                category as (
                    SELECT
                    id as value,
                    name as title
                    FROM
                    " . TBL_ITEM_CATEGORY . "
    
                    WHERE client_id='$client_id' AND status=" . STATUS . "
                ),
    
                categoryArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'value', c.value,
                        'title', c.title
                    )) as categories
                    FROM category c),

                    customer as (
                        SELECT
                        name as title,
                        id as value,
                        CASE WHEN cate='lba' THEN '' ELSE '- Vendor' END AS customerCategory
        
                        FROM " . TBL_CUSTOMERS . "
        
                        WHERE client_id='$client_id' AND status=" . STATUS ." AND name !='default vendor' $customerName
                        ORDER BY date, tm ASC
                        $limitCustomer
                        
                    ),

                    walkInCustomer as (
                        SELECT
                        name as title,
                        id as value
        
                        FROM " . TBL_CUSTOMERS . "
        
                        WHERE client_id='$client_id' AND status=" . STATUS ." AND name = 'walk-in customer'
                        LIMIT 1
                    ),
        
                    customerArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', CONCAT(c.title, ' ', c.customerCategory),
                            'value', c.value
                            
                        )) as customers
                        FROM customer c),

                    walkInCustomerArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', c.title,
                            'value', c.value
                        )) as walkInCustomer
                        FROM walkInCustomer c
                    ),

                    bankAccount as (
                        SELECT i.id as value, i.name as title, b.logo as icon FROM " . TBL_BANK_ACCOUNT . " i
                        LEFT JOIN " . TBL_APP_BANK . " b
                        ON b.id=i.bank_id AND b.status=" . STATUS . " AND b.status = " . STATUS . "
                        WHERE i.client_id = '$client_id' AND i.status = " . STATUS . "
                    ),

                    bankAccountArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', c.title,
                            'value', c.value,
                            'icon', c.icon
                        )) as bankAccounts
                        FROM bankAccount c)

            SELECT * FROM combinedArr, currencyArr, categoryArr, customerArr, bankAccountArr, walkInCustomerArr
            
        ";

        // echo $query;exit;

        $data = DB::select($query);

        foreach ($data as $row) {
            // $row->salesItems = $row->salesItems ? $row->salesItems : [];
            if (property_exists($row, 'salesItems')) {
                $row->salesItems = $row->salesItems ? json_decode(stripslashes($row->salesItems), true) : [];
            }
            if (property_exists($row, 'currencies')) {
                $row->currencies = $row->currencies ? json_decode(stripslashes($row->currencies), true) : [];
            }
            if (property_exists($row, 'categories')) {
                $row->categories = $row->categories ? json_decode(stripslashes($row->categories), true) : [];
            }
            if (property_exists($row, 'customers')) {
                $row->customers = $row->customers ? json_decode(stripslashes($row->customers), true) : [];
            }
            if (property_exists($row, 'bankAccounts')) {
                $row->bankAccounts = $row->bankAccounts ? json_decode(stripslashes($row->bankAccounts), true) : [];
            }

            if (property_exists($row, 'walkInCustomer')) {
                $row->walkInCustomer = $row->walkInCustomer ? json_decode(stripslashes($row->walkInCustomer), true) : [];
            }
        }

        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    public function overAllSalesItems(Request $request) {
        $client_id = Auth::user()->client_id;
        $location_id = Auth::user()->location_id;
        
        $inventoryEvaluation = Location::where("client_id", $client_id)
        ->where("id", $location_id)
        ->value("inventory_evaluation");

        // weighted average cost price
        if($inventoryEvaluation==STATUS) {
            return $this->getOptimisedSalesItems($request);
        }

        // current market pricing
        if($inventoryEvaluation==ONE_STATUS) {
            return $this->getCmpOptimisedSalesItems($request);
        }

        // FIFO cost price
        if($inventoryEvaluation==TWO_STATUS) {
            // ...

        }
    }

    public function getSalesItems(Request $request) {

        $client_id = Auth::user()->client_id;
        $location_id = empty($request->location_id) ? Auth::user()->location_id : $request->location_id;
        $store_id = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
        $search = !empty($request->text) ? " AND i.name LIKE '%$request->text%'" : "";
        $category = !empty($request->category) ? "AND i.item_category_id='$request->category'" : "";
        $customerName = !empty($request->customerName) ? "AND name LIKE '%$request->customerName%'" : "";
        $limit = "LIMIT $request->count, 50";
        $limitCustomer = "LIMIT $request->customerCount, 50";

        $query = "
            WITH products as (
                SELECT i.id, i.name, i.reorder_level, i.sales_type, p.m_metric, p.m_category_id, i.item_category_id, p.cal_qty, p.rate, c.name as category, i.income_account, i.vat_status, i.item_class_id
                FROM " . ONLINE_RETAILS .".items i

                LEFT JOIN " . ONLINE_RETAILS .".item_price p 
                ON p.item_id = i.id AND p.location_id = '$location_id' AND p.client_id = '$client_id' AND p.status = 0

                LEFT JOIN " . ONLINE_RETAILS .".measurement m
                ON m.m_category_id = p.m_category_id AND m.hierarchy = p.m_metric AND m.client_id = '$client_id' AND m.location_id = '$location_id'

                LEFT JOIN " . ONLINE_RETAILS .".item_category c
                ON c.id = i.item_category_id AND c.client_id = '$client_id' AND c.status = 0

                WHERE i.client_id = '$client_id' AND i.status = 0 AND i.disabled_status = 0 $search $category ORDER BY i.name
              
            ),

            inventoryDetail As (
                SELECT p.id, s.item_id, p.name, p.reorder_level, s.effect, s.qty, p.m_category_id, p.item_category_id, p.cal_qty, p.rate, p.category, p.sales_type, p.income_account, p.vat_status, p.item_class_id, s.amount
                FROM products p

                LEFT JOIN  " . ONLINE_RETAILS .".stock s ON s.item_id = p.id AND s.m_metric = p.m_metric and s.location_id = '$location_id' AND s.client_id = '$client_id' AND s.store_id = '$store_id' AND s.control_id != 1
                
                WHERE s.item_id IS NOT NULL 
                order by name
            ),

            pmsm As (
                SELECT i.cal_qty, i.id, i.item_id, i.rate, i.m_metric, m.abr, m.name as measurement 
                FROM " . ONLINE_RETAILS .".item_price i
                LEFT JOIN " . ONLINE_RETAILS .".measurement m
                ON m.m_category_id = i.m_category_id AND m.hierarchy = i.m_metric AND m.client_id = '$client_id' AND m.location_id = '$location_id'
                WHERE i.primary_measurement = 1 AND i.client_id = '$client_id' AND i.location_id = '$location_id' AND i.status = 0
            ),

            newStockQty As (
                SELECT a.*,b.abr, b.measurement, b.cal_qty as pQty, b.rate as price, b.m_metric
                FROM inventoryDetail a
                LEFT JOIN pmsm b 
                ON b.item_id = a.item_id
            ),

            crStockQty  As (
                SELECT item_id, name, m_category_id,item_category_id, effect, price,
                SUM((cal_qty * qty)/pQty) as dispensed
               
                FROM newStockQty
                WHERE effect = 'cr'
                group by item_id
            ),

            drStockQty  As (
                SELECT id, item_id,qty, name, abr, cal_qty, pQty, measurement, m_category_id,item_category_id, effect, price, ROUND(reorder_level/pQty,2) as reorder_level, ROUND(SUM((cal_qty * qty)/pQty),0) as received, category, m_metric, sales_type, income_account, vat_status, item_class_id, amount, ROUND(SUM(amount) / SUM(qty * cal_qty), 0) as cp
               
                FROM newStockQty
                WHERE effect = 'dr'
                group by item_id
            ),

            inventoryBal As (
                SELECT dr.id, dr.name, round(dr.pQty, 0) as cal_qty, dr.abr as msm, dr.measurement, round(dr.price,0) as price, dr.reorder_level, round(dr.received - coalesce(cr.dispensed,0) ,2) as qty, dr.category, 0 as discount, '' as description, 0 as up_sell, dr.m_category_id, dr.item_category_id, dr.m_metric, dr.sales_type, dr.income_account, dr.vat_status, dr.item_class_id,
                (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'title', m.abr,
                        'value', round(m.hierarchy,0), 
                        'cal_qty', round(p.cal_qty,0), 
                        'price', round(p.rate,0)
                    ))

                    FROM " . ONLINE_RETAILS .".item_price p

                    LEFT JOIN " . ONLINE_RETAILS .".measurement m
                    ON p.m_category_id = m.m_category_id AND p.m_metric = m.hierarchy AND m.client_id='$client_id' AND m.location_id='$location_id'
                    
                    WHERE dr.id = p.item_id AND p.client_id='$client_id' AND p.location_id='$location_id' and p.status=0
                    GROUP BY p.item_id

                ) AS measurements,
                dr.cp
                

                FROM drStockQty dr
                LEFT JOIN crStockQty cr 
                
                ON  cr.item_id = dr.item_id
                $limit
            ),

            combined as (SELECT * FROM inventoryBal),
           
            combinedArr as (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'id', s.id,
                    'name', s.name, 
                    'cal_qty', s.cal_qty, 
                    'current_cal_qty', s.cal_qty,
                    'msm', s.msm,
                    'measurement', s.measurement,
                    'price', s.price,
                    'reorder_level', s.reorder_level,
                    'qty', s.qty,
                    'category', s.category,
                    'discount', s.discount,
                    'description', s.description,
                    'up_sell', s.up_sell,
                    'm_category_id', s.m_category_id,
                    'item_category_id', s.item_category_id,
                    'm_metric', s.m_metric,
                    'sales_type', s.sales_type,
                    'income_account', s.income_account,
                    'vat_status', s.vat_status,
                    'item_class_id', s.item_class_id,
                    'measurements', s.measurements,
                    'currentQty', 1,
                    'cp',cp
                )) as salesItems
                FROM combined s
                ),

                currency as (
                    SELECT
                    c.id as value,
                    x.name as title,
                    c.primary_currency
                    FROM
                    " . TBL_CURRENCY . " c
                    LEFT JOIN " . TBL_CURRENCY_CATEGORY . " x 
                    ON c.currency_cate_id=x.id AND x.status=" . STATUS . "
                    WHERE c.client_id='$client_id' AND c.location_id='$location_id' AND c.status=" . STATUS . "
                ),

                currencyArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'value', c.value,
                        'title', c.title,
                        'primary_currency', c.primary_currency
                    )) as currencies
                    FROM currency c),
    
                category as (
                    SELECT
                    id as value,
                    name as title
                    FROM
                    " . TBL_ITEM_CATEGORY . "
    
                    WHERE client_id='$client_id' AND status=" . STATUS . "
                ),
    
                categoryArr as (
                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                        'value', c.value,
                        'title', c.title
                    )) as categories
                    FROM category c),

                    customer as (
                        SELECT
                        name as title,
                        id as value,
                        CASE WHEN cate='lba' THEN '' ELSE '- Vendor' END AS customerCategory
        
                        FROM " . TBL_CUSTOMERS . "
        
                        WHERE client_id='$client_id' AND status=" . STATUS ." AND name !='default vendor' $customerName
                        ORDER BY date, tm ASC
                        $limitCustomer
                        
                    ),

                    walkInCustomer as (
                        SELECT
                        name as title,
                        id as value
        
                        FROM " . TBL_CUSTOMERS . "
        
                        WHERE client_id='$client_id' AND status=" . STATUS ." AND name = 'walk-in customer'
                        LIMIT 1
                    ),
        
                    customerArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', CONCAT(c.title, ' ', c.customerCategory),
                            'value', c.value
                            
                        )) as customers
                        FROM customer c),

                    walkInCustomerArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', c.title,
                            'value', c.value
                        )) as walkInCustomer
                        FROM walkInCustomer c
                    ),

                    bankAccount as (
                        SELECT i.id as value, i.name as title, b.logo as icon FROM " . TBL_BANK_ACCOUNT . " i
                        LEFT JOIN " . TBL_APP_BANK . " b
                        ON b.id=i.bank_id AND b.status=" . STATUS . " AND b.status = " . STATUS . "
                        WHERE i.client_id = '$client_id' AND i.status = " . STATUS . "
                    ),

                    bankAccountArr as (
                        SELECT JSON_ARRAYAGG(JSON_OBJECT(
                            'title', c.title,
                            'value', c.value,
                            'icon', c.icon
                        )) as bankAccounts
                        FROM bankAccount c)

            SELECT * FROM combinedArr, currencyArr, categoryArr, customerArr, bankAccountArr, walkInCustomerArr
            
        ";

        $data = DB::select($query);

        foreach ($data as $row) {
            // $row->salesItems = $row->salesItems ? $row->salesItems : [];
            if (property_exists($row, 'salesItems')) {
                $row->salesItems = $row->salesItems ? json_decode(stripslashes($row->salesItems), true) : [];
            }
            if (property_exists($row, 'currencies')) {
                $row->currencies = $row->currencies ? json_decode(stripslashes($row->currencies), true) : [];
            }
            if (property_exists($row, 'categories')) {
                $row->categories = $row->categories ? json_decode(stripslashes($row->categories), true) : [];
            }
            if (property_exists($row, 'customers')) {
                $row->customers = $row->customers ? json_decode(stripslashes($row->customers), true) : [];
            }
            if (property_exists($row, 'bankAccounts')) {
                $row->bankAccounts = $row->bankAccounts ? json_decode(stripslashes($row->bankAccounts), true) : [];
            }

            if (property_exists($row, 'walkInCustomer')) {
                $row->walkInCustomer = $row->walkInCustomer ? json_decode(stripslashes($row->walkInCustomer), true) : [];
            }
        }
        DB::disconnect();
        return response()->json(['results' => count($data), 'data' => $data, 'success' => true], 200);
    }

    private function validation($request) {
        $customer = $this->customerVendorController->customerName(Auth::user()->client_id, $request->customer);

        $validationRules = [
            'bank' => $request->pmethod === POS || $request->pmethod === BANK_TRANSFER ? 'required' : '',
            'total' => 'required',
            'customer' => 'required',
            'txn_date' => 'required',
            'currency' => 'required',
            'token' => 'required',
            'authoriser' => $request->pmethod === COMPLIMENTARY ? 'required' : '',
            'tender' => $request->pmethod === POS || $request->pmethod === CASH || $request->pmethod === BANK_TRANSFER ? 'required' : ''
        ];

        if($request->tender < $request->total && ($request->pmethod === POS || $request->pmethod === CASH || $request->pmethod === BANK_TRANSFER)) {
            return response()->json(['message' => 'Tender cannot be less than the sales value, use discount instead', 'error' => true], 400);
        }

        if($request->total - ($request->posAmount + $request->cashAmount + $request->bankAmount) > STATUS && $request->pmethod === PAYMENT_SPLIT && $customer === WALK_IN_CUSTOMER) {
            return response()->json(['message' => 'You have to receive full amount from a walk-in customer when using payment split', 'error' => true], 400);
        }

        if($request->pmethod === BALANCE && $customer === WALK_IN_CUSTOMER) {
            return response()->json(['message' => 'You cannot use balance for Walk-in customer', 'error' => true], 400);
        }

        $itemId = $this->validateQty($request);
        
        if($itemId !== false) {
            $itemName = $this->productController->getItemName($itemId);
            return response()->json(['message' => "Insufficient qty for $itemName, kindly return to sales point to get its current quantity.", 'error' => true], 400);
        }

        $item = $this->validateCostPrice($request);
        if($item !== false) {
            return response()->json(['message' => "The unit cost price of $item is higher than its selling price", 'error' => true], 400);
        };
        
        // Create a validator with the defined rules and the request data
        $validator = Validator::make($request->all(), $validationRules);
        // Check if validation fails and return an error response if it does
        
        if ($validator->fails()) {
            return response()->json(['message' => $validator->errors()->first(), 'error' => true], 400);
        }
    }

    public function validateQty($request){
        foreach($request->preview as $prev) {
            $qty = $this->inventoryController->getQtyBalance($prev['id']);
            $requestedQty = $prev['prevQty'] * $prev['cal_qty'];
           
            if($requestedQty > $qty) {
                return $prev['id'];
            }
        }
        
        return false;
    }

    public function validateCostPrice($request){
        foreach($request->preview as $prev) {
            $price = $this->priceController->itemPrice($prev['id'], $prev['m_metric'], $prev['m_category_id'], Auth::user()->location_id);
            $primaryStore = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
            $cp = $this->inventoryController->inventoryEvaluationPrice($prev['id'], $primaryStore);

            if($price < $cp) {
                $itemName = $this->productController->getItemName($prev['id']);
                return $itemName;
            }
        }

        return false;
    }

    public function saveStock($request) {
        foreach($request->preview as $prev) {
            $primaryStore = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
            $cp = $this->inventoryController->inventoryEvaluationPrice($prev['id'], $primaryStore);
            $total = $cp * $prev['prevQty'];
            
            $this->inventoryController->insertStock($request->token, $request->txn_date, $prev['id'], $request->customer, $prev['item_category_id'], $prev['prevQty'], $cp, $request->currency, $prev['m_metric'], $prev['m_category_id'], $primaryStore, EFFECT_CREDIT, null, $total);
        }
    }

    private function calcDiscount($discount, $amount, $absoluteDiscount) {
        if($discount > STATUS){
            if($absoluteDiscount === true) {
                return $discount;
            } else {
                return ($discount / 100) * $amount;
            }
        }

        return STATUS;
    }

    public function saveItemLog($request, $customer){
        
        $vat_value = 0;
        $queuer="";
        $generalDiscount = $request->generalDiscount ? $request->generalDiscount : STATUS;
        
        foreach($request->preview as $prev) {
            $primaryStore = $this->storeController->getPrimaryStore(Auth::user()->client_id, Auth::user()->location_id);
            $cp = $this->inventoryController->inventoryEvaluationPrice($prev['id'], $primaryStore);
            
            $calcDiscount = $generalDiscount > STATUS ? 
            $this->calcDiscount($generalDiscount, $prev['amount'], $request->absoluteDiscount) : 
            $this->calcDiscount($prev['discount'], $prev['amount'], $request->absoluteDiscount);
            $price = $prev['up_sell'] > STATUS ? $prev['up_sell'] : $prev['price'];

            $tender = $request->pmethod === PAYMENT_SPLIT ? 
            $request->posAmount + $request->cashAmount + $request->bankAmount : 
            $request->tender;

            $this->itemLogController->insertItemLog($request->customer, $customer, $prev['amount'], $cp, $price, $prev['prevQty'], $request->token, $prev['id'], $prev['item_category_id'], $primaryStore, $prev['cal_qty'], $prev['m_metric'], $prev['m_category_id'], $calcDiscount, $prev['sales_type'], $prev['description'], $prev['item_class_id'], $prev['vat_status'], $vat_value, $prev['income_account'], $queuer, $request->txn_date, $tender);
        }
    }

    private function cashTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue) {

        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_CASH, Auth::user()->client_id);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_DEBIT, $jvalue, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue, $request->pmethod);
    }

    private function bankPosTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue) {

        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_BANK, Auth::user()->client_id);
        $bank_id = $this->bankAccountController->getBankIdByBankAccountId($request->bank);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_DEBIT, $jvalue, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue, $request->pmethod, $request->bank, $bank_id);

        $cashBack = $request->tender - $jvalue;

        if($cashBack > STATUS) {
            $this->cashBackTxn($request, $cashBack, $customer, $currencyUnit);
        }
    }

    private function cashBackTxn($request, $jvalue, $customer, $currencyUnit){
        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_CASH, Auth::user()->client_id);
        $currencyValue = $jvalue * $currencyUnit;

        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_CREDIT, $jvalue, $request->token, $currencyUnit, $request->txn_date, $request->customer, "being balance returned to customer", $customer, "s", $request->currency, $currencyValue);
    }

    private function salesTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue) {

        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_SALES, Auth::user()->client_id);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_CREDIT, $jvalue, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue);
    }

    private function complimentaryTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue) {

        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_COMPLIMENTARY, Auth::user()->client_id);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_DEBIT, $jvalue, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue, $request->pmethod, "", "", $request->authoriser);
    }

    private function customerBalanceTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue) {

        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_RECEIVABLE, Auth::user()->client_id);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_DEBIT, $jvalue, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue, "charged to customer account");
    }

    private function saveCOGTxn($request, $currencyUnit, $being, $customer) {
        $acctLegs = $this->chartOfAccountController->getAccountLegs(CHART_COST_OF_GOODS, CHART_STOCK, Auth::user()->client_id);
        $cog = $this->inventoryController->getCOGByToken($request->token);
  
        $this->txnLogController->createTxnLog($acctLegs, $cog, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency);
    }

    private function savePaymentSplitPosTxn($request, $currencyUnit, $being, $customer, $currencyValue){
        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_BANK, Auth::user()->client_id);
        $bank_id = $this->bankAccountController->getBankIdByBankAccountId($request->pos);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_DEBIT, $request->posAmount, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue, POS, $request->bank, $bank_id, "", "", ONE_STATUS);
    }

    private function savePaymentSplitBankTxn($request, $currencyUnit, $being, $customer, $currencyValue){
        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_BANK, Auth::user()->client_id);
        $bank_id = $this->bankAccountController->getBankIdByBankAccountId($request->bank);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_DEBIT, $request->bankAmount, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue, BANK_TRANSFER, $request->bank, $bank_id, "", "", ONE_STATUS);
    }

    private function savePaymentSplitCashTxn($request, $currencyUnit, $being, $customer, $currencyValue) {

        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_CASH, Auth::user()->client_id);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_DEBIT, $request->cashAmount, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue, CASH, "", "", "", "", ONE_STATUS);
    }

    private function savePaymentSplitChargedToCustomerAccountTxn($request, $currencyUnit, $being, $customer, $currencyValue, $amount) {

        $acctLeg = $this->chartOfAccountController->getSingleAccountLeg(CHART_RECEIVABLE, Auth::user()->client_id);
        
        $this->txnLogController->createSingleTxnLog($acctLeg, EFFECT_DEBIT, $amount, $request->token, $currencyUnit, $request->txn_date, $request->customer, $being, $customer, "s", $request->currency, $currencyValue, "charged to customer account", "", "", "", "", ONE_STATUS);
    }

    private function paymentSplitTxn($request, $currencyUnit, $being, $customer, $currencyValue){
        if($request->posAmount > STATUS){
            $this->savePaymentSplitPosTxn($request, $currencyUnit, $being, $customer, $currencyValue);
        }
        if($request->bankAmount > STATUS){
            $this->savePaymentSplitBankTxn($request, $currencyUnit, $being, $customer, $currencyValue);
        }
        if($request->cashAmount > STATUS){
            $this->savePaymentSplitCashTxn($request, $currencyUnit, $being, $customer, $currencyValue);
        }

        $creditAmount = $request->total - ($request->posAmount + $request->cashAmount + $request->bankAmount);
        if($creditAmount > STATUS){
            $this->savePaymentSplitChargedToCustomerAccountTxn($request, $currencyUnit, "being amount charged to customer account", $customer, $currencyValue, $creditAmount);
        }
        
    }

    public function saveTxnLog($request, $customer) {
        $client_id=Auth::user()->client_id;
        $location_id=Auth::user()->location_id;

        $currencyUnit = $this->currencyController->getCurrencyUnit($client_id, $location_id, $request->currency);
        $jvalue = $this->itemLogController->getDiscountedAmount($request->token);
        $being = "being sales made";
        $currencyValue = $currencyUnit * $jvalue;
        
        if($request->pmethod === CASH){
            $this->cashTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue);
        } else if($request->pmethod === POS || $request->pmethod === BANK_TRANSFER) {
            $this->bankPosTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue);
        } else if($request->pmethod === COMPLIMENTARY){
            $this->complimentaryTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue);
        } else if($request->pmethod === BALANCE){
            $this->customerBalanceTxn($request, $jvalue, $currencyUnit, "being amount charged to customer account", $customer, $currencyValue);
        } else if($request->pmethod === PAYMENT_SPLIT){
            $this->paymentSplitTxn($request, $currencyUnit, $being, $customer, $currencyValue);
        }
        
        $this->salesTxn($request, $jvalue, $currencyUnit, $being, $customer, $currencyValue);
        $this->saveCOGTxn($request, $currencyUnit, "being cost of goods sold", $customer);
    }
 
    public function sales(Request $request) {

        $validation = $this->validation($request);

        if($validation) {
            return $validation;
        }

        $client_id=Auth::user()->client_id;
        $location_id=Auth::user()->location_id;

        $customer = $this->customerVendorController->customerName($client_id, $request->customer);

        $this->saveStock($request);
        $this->saveItemLog($request, $customer);
        $this->saveTxnLog($request, $customer);

        $receipt = "
            SELECT
            l.token,
            ROUND(SUM(l.amount), 0) as amount,
            ROUND(SUM(l.discount), 0) as discount,
            ROUND(SUM(l.vat_value), 0) as vat,
            txn.comp as authoriser,
            l.customer,
            l.txn_date,
            l.tm,
            u.name,
            l.tender,
            (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'item', i.name,
                    'amount', round(l.amount, 2),
                    'qty', l.qty,
                    'abr', m.abr,
                    'price', round(l.price, 2),
                    'discount', round(l.discount, 2),
                    'description', l.description,
                    'vat_value', round(l.vat_value, 2),
                    'vat_status', round(l.vat_status, 2)
                ))
            ) as receipt,

            (
                SELECT JSON_ARRAYAGG(JSON_OBJECT(
                    'pmethod', p.pmethod,
                    'amount', p.jvalue
                ))
            
                FROM " . TBL_LOG . " p
                
                WHERE p.token='$request->token' AND p.pmethod != '' AND p.control_id != " . ONE_STATUS . " AND p.client_id='$client_id' AND p.location_id='$location_id'
            ) AS pmethod

            FROM " . TBL_PRODUCT_LOG . " l

            INNER JOIN " . TBL_ITEMS . " i
            ON i.id=l.item_id AND i.status = " . STATUS . " AND i.client_id='$client_id'

            LEFT JOIN " . TBL_USER . " u
            ON u.id=l.user_id AND u.client_id='$client_id' AND u.location_id='$location_id'

            LEFT JOIN " .TBL_MEASUREMENT . " m
            ON m.m_category_id=l.m_category_id AND m.status = " . STATUS . " AND m.client_id='$client_id' AND m.location_id='$location_id' AND m.hierarchy=l.m_metric

            LEFT JOIN " . TBL_LOG . " txn
            ON txn.token='$request->token' AND txn.control_id != " . ONE_STATUS . " AND txn.client_id='$client_id' AND txn.location_id='$location_id' AND txn.comp != ''

            WHERE l.client_id='$client_id' AND l.location_id='$location_id' AND l.control_id != " . ONE_STATUS . " AND l.token = '$request->token'
            
        ";

        $data = DB::select($receipt);
        DB::disconnect();

        return response()->json(['message' => 'Sales saved successfully', 'data' => $data, 'success' => true], 201);
    }

}