<?php

namespace App\Http\Middleware;

use Closure;
use JWTAuth;
use Exception;
use Tymon\JWTAuth\Http\Middleware\BaseMiddleware;
use Illuminate\Support\Facades\Response;

class JwtMiddleware extends BaseMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                throw new Exception('User Not Found');
            }
        } catch (Exception $e) {
            if ($e instanceof \Tymon\JWTAuth\Exceptions\TokenInvalidException) {
                return response()->json(['message' => 'Token Invalid', 'data' => '', 'error' => true], 400);
            } else if ($e instanceof \Tymon\JWTAuth\Exceptions\TokenExpiredException) {
                return response()->json(['message' => 'Token Expired', 'data' => '', 'error' => true], 400);
            } else {
                if ($e->getMessage() === 'User Not Found') {
                    return response()->json(['message' => 'User Not Found', 'data' => '', 'error' => true], 400);
                } else {
                    return response()->json(['message' => 'Authorization Token not found', 'data' => '', 'error' => true], 400);
                }
            }
        }
        return $next($request);
    }
}
