<?php

namespace App\Http\Middleware;

use Closure;
use Tymon\JWTAuth\Facades\JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Exceptions\TokenExpiredException;
use Symfony\Component\HttpFoundation\Response;
use Tymon\JWTAuth\Exceptions\TokenInvalidException;
use App\Traits\ApiResponse;

class JwtOrCookieAuth
{
    use ApiResponse; // Use the ApiResponse trait

    public function handle(Request $request, Closure $next)
    {
        try {
            // Retrieve the token from cookies
            $token = $request->cookie('accessToken');


            // if (!$token) {
            //     return $this->error('Token is missing or expired.', 401, ['action'=> 'refresh']);
            // }

            // Validate the token and authenticate the user
            $user = JWTAuth::setToken($token)->authenticate();

            // Attach user object to the request for further processing
            $request->merge(['user' => $user]);

        } catch (TokenExpiredException $e) {
            return $this->error(['Token has expired'], 401, ['action'=> 'refresh']);
        } catch (TokenInvalidException $e) {
            return $this->error('Token is invalid', 401);
        } catch (JWTException $e) {
            return $this->error('Token error', 400);
        }

        return $next($request);
    }
}
