<?php

namespace App\Http\Requests;

use App\Rules\UserExists;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Contracts\Validation\Validator;
use Illuminate\Http\Exceptions\HttpResponseException;
use Illuminate\Validation\ValidationException;


class RegistrationRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
        return [
            'name' => 'required|string|max:255',
            'username' => [
                'required',
                'string',
                new UserExists(), // Check user existence based on username/email
            ],
            'email' => [
                'required',
                'string',
                new UserExists(), // Check user existence based on username/email
            ],
            'phone' => 'required|string|max:15',
            'password' => 'required|string|min:8',
            'referral_code' => 'nullable|string|max:50',
            'acceptTerms' => 'nullable|boolean',
        ];
    }

    /**
     * Get custom messages for validator errors.
     *
     * @return array
     */
    public function messages()
    {
        return [
            'name.required' => 'The name field is required.',
            'username.required' => 'The username field is required.',
            'email.required' => 'The email field is required.',
            'email.email' => 'The email must be a valid email address.',
            'phone.required' => 'The phone number is required.',
            'password.required' => 'The password field is required.',
            'password.min' => 'The password must be at least 8 characters.',
        ];
    }

    /**
     * Handle a failed validation attempt.
     *
     * @param  \Illuminate\Contracts\Validation\Validator  $validator
     * @return void
     *
     * @throws \Illuminate\Validation\ValidationException
     */
    protected function failedValidation(Validator $validator)
    {
         // Extract the first error message from the validation errors
         $message = $validator->errors()->first();

         $response = [
            'success' => false,
            'message' => $message, // Extract the message or fallback to default
        ];

        // Throw the response as an exception with a 422 status code
        throw new HttpResponseException(
            response()->json($response) // Use provided code or default to 422
        );
        // throw new ValidationException($validator, $response);
    }
}
