<?php

namespace App\Logging;
use Monolog\Logger;

use Monolog\Handler\AbstractProcessingHandler;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Log;

class TeamsWebhookHandler2 extends AbstractProcessingHandler
{
    protected $webhookUrl;
    protected $client;

    public function __construct($level = Logger::DEBUG, bool $bubble = true)
    {
        $this->webhookUrl = env('TEAMS_WEBHOOK_URL');;
        $this->client = new Client();  // Guzzle HTTP client
    }

    protected function write(array $record): void
    {
        $message = $record['message'];
        $level = strtoupper($record['level_name']);
        $context = json_encode($record['context'], JSON_PRETTY_PRINT);

        $payload = [
            'text' => "*$level* - $message\nContext: $context"
        ];

        try {
            $this->client->post($this->webhookUrl, [
                'json' => $payload,
            ]);
        } catch (\Exception $e) {
            // Handle the case if the request fails
            Log::error('Failed to send log to Teams: ' . $e->getMessage());
        }
    }
}
