<?php

namespace App\Models\Account;
use Eloquent as Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Tymon\JWTAuth\Contracts\JWTSubject;
use Illuminate\Auth\Notifications\VerifyEmail as VerifyEmailNotification;
use Illuminate\Notifications\Messages\MailMessage;


/**
 * Summary of User
 */
class User extends Authenticatable implements JWTSubject, MustVerifyEmail
{
    use SoftDeletes;
    use Notifiable;


 
    public $table = 'user';
 
    protected $connection = 'mysql';
    public $timestamps = false;

    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';
  
    protected $dates = ['deleted_at'];

    protected $fillable = [
        'name',
        'location_id',
        'salt',
        'password',
        'username',
        'username2',
        'over_view',
        'deletion',
        'status',
        'discount',
        'up_sell',
        'back_date',
        'verification_status',
        'email_verification',
        'sms_verification',
        'master_status',
        'edit',
        'sales_return',
        'app',
        'allowed_on_desktop',
        'dashboard',
        'stock_cp',
        'allowed_on_web',
        'allowed_on_mobile',
        'position',
        'office',
        'location_overview',
        'store_master',
        'store_id',
        'date',
        'tm',
        'client_id',
        'update_date',
        'phone',
        'sales_priviledge',
        
        'confirm_bank_pos',
        'bank_priviledge'
    ];

    public function sendEmailVerificationNotification()
    {
        $this->notify(new \App\Notifications\EmailVerificationNotification);
    }
    

    /**
     * Get the notification's delivery channels.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function preferredNotificationChannel()
    {
        // You can customize the notification channel here if needed
        // For example, return ['mail'] to only use email notifications
        return config('app.notification_channel', ['mail']);
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {
        $verificationUrl = $this->verificationUrl($notifiable);

        return (new MailMessage)
            ->subject('Verify your email address')
            ->line('Click the button below to verify your email address.')
            ->action('Verify Email Address', $verificationUrl)
            ->line('If you did not create an account, no further action is required.');
    }



    

    /**
     * Summary of getJWTIdentifier
     * @return mixed
     */
    public function getJWTIdentifier()
    {
        return $this->getKey();
    }
    /**
     * Summary of getJWTCustomClaims
     * @return array
     */
    public function getJWTCustomClaims()
    {
        return [];
    }
}
