<?php

namespace App\Models\Retails;

use App\Models\BaseModel;
use Eloquent as Model;
use Illuminate\Database\Eloquent\SoftDeletes;


/**
 * @SWG\Definition(
 *      definition="Countries",
 *      required={"name", "updated_at", "flag"},
 *      @SWG\Property(
 *          property="id",
 *          description="id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="name",
 *          description="name",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="iso3",
 *          description="iso3",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="iso2",
 *          description="iso2",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="phonecode",
 *          description="phonecode",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="capital",
 *          description="capital",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="currency",
 *          description="currency",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="currency_symbol",
 *          description="currency_symbol",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="tld",
 *          description="tld",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="native",
 *          description="native",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="region",
 *          description="region",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="subregion",
 *          description="subregion",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="timezones",
 *          description="timezones",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="translations",
 *          description="translations",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="latitude",
 *          description="latitude",
 *          type="number",
 *          format="number"
 *      ),
 *      @SWG\Property(
 *          property="longitude",
 *          description="longitude",
 *          type="number",
 *          format="number"
 *      ),
 *      @SWG\Property(
 *          property="emoji",
 *          description="emoji",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="emojiU",
 *          description="emojiU",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="created_at",
 *          description="created_at",
 *          type="string",
 *          format="date-time"
 *      ),
 *      @SWG\Property(
 *          property="updated_at",
 *          description="updated_at",
 *          type="string",
 *          format="date-time"
 *      ),
 *      @SWG\Property(
 *          property="flag",
 *          description="flag",
 *          type="boolean"
 *      ),
 *      @SWG\Property(
 *          property="wikiDataId",
 *          description="Rapid API GeoDB Cities",
 *          type="string"
 *      )
 * )
 */
class Countries extends BaseModel
{
    //use SoftDeletes;
    protected $connection = 'mysql_retail';

    public $table = 'countries';

    public $timestamps = false;
    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';


    // protected $dates = ['deleted_at'];



    public $fillable = [
        'name',
        'iso3',
        'iso2',
        'phonecode',
        'capital',
        'currency',
        'currency_symbol',
        'tld',
        'native',
        'region',
        'subregion',
        'timezones',
        'translations',
        'latitude',
        'longitude',
        'emoji',
        'emojiU',
        'flag',
        'wikiDataId',
        'code',
        'currency_code'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'name' => 'string',
        'iso3' => 'string',
        'iso2' => 'string',
        'phonecode' => 'string',
        'capital' => 'string',
        'currency' => 'string',
        'currency_symbol' => 'string',
        'tld' => 'string',
        'native' => 'string',
        'region' => 'string',
        'subregion' => 'string',
        'timezones' => 'string',
        'translations' => 'string',
        'latitude' => 'decimal:8',
        'longitude' => 'decimal:8',
        'emoji' => 'string',
        'emojiU' => 'string',
        'flag' => 'boolean',
        'wikiDataId' => 'string'
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'name' => 'required|string|max:100',
        'iso3' => 'nullable|string|max:3',
        'iso2' => 'nullable|string|max:2',
        'phonecode' => 'nullable|string|max:255',
        'capital' => 'nullable|string|max:255',
        'currency' => 'nullable|string|max:255',
        'currency_symbol' => 'nullable|string|max:255',
        'tld' => 'nullable|string|max:255',
        'native' => 'nullable|string|max:255',
        'region' => 'nullable|string|max:255',
        'subregion' => 'nullable|string|max:255',
        'timezones' => 'nullable|string',
        'translations' => 'nullable|string',
        'latitude' => 'nullable|numeric',
        'longitude' => 'nullable|numeric',
        'emoji' => 'nullable|string|max:191',
        'emojiU' => 'nullable|string|max:191',
        'created_at' => 'nullable',
        'updated_at' => 'required',
        'flag' => 'required|boolean',
        'wikiDataId' => 'nullable|string|max:255'
    ];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function regions()
    {
        return $this->hasMany(\App\Models\Region::class, 'country');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function settings()
    {
        return $this->hasMany(\App\Models\Setting::class, 'country');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function users()
    {
        return $this->hasMany(\App\Models\User::class, 'country');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function wastifyStates()
    {
        return $this->hasMany(\App\Models\WastifyState::class, 'country');
    }
}
