<?php

namespace App\Models\Retails;


use App\Models\BaseModel;
use Eloquent as Model;
use Illuminate\Database\Eloquent\SoftDeletes;


/**
 * @SWG\Definition(
 *      definition="States",
 *      required={"name", "country_id", "country_code", "updated_at", "flag"},
 *      @SWG\Property(
 *          property="id",
 *          description="id",
 *          type="string",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="name",
 *          description="name",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="country_id",
 *          description="country_id",
 *          type="string",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="country_code",
 *          description="country_code",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="fips_code",
 *          description="fips_code",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="iso2",
 *          description="iso2",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="latitude",
 *          description="latitude",
 *          type="number",
 *          format="number"
 *      ),
 *      @SWG\Property(
 *          property="longitude",
 *          description="longitude",
 *          type="number",
 *          format="number"
 *      ),
 *      @SWG\Property(
 *          property="created_at",
 *          description="created_at",
 *          type="string",
 *          format="date-time"
 *      ),
 *      @SWG\Property(
 *          property="updated_at",
 *          description="updated_at",
 *          type="string",
 *          format="date-time"
 *      ),
 *      @SWG\Property(
 *          property="flag",
 *          description="flag",
 *          type="boolean"
 *      ),
 *      @SWG\Property(
 *          property="wikiDataId",
 *          description="Rapid API GeoDB Cities",
 *          type="string"
 *      )
 * )
 */
class States extends BaseModel
{
   // use SoftDeletes;

   protected $connection = 'mysql_retail';
    public $table = 'states';
    
    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';
    public $timestamps = false;


  // protected $dates = ['deleted_at'];



    public $fillable = [
        'name',
        'country_id',
        'country_code',
        'fips_code',
        'iso2',
        'latitude',
        'longitude',
        'flag',
        'wikiDataId'
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'string',
        'name' => 'string',
        'country_id' => 'string',
        'country_code' => 'string',
        'fips_code' => 'string',
        'iso2' => 'string',
        'latitude' => 'string',
        'longitude' => 'string',
        'flag' => 'string',
        'wikiDataId' => 'string'
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'name' => 'required|string|max:255',
        'country_id' => 'required|string',
        'country_code' => 'required|string|max:2',
        'fips_code' => 'nullable|string|max:255',
        'iso2' => 'nullable|string|max:255',
        'latitude' => 'nullable|numeric',
        'longitude' => 'nullable|numeric',
        'created_at' => 'nullable',
        'updated_at' => 'required',
        'flag' => 'required|boolean',
        'wikiDataId' => 'nullable|string|max:255'
    ];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function regions()
    {
        return $this->hasMany(\App\Models\Region::class, 'region_state');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function users()
    {
        return $this->hasMany(\App\Models\User::class, 'state');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function wastifyStates()
    {
        return $this->hasMany(\App\Models\WastifyState::class, 'region_state');
    }
}
