<?php

namespace App\QueryBuilders;

use App\QueryBuilders\Contracts\QueryBuilderInterface;
use InvalidArgumentException;

class QueryBuilderResolver
{
    /**
     * Mapping of query builder keys to their class names.
     *
     * @var array
     */
    protected array $builders = [
        'user-rights' => UserRightsBuilder::class,
        // Add other builders here...
    ];

    /**
     * Resolve a query builder by its key.
     *
     * @param  string $builderKey
     * @return QueryBuilderInterface
     * @throws InvalidArgumentException
     */
    public function resolve(string $builderKey): QueryBuilderInterface
    {
        if (!isset($this->builders[$builderKey])) {
            throw new InvalidArgumentException("Query builder [$builderKey] not found.");
        }

        // Use Laravel's service container to resolve the builder
        $builder = app($this->builders[$builderKey]);

        // Ensure the resolved class implements the interface
        if (!$builder instanceof QueryBuilderInterface) {
            throw new InvalidArgumentException("Query builder [$builderKey] must implement QueryBuilderInterface.");
        }

        return $builder;
    }
}
