<?php

namespace App\QueryBuilders;

use App\QueryBuilders\Contracts\QueryBuilderInterface;
use App\Enums\OriginEnum;

class UserRightsBuilder implements QueryBuilderInterface
{
    public function build(array $params): string
    {
        // Destructure the array into individual variables
        $clientId = $params['client_id'] ?? null;
        $locationId = $params['location_id'] ?? null;
        $userId = $params['user_id'] ?? null;
        $planId = $params['plan_id'] ?? null;
        $origin = $params['origin'] ?? null;

        if (!$clientId || !$locationId || !$userId || !$planId || !$origin) {
            throw new \InvalidArgumentException("Missing required parameters for building the query.");
        }

        $platform1 = $origin === OriginEnum::DESKTOP->value ? "AND x.desktop=1" : "AND x.mobile=1";
        $platform2 = $origin === OriginEnum::DESKTOP->value ? "AND sub.desktop=1" : "AND sub.mobile=1";
        return "
            WITH rights AS (
                SELECT
                    x.menu_id AS subMenuId,
                    m.module_title,
                    m.module_icon,
                    r.plan_id,
                    m.id,
                    r.module_id,
                    r.menu_id,
                    r.status,
                    m.module_name,
                    x.menu_name,
                    x.path,
                    x.screen,
                    x.menu_title,
                    r.user,
                    x.sub_module,
                    x.sub_menu
                FROM ".TBL_RIGHTS." r
                LEFT JOIN ".TBL_MODULE." m ON m.module_icon != '' AND m.status=0
                LEFT JOIN  ".TBL_SUB_MENU." x ON x.id = r.menu_id AND x.module_id = r.module_id AND x.is_menu = 1 $platform1 AND x.status=0
                WHERE r.module_id=m.id
                  AND r.client_id = '$clientId'
                  AND r.location_id = '$locationId'
                  AND r.module_id IS NOT NULL
                  AND r.plan_id = '$planId'
                  AND r.status = 0
                  AND x.menu_name IS NOT NULL
                ORDER BY m.id ASC
            ),
            rightMenu AS (
                SELECT
                    JSON_OBJECT(
                        'module_id', r.module_id,
                        'menu_title', r.module_title,
                        'path', 'javascript:void(0)',
                        'icon', r.module_icon,
                        'category', 'general',
                        'isMenuOpen', false,
                        'menu', r.module_name,
                        'fullPageMenu', false,
                        'desc', true,
                        'content', '',
                        'status', 0,
                        'child_routes', JSON_ARRAYAGG(
                            JSON_OBJECT(
                                'sub_module', r.sub_module,
                                'module_id', r.module_id,
                                'menu_id', r.menu_id,
                                'id', r.subMenuId,
                                'sub_menu', r.sub_menu,
                                'menu_title', r.menu_title,
                                'menu_name', r.menu_name,
                                'path', r.path,
                                'child_routes', null,
                                'icon', 'policy',
                                'fullPageMenu', false,
                                'isMenuOpen', false,
                                'exact', true,
                                'third_child_routes', (
                                    SELECT JSON_ARRAYAGG(JSON_OBJECT(
                                        'sub_module', sub.sub_module,
                                        'module_id', sub.module_id,
                                        'id', sub.id,
                                        'menu_id', sub.id,
                                        'sub_menu', sub.sub_menu,
                                        'menu_title', sub.menu_title,
                                        'path', sub.path,
                                        'menu_name', sub.menu_name,
                                        'child_routes', null,
                                        'icon', 'policy',
                                        'fullPageMenu', false,
                                        'isMenuOpen', false,
                                        'third_child_routes', null,
                                        'exact', true
                                    ))
                                    FROM  " . TBL_SUB_MENU . "  sub
                                    WHERE sub.menu_id=r.id AND sub.is_menu=1 $platform2 AND sub.status=0
                                ),
                                'status', r.status
                            )
                        )
                    ) AS menu,
                    r.user
                FROM rights r
                GROUP BY r.module_id, r.user
            ),
            users AS (
                SELECT
                    (SELECT JSON_ARRAYAGG(m.menu) FROM rightMenu m WHERE m.user=u.id) AS defaultRights
                FROM " . TBL_USER . " u
                WHERE
                    u.client_id='$clientId'
                    AND u.location_id='$locationId'
                    AND u.id='$userId'
            )
            SELECT * FROM users;
        ";
    }
}
