<?php

namespace App\Repositories;

use App\Models\Retails\Location;
use App\Repositories\Contracts\LocationRepositoryInterface;
use Carbon\Carbon;

class LocationRepository implements LocationRepositoryInterface
{
    /**
     * Get the plan ID for a specific client and location.
     *
     * @param int $clientId
     * @param int $locationId
     * @return int|null
     */
    public function getPlanId(int $clientId, int $locationId): ?int
    {
        // Use the Location model directly instead of chaining `table`.
        $result = Location::where('client_id', $clientId)
            ->where('id', $locationId)
            ->value('app_plan_id');

        return $result;
    }

    public function getDefaultLocations(int $clientId, ?int $locationId, bool $locationOverview): array
    {
        $query = Location::where('client_id', $clientId)->where('status', 0);

        if (!$locationOverview) {
            $query->where('id', $locationId);
        }

        return $query->orderBy('id', 'ASC')->get()->toArray();
    }


    public function isExpired(int $clientId, int $locationId): bool
    {
        $location = Location::select('end_date')
            ->where('status', STATUS)
            ->where('client_id', $clientId)
            ->where('id', $locationId)
            ->first();

        if (!$location || !$location->end_date) {
            return 'inactive'; // Return 'inactive' if location or end_date doesn't exist
        }

        return Carbon::parse($location->end_date)->isPast();
    }


    public function getSubscriptionStatus(int $clientId, int $locationId): string
    {
        return $this->isExpired($clientId, $locationId) ? 'expired' : 'active';
    }
}
