<?php

namespace App\Repositories;

use App\Models\Account\User;
use App\Repositories\Contracts\UserRepositoryInterface;

class UserRepository implements UserRepositoryInterface
{
    public function findByEmail(string $email)
    {
        return User::where('email', $email)->first();
    }


     /**
     * Find a user by email or username.
     *
     * @param string $input
     * @return User|null
     */
    public function findByEmailOrUsername(string $input): ?User
    {
        // location_overview: res.data.data.location_overview,
        // edit: res.data.data.edit,
        // deletion: res.data.data.deletion,
        // back_date: res.data.data.back_date,
        // discount: res.data.data.discount,
        // master_status: res.data.data.master_status,
        // up_sell: res.data.data.up_sell,
        // store_master: res.data.data.store_master,
        // sales_priviledge: res.data.data.sales_priviledge,
        // bank_priviledge: res.data.data.bank_priviledge,
        // stock_cp: res.data.data.stock_cp,
        // sales_return: res.data.data.sales_return,
        // confirm_bank_pos: res.data.data.confirm_bank_pos,
        // dashboard: res.data.data.dashboard,
        // rights: res.data.defaultRights,

        $columns = [
            'id',
            'status',
            'username',
            'username2',
            'password',
            'salt',
            'allowed_on_web',
            'allowed_on_desktop',
            'allowed_on_mobile',
            'client_id',
            'location_id',
            'name',
            'password2',
            'salt2',
            'verification_status',
            'location_overview',
            'edit',
            'deletion',
            'back_date',
            'discount',
            'master_status',
            'up_sell',
            'store_master',
            'sales_priviledge',
            'bank_priviledge',
            'stock_cp',
            'sales_return',
            'confirm_bank_pos',
            'dashboard',
        ];

        // Case-insensitive check for email or username
        return User::select($columns)
        ->whereRaw('LOWER(username) = ?', [strtolower($input)])
        ->orWhereRaw('LOWER(username2) = ?', [strtolower($input)])
        ->first();
    }

    public function getDefaultUsers(int $clientId, bool $overview, int $userId, int $locationId): array
    {
        $query = User::where('client_id', $clientId)
            ->where('location_id', $locationId);

        if (!$overview) {
            $query->where('id', $userId);
        }

        return $query->get()->toArray();
    }

    public function getDefaultUsersPinCode(int $clientId, int $locationId): array
    {
        return User::where('client_id', $clientId)
            ->where('location_id', $locationId)
            ->get()
            ->toArray();
    }
}
