<?php

namespace App\Services\Auth;

use App\Repositories\Contracts\UserRepositoryInterface;
use App\Services\Contracts\TokenServiceInterface;
use App\Services\Contracts\PasswordServiceInterface;
use App\Services\UserCacheService;

class AuthService
{
    protected $userRepository;
    protected $tokenService;
    protected $passwordService;

    public function __construct(UserRepositoryInterface $userRepository, TokenServiceInterface $tokenService, PasswordServiceInterface $passwordService)
    {
        $this->userRepository = $userRepository;
        $this->tokenService = $tokenService;
        $this->passwordService = $passwordService;
    }

    public function authenticate(array $credentials)
    {
        // Find user by email or username
        // $user = $this->userRepository->findByEmailOrUsername($credentials['username']);

        // Get the cached user from the singleton
        $user = UserCacheService::getInstance()->getUser();


        // \Log::info('Request data:', ['user' => $user]);


        if (!$user || !$this->passwordService->verifyPassword(
            $credentials['password'],
            $user -> password,
            $user -> salt
        )) {
            // Return consistent error response
            return [
                'success' => false,
                'message' => 'Incorrect email/username or password.',
            ];
        }

        $tokens = $this->tokenService->generateTokens($user);
        // Return tokens and user data
        return [
            'success' => true,
            'message' => 'Login successful.',
            'accessToken' => $tokens['accessToken'],
            'refreshToken' => $tokens['refreshToken'],
        ];
    }

    public function logout(string $token)
    {
        $this->tokenService->invalidateToken($token);
    }

    public function refreshToken(string $refreshToken)
    {
        $newAccessToken = $this->tokenService->refreshToken($refreshToken);

        return [
            'success' => true,
            'accessToken' => $newAccessToken,
        ];
    }
}
