<?php
namespace App\Services\Auth;

use App\Services\General\GeneralService;
use Illuminate\Support\Facades\DB;
use App\Models\Account\DeviceLog;
Use App\Models\Account\Licensee;
use App\Models\Account\User;
use App\Models\Retails\Location;
use App\Mail\UserSignUpMail;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Auth\LaraTermii;
use Carbon\Carbon;
use JWTAuth;
use Response;
use Validator;
use App\Services\Auth\RegisterService;

class RegisterService extends GeneralService
{
   
    public function validateRequest(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|unique:licensee',
            'password' => 'required|min:8|confirmed',
            'phone' => 'required|size:11|unique:licensee',
            'email' => 'required|email|unique:licensee|regex:/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/|max:255',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation failed', 'errors' => $validator->errors()], 400);
        }

        return null;
    }

    public function createLicensee($request, $currentDate, $gst_value, $salt, $pass)
    {
        return Licensee::create([
            'name' => $request->name,
            'status' => STATUS,
            'address' => $request->address,
            'start_date' => $currentDate,
            'end_date' => $currentDate,
            'email' => $request->email,
            'tm' => Carbon::now()->toTimeString(),
            'phone' => $request->phone,
            'multi' => '1 Location',
            'user_id' => STATUS,
            'source' => $request->where,
            'app' => APP_ID,
            'advisor_id' => $request->advisorId,
            'currency_sign' => $request->currency_sign,
            'timezone' => $request->timezone,
            'device_name' => $request->macAddress1,
            'license_no' => $this->generateUniqueNumber('licensee', 'license_no')
        ]);
    }

    public function createLocation($request, $currentDate, $licensee, $gst_value)
    {
        return Location::create([
            'name' => $request->name,
            'app_plan_id' => ONE_STATUS,
            'status' => STATUS,
            'date' => $currentDate,
            'email' => $request->email,
            'address' => $request->address,
            'logo' => 'camera.png',
            'tm' => Carbon::now()->toTimeString(),
            'timezone' => $request->timezone,
            'vat_value' => $gst_value,
            'phone' => $request->phone,
            'country' => $request->country,
            'client_id' => $licensee->id,
            'allowed_users' => ONE_STATUS,
            'allowed_store' => ONE_STATUS,
            'multi_store' => ONE_STATUS,
            'allowed_items' => 50,
            'subscription_status' => STATUS,
            'update_date' => $currentDate,
            'start_date' => $currentDate,
            'end_date' => $currentDate,
        ]);
    }

    public function createUser($request, $currentDate, $licensee, $location, $salt, $pass)
    {
        return User::create([
            'name' => $request->name,
            'location_id' => $location->id,
            'salt' => $salt,
            'password' => $pass,
            'username' => $request->email,
            'over_view' => ONE_STATUS,
            'deletion' => ONE_STATUS,
            'status' => STATUS,
            'discount' => ONE_STATUS,
            'up_sell' => ONE_STATUS,
            'back_date' => ONE_STATUS,
            'verification_status' => ONE_STATUS,
            'sales_priviledge' => TWO_STATUS,
            'email_verification' => ONE_STATUS,
            'master_status' => ONE_STATUS,
            'edit' => ONE_STATUS,
            'confirm_bank_pos' => ONE_STATUS,
            'sales_return' => ONE_STATUS,
            'app' => APP_ID,
            'allowed_on_desktop' => ONE_STATUS,
            'dashboard' => 'dashboard',
            'stock_cp' => ONE_STATUS,
            'bank_priviledge' => ONE_STATUS,
            'allowed_on_web' => ONE_STATUS,
            'allowed_on_mobile' => STATUS,
            'position' => 'control',
            'office' => 'control',
            'location_overview' => ONE_STATUS,
            'store_master' => ONE_STATUS,
            'store_id' => 'all',
            'date' => $currentDate,
            'tm' => $currentDate,
            'client_id' => $licensee->id,
            'update_date' => $currentDate,
        ]);
    }

    public function createDeviceLog($licensee, $location, $deviceName, $request, $user, $currentDate)
    {
        DeviceLog::create([
            'client_id' => $licensee->id,
            'location_id' => $location->id,
            'mac_address' => $deviceName,
            'mac_address_2' => $request->macAddress2,
            'user_id' => $user->id,
            'email' => $request->email,
            'timezone' => $request->timezone,
            'status' => STATUS,
            'date' => $currentDate,
        ]);
    }

    public function sendOTP($request, $userId)
    {
        $termii = new \Zeevx\LaraTermii\LaraTermii('TLYL7YLhThb8NcOaX4xpa0BtLR0GQ0LlD6exoB1EYkr5pY7uho7jhzy4g38ycQ');
        $otpDetails = $termii->sendOTP($this->standardPhone($request->phone), 'N-Alert', 'ALPHANUMERIC', 10, 5, 6, '< 123456 >', 'Your pin is < 123456 >', 'dnd');
        $decode = json_decode($otpDetails);
        $pinId = $decode->pinId;
        User::where('id', $userId)->update(['pin_id' => $pinId]);
    }

    public function sendEmailVerification($request, $userId)
    {
        $arr = ['fullname' => $request->name, 'auth_code' => '1212122'];
        Mail::to($request->email)->send(new UserSignUpMail($arr));
    }

    // You can move other private helper methods here as well.
}
