<?php

namespace App\Services\Auth;

use App\Repositories\Contracts\LocationRepositoryInterface;
use Tymon\JWTAuth\Facades\JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;
use App\Services\Contracts\TokenServiceInterface;

class TokenService implements TokenServiceInterface
{

    protected $locationRepository;


    public function __construct(LocationRepositoryInterface $locationRepository)
    {
        $this->locationRepository = $locationRepository;
    }

    /**
     * Generate access and refresh tokens for a user.
     *
     * @param  mixed  $user
     * @return array
     */
    public function generateTokens($user): array
    {

        // Ensure $user is an instance of User and implements JWTSubject
        if (!$user instanceof \Tymon\JWTAuth\Contracts\JWTSubject) {
            throw new \InvalidArgumentException('User must implement JWTSubject.');
        }

        // $subscription_status = $this->locationRepository->getSubscriptionStatus($user->client_id, $user->location_id);

        $accessToken = JWTAuth::claims([
            'user_id' => $user->id,
            'email' => $user->username,
            // 'phone' => $user->phone,
            // 'username' => $user->username2,
            // 'location_id' => $user->location_id,
            // 'name' => $user->name,
            // 'verification_status' => $user->verification_status,
            // 'subscription_status' => $subscription_status
        ])->fromUser($user);

        $refreshToken = JWTAuth::customClaims(['type' => 'refresh'])->fromUser($user);

        return [
            'accessToken' => $accessToken,
            'refreshToken' => $refreshToken,
        ];
    }



    public function generateStaticToken()
    {
        $customClaims = [
            'role' => 'admin',
            'permissions' => ['create', 'update', 'delete'],
            'app_name' => 'MyApp',
        ];

        // Create a token with custom claims
        $token = JWTAuth::claims($customClaims)->attempt([
            'email' => 'dummy@example.com',
            'password' => 'dummy_password',
        ]);

        return $token;
    }


    /**
     * Refresh the access token using a refresh token.
     *
     * @param  string  $refreshToken
     * @return string
     */
    public function refreshToken(string $refreshToken): string
    {
        try {
            $newToken = JWTAuth::setToken($refreshToken)->refresh();
            return $newToken;
        } catch (JWTException $e) {
            throw new \Exception('Unable to refresh token: ' . $e->getMessage());
        }
    }

    /**
     * Invalidate a token (logout functionality).
     *
     * @param  string  $token
     * @return bool
     */
    public function invalidateToken(string $token): bool
    {
        try {
            JWTAuth::setToken($token)->invalidate();
            return true;
        } catch (JWTException $e) {
            throw new \Exception('Unable to invalidate token: ' . $e->getMessage());
        }
    }
}
