<?php

namespace App\Services\Contracts;

interface PasswordServiceInterface
{
    /**
     * Hash a password using a salt.
     *
     * @param string $password The password to hash.
     * @param string $salt The salt to use in hashing.
     * @return string The hashed password.
     */
    public function hashPass(string $password, string $salt): string;

    /**
     * Generate a random salt.
     *
     * @return string The generated salt.
     */
    public function generateSalt(): string;

    /**
     * Verify if a password matches a hashed password when combined with a salt.
     *
     * @param string $inputPassword The password input to verify.
     * @param string $storedHash The stored hashed password to compare against.
     * @param string $salt The salt used in hashing the stored password.
     * @return bool True if the password is valid, false otherwise.
     */
    public function verifyPassword(string $inputPassword, string $storedHash, string $salt): bool;
}
