<?php

namespace App\Services;

use App\Repositories\Contracts\CurrencyRepositoryInterface;
use App\Utils\FileReaderUtility;
use App\Repositories\Contracts\CustomQueryRepositoryInterface;
use App\Repositories\Contracts\LocationRepositoryInterface;
use App\Repositories\CurrencyRepository;

class DataLoaderService
{
    protected $customQueryRepository;
    protected $locationRepository;
    protected $currencyRepository;

    public function __construct(CustomQueryRepositoryInterface $customQueryRepository, LocationRepositoryInterface $locationRepository, CurrencyRepositoryInterface $currencyRepository)
    {
        $this->customQueryRepository = $customQueryRepository;
        $this->locationRepository = $locationRepository;
        $this->currencyRepository = $currencyRepository;
    }

    /**
     * Load default data for the user.
     *
     * @param  object $user
     * @return array
     */
    public function loadDefaults($user, $origin)
    {
        return [
            // 'defaultRights' => $this->getUserRights($user,$origin),
            'defaultCurrency' => $this->getDefaultCurrency(),
            // 'defaultVendor' => $this->getDefaultVendor($user->client_id),
            // 'defaultWalkInCustomer' => $this->getDefaultWalkInCustomer($user->client_id),
            // 'defaultLocations' => $this->getDefaultLocations($user),
            // 'tables' => $this->readDatabaseFile(),
        ];
    }

    /**
     * Get user rights.
     *
     * @param  object $user
     * @return mixed
     */
    private function getUserRights($user, $origin)
    {
        $planId = $this->locationRepository->getPlanId($user->client_id, $user->location_id);

        // Define the required parameters for the query
        $params = [
            'client_id' => $user->client_id,
            'location_id' => $user->location_id,
            'user_id' => $user->id,
            'plan_id'=> $planId,
            'origin' => $origin, // Assuming the origin is part of the user data
        ];

        // Use the CustomQueryRepository to execute the 'user-rights' query
        $data = $this->customQueryRepository->executeQuery('user-rights', $params);

        // \Log::info('Request data:', ['data' => $data]);

        foreach ($data as $row) {
            if (property_exists($row, 'defaultRights')) {
                $row->defaultRights = $row->defaultRights ? json_decode(stripslashes($row->defaultRights), true) : [];
            }
        }

        return $data[0]->defaultRights ?? [];
    }

    /**
     * Get default currency.
     *
     * @return mixed
     */
    private function getDefaultCurrency()
    {
       return $this->currencyRepository->getDefaultCurrency();
        // return $this->customQueryRepository->getDefaultCurrency();
    }

    /**
     * Get default vendor for the client.
     *
     * @param  int $clientId
     * @return mixed
     */
    private function getDefaultVendor($clientId)
    {
        // return $this->customQueryRepository->getDefaultVendor($clientId);
    }

    /**
     * Get default walk-in customer for the client.
     *
     * @param  int $clientId
     * @return mixed
     */
    private function getDefaultWalkInCustomer($clientId)
    {
        // return $this->customQueryRepository->getDefaultWalkInCustomer($clientId);
    }

    /**
     * Get default locations for the user.
     *
     * @param  object $user
     * @return mixed
     */
    private function getDefaultLocations($user)
    {
        // return $this->customQueryRepository->getDefaultLocations($user->id, $user->client_id, $user->location_id);
    }

    /**
     * Read the database file.
     *
     * @return string
     */
    private function readDatabaseFile()
    {
        $fileName = env('DESKTOP_DB_FILE', 'desktopDB');
        return FileReaderUtility::readDatabaseFile($fileName);
    }
}
