<?php

namespace App\Services;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

class ExpoNotificationService
{
    protected $client;
    protected $apiUrl = 'https://exp.host/--/api/v2/push/send';

    public function __construct()
    {
        $this->client = new Client();
    }


    public function sendNotification($to, $title, $body, $data = [], $sound = 'default')
    {
        $payload = [
            'to' => $to,
            'title' => $title,
            'body' => $body,
            'sound' => $sound,
            'data' => $data,
            'priority' => 'high'
        ];

        try {
            $response = $this->client->post($this->apiUrl, [
                'headers' => [
                    'Accept' => 'application/json',
                    'Content-Type' => 'application/json',
                ],
                'json' => $payload,
            ]);

            return json_decode($response->getBody()->getContents(), true);
        } catch (RequestException $e) {
            return [
                'success' => false,
                'message' => 'Failed to send notification',
                'error' => $e->getMessage(),
            ];
        }
    }
}
