<?php

namespace App\Services;

use App\Enums\LogType;
use App\Enums\OriginEnum;
use App\Services\Contracts\LogServiceInterface;
use Illuminate\Support\Facades\Log;
use Throwable;

class LogService implements LogServiceInterface
{

    public function critical(string $process, Throwable|array $error, array $context = []): void
    {
        // Handle different types of error input
        if ($error instanceof Throwable) {
            // Extract details from the exception
            $context = array_merge($context, [
                'platform' => OriginEnum::WEB->label(),
                'exception_message' => $error->getMessage(),
                'file' => $error->getFile(),
                'line' => $error->getLine(),
                'trace' => $error->getTraceAsString(),
            ]);
        } else {
            // If $error is not an exception, merge directly into context
            $context = array_merge($context, $error);
        }

        // Add standard fields
        $context = array_merge($context, [
            'timestamp' => now()->toISOString(),
            'severity' => 'critical',
        ]);

        // Log the critical event
        $this->log('critical', $process, LogType::CRITICAL->value, $context);
    }


    public function support(string $message, string $logType,  array $context = []): void
    {
        $this->log('support', $message, $logType, $context);
    }

    protected function log(string $channel, string $message, string $logType, array $context = []): void
    {
        // Add standard fields to the context
        $context = array_merge($context, [
            'channel' => $channel,
            'severity' => $logType,   // Include the log type (e.g., critical, warning)
            'timestamp' => now()->toISOString(), // Add timestamp for consistency
        ]);

        // Determine the appropriate log level based on the log type
        $logLevel = match ($logType) {
            LogType::CRITICAL->value => 'critical',
            LogType::WARNING->value => 'warning',
            LogType::INFO->value => 'info',
            default => 'info', // Default to info for unrecognized log types
        };

        // Log the message using the determined level
        Log::channel('teams')->log($logLevel, $message, $context);
    }
}
