<?php

namespace App\Services;

class UserCacheService
{
    private static ?self $instance = null;
    private ?string $user = null; // Store the serialized user as a string

    /**
     * Private constructor to prevent direct instantiation.
     */
    private function __construct() {}

    /**
     * Get the Singleton instance.
     *
     * @return static
     */
    public static function getInstance(): self
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * Store the serialized user in the cache.
     *
     * @param object $user
     * @return void
     */
    public function setUser(object $user): void
    {
        $this->user = serialize($user);
    }

    /**
     * Retrieve and unserialize the cached user.
     *
     * @return object|null
     */
    public function getUser(): ?object
    {
        return $this->user ? unserialize($this->user) : null;
    }

    /**
     * Clear the cached user.
     *
     * @return void
     */
    public function clear(): void
    {
        $this->user = null;
    }

    /**
     * Check if a user is cached.
     *
     * @return bool
     */
    public function hasUser(): bool
    {
        return $this->user !== null;
    }
}
