<?php

use App\Http\Controllers\Account\CashLodgement\CashLodgementController;
use App\Http\Controllers\Account\ChartOfAccount\ChartOfAccountController;
use App\Http\Controllers\Account\Float\FloatController;
use App\Http\Controllers\Account\FloatDisbursement\FloatDisbursementController;
use App\Http\Controllers\Account\Log\TxnLogController;
use App\Http\Controllers\Auth\ForgotPasswordController;
use App\Http\Controllers\Auth\LoginController;
use App\Http\Controllers\Auth\LogoutController;
use App\Http\Controllers\Auth\OtpValidationController;
use App\Http\Controllers\Auth\RegisterController;
use App\Http\Controllers\Folio\Bank\BankAccountController;
use App\Http\Controllers\Folio\Bank\BankController;
use App\Http\Controllers\Folio\CustomerVendor\CustomerVendorController;
use App\Http\Controllers\Folio\Location\LocationController;
use App\Http\Controllers\General\BillingController;
use App\Http\Controllers\General\DashboardController;
use App\Http\Controllers\General\GeneralController;
use App\Http\Controllers\General\LoadDefaultController;
use App\Http\Controllers\General\SubscriptionController;
use App\Http\Controllers\General\SyncController;
use App\Http\Controllers\General\NewSyncController;
use App\Http\Controllers\Inventory\Inventory\InventoryController;
use App\Http\Controllers\Inventory\Inventory\StockAdjustmentController;
use App\Http\Controllers\Inventory\Inventory\StockRequisitionController;
use App\Http\Controllers\Inventory\Product\PriceController;
use App\Http\Controllers\Inventory\Product\ProductController;
use App\Http\Controllers\Settings\Currency\CurrencyController;
use App\Http\Controllers\Settings\Device\DeviceController;
use App\Http\Controllers\Settings\Measurement\MeasurementController;
use App\Http\Controllers\Settings\Store\StoreController;
use App\Http\Controllers\Settings\User\UserController;
use App\Http\Controllers\Transactions\Sales\ItemLogController;
use App\Http\Controllers\Transactions\Sales\SalesController;
use App\Http\Controllers\CustomerSupport\CustomerSupportController;
use App\Http\Controllers\Settings\MobileUpdateController\MobileUpdateController;
use Illuminate\Support\Facades\Route;
use App\Mail\TestEmail;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Auth\ChangePasswordController;
use App\Http\Controllers\Sync\PushController;
use App\Http\Controllers\Sync\PushControllerDev;
use App\Http\Controllers\Sync\PullController;
use App\Http\Controllers\Sync\SyncSettingController;
use App\Http\Controllers\General\SmsController;
use App\Http\Controllers\CustomerEngagement\CampaignController;
use App\Http\Controllers\CustomerEngagement\CustomerRetargetingController;
use App\Http\Controllers\CustomerEngagement\BuyingPatternController;
use App\Http\Controllers\General\NotificationController;
use App\Http\Controllers\General\ZohoCampaignController;
use App\Http\Controllers\Settings\License\LicenseController;
use App\Http\Controllers\Auth\AuthController;
use App\Http\Controllers\Web\PlansBillingsController;
use App\Http\Controllers\Web\WebDashboardController;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/


Route::get('/broadcast-notifications', [NotificationController::class, 'broadcastExpoNotification']);
Route::post('/send-expo-notification', [NotificationController::class, 'sendExpoNotification']);


Route::get('/process-queue-logs', [SyncController::class, 'processQueueLogs']);

Route::get('/get-country-timezone', [LocationController::class, 'getCountryTimezone']);


Route::get('/', function () {
    return view('welcome');
});

Route::get('/test', function () {
    return response('Testing API..');
});

Route::get('/send-test-email', function () {
    $data = ['message' => 'This is a test email.'];
    Mail::to('iadeyinka03@gmail.com')->send(new TestEmail($data));

    return 'Email sent!';
});

  //zoho
  Route::post('/zoho/oauth', [ZohoCampaignController::class, 'redirectToZoho'])->name('zoho.oauth');
  Route::get('/zoho/callback', [ZohoCampaignController::class, 'handleZohoCallback'])->name('zoho.callback');
  Route::post('/zoho/send-emails', [ZohoCampaignController::class, 'sendEmailsToZohoCampaign'])->name('zoho.sendEmails');
  

Route::post('/login', [LoginController::class, 'Login']);   //done
Route::post('/register', [RegisterController::class, 'newClient']);


Route::get('/send-zepto-mail', [RegisterController::class, 'sendTestEmail']);
Route::post('/validate-account-otp', [OtpValidationController::class, 'ValidateAccountOtp']);
Route::post('/validate-account-otp-device', [OtpValidationController::class, 'ValidateAccountOtpDevice']);
Route::post('/validate-account-otp-web', [OtpValidationController::class, 'ValidateAccountOtpWeb']);
Route::post('/forget-account-sendotp', [ForgotPasswordController::class, 'ForgotPasswordSend']);
Route::post('/update-forgot-password', [ForgotPasswordController::class, 'UpdateForgotPassword']);

Route::get('/default-table', [LoadDefaultController::class, 'getDefaultTables']);
Route::get('/subscription-plans', [SubscriptionController::class, 'getSubscriptionPlans']);
Route::get('/subscription-plans2', [SubscriptionController::class, 'getSubscriptionPlans2']);
Route::post('/subscription-summary', [SubscriptionController::class, 'subscriptionSummary']);

Route::post('/addon-summary', [SubscriptionController::class, 'addonSummary']);




Route::get('/klakpad-product', [GeneralController::class, 'getApp']);
Route::get('/dashboard-illustration', [DashboardController::class, 'dashboardIllustration']);

Route::post('/paystack-webhook', [BillingController::class, 'handlePaystackWebhook']);

// Mobile App Update
Route::get('/get-mobile-app-update', [MobileUpdateController::class, 'getMobileAppUpdate']);

Route::post('/insert-customer-campaign', [CampaignController::class, 'insertCustomerCampaign']);

Route::get('/get-access-token', [RegisterController::class, 'accessToken']);

Route::get('/get-cell-range', [RegisterController::class, 'getCellRange']);

Route::post('/new-client-excel', [RegisterController::class, 'newClientExcel']);

Route::post('/registrants', [LicenseController::class, 'registrants']);

 // Authentication Routes
Route::prefix('auth')->group(function () {
    // Login route
    Route::post('/login', [AuthController::class, 'login']);

    // Logout route - protected by middleware
    Route::post('/logout', [AuthController::class, 'logout'])->middleware('jwtorcookie.auth');

    // Session route - protected by middleware
    Route::get('/session', [AuthController::class, 'checkSession'])->middleware('jwtorcookie.auth');

    // Refresh token route - protected by middleware
    Route::post('/refresh', [AuthController::class, 'refresh']);

    // Refresh token route - protected by middleware
    Route::post('/signup', [AuthController::class, 'register']);
});





//middleware
Route::group(['middleware' => ['jwt.verify']], function () {
    Route::post('/load-default-segment', [LoadDefaultController::class, 'loadDefaultSegment']);
    Route::post('/logout', [LogoutController::class, 'logout']);

    Route::post('/edit-phone-number', [RegisterController::class, 'editPhoneNumber']);

    // Route::group(['middleware' => ['jwt.verify', 'verified']], function () {

    Route::get('/dashboard', [DashboardController::class, 'dashboard']);
    Route::get('/web-dashboard', [DashboardController::class, 'webDashboardData']);

    //account
    Route::get('/get-all-base-measurement', [MeasurementController::class, 'getAllBaseMeasurement']);
    Route::get('/get-non-base-measurement/{id}', [MeasurementController::class, 'getNonBaseMeasurement']);
    Route::get('/get-default-measurement', [MeasurementController::class, 'getDefaultMeasurement']);
    Route::get('/get-default-measurement-category', [MeasurementController::class, 'getDefaultMeasurementCategory']);
    Route::patch('/set-primary-measurement', [MeasurementController::class, 'setPrimaryMeasurement']);
    Route::get('/get-measurement-by-category', [MeasurementController::class, 'getMeausrementByCategory']);
    

    Route::get('/get-revenue-accounts', [ChartOfAccountController::class, 'getRevenueAccounts']);
    Route::get('/get-account-type', [ChartOfAccountController::class, 'getAccountTypes']);
    Route::get('/get-stock-adjustment-accounts', [ChartOfAccountController::class, 'getStockAdjustMentAccounts']);

    //cash lodge
    Route::post('/lodge-cash', [CashLodgementController::class, 'lodgeCash']);
    Route::get('/all-cash-lodgement', [CashLodgementController::class, 'getAllCashLodgement']);
    Route::patch('/lodge-cash/{id?}', [CashLodgementController::class, 'editCashLodge']);
    Route::delete('/lodge-cash/{id?}', [CashLodgementController::class, 'deleteCashLodge']);

    Route::post('/disburse-cash', [FloatDisbursementController::class, 'disburseCash']);
    Route::post('/all-cash-disbursement', [FloatDisbursementController::class, 'getAllCashDisbursement']);
    Route::post('/all-disbursement-by-location', [FloatDisbursementController::class, 'getGroupedCashDisbursementByLocation']);
    Route::post('/all-disbursement-by-location/{id?}', [FloatDisbursementController::class, 'getFloatDisbursementByLocation']);
    

    Route::get('/currencyandaccount-for-float', [FloatController::class, 'currencyAndAccForFloat']);
    Route::post('/post-float', [FloatController::class, 'postFloat']);
    Route::get('/get-float', [FloatController::class, 'getFloat']);
    Route::get('/get-cash', [FloatController::class, 'getCash']);
    Route::patch('/edit-float/{id?}', [FloatController::class, 'editFloat']);
    Route::get('/get-float-balance', [FloatController::class, 'getFloatBalance']);
    Route::post('/change-password', [ChangePasswordController::class, 'changePassword']);

    //product
    Route::post('/product-category', [ProductController::class, 'createProductCategory']);
    Route::patch('/product-category/{id?}', [ProductController::class, 'editProductCategory']);
    Route::post('/get-product-category', [ProductController::class, 'getIProductCategory']);
    Route::get('/product-category-dropdown', [ProductController::class, 'productCatDropdown']);
    Route::get('/product-dropdown', [ProductController::class, 'productDropdown']);
    Route::get('/get-product-data', [ProductController::class, 'getProductData']);
    Route::post('/product-list', [ProductController::class, 'productList']);
    Route::patch('/delete-product-category/{id}', [ProductController::class, 'deleteProductCategory']);
    Route::post('/product', [ProductController::class, 'createProduct']);
    Route::patch('/product/{id?}', [ProductController::class, 'deleteProduct']);
    Route::patch('/edit-product/{id?}', [ProductController::class, 'editProduct']);
    Route::get('/product/{id?}', [ProductController::class, 'getProductById']);
    Route::get('/product', [ProductController::class, 'getAllProducts']);
    Route::patch('/toggle-product-visibility/{id}/{disabled_status}', [ProductController::class, 'toglleProductVisibility']);

    // price
    Route::get('/product-price-info/{id?}', [PriceController::class, 'getProductPriceInfo']);

    // customer
    Route::post('/add-customer', [CustomerVendorController::class, 'addCustomer']);
    Route::patch('/edit-customer/{id?}', [CustomerVendorController::class, 'editCustomer']);
    Route::patch('/delete-customer/{id?}', [CustomerVendorController::class, 'deleteCustomer']);
    Route::get('/customer/{id?}', [CustomerVendorController::class, 'getCustomerbyId']);
    Route::get('/customer-dropdown', [CustomerVendorController::class, 'customerDropdown']);
    Route::get('/get-all-customer-dropdown', [CustomerVendorController::class, 'getAllCustomerDropdown']);
    Route::get('/get-all-customer', [CustomerVendorController::class, 'getAllCustomers']);

    Route::get('/get-all-customer-with-bankacc', [CustomerVendorController::class, 'getAllCustomersWithBankAccounts']);

    Route::post('/make-payment', [CustomerVendorController::class, 'makePayment']);
    Route::post('/customer-balance/{id?}', [CustomerVendorController::class, 'getCustomerBalance']);
    Route::post('/customer-transaction/{id?}', [CustomerVendorController::class, 'getCustomerTransactionList']);
    Route::post('/customer-purchases/{id?}', [CustomerVendorController::class, 'getCustomerPurchaseList']);
    Route::post('/vendor-purchases/{id?}', [CustomerVendorController::class, 'getVendorPurchaseList']);

    //measurement
    Route::get('/get-base-measurement', [MeasurementController::class, 'getBaseMeasurement']);
    Route::get('/get-all-measurement-dropdown', [MeasurementController::class, 'getAllMeausrementDropdown']);
    Route::get('/get-measurement-by-category-dropdown/{m_category_id?}', [MeasurementController::class, 'getMeausrementByCategoryDropdown']);
    Route::get('/get-measurement-by-category-and-metric-dropdown/{m_category_id?}/{item_id?}', [MeasurementController::class, 'getMeausrementByCategoryAndMetricDropdown']);
    Route::get('/get-measurements', [MeasurementController::class, 'getMeasurementInfo']);
    Route::get('/get-measurement-category', [MeasurementController::class, 'getMeasurementCategoryInfo']);
    

    // inventory
    Route::post('/save-single-inventory', [InventoryController::class, 'saveSingleInventory']);
    Route::post('/save-inventory', [InventoryController::class, 'saveInventory']);
    Route::post('/get-inventory/{item_id?}', [InventoryController::class, 'getOptimisedInventory']);
    Route::post('/get-inventory-dependencies/{item_id?}', [InventoryController::class, 'getInventoryDependencies']);
    // Route::post('/get-optimised-inventory/{item_id?}', [InventoryController::class, 'getOptimisedInventory']);
    Route::post('/get-stock-movement', [InventoryController::class, 'getStockMovement']);
    Route::post('/get-purchase-log', [InventoryController::class, 'getPurchaseLog']);
    Route::post('/get-cost-price/{item_id?}', [InventoryController::class, 'itemAverageCostPrice']);
    Route::patch('/delete-single-inventory/{id}', [InventoryController::class, 'deleteSinglePurchaseLog']);
    Route::post('/get-qty-balance/{id}', [InventoryController::class, 'getQtyBalance']);
    Route::post('/get-cog-list', [InventoryController::class, 'cogList']);
    Route::post('/get-cog-summary', [InventoryController::class, 'cogSummary']);
    Route::post('/create-inventory-template', [InventoryController::class, 'createInventoryTemplate']);
    Route::post('/upload-inventory', [InventoryController::class, 'uploadInventory']);
    Route::get('/inventory-summary', [InventoryController::class, 'stockSummary']);
    Route::get('/inventory-data', [InventoryController::class, 'stockData']);
    Route::get('/current-market-pricing', [InventoryController::class, 'currentMarketingPrice']);
    
    

    // stock requisition
    Route::post('/send-request', [StockRequisitionController::class, 'sendRequest']);
    Route::post('/get-request', [StockRequisitionController::class, 'fetchRequest']);
    Route::patch('/cancel-request', [StockRequisitionController::class, 'cancelRequest']);
    Route::patch('/decline-request', [StockRequisitionController::class, 'declineRequest']);
    Route::patch('/edit-request', [StockRequisitionController::class, 'editRequest']);
    Route::post('/release-request', [StockRequisitionController::class, 'releaseRequest']);
    Route::post('/accept-request', [StockRequisitionController::class, 'acceptRequest']);
    Route::post('/get-fulfilled', [StockRequisitionController::class, 'fetchFulfilled']);
    Route::post('/get-released', [StockRequisitionController::class, 'fetchReleased']);
    Route::post('/stock-requisition-summary', [StockRequisitionController::class, 'stockRequisitionSummary']);

    // stock adjustment
    Route::post('/save-stock-adjustment', [StockAdjustmentController::class, 'saveStockAdjustment']);
    Route::post('/get-stock-adjustment', [StockAdjustmentController::class, 'getStockAdjustment']);

    //banks
    Route::get('/get-banks', [BankController::class, 'getBanks']);

    //bank account
    Route::post('/save-bank-account', [BankAccountController::class, 'saveBankAccount']);
    Route::get('/get-bank-account', [BankAccountController::class, 'getBankAccounts']);
    Route::patch('/edit-bank-account/{id}', [BankAccountController::class, 'editBankAccount']);
    Route::get('/get-bank-summary', [BankAccountController::class, 'getBankSummary']);
    Route::get('/get-bank-account-statement/{id}', [BankAccountController::class, 'getBankAccountStatement']);
    Route::patch('/delete-bank-account/{id}', [BankAccountController::class, 'deletebank']);
    Route::get('/get-single-bank-account/{id}', [BankAccountController::class, 'getSingleBankAccount']);

    Route::get('/resolve-bank', [BillingController::class, 'resolveBankAccount']);


    Route::get('/get-bank-account-dropdown', [BankAccountController::class, 'getBankAccountDropdown']);

    Route::get('/get-bank-details', [BankAccountController::class, 'getBankDetails']);

    // location

    Route::get('/location-dropdown', [LocationController::class, 'locationDropdown']);
    Route::post('/get-locations', [LocationController::class, 'getLocations']);
    Route::post('/get-one-location/{id}', [LocationController::class, 'getOneLocation']);
    Route::post('/get-location-summary', [LocationController::class, 'getLocationSummary']);
    Route::post('/add-location', [LocationController::class, 'addLocation']);
    Route::post('/edit-location', [LocationController::class, 'editLocation']);
    Route::post('/get-date-time-format-zone', [LocationController::class, 'getDateTimeFormatZone']);
    Route::post('/set-inventory-evaluation', [LocationController::class, 'setInventoryEvaluation']);

    // store
    Route::get('/store-dropdown', [StoreController::class, 'storeDropdown']);

    // currency
    Route::get('/currency-dropdown', [CurrencyController::class, 'currencyDropdown']);
    Route::get('/get-primary-currency', [CurrencyController::class, 'getPrimaryCurrencyInfo']);


    // sales
    // Route::get('/get-sales-items', [SalesController::class, 'getSalesItems']);
    Route::post('/get-sales-items', [SalesController::class, 'overAllSalesItems']);
    Route::post('/sales', [SalesController::class, 'sales']);

    //users
    Route::get('/menu', [UserController::class, 'getModulesWithMenus']);

    Route::get('/getRegisterUserProcess', [UserController::class, 'registerUserProcess']);

    Route::get('/get-desktop-user-rights', [UserController::class, 'getDesktopAllUsersByClientId']);

    Route::post('/add-user', [UserController::class, 'addUser']);
    Route::patch('/edit-user/{id}', [UserController::class, 'editUser']);
    Route::get('/user/user-dropdown', [UserController::class, 'userDropdown']);
    Route::get('/user/{id}', [UserController::class, 'getUserById']);
    Route::get('/get-user-rights', [UserController::class, 'getUserRights']);
    Route::get('/all-users-client', [UserController::class, 'getAllUsersByClientId']);
    Route::patch('/toggle-user-status/{id}', [UserController::class, 'toggleUserStatus']);
    Route::get('/get-user-desktop-rights', [UserController::class, 'getUserDesktopRights']);
    Route::get('/desktop-rights', [UserController::class, 'getDesktopRights']);

    // general
    Route::patch('/undo-transaction', [GeneralController::class, 'undoTransaction']);
    Route::get('/get-default-currency', [GeneralController::class, 'getDefaultCurrency']);
    Route::post('/send-email-message', [GeneralController::class, 'sendEmailMessage']);






    //sync
    Route::post('/sync', [SyncController::class, 'sync']);
    Route::post('/new-sync', [NewSyncController::class, 'sync']);
    Route::get('/get-sync-setting', [SyncSettingController::class, 'getSynchronization']);
    Route::post('/set-sync-setting', [SyncSettingController::class, 'setSynchronization']); 

    // Push
    Route::post('/push', [PushController::class, 'pushData']);

    Route::post('/push-copy', [PushControllerDev::class, 'pushDataCopy']);

    // Pull
    Route::post('/pull', [PullController::class, 'pullData']);
    Route::get('/data-to-pull/{cloud_id}', [PullController::class, 'dataToPull']);
    
    // Sales list
    Route::post('/get-sales-list', [ItemLogController::class, 'salesList']);
    Route::post('/get-sales-summary', [ItemLogController::class, 'salesSummary']);
    Route::post('/get-payment-analysis', [TxnLogController::class, 'paymentAnalysis']);
    Route::post('/get-single-payment-analysis', [TxnLogController::class, 'singlePaymentAnalysis']);

    // Device log
    Route::get('/get-devices', [DeviceController::class, 'getDevices']);
    Route::patch('/set-device/{id}', [DeviceController::class, 'setDevice']);
    Route::patch('/set-device-name/{id}', [DeviceController::class, 'setDeviceName']);
    

    //billing
    Route::post('/initialise-payment', [BillingController::class, 'initialisePaystackPayment']);
    Route::post('/topup-wallet', [BillingController::class, 'topupWallet']);
    Route::post('/check-coupon', [BillingController::class, 'confirmCoupon']);
    Route::get('/transaction-history', [BillingController::class, 'getTransactionHistory']);
    Route::get('/subsribed-history', [BillingController::class, 'fetchLocations']);

    

    

    //subscription
    Route::post('/subscribe-with-wallet', [SubscriptionController::class, 'subscribeWithWallet']);
    //subscription
    Route::post('/subscribe-for-free', [SubscriptionController::class, 'subscribeForFree']);
    
    Route::post('/validate-card-payment', [SubscriptionController::class, 'validatePaymentWithCards']);
    Route::post('/subscribe-with-card', [SubscriptionController::class, 'subscribeWithCard']);
    Route::get('/client-location', [SubscriptionController::class, 'getClientLocations']);


    Route::get('/sms-history', [SmsController::class, 'SmsHistory']);
    Route::post('/buy-sms', [SmsController::class, 'buySms']);
    Route::post('/send-sms-message', [SmsController::class, 'sendSMSMessage']);

    

    Route::get('/remote-queue', [SyncController::class, 'getRemoteQuAccess']);
    Route::get('/copy-queuelog-row', [SyncController::class, 'copyMissingQLRow']);

    // Customer Support
    Route::patch('/update-item-as-needed/{item_id}', [CustomerSupportController::class, 'updateItemAsNeeded']);
    Route::post('/update-item-sales-type', [CustomerSupportController::class, 'updateItemSalesType']);
    Route::patch('/update-stock-as-needed/{id}', [CustomerSupportController::class, 'updateStockAsNeeded']);
    Route::patch('/update-stock-adjustment-as-needed/{id}', [CustomerSupportController::class, 'updateStockAdjustmentAsNeeded']);
    Route::patch('/update-txn-log-dr-as-needed/{id}', [CustomerSupportController::class, 'updateTxnLogDrAsNeeded']);
    Route::patch('/update-txn-log-cr-as-needed/{id}', [CustomerSupportController::class, 'updateTxnLogCrAsNeeded']);
    Route::post('/contact-us', [CustomerSupportController::class, 'contactUs']);
    Route::post('/sales-txn-log', [CustomerSupportController::class, 'saveTxnLogAsNeeded']);
    Route::post('/update-txn-log', [CustomerSupportController::class, 'updateTxnLogAsNeeded']);
    Route::post('/generate-support-link-auth', [CustomerSupportController::class, 'generateSupportLinkAuth']);
    
    

    // Customer Engagement
    Route::post('/load-customer-campaign', [CampaignController::class, 'loadCustomerCampaign']);
    Route::post('/save-customer-campaign', [CampaignController::class, 'saveCustomerCampaign']);
    Route::get('/get-customer-campaign', [CampaignController::class, 'getCustomerCampaign']);
    Route::post('/send-email-to-patronage', [CampaignController::class, 'sendEmailToPatronage']);
    Route::post('/send-sms-to-patronage', [CampaignController::class, 'sendSmsToPatronage']);
    Route::post('/send-email-to-customer-retargeting', [CampaignController::class, 'sendEmailToCustomerRetargeting']);
    Route::post('/send-sms-to-customer-retargeting', [CampaignController::class, 'sendSmsToCustomerRetargeting']);
    Route::post('/send-email-to-all-customers', [CampaignController::class, 'sendEmailToAllCustomers']);
    Route::post('/send-sms-to-all-customers', [CampaignController::class, 'sendSmsToAllCustomers']);

    Route::post('/get-customers-buying-patterns', [BuyingPatternController::class, 'getBuyingPatterns']);
    Route::post('/get-customer-buying-patterns', [BuyingPatternController::class, 'getCustomerBuyingPatterns']);
    Route::post('/get-customer-most-items', [BuyingPatternController::class, 'getCustomerMostBoughtItems']);
    Route::post('/get-customer-most-visited', [BuyingPatternController::class, 'getCustomerMostVisitedLocation']);

    Route::post('/create-segment', [CustomerRetargetingController::class, 'createSegment']);
    Route::get('/get-segment', [CustomerRetargetingController::class, 'getSegment']);
    Route::post('/get-customer-retargeting', [CustomerRetargetingController::class, 'getCustomerRetargeting']);
    Route::post('/send-email-to-targeted-customers', [CustomerRetargetingController::class, 'sendEmailToTargetedCustomers']);
    Route::post('/send-sms-to-targeted-customers', [CustomerRetargetingController::class, 'sendSMSToTargetedCustomers']);


});



Route::group([
    'prefix' => 'web',
    'middleware' => 'jwtorcookie.auth',
], function () {
    Route::get('/todos', [WebDashboardController::class, 'todoNotifications']);
    Route::get('/subscription-plans', [WebDashboardController::class, 'userSubscriptionPlans']);
    Route::get('/client-users', [WebDashboardController::class, 'clientUsers']);
    Route::get('/user-wallet', [WebDashboardController::class, 'userWallet']);
    Route::get('/billings/{appId?}', [PlansBillingsController::class, 'billingHistory']);
    Route::get('/plans/{appId?}', [PlansBillingsController::class, 'planHistory']);
});

